@@ -3,7 +3,7 @@
 // GkClient.h
 //
 // Copyright (c) Citron Network Inc. 2001-2003
-// Copyright (c) 2002-2013, Jan Willamowius
+// Copyright (c) 2002-2017, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -13,7 +13,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef GKCLIENT_H
-#define GKCLIENT_H "@(#) $Id: GkClient.h,v 1.56 2013/10/07 18:27:06 willamowius Exp $"
+#define GKCLIENT_H "@(#) $Id$"
 
 #include "gk_const.h"
 #include "Toolkit.h"
@@ -48,12 +48,15 @@ class SignalingMsg;
 template <class> class H225SignalingMsg;
 typedef H225SignalingMsg<H225_Setup_UUIE> SetupMsg;
 
+bool IsOIDForAlgo(const PString & oid, const PCaselessString & algo);
+
 #ifdef HAS_H46023
 class H460_FeatureStd;
 class STUNClient;
 class UDPProxySocket;
 class H46024B_AlternateAddress;
 class H46024B_ArrayOf_AlternateAddress;
+
 class CallH46024Sockets
 {
 public:
@@ -76,7 +79,7 @@ protected:
 
 typedef std::map<H225_CallIdentifier, std::list<CallH46024Sockets> > GkNATSocketMap;
 
-#endif
+#endif // HAS_H46023
 
 class GkClient {
 public:
@@ -97,21 +100,21 @@ public:
 	bool UseTLS() const { return m_useTLS; }
 
 	bool UsesAdditiveRegistration() const;
-	bool AdditiveRegister(H225_ArrayOf_AliasAddress & aliases, int & rejectReason, 
+	bool AdditiveRegister(H225_ArrayOf_AliasAddress & aliases, int & rejectReason,
 						H225_ArrayOf_ClearToken * tokens, H225_ArrayOf_CryptoH323Token * cryptotokens);
 	bool AdditiveUnRegister(const H225_ArrayOf_AliasAddress & aliases);
 	void AppendLocalAlias(const H225_ArrayOf_AliasAddress & aliases);
 	void RemoveLocalAlias(const H225_ArrayOf_AliasAddress & aliases);
 
-	bool OnSendingGRQ(H225_GatekeeperRequest &grq);
-	bool OnSendingRRQ(H225_RegistrationRequest &rrq);
-	bool OnSendingARQ(H225_AdmissionRequest &arq, Routing::AdmissionRequest &req);
-	bool OnSendingLRQ(H225_LocationRequest &lrq, Routing::LocationRequest &req);
-	bool OnSendingARQ(H225_AdmissionRequest &arq, Routing::SetupRequest &req, bool answer = false);
-	bool OnSendingARQ(H225_AdmissionRequest &arq, Routing::FacilityRequest &req);
-	bool OnSendingDRQ(H225_DisengageRequest &drq, const callptr &call);
-	bool OnSendingURQ(H225_UnregistrationRequest &urq);
-	
+	bool OnSendingGRQ(H225_GatekeeperRequest & grq);
+	bool OnSendingRRQ(H225_RegistrationRequest & rrq);
+	bool OnSendingARQ(H225_AdmissionRequest & arq, Routing::AdmissionRequest & req);
+	bool OnSendingLRQ(H225_LocationRequest & lrq, Routing::LocationRequest & req);
+	bool OnSendingARQ(H225_AdmissionRequest & arq, Routing::SetupRequest & req, bool answer = false);
+	bool OnSendingARQ(H225_AdmissionRequest & arq, Routing::FacilityRequest & req);
+	bool OnSendingDRQ(H225_DisengageRequest & drq, const callptr & call);
+	bool OnSendingURQ(H225_UnregistrationRequest & urq);
+
 	bool SendARQ(Routing::AdmissionRequest &);
 	bool SendLRQ(Routing::LocationRequest &);
 	bool SendARQ(Routing::SetupRequest &, bool answer = false);
@@ -124,16 +127,16 @@ public:
 
 	bool RewriteE164(H225_AliasAddress & alias, bool);
 	bool RewriteE164(H225_ArrayOf_AliasAddress & alias, bool);
-	bool RewriteE164(SetupMsg &setup, bool);
+	bool RewriteE164(SetupMsg & setup, bool);
 
 	/** Fills LRQ with approtiation tokens/cryptoTokens containing
-		configured username/password data. 
-		Declared outside SetPassword template because it should not 
+		configured username/password data.
+		Declared outside SetPassword template because it should not
 		depend on authMode.
 	*/
 	void SetNBPassword(
-		H225_LocationRequest& lrq, /// LRQ message to be filled with tokens
-		const PString& id // username to be put inside tokens/cryptoTokens
+		H225_LocationRequest & lrq, /// LRQ message to be filled with tokens
+		const PString & id // username to be put inside tokens/cryptoTokens
 		);
 
 	/** Fills LRQ with approtiation tokens/cryptoTokens containing,
@@ -141,26 +144,31 @@ public:
 		section.
 	*/
 	void SetNBPassword(
-		H225_LocationRequest& lrq /// LRQ message to be filled with tokens
+		H225_LocationRequest & lrq /// LRQ message to be filled with tokens
 		)
 	{
         SetPassword(lrq);
 	}
-		
+
 	template<class RAS> void SetPassword(RAS & rasmsg, const PString & id)
 	{
 		for (PINDEX i = 0; i < m_h235Authenticators->GetSize();  i++) {
 			H235Authenticator * authenticator = (H235Authenticator *)(*m_h235Authenticators)[i].Clone();
 
-			if (authenticator && authenticator->IsSecuredPDU(rasmsg.GetTag(), FALSE)) {
-				authenticator->SetLocalId(id);
-				authenticator->SetPassword(m_password);
+			if (authenticator) {
+                authenticator->SetLocalId(id);
+                authenticator->SetPassword(m_password);
+                if (IsOIDForAlgo(m_authAlgo.AsString(), authenticator->GetName())
+                        && authenticator->IsSecuredPDU(rasmsg.GetTag(), FALSE)) {
+                    authenticator->SetLocalId(id);
+                    authenticator->SetPassword(m_password);
 
-				if (authenticator->PrepareTokens(rasmsg.m_tokens, rasmsg.m_cryptoTokens)) {
-					PTRACE(4, "GKClient\tPrepared PDU with authenticator " << authenticator);
-				}
-			}
-			delete authenticator;
+                    if (authenticator->PrepareTokens(rasmsg.m_tokens, rasmsg.m_cryptoTokens)) {
+                        PTRACE(4, "GKClient\tPrepared PDU with authenticator " << authenticator);
+                    }
+                }
+                delete authenticator;
+            }
 		}
 		if (rasmsg.m_tokens.GetSize() > 0)
 			rasmsg.IncludeOptionalField(RAS::e_tokens);
@@ -168,6 +176,7 @@ public:
 		if (rasmsg.m_cryptoTokens.GetSize() > 0)
 			rasmsg.IncludeOptionalField(RAS::e_cryptoTokens);
 	}
+
 	template<class RAS> void SetPassword(RAS & rasmsg)
 	{
 		SetPassword(rasmsg, m_h323Id.GetSize() > 0 ? m_h323Id[0] :
@@ -175,6 +184,12 @@ public:
 			);
 	}
 
+private:
+	/* No copy constructor allowed */
+	GkClient(const GkClient &);
+	/* No operator= allowed */
+	GkClient & operator=(const GkClient &);
+
 private:
 	bool Discovery();
 	void Register();
@@ -195,11 +210,13 @@ private:
 	bool OnBRQ(RasMsg *);
 	bool OnIRQ(RasMsg *);
 
-	bool RewriteString(PString &, bool) const;
+	bool RewriteString(PString & alias, bool fromInternal) const;
 	void SetClearTokens(H225_ArrayOf_ClearToken &, const PString &);
 	void SetCryptoTokens(H225_ArrayOf_CryptoH323Token &, const PString &);
-	void SetRasAddress(H225_ArrayOf_TransportAddress &);
-	void SetCallSignalAddress(H225_ArrayOf_TransportAddress &);
+	void SetRasAddress(H225_ArrayOf_TransportAddress & addr);
+	void SetRasAddress(H225_TransportAddress & addr);
+	void SetCallSignalAddress(H225_ArrayOf_TransportAddress & addr);
+	void SetCallSignalAddress(H225_TransportAddress & addr);
 
 	RasServer *m_rasSrv;
 
@@ -232,23 +249,23 @@ private:
 
 	AlternateGKs *m_gkList;
 	bool m_useAltGKPermanent;
-	int m_authMode;
+	PASN_ObjectId m_authAlgo;
 
 	Toolkit::RewriteData *m_rewriteInfo;
 
 	GkClientHandler *m_handlers[4];
 
 	NATClient *m_natClient;
-	
+
 	enum ParentVendors {
 		ParentVendor_Generic,
 		ParentVendor_GnuGk,
 		ParentVendor_Cisco
 	};
-	
+
 	/// vendor of the parent gatekeeper
 	int m_parentVendor;
-	
+
 	enum EndpointTypes {
 		EndpointType_Terminal,
 		EndpointType_Gateway
@@ -266,6 +283,8 @@ private:
 	/// list of Authenticators
 	H235Authenticators * m_h235Authenticators;
 
+    // enable GnuGk's old NAT traversal method
+    bool m_enableGnuGkNATTraversal;
 	// enable H.460.18 (offer to parent)
 	bool m_enableH46018;
 	// registered with H.460.18 support
@@ -304,7 +323,7 @@ public:
 	// NAT type detected
 	void H46023_TypeDetected(int nattype);
     // Create socket pair
-	bool H46023_CreateSocketPair(const H225_CallIdentifier & id, WORD sessionID, UDPProxySocket * & rtp, UDPProxySocket * & rtcp, bool & nated);
+	bool H46023_CreateSocketPair(const H225_CallIdentifier & id, PINDEX callNo, WORD sessionID, UDPProxySocket * & rtp, UDPProxySocket * & rtcp, bool & nated);
 	// Set the NAT Strategy
 	void H46023_SetNATStategy(const H225_CallIdentifier & id, unsigned nat);
 	// Find the NAT Strategy
@@ -312,9 +331,9 @@ public:
 	// Set the socketPair
 	void H46023_SetSocketPair(const H225_CallIdentifier & id, WORD sessionID, UDPProxySocket * rtp, UDPProxySocket * rtcp);
 	// Set Alternates (Annex A)
-	void H46023_SetAlternates(const H225_CallIdentifier & id, WORD session, PString cui, 
+	void H46023_SetAlternates(const H225_CallIdentifier & id, WORD session, PString cui,
 							unsigned muxID, H323TransportAddress m_rtp, H323TransportAddress m_rtcp);
-	void H46023_LoadAlternates(const H225_CallIdentifier & id, WORD session, PString & cui, 
+	void H46023_LoadAlternates(const H225_CallIdentifier & id, WORD session, PString & cui,
 							unsigned & muxID, H323TransportAddress & m_rtp, H323TransportAddress & m_rtcp);
 	// Set Alternates (Annex B)
 	void H46023_SetAlternates(const H225_CallIdentifier & id, const H46024B_ArrayOf_AlternateAddress & alternates);
