@@ -3,7 +3,7 @@
 // GkStatus.h	thread listening for connections to receive
 //		status updates from the gatekeeper
 //
-// Copyright (c) 2000-2011, Jan Willamowius
+// Copyright (c) 2000-2018, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -13,9 +13,10 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef GKSTATUS_H
-#define GKSTATUS_H "@(#) $Id: GkStatus.h,v 1.45 2013/11/07 13:08:10 willamowius Exp $"
+#define GKSTATUS_H "@(#) $Id$"
 
 #include <map>
+#include <list>
 #include "yasocket.h"
 #include "singleton.h"
 
@@ -43,24 +44,26 @@ public:
 	GkStatus();
 	virtual ~GkStatus();
 
+    virtual void LoadConfig();
+
 	/** Authenticate new telnet connection. Ask for username/password
 		if required. If the authentication is successful, the client
 		is added to the list of active clients. If it fails, client
 		instance is deleted.
-	*/	
+	*/
 	void AuthenticateClient(
 		/// new status interface client to be authenticated
-		StatusClient* newClient
+		StatusClient * newClient
 		);
 
-	/** Broadcast the message to all active client connected 
+	/** Broadcast the message to all active client connected
 		to the status interface. The message is sent to each client
 		only if client trace level is greater or equal to the trace level
 		of the message.
 	*/
 	void SignalStatus(
 		/// message string to be broadcasted
-		const PString& msg, 
+		const PString & msg,
 		/// trace level at which the message should be broadcasted
 		/// if the current output trace level is less than this value,
 		/// the client will not receive this message
@@ -68,7 +71,7 @@ public:
 		);
 
 	/** Disconnect the specified status interface client.
-		
+
 		@return
 		true if the status interface client with the given session ID
 		has been found and disconnected.
@@ -90,15 +93,19 @@ public:
 	*/
 	void ShowUsers(
 		/// client that requested the list of all active clients
-		StatusClient* requestingClient
+		StatusClient * requestingClient
 		) const;
 
-	/** Print help (list of status interface commands) 
+    /** Print the event backlog to the requesting client.
+    */
+    void PrintEventBacklog(StatusClient * requestingClient) const;
+
+	/** Print help (list of status interface commands)
 		to the requesting client.
 	*/
 	void PrintHelp(
 		/// client that requested the help message
-		StatusClient* requestingClient
+		StatusClient * requestingClient
 		) const;
 
 	enum StatusInterfaceCommands {
@@ -117,9 +124,10 @@ public:
 		e_DisconnectEndpoint,          /// disconnect a call by endpoint ID
 		e_DisconnectSession,           /// disconnect a user from status port
 		e_ClearCalls,                  /// disconnect all calls
-		e_UnregisterAllEndpoints,      /// force unregisterung of all andpoints
-		e_UnregisterIp,                /// force unregisterung of one andpoint by IP number
-		e_UnregisterAlias,             /// force unregisterung of one andpoint by alias
+		e_UnregisterAllEndpoints,      /// force unregistering of all endpoints
+		e_UnregisterIp,                /// force unregistering of one endpoint by IP number
+		e_UnregisterEP,                /// force unregistering of one endpoint by endpoint ID
+		e_UnregisterAlias,             /// force unregistering of one endpoint by alias
 		e_TransferCall,                /// transfer call from one endpoint to another
 		e_RerouteCall,                 /// transfer call with pause and reroute
 		e_MakeCall,                    /// establish a new call from endpoint A to endpoint B
@@ -135,7 +143,9 @@ public:
 		e_Shutdown,                    /// Shutdown the program
 		e_RouteToAlias,                /// Route a call to a specified alias
 		e_RouteToGateway,              /// Route a call to a specified alias + destinationCallSignalAddr
+		e_RouteToInternalGateway,      /// Route a call to a specified alias + destinationCallSignalAddr, but don't let caller know about new destination
 		e_BindAndRouteToGateway,       /// Route a call to a specified alias + destinationCallSignalAddr + specify bind IP
+		e_BindAndRouteToInternalGateway, /// Route a call to a specified alias + destinationCallSignalAddr + specify bind IP, but don't let caller know about new destination
 		e_RouteReject,                 /// Reject to Route a call upon ARQ (send ARJ)
 		e_Trace,                       /// change trace level for status interface output
 		e_RotateLog,                   /// Force log file rotation
@@ -155,22 +165,28 @@ public:
 		e_ResetCallCounters,           /// Reset the call counters
 		e_PrintEndpointQoS,            /// Print QoS values for all endpoints
 		e_PrintAllConfigSwitches,      /// print all known config switches
+		e_PrintEventBacklog,           /// print buffered events
+		e_PrintNeighbors,              /// print list of neighbors
+		e_PrintCallInfo,               /// print detailed information for a call
+		e_MaintenanceMode,             /// switch in or out of maintenance mode
+		e_GetLicenseStatus,            /// get license status
+		e_GetServerID,                 /// get server ID
 		e_numCommands
 		/// Number of different strings
 	};
 
 	/** Parse the text message into the status interface command.
-	
+
 		@return
-		The command code (see #StatusInterfaceCommands enum#) 
+		The command code (see #StatusInterfaceCommands enum#)
 		and 'args' filled with command tokens
 		or -1 if no corresponding command has been found.
 	*/
 	int ParseCommand(
 		/// message to be parsed
-		const PString& msg,
+		const PString & msg,
 		/// message split into tokens upon successful return
-		PStringArray& args
+		PStringArray & args
 		);
 
 protected:
@@ -186,6 +202,12 @@ protected:
 
 	unsigned m_statusClients;
 	unsigned m_maxStatusClients;
+
+	// event backlog
+	std::list<PString> m_eventBacklog;
+	PMutex m_eventBacklogMutex;
+	unsigned m_eventBacklogLimit;
+	PRegularExpression m_eventBacklogRegex;
 };
 
 /** Listen for incoming connections to the status interface port
@@ -199,11 +221,11 @@ public:
 	/// create the new listener socket
 	StatusListener(
 		/// address the socket is to be bound to
-		const Address& addr, 
+		const Address & addr,
 		/// port number the socket is to be bound to
 		WORD port
 		);
-		
+
 	virtual ~StatusListener();
 
 	/** Create a new StatusClient socket that will be used
