@@ -3,7 +3,7 @@
 // Neighboring System for GNU Gatekeeper
 //
 // Copyright (c) Citron Network Inc. 2002-2003
-// Copyright (c) 2004-2012, Jan Willamowius
+// Copyright (c) 2004-2019, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -13,7 +13,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef NEIGHBOR_H
-#define NEIGHBOR_H "@(#) $Id: Neighbor.h,v 1.36 2013/10/31 17:26:08 willamowius Exp $"
+#define NEIGHBOR_H "@(#) $Id$"
 
 #include <list>
 #include <map>
@@ -36,12 +36,12 @@ class RasServer;
 
 namespace Neighbors {
 
-
 using Routing::AdmissionRequest;
 using Routing::LocationRequest;
 using Routing::SetupRequest;
 using Routing::FacilityRequest;
 
+class NeighborPingThread;
 
 struct PrefixInfo {
 	PrefixInfo() : m_length(0), m_priority(0) { }
@@ -79,12 +79,12 @@ public:
 	PString GetGkId() const { return m_gkid; }
 	PIPSocket::Address GetIP() const;
 	WORD GetPort() const;
-	H225_LocationRequest & BuildLRQ(H225_RasMessage &, WORD, const H225_ArrayOf_AliasAddress &);
+	H225_LocationRequest & BuildLRQ(H225_RasMessage & lrq_ras, WORD seq, const H225_ArrayOf_AliasAddress & dest);
 
 	// new virtual functions
 
 	// the real constructor, get profile of this neighbor
-	virtual bool SetProfile(const PString &, const PString &);
+	virtual bool SetProfile(const PString & id, const PString & type, bool reload);
     // Sent profile based on SRV Record
 	virtual bool SetProfile(const PString &, const H323TransportAddress &);
 
@@ -94,6 +94,9 @@ public:
 	// send a H.460.18 keepAlive (triggered by a timer)
 	void SendH46018GkKeepAlive(GkTimer* timer);
 	void SetH46018GkKeepAliveInterval(int interval);
+	// send a LRQ Ping (triggered by a timer)
+	void SendLRQPing(GkTimer* timer);
+	void SetLRQPingInterval(int interval);
 
 	// get PrefixInfo for a given aliases
 	// if an alias is matched, set dest to the alias
@@ -117,6 +120,9 @@ public:
 	virtual bool IsAcceptable(RasMsg *ras) const;
 	virtual bool UseTLS() const { return m_useTLS; }
 
+	virtual void SetDisabled(bool val) { m_disabled = val; }
+	virtual bool IsDisabled() const { return m_disabled; }
+
 protected:
 	void SetForwardedInfo(const PString &);
 
@@ -139,9 +145,14 @@ protected:
 	PString m_sendAuthUser, m_sendPassword;	// user + password to send to neighbor
 	int m_keepAliveTimerInterval;
 	GkTimerManager::GkTimerHandle m_keepAliveTimer;
+	int m_lrqPingInterval;
+	GkTimerManager::GkTimerHandle m_lrqPingTimer;
+    NeighborPingThread * m_pingThread;
 	bool m_H46018Server;
 	bool m_H46018Client;
 	bool m_useTLS;
+	bool m_disabled; // is this neighbor disabled (eg. because of no responses)
+	bool m_loopDetection;
 };
 
 class NeighborList {
@@ -158,6 +169,9 @@ public:
 	bool IsTraversalClient(const PIPSocket::Address &) const;
 	bool IsTraversalServer(const PIPSocket::Address &) const;
 
+	// is IP a neighbor and is it not disabled ?
+	bool IsAvailable(const PIPSocket::Address & ip);
+
 	// return the neighbor's ID from the list by signal address
 	PString GetNeighborIdBySigAdr(const H225_TransportAddress & sigAd);
 	PString GetNeighborIdBySigAdr(const PIPSocket::Address & sigAd);
@@ -166,10 +180,10 @@ public:
 	PString GetNeighborGkIdBySigAdr(const H225_TransportAddress & sigAd);
 	PString GetNeighborGkIdBySigAdr(const PIPSocket::Address & sigAd);
 
-	// return the neighbor's use eof TLS from the list by signal address
+	// return the neighbor's use of TLS from the list by signal address
 	bool GetNeighborTLSBySigAdr(const H225_TransportAddress & sigAd);
 	bool GetNeighborTLSBySigAdr(const PIPSocket::Address & sigAd);
- 
+
 	operator List & () { return m_neighbors; }
 	operator const List & () const { return m_neighbors; }
 
