@@ -1,9 +1,10 @@
+
 //////////////////////////////////////////////////////////////////
 //
 // ProxyChannel.h
 //
 // Copyright (c) Citron Network Inc. 2001-2003
-// Copyright (c) 2002-2013, Jan Willamowius
+// Copyright (c) 2002-2025, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -13,7 +14,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef PROXYCHANNEL_H
-#define PROXYCHANNEL_H "@(#) $Id: ProxyChannel.h,v 1.163 2014/01/16 10:14:32 willamowius Exp $"
+#define PROXYCHANNEL_H "@(#) $Id$"
 
 #include <vector>
 #include <list>
@@ -71,12 +72,8 @@ const WORD DEFAULT_PACKET_BUFFER_SIZE = 2048;
 
 void PrintQ931(int, const char *, const char *, const Q931 *, const H225_H323_UserInformation *);
 
-bool GetUUIE(const Q931 & q931, H225_H323_UserInformation & uuie);
-
-void SetUUIE(Q931 & q931, const H225_H323_UserInformation & uuie);
-
-ssize_t UDPSendWithSourceIP(int fd, void * data, size_t len, const H323TransportAddress & toAddress);
-ssize_t UDPSendWithSourceIP(int fd, void * data, size_t len, const PIPSocket::Address & ip, WORD port);
+ssize_t UDPSendWithSourceIP(int fd, void * data, size_t len, const IPAndPortAddress & toAddress, const PIPSocket::Address * gkIP);
+ssize_t UDPSendWithSourceIP(int fd, void * data, size_t len, const PIPSocket::Address & ip, WORD port, const PIPSocket::Address * gkIP);
 
 
 class ProxySocket : public USocket {
@@ -87,7 +84,8 @@ public:
 		Forwarding,
 		Closing,
 		Error,
-		DelayedConnecting	// H.460.18
+		DelayedConnecting,	// H.460.18
+		NoDataAndDone       // H.460.18 H.245 multiplexed connection
 	};
 
 	ProxySocket(
@@ -101,7 +99,7 @@ public:
 	virtual Result ReceiveData();
 	virtual bool ForwardData();
 	virtual bool EndSession();
-	virtual void OnError() {}
+	virtual void OnError() { }
 
 	bool IsConnected() const { return connected; }
 	void SetConnected(bool c) { connected = c; }
@@ -112,8 +110,8 @@ public:
 
 private:
 	ProxySocket();
-	ProxySocket(const ProxySocket&);
-	ProxySocket& operator=(const ProxySocket&);
+	ProxySocket(const ProxySocket &);
+	ProxySocket& operator=(const ProxySocket &);
 
 protected:
 	BYTE *wbuffer;
@@ -126,6 +124,9 @@ private:
 
 class TCPProxySocket : public ServerSocket, public ProxySocket {
 public:
+    enum H225KeepAliveMethod { TPKTH225, EmptyFacility, Information, Notify, Status, StatusInquiry, NoneH225 };
+    enum H245KeepAliveMethod { TPKTH245, UserInput, NoneH245 };
+
 	TCPProxySocket(const char * t, TCPProxySocket * s = NULL, WORD p = 0);
 	virtual ~TCPProxySocket();
 
@@ -143,14 +144,17 @@ public:
 
 	void DetachRemote();
   	void RemoveRemoteSocket();
-	void SetRemoteSocket(TCPProxySocket * ret) { remote=ret; }
+	void SetRemoteSocket(TCPProxySocket * ret) { remote = ret; }
 	void LockRemote() { m_remoteLock.Wait(); }
 	void UnlockRemote() { m_remoteLock.Signal(); }
 
+    virtual int GetOSSocket() const { return os_handle; }
+	virtual void SetOSSocket(int sock, const PString & name) { os_handle = sock; SetName(name); }
+
 private:
 	TCPProxySocket();
-	TCPProxySocket(const TCPProxySocket&);
-	TCPProxySocket& operator=(const TCPProxySocket&);
+	TCPProxySocket(const TCPProxySocket &);
+	TCPProxySocket& operator=(const TCPProxySocket &);
 
 protected:
 	struct TPKTV3 {
@@ -167,7 +171,12 @@ protected:
 	PBYTEArray buffer;
 
 public:
-	bool InternalWrite(const PBYTEArray &);
+	bool InternalWrite(const PBYTEArray & buf);
+	void SendKeepAlive(GkTimer * timer);
+	void SendEmptyTPKTKeepAlive();
+	void RegisterKeepAlive(int h46018_interval = 0);
+	void UnregisterKeepAlive();
+	bool UsesH460KeepAlive() const { return (m_keepAliveTimer != GkTimerManager::INVALID_HANDLE) && m_h46018KeepAlive; }
 
 protected:
 	bool SetMinBufSize(WORD);
@@ -175,31 +184,42 @@ protected:
 	BYTE *bufptr;
 	TPKTV3 tpkt;
 	unsigned tpktlen;
+
+	bool m_h46018KeepAlive;
+	H225KeepAliveMethod m_h460KeepAliveMethodH225;
+	H245KeepAliveMethod m_h460KeepAliveMethodH245;
+	H225KeepAliveMethod m_nonStdKeepAliveMethodH225;
+	H245KeepAliveMethod m_nonStdKeepAliveMethodH245;
+	int m_keepAliveInterval;
+	GkTimerManager::GkTimerHandle m_keepAliveTimer;
 };
 
 class RTPLogicalChannel;
+
 class UDPProxySocket : public UDPSocket, public ProxySocket {
 public:
 #ifndef LARGE_FDSET
 	PCLASSINFO( UDPProxySocket, UDPSocket )
 #endif
 
-	UDPProxySocket(const char * t, const H225_CallIdentifier & id);
+	UDPProxySocket(const char * t, PINDEX no);
 	~UDPProxySocket();
 
 	void UpdateSocketName();
-	void RemoveCallPtr() { m_call = NULL; }
-	void SetDestination(H245_UnicastAddress &, callptr &);
-	void SetForwardDestination(const Address &, WORD, H245_UnicastAddress *, callptr &);
-	void SetReverseDestination(const Address &, WORD, H245_UnicastAddress *, callptr &);
-	typedef void (UDPProxySocket::*pMem)(const Address &, WORD, H245_UnicastAddress *, callptr &);
+	void RemoveCallPtr() { PWaitAndSignal lock(m_callMutex); m_call = NULL; }
+	void SetRTCPDestination(const H245_UnicastAddress & addr, const PIPSocket::Address & sourceIP, bool isUnidirectional);
+    void ZeroRTCPDestination();
+	void SetForwardDestination(const Address & srcIP, WORD srcPort, H245_UnicastAddress * dstAddr, callptr & call, bool onlySetDest, bool onlySetSrc);
+	void SetReverseDestination(const Address & srcIP, WORD srcPort, H245_UnicastAddress * dstAddr, callptr & call, bool onlySetDest, bool onlySetSrc);
+	typedef void (UDPProxySocket::*pMem)(const Address & srcIP, WORD srcPort, H245_UnicastAddress * dstAddr, callptr & call, bool onlySetDest, bool onlySetSrc);
 
-	bool Bind(const Address &localAddr, WORD pt);
+	bool Bind(const Address & localAddr, WORD pt);
 	int GetOSSocket() const { return os_handle; }
 	void SetNAT(bool);
 	bool isMute() { return mute; }
 	void SetMute(bool toMute) { mute = toMute; }
 	void OnHandlerSwapped() { std::swap(fnat, rnat); }
+	WORD GetRTPSessionID() const { return m_sessionID; }
 	void SetRTPSessionID(WORD id) { m_sessionID = id; }
 #ifdef HAS_H235_MEDIA
 	void SetEncryptingRTPChannel(RTPLogicalChannel * lc) { m_encryptingLC = lc; }
@@ -214,8 +234,8 @@ public:
 	bool UsesH46019() const { return m_useH46019; }
 	void SetH46019UniDirectional(bool val) { m_h46019uni = val; }
 	void AddKeepAlivePT(BYTE pt);
-	void SetMultiplexDestination(const H323TransportAddress & toAddress, H46019Side side);
-	void SetMultiplexID(PUInt32b multiplexID, H46019Side side);
+	void SetMultiplexDestination(const IPAndPortAddress & toAddress, H46019Side side);
+	void SetMultiplexID(DWORD multiplexID, H46019Side side);
 	void SetMultiplexSocket(int multiplexSocket, H46019Side side);
 #endif
 
@@ -223,23 +243,33 @@ public:
 	virtual Result ReceiveData();
 	virtual bool OnReceiveData(void *, PINDEX, Address &, WORD &) { return true; }
 
+    void GetPorts(PIPSocket::Address & _fSrcIP, PIPSocket::Address & _fDestIP, PIPSocket::Address & _rSrcIP, PIPSocket::Address & _rDestIP,
+                    WORD & _fSrcPort, WORD & _fDestPort, WORD & _rSrcPort, WORD & _rDestPort) const;
+    void ZeroAllIPs();
+    void ForwardAndReverseSeen() { PTRACE(7, "JW RTP ForwardAndReverseSeen"); m_forwardAndReverseSeen = true; }
+    bool IsRTPInactive() const;
+    void ApplyPortDetectionCache();
+    void CachePortDetectionData(Address fromIP, WORD fromPort);
+    void DoPortDetection(WORD localport, Address fromIP, WORD fromPort);
+
 protected:
 	virtual bool WriteData(const BYTE *, int);
 	virtual bool Flush();
 	virtual bool ErrorHandler(PSocket::ErrorGroup);
 
-	void SetMediaIP(const PString & direction, const Address & ip);
+	void SetMediaIP(bool isSRC, const Address & ip);
 
 	// RTCP handler
 	void BuildReceiverReport(const RTP_ControlFrame & frame, PINDEX offset, bool dst);
 
-	H225_CallIdentifier m_callID;
+	PINDEX m_callNo;
 	callptr * m_call;
+	PMutex m_callMutex;
 
 private:
 	UDPProxySocket();
-	UDPProxySocket(const UDPProxySocket&);
-	UDPProxySocket& operator=(const UDPProxySocket&);
+	UDPProxySocket(const UDPProxySocket &);
+	UDPProxySocket& operator=(const UDPProxySocket &);
 
 protected:
 	Address fSrcIP, fDestIP, rSrcIP, rDestIP;
@@ -248,7 +278,6 @@ protected:
 	bool mute;
 	bool m_isRTPType;
 	bool m_isRTCPType;
-	bool m_dontQueueRTP;
 	bool m_EnableRTCPStats;
 	WORD m_sessionID;
 	RTPLogicalChannel * m_encryptingLC;
@@ -256,8 +285,7 @@ protected:
 #ifdef HAS_H46018
 	bool m_h46019fc;
 	bool m_useH46019;
-	bool m_h46019uni;
-	bool m_h46019DetectionDone;
+	bool m_h46019uni; // is this channel unidirectional/H.239 (not only when using H.460.19)
 	PMutex m_h46019DetectionLock;
 	bool m_checkH46019KeepAlivePT;
 	WORD m_keepAlivePT_1;
@@ -265,14 +293,40 @@ protected:
 	PTime m_channelStartTime;
 	// two (!), one or zero parties in a call through a UDPProxySocket may by multiplexed
 	// UDPProxySocket always receives regular RTP, but may send out multiplexed
-	H323TransportAddress m_multiplexDestination_A;	// OLC side of first logical channel in this session
-	PUInt32b m_multiplexID_A;	// ID _to_ A side (only valid if m_multiplexDestination_A is set)
+	IPAndPortAddress m_multiplexDestination_A;	// OLC side of first logical channel in this session
+	DWORD m_multiplexID_A;	// ID _to_ A side (only valid if m_multiplexDestination_A is set)
 	int m_multiplexSocket_A;	// only valid if m_multiplexDestination_A is set
-	H323TransportAddress m_multiplexDestination_B;	// OLCAck side of first logical channel in this session
-	PUInt32b m_multiplexID_B;	// ID _to_ B side (only valid if m_multiplexDestination_B is set)
+	IPAndPortAddress m_multiplexDestination_B;	// OLCAck side of first logical channel in this session
+	DWORD m_multiplexID_B;	// ID _to_ B side (only valid if m_multiplexDestination_B is set)
 	int m_multiplexSocket_B;	// only valid if m_multiplexDestination_ is set)
 	PMutex m_multiplexMutex;	// protect multiplex IDs, addresses and sockets against access from concurrent threads
 #endif
+#ifdef HAS_H235_MEDIA
+	bool m_haveShownPTWarning;	// flag to show the PayloadType warning only once (not for every RTP packet)
+#endif
+    bool m_ignoreSignaledIPs;   // ignore all RTP/RTCP IPs in signaling, do full auto-detect
+    bool m_ignoreSignaledPrivateH239IPs;   // also ignore private IPs signaled in H239 streams
+    bool m_ignoreSignaledAllH239IPs;   // also ignore all IPs signaled in H239 streams
+    list<NetworkAddress> m_ignorePublicH239IPs;   // do auto-detect for H.239 on this network
+    list<NetworkAddress> m_keepSignaledIPs;   // don't do auto-detect on this network (by signaled IP)
+    list<NetworkAddress> m_keepSignaledIPsFrom;   // don't do auto-detect on this network (by source IP)
+    bool m_restrictRTPSources;
+    NetworkAddress m_restrictRTPNetwork_A;
+    NetworkAddress m_restrictRTPNetwork_B;
+	bool m_portDetectionDone;
+	bool m_forwardAndReverseSeen;   // did we see logical channels for both directions, yet ?
+	bool m_legacyPortDetection;
+	int m_inactivityTimeout;
+	time_t m_lastPacketFromForwardSrc;
+	time_t m_lastPacketFromReverseSrc;
+	int m_portDetectionTimeout;
+	time_t m_firstMedia;
+	bool m_mediaFailDetected;
+	bool m_RTPMultiplexingEnabled;
+	list<NetworkAddress> m_allowAnyRTPSourcePortForH239; // list of IPs/networks
+	bool m_cachePortDetection;
+	int m_cachePortDetectionDuration;
+	std::map<IPAndPortAddress, time_t> m_portDetectionCache;
 };
 
 #if H323_H450
@@ -308,22 +362,47 @@ public:
 
 	bool IsH245Tunneling() const { return m_h245Tunneling; }
 	bool IsH245TunnelingTranslation() const { return m_h245TunnelingTranslation; }
+	void QueueH245Message(PASN_OctetString & msg) { m_h245Queue.push(msg); }
 	PASN_OctetString * GetNextQueuedH245Message();
-	unsigned GetH245MessageQueueSize() const { return  m_h245Queue.size(); }
+	unsigned GetH245MessageQueueSize() const { return m_h245Queue.size(); }
 	bool HandleH245Mesg(PPER_Stream &, bool & suppress, H245Socket * h245sock = NULL);
 	void SendPostDialDigits();
 	void OnH245ChannelClosed() { m_h245socket = NULL; }
-	Address GetLocalAddr() { return localAddr; }
-	Address GetPeerAddr() { return peerAddr; }
-	Address GetMasqAddr() { return masqAddr; }
+	Address GetLocalAddr() const { return localAddr; }
+#ifdef HAS_AVAYA_SUPPORT
+	WORD GetLocalPort() const { return localPort; }
+#endif
+	Address GetPeerAddr() const { return peerAddr; }
+	Address GetMasqAddr() const { return masqAddr; }
 	PINDEX GetCallNumber() const { return m_call ? m_call->GetCallNumber() : 0; }
 	H225_CallIdentifier GetCallIdentifier() const { return m_call ? m_call->GetCallIdentifier() : 0; }
-	void BuildFacilityPDU(Q931 &, int, const PObject * = NULL);
+	PString GetCallIdentifierAsString() const { return m_call ? AsString(m_call->GetCallIdentifier()) : "unknown"; }
+	void BuildFacilityPDU(Q931 &, int, const PObject * = NULL, bool h46017 = false
+#ifdef HAS_AVAYA_SUPPORT
+											, bool avaya = false
+#endif
+														);
 	void BuildProgressPDU(Q931 &, PBoolean fromDestination);
-	void BuildProceedingPDU(Q931 & ProceedingPDU, const H225_CallIdentifier & callId, unsigned crv);
+	void BuildNotifyPDU(Q931 &, PBoolean fromDestination, int indication = 0);
+	void BuildStatusPDU(Q931 &, PBoolean fromDestination);
+	void BuildStatusInquiryPDU(Q931 &, PBoolean fromDestination);
+	void BuildInformationPDU(Q931 &, PBoolean fromDestination
+#ifdef HAS_AVAYA_SUPPORT
+								, const unsigned char * ccmsAvaya = NULL, unsigned int ccmsAvayaLen = 0, unsigned callref = 0
+#endif
+																				);
+	void BuildProceedingPDU(Q931 & ProceedingPDU, const H225_CallIdentifier & callId, unsigned crv
+#ifdef HAS_AVAYA_SUPPORT
+													, bool avaya = false
+#endif
+																);
+#ifdef HAS_AVAYA_SUPPORT
+	void BuildConnectPDU(Q931 & ConnectPDU, const H225_CallIdentifier & callId, unsigned crv, const PObject * = NULL, bool avaya = true);
+#endif
 	void BuildSetupPDU(Q931 &, const H225_CallIdentifier & callid, unsigned crv, const PString & destination, bool h245tunneling);
+	WORD GetCRV() const { return m_crv; }
 	void RemoveCall();
-	bool RerouteCall(CallLeg which, const PString & destination, bool h450transfer);
+	bool RerouteCall(CallLeg which, const PString & destination);
 	void RerouteCaller(PString destination);
 	void RerouteCalled(PString destination);
 	void PerformConnecting();
@@ -334,12 +413,18 @@ public:
 	Result RetrySetup();
 	void TryNextRoute();
 	void RemoveH245Handler();
-	void SaveTCS(const H245_TerminalCapabilitySet & tcs) { m_savedTCS = tcs; }
 	H245_TerminalCapabilitySet GetSavedTCS() const { return m_savedTCS; }
+	unsigned GetNextTCSSeq() { return ++m_tcsRecSeq; }
 	bool SendTunneledH245(const PPER_Stream & strm);
 	bool SendTunneledH245(const H245_MultimediaSystemControlMessage & h245msg);
-#ifdef HAS_H235_MEDIA
+    void SendFacilityKeepAlive();
+    void SendInformationKeepAlive();
+    void SendNotifyKeepAlive();
+    void SendStatusKeepAlive();
+    void SendStatusInquiryKeepAlive();
+
 	bool IsH245Master() const { return m_isH245Master; }
+#ifdef HAS_H235_MEDIA
     bool HandleH235TCS(H245_TerminalCapabilitySet & tcs);
     bool HandleH235OLC(H245_OpenLogicalChannel & olc);
 	void SendEncryptionUpdateCommand(WORD flcn, BYTE oldPT, BYTE plainPT);
@@ -347,15 +432,15 @@ public:
 #endif
 	H245Socket * GetH245Socket() const { return m_h245socket; }
 	void SetH245Socket(H245Socket * sock) { m_h245socket = sock; }
-	bool CompareH245Socket(H245Socket * sock) const { return sock == m_h245socket; }	// intentionally comparing pointers
-	
+	bool CompareH245Socket(const H245Socket * sock) const { return sock == m_h245socket; }	// intentionally comparing pointers
+
 protected:
 	void SetRemote(CallSignalSocket *);
-	bool CreateRemote(H225_Setup_UUIE &setupBody);
+	bool CreateRemote(H225_Setup_UUIE & setupBody);
 
 public:
 #ifdef HAS_H46017
-	bool SendH46017Message(const H225_RasMessage & ras);
+	bool SendH46017Message(H225_RasMessage ras, GkH235Authenticators * authenticators);
 	void CleanupCall();
 #endif
 #ifdef HAS_H46026
@@ -363,17 +448,22 @@ public:
 	void PollPriorityQueue();
 #endif
 	bool MaintainConnection() const { return m_maintainConnection; }
+	bool IsCaller() const { return m_callerSocket; }
 
 #ifdef HAS_H46018
-	bool IsCaller() const { return m_callerSocket; }
 	bool IsTraversalClient() const;
 	bool IsTraversalServer() const;
-	bool CreateRemote(const H225_TransportAddress & addr);
+	bool CreateRemote(const H225_TransportAddress & addr
+#ifdef HAS_AVAYA_SUPPORT
+							, bool bForwardData = true
+#endif
+										);
 	bool OnSCICall(const H225_CallIdentifier & callID, H225_TransportAddress sigAdr, bool useTLS);
 	bool IsCallFromTraversalServer() const { return m_callFromTraversalServer; }
 	bool IsCallToTraversalServer() const { return m_callToTraversalServer; }
-	void SetSessionMultiplexDestination(WORD session, bool isRTCP, const H323TransportAddress & toAddress, H46019Side side);
+	void SetSessionMultiplexDestination(WORD session, bool isRTCP, const IPAndPortAddress & toAddress, H46019Side side);
 	const H245Handler * GetH245Handler() const { return m_h245handler; }
+	void SetH245OSSocket(int socket, const PString & name);
 #endif
 	void LockH245Handler() { m_h245handlerLock.Wait(); }
 	void UnlockH245Handler() { m_h245handlerLock.Signal(); }
@@ -382,10 +472,21 @@ public:
 	bool IsH46024Call(const H225_Setup_UUIE & setupBody);
 #endif
 
+	bool SetupResponseTokens(SignalingMsg * msg, GkH235Authenticators * auth, const endptr & ep);
+
 	CallSignalSocket * GetRemote() const { return dynamic_cast<CallSignalSocket *>(remote); }
 
+    bool IsRTPInactive(short session) const;
+    void AbortLogicalChannel(short session);
+
+#ifdef HAS_AVAYA_SUPPORT
+	PString	GetAvayaKeypad() const { return m_Avaya_Keypad; }
+	void SetAvayaKeypad(PString pad) { m_Avaya_Keypad = pad; }
+#endif
+
 protected:
 	void ForwardCall(FacilityMsg *msg);
+	void RerouteCall(FacilityMsg *msg);
 
 	/// signaling message handlers
 	void OnSetup(SignalingMsg *msg);
@@ -419,28 +520,37 @@ protected:
 		}
 		return false;
 	}
-	
+
 	template<class UUIE> bool HandleFastStart(UUIE & uu, bool fromCaller)
 	{
 		if (!uu.HasOptionalField(UUIE::e_fastStart))
 			return false;
-			
+
+        if (GkConfig()->GetBoolean(RoutedSec, "DisableFastStart", false)) {
+            uu.RemoveOptionalField(UUIE::e_fastStart);
+            return true;
+        }
+
 		if (!fromCaller && m_call)
 			m_call->SetFastStartResponseReceived();
-			
+
 		return m_h245handler != NULL ? OnFastStart(uu.m_fastStart, fromCaller) : false;
 	}
 
 private:
-	CallSignalSocket(const CallSignalSocket&);
-	CallSignalSocket& operator=(const CallSignalSocket&);
+	CallSignalSocket(const CallSignalSocket &);
+	CallSignalSocket & operator=(const CallSignalSocket &);
 
 	void InternalInit();
 	void BuildReleasePDU(Q931 &, const H225_CallTerminationCause *) const;
 	// if return false, the h245Address field will be removed
 	bool SetH245Address(H225_TransportAddress &);
 
-	bool InternalConnectTo();
+	bool InternalConnectTo(
+#ifdef HAS_AVAYA_SUPPORT
+				bool bForwardData = true
+#endif
+							);
 	bool ForwardCallConnectTo();
 
 	/** @return
@@ -448,9 +558,9 @@ private:
 	*/
 	PString GetCallingStationId(
 		/// Q.931/H.225 Setup message with additional data
-		const SetupMsg& setup,
+		const SetupMsg & setup,
 		/// additional data
-		SetupAuthData& authData
+		SetupAuthData & authData
 		) const;
 
 	/** @return
@@ -458,15 +568,15 @@ private:
 	*/
 	PString GetCalledStationId(
 		/// Q.931/H.225 Setup message with additional data
-		const SetupMsg& setup,
+		const SetupMsg & setup,
 		/// additional data
-		SetupAuthData& authData
+		SetupAuthData & authData
 		) const;
 
 	/// @return	a number dialed by the user
 	PString GetDialedNumber(
 		/// Q.931/H.225 Setup message with additional data
-		const SetupMsg& setup
+		const SetupMsg & setup
 		) const;
 
 	void SetCallTypePlan(Q931 *q931);
@@ -477,7 +587,10 @@ protected:
 	// but the local address that remote socket bind to
 	// they may be different in multi-homed environment
 	Address localAddr, peerAddr, masqAddr;
-	WORD peerPort;
+    WORD peerPort;
+#ifdef HAS_AVAYA_SUPPORT
+	WORD localPort;
+#endif
 
 private:
 	WORD m_crv;
@@ -501,8 +614,12 @@ private:
 	unsigned m_h225Version;
 	/// raw Setup data as received from the caller (for failover)
 	PBYTEArray m_rawSetup;
+	/// raw Connect data as received from the called (for reroute CALLED)
+	PBYTEArray m_rawConnect;
 	PMutex infomutex;    // Information PDU processing Mutex
 	H245_TerminalCapabilitySet m_savedTCS;	// saved tcs to re-send
+	unsigned m_tcsRecSeq;
+	unsigned m_tcsAckRecSeq;
 #ifdef HAS_H46017
 	bool m_h46017Enabled;
 	TCPProxySocket * rc_remote; // copy of the remote pointer that may be only used to send RC on call end
@@ -512,11 +629,15 @@ private:
 	bool m_callToTraversalServer;
 	bool m_senderSupportsH46019Multiplexing;
 #endif
-#ifdef HAS_H235_MEDIA
 	bool m_isH245Master;
-#endif
 #ifdef HAS_H46026
 	H46026ChannelManager * m_h46026PriorityQueue;
+#endif
+    // remember that this socket was forwarded so we don't accidentally delete
+    // RTP multiplex sessions (H.460.19/H.460.26) when we delete the socket object
+    bool m_socketWasForwarded;
+#ifdef HAS_AVAYA_SUPPORT
+	PString m_Avaya_Keypad;
 #endif
 };
 
@@ -555,10 +676,14 @@ public:
 
 	virtual bool Connect(const Address & addr);
 	virtual PBoolean Connect(const Address & iface, WORD localPort, const Address & addr);	// override from TCPProxySocket
+#ifdef LARGE_FDSET
+	virtual bool Read(void * buf, int sz, bool wantZeroReads = false);
+#else
 	virtual bool Read(void * buf, int sz);
-	virtual int GetLastReadCount() const { return m_lastReadCount; }
+#endif
+	virtual PINDEX GetLastReadCount() const { return m_lastReadCount; }
 	virtual bool Write(const void * buf, int sz);
-	virtual int GetLastWriteCount() const { return m_lastWriteCount; }
+	virtual PINDEX GetLastWriteCount() const { return m_lastWriteCount; }
 	virtual void Dispatch();
 
 protected:
@@ -571,6 +696,21 @@ protected:
 
 #ifdef HAS_H46018
 
+class MultiplexH245Listener : public TCPListenSocket {
+#ifndef LARGE_FDSET
+	PCLASSINFO ( MultiplexH245Listener, TCPListenSocket )
+#endif
+public:
+	MultiplexH245Listener(const Address &, WORD);
+	~MultiplexH245Listener();
+
+	// override from class TCPListenSocket
+	virtual ServerSocket *CreateAcceptor() const;
+
+protected:
+	Address m_addr;
+};
+
 class MultiplexRTPListener : public UDPSocket {
 #ifndef LARGE_FDSET
 	PCLASSINFO ( MultiplexRTPListener, UDPSocket )
@@ -596,34 +736,41 @@ class RTPLogicalChannel;
 class H46019Session
 {
 public:
-	H46019Session(const H225_CallIdentifier & callid, WORD session, void * openedBy);
+	H46019Session(PINDEX callno, WORD session, void * openedBy);
+    ~H46019Session();
+
+	H46019Session(const H46019Session & other);
+	H46019Session & operator=(const H46019Session & other);
+	H46019Session & Merge(const H46019Session & other);
 
 	void Dump() const;
 
-	bool IsValid() const { return (m_session != INVALID_RTP_SESSION) || (m_flcn > 0); }
-	bool sideAReady(bool isRTCP) const { return isRTCP ? IsSet(m_addrA_RTCP) : IsSet(m_addrA); }
-	bool sideBReady(bool isRTCP) const { return isRTCP ? IsSet(m_addrB_RTCP) : IsSet(m_addrB); }
+	bool IsValid() const { return !m_deleted && ((m_session != INVALID_RTP_SESSION) || (m_flcn > 0)); }
+	bool sideAReady(bool isRTCP) const { return isRTCP ? m_addrA_RTCP.IsSet() : m_addrA.IsSet(); }
+	bool sideBReady(bool isRTCP) const { return isRTCP ? m_addrB_RTCP.IsSet() : m_addrB.IsSet(); }
 	H46019Session SwapSides() const; // return a copy with side A and B swapped
 
-	bool IsKeepAlive(void * data, unsigned len, bool isRTCP) const { return isRTCP ? true : (len == 12); }
+	static bool IsKeepAlive(unsigned len, bool isRTCP) { return isRTCP ? true : (len == 12); }
 
-	void HandlePacket(PUInt32b receivedMultiplexID, const H323TransportAddress & fromAddress, void * data, unsigned len, bool isRTCP);
-	static void Send(PUInt32b sendMultiplexID, const H323TransportAddress & toAddress, int ossocket, void * data, unsigned len, bool bufferHasRoomForID=false);
+	void HandlePacket(DWORD receivedMultiplexID, const IPAndPortAddress & fromAddress, void * data, unsigned len, bool isRTCP);
+	static void Send(DWORD sendMultiplexID, const IPAndPortAddress & toAddress, int ossocket, void * data, unsigned len, bool bufferHasRoomForID, PIPSocket::Address * gkIP, bool isRetry = false);
 
-//protected:
-	H225_CallIdentifier m_callid;
-	WORD m_session;
+public:
+    bool m_deleted; // logically deleted, but still in list so other threads can leave methods
+    time_t m_deleteTime;
+	PINDEX m_callno;
+	WORD m_session;     // RTP session ID
 	WORD m_flcn;		// only used to assign master assigned RTP session IDs
 	void * m_openedBy;	// side A (pointer to H245ProxyHandler used as an ID)
 	void * m_otherSide;	// side B (pointer to H245ProxyHandler used as an ID)
-	H323TransportAddress m_addrA;
-	H323TransportAddress m_addrA_RTCP;
-	H323TransportAddress m_addrB;
-	H323TransportAddress m_addrB_RTCP;
-	PUInt32b m_multiplexID_fromA;
-	PUInt32b m_multiplexID_toA;
-	PUInt32b m_multiplexID_fromB;
-	PUInt32b m_multiplexID_toB;
+	IPAndPortAddress m_addrA;
+	IPAndPortAddress m_addrA_RTCP;
+	IPAndPortAddress m_addrB;
+	IPAndPortAddress m_addrB_RTCP;
+	DWORD m_multiplexID_fromA;
+	DWORD m_multiplexID_toA;
+	DWORD m_multiplexID_fromB;
+	DWORD m_multiplexID_toB;
 	int m_osSocketToA;
 	int m_osSocketToA_RTCP;
 	int m_osSocketToB;
@@ -632,9 +779,11 @@ public:
 #ifdef HAS_H235_MEDIA
 	RTPLogicalChannel * m_encryptingLC;
 	RTPLogicalChannel * m_decryptingLC;
-	PUInt32b m_encryptMultiplexID;
-	PUInt32b m_decryptMultiplexID;
+	DWORD m_encryptMultiplexID;
+	DWORD m_decryptMultiplexID;
 #endif
+    time_t m_lastPacketFromA;   // time stamps for inactivity detection
+    time_t m_lastPacketFromB;
 };
 
 class MultiplexedRTPReader : public SocketsReader {
@@ -659,35 +808,46 @@ public:
 	MultiplexedRTPHandler();
 	virtual ~MultiplexedRTPHandler();
 
-	virtual void OnReload() { /* currently not runtime changable */ }
+	virtual void OnReload() { /* currently not runtime changeable */ }
 
 	virtual void AddChannel(const H46019Session & cha);
-	virtual void UpdateChannelSession(const H225_CallIdentifier & callid, WORD flcn, void * openedBy, WORD session);
+	virtual void UpdateChannelSession(PINDEX callno, WORD flcn, void * openedBy, WORD session);
 	virtual void UpdateChannel(const H46019Session & cha);
-	virtual H46019Session GetChannel(const H225_CallIdentifier & callid, WORD session) const;
-	virtual H46019Session GetChannelSwapped(const H225_CallIdentifier & callid, WORD session, void * openedBy) const;
-	virtual void RemoveChannels(H225_CallIdentifier callid);	// pass by value in case call gets removed
+	virtual H46019Session GetChannel(PINDEX callno, WORD session) const;
+	virtual H46019Session GetChannelSwapped(PINDEX callno, WORD session, void * openedBy) const;
+	virtual void RemoveChannels(PINDEX callno);
+	virtual void RemoveChannel(PINDEX callno, WORD session);
 #ifdef HAS_H235_MEDIA
-	virtual void RemoveChannel(H225_CallIdentifier callid, RTPLogicalChannel * rtplc);
+	virtual void RemoveChannel(PINDEX callno, RTPLogicalChannel * rtplc);
 #endif
 	virtual void DumpChannels(const PString & msg = "") const;
 
-	virtual bool HandlePacket(PUInt32b receivedMultiplexID, const H323TransportAddress & fromAddress, void * data, unsigned len, bool isRTCP);
+	virtual bool HandlePacket(DWORD receivedMultiplexID, const IPAndPortAddress & fromAddress, void * data, unsigned len, bool isRTCP);
 #ifdef HAS_H46026
-	virtual bool HandlePacket(const H225_CallIdentifier & callid, const H46026_UDPFrame & data);
+	virtual bool HandlePacket(PINDEX callno, const H46026_UDPFrame & data);
 #endif
 
 	virtual int GetRTPOSSocket() const { return m_reader ? m_reader->GetRTPOSSocket() : INVALID_OSSOCKET; }
 	virtual int GetRTCPOSSocket() const { return m_reader ? m_reader->GetRTCPOSSocket() : INVALID_OSSOCKET; }
 
-	virtual PUInt32b GetMultiplexID(const H225_CallIdentifier & callid, WORD session, void * to);
-	virtual PUInt32b GetNewMultiplexID();
+	virtual DWORD GetMultiplexID(PINDEX callno, WORD session, void * to);
+	virtual DWORD GetNewMultiplexID();
+
+	bool GetDetectedMediaIP(PINDEX callno, WORD sessionID, bool forCaller, /* out */ PIPSocket::Address & addr, WORD & port) const;
+
+	// delete sessions marked as deleted
+	void SessionCleanup(GkTimer* timer);
 
 protected:
 	MultiplexedRTPReader * m_reader;
 	mutable PReadWriteMutex m_listLock;
 	list<H46019Session> m_h46019channels;
-	PUInt32b idCounter; // we should make sure this counter is _not_ reset on reload
+	DWORD m_idCounter; // we should make sure this counter is _not_ reset on reload
+	GkTimerManager::GkTimerHandle m_cleanupTimer;
+	int m_deleteDelay;    // how long to wait before deleting a session marked for delete in sec.
+	bool m_inactivityCheck;
+	int m_inactivityTimeout;
+	short m_inactivityCheckSession;
 };
 #endif
 
@@ -698,22 +858,21 @@ protected:
 class H46026Session
 {
 public:
-	H46026Session() : m_isValid(false) { }
-	H46026Session(const H225_CallIdentifier & callid, WORD session, int osRTPSocket, int osRTCPSocket,
-					const H323TransportAddress & toRTP, const H323TransportAddress & toRTCP);
+	H46026Session();
+	H46026Session(PINDEX callno, WORD session, int osRTPSocket, int osRTCPSocket,
+					const IPAndPortAddress & toRTP, const IPAndPortAddress & toRTCP);
 
 	void Send(void * data, unsigned len, bool isRTCP);
 	bool IsValid() const { return m_isValid; }
 	void Dump() const;
 
-//protected:
 	bool m_isValid;
-	H225_CallIdentifier m_callid;
+	PINDEX m_callno;
 	WORD m_session;
 	int m_osRTPSocket;
 	int m_osRTCPSocket;
-	H323TransportAddress m_toAddressRTP;
-	H323TransportAddress m_toAddressRTCP;
+	IPAndPortAddress m_toAddressRTP;
+	IPAndPortAddress m_toAddressRTCP;
 
 #ifdef HAS_H235_MEDIA
 	RTPLogicalChannel * m_encryptingLC;
@@ -727,21 +886,21 @@ public:
 	H46026RTPHandler();
 	virtual ~H46026RTPHandler();
 
-	virtual void OnReload() { /* currently not runtime changable */ }
+	virtual void OnReload() { /* currently not runtime changeable */ }
 
 	virtual void AddChannel(const H46026Session & chan);
 	virtual void ReplaceChannel(const H46026Session & chan);
-	virtual void UpdateChannelRTP(const H225_CallIdentifier & callid, WORD session, H323TransportAddress toRTP);
-	virtual void UpdateChannelRTCP(const H225_CallIdentifier & callid, WORD session, H323TransportAddress toRTCP);
-	virtual void RemoveChannels(H225_CallIdentifier callid);	// pass by value in case call gets removed
-	H46026Session FindSession(const H225_CallIdentifier & callid, WORD session) const;
+	virtual void UpdateChannelRTP(PINDEX callno, WORD session, IPAndPortAddress toRTP);
+	virtual void UpdateChannelRTCP(PINDEX callno, WORD session, IPAndPortAddress toRTCP);
+	virtual void RemoveChannels(PINDEX callno);	// pass by value in case call gets removed
+	H46026Session FindSession(PINDEX callno, WORD session) const;
 #ifdef HAS_H235_MEDIA
-	virtual void UpdateChannelEncryptingLC(const H225_CallIdentifier & callid, WORD session, RTPLogicalChannel * lc);
-	virtual void UpdateChannelDecryptingLC(const H225_CallIdentifier & callid, WORD session, RTPLogicalChannel * lc);
+	virtual void UpdateChannelEncryptingLC(PINDEX callno, WORD session, RTPLogicalChannel * lc);
+	virtual void UpdateChannelDecryptingLC(PINDEX callno, WORD session, RTPLogicalChannel * lc);
 #endif
 	virtual void DumpChannels(const PString & msg = "") const;
 
-	virtual bool HandlePacket(const H225_CallIdentifier & callid, H46026_UDPFrame & data);
+	virtual bool HandlePacket(PINDEX callno, H46026_UDPFrame & data);
 
 protected:
 	mutable PReadWriteMutex m_listLock;
@@ -753,7 +912,7 @@ protected:
 
 class ProxyHandler : public SocketsReader {
 public:
-	ProxyHandler(const PString& name);
+	ProxyHandler(const PString & name);
 	virtual ~ProxyHandler();
 
 	void Insert(TCPProxySocket *);
@@ -780,9 +939,9 @@ private:
 	void DetachSocket(IPSocket *socket);
 
 	ProxyHandler();
-	ProxyHandler(const ProxyHandler&);
-	ProxyHandler& operator=(const ProxyHandler&);
-	
+	ProxyHandler(const ProxyHandler &);
+	ProxyHandler& operator=(const ProxyHandler &);
+
 private:
 	std::list<PTime *> m_removedTime;
 	/// time to wait before deleting a closed socket
@@ -811,9 +970,9 @@ public:
 	void LoadConfig();
 
 private:
-	HandlerList(const HandlerList&);
-	HandlerList& operator=(const HandlerList&);
-	
+	HandlerList(const HandlerList &);
+	HandlerList& operator=(const HandlerList &);
+
 private:
 	/// signaling/H.245/T.120 proxy handling threads
 	std::vector<ProxyHandler *> m_sigHandlers;
