@@ -6,7 +6,7 @@
 // Avoid including large h225.h in RasSrv.h
 //
 // Copyright (c) Citron Network Inc. 2001-2003
-// Copyright (c) 2006-2011, Jan Willamowius
+// Copyright (c) 2006-2015, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -16,24 +16,25 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef RASPDU_H
-#define RASPDU_H "@(#) $Id: RasPDU.h,v 1.31 2013/08/02 22:12:07 willamowius Exp $"
+#define RASPDU_H "@(#) $Id$"
 
 #include <list>
+#include <utility>
 #include "yasocket.h"
 #include "factory.h"
 #include "rasinfo.h"
+#include "gkh235.h"
+#include <h235auth.h>
+#include "RasTbl.h"
+#include "Toolkit.h"
 
 
-class Toolkit;
-class GkStatus;
-class RegistrationTable;
-class CallTable;
 class RasListener;
 class MulticastListener;
 class CallSignalListener;
 class TLSCallSignalListener;
+class MultiplexH245Listener;
 class StatusListener;
-class MultiplexRTPListener;
 class RasServer;
 class CallSignalSocket;
 
@@ -52,7 +53,7 @@ public:
 #ifdef HAS_H46017
 	bool Read(const PBYTEArray & buffer);
 #endif
-	bool Reply() const;
+	bool Reply(GkH235Authenticators * authenticators);
 
 	PPER_Stream m_rasPDU;
 	H225_RasMessage m_recvRAS;
@@ -72,7 +73,7 @@ public:
 	virtual ~RasListener();
 
 	GatekeeperMessage *ReadRas();
-	bool SendRas(const H225_RasMessage &, const Address &, WORD);
+	bool SendRas(H225_RasMessage &, const Address &, WORD, GkH235Authenticators * auth);
 
 	WORD GetSignalPort() const { return m_signalPort; }
 	void SetSignalPort(WORD pt) { m_signalPort = pt; }
@@ -106,40 +107,45 @@ public:
 
 	// new virtual function
 	virtual bool Process() = 0;
-   
+
 	virtual int GetSeqNum() const = 0;
 	virtual H225_NonStandardParameter *GetNonStandardParam() = 0;
 
 	// override from class Task
 	virtual void Exec();
 
-	bool IsFrom(const PIPSocket::Address &, WORD) const;
+	bool IsFrom(const PIPSocket::Address & addr, WORD pt) const;
 	unsigned GetTag() const { return m_msg->GetTag(); }
 	const char *GetTagName() const { return m_msg->GetTagName(); }
 
+	/// Get an address the message has been received from
+	void GetPeerAddr(PIPSocket::Address & addr, WORD & port) const;
+	void GetPeerAddr(PIPSocket::Address & addr) const;
+
 	void GetRasAddress(H225_TransportAddress &) const;
 	void GetCallSignalAddress(H225_TransportAddress &) const;
 
 	bool EqualTo(const RasMsg *) const;
 	bool operator==(const RasMsg & other) const { return EqualTo(&other); }
 
-	bool Reply() const { return m_msg->Reply(); }
+	bool Reply(GkH235Authenticators * authenticators) const { return m_msg->Reply(authenticators); }
 
 	GatekeeperMessage *operator->() { return m_msg; }
 	const GatekeeperMessage *operator->() const { return m_msg; }
+	GatekeeperMessage * GetMsg() const { return m_msg; }
 
 	static void Initialize();
 protected:
-	RasMsg(GatekeeperMessage *m) : m_msg(m) {}
+	RasMsg(GatekeeperMessage * m) : m_msg(m), m_authenticators(NULL) { }
 	RasMsg(const RasMsg &);
 
 	static bool PrintStatus(const PString &);
-	
-	GatekeeperMessage *m_msg;
+
+	GatekeeperMessage * m_msg;
+	GkH235Authenticators * m_authenticators;
 
 	// just pointers to global singleton objects
 	// cache for faster access
-	static Toolkit *Kit;
 	static RegistrationTable *EndpointTbl;
 	static CallTable *CallTbl;
 	static RasServer *RasSrv;
@@ -149,9 +155,9 @@ template<class RAS>
 class RasPDU : public RasMsg {
 public:
 	typedef RAS RasClass;
-	
-	RasPDU(GatekeeperMessage *m) : RasMsg(m), request(m->m_recvRAS) {}
-	virtual ~RasPDU() {}
+
+	RasPDU(GatekeeperMessage *m) : RasMsg(m), request(m->m_recvRAS) { }
+	virtual ~RasPDU() { }
 
 	// override from class RasMsg
 	virtual bool Process() { return false; }
@@ -162,11 +168,13 @@ public:
 	operator const RAS & () const { return request; }
 
 	H225_RasMessage & BuildConfirm();
-	H225_RasMessage & BuildReject(unsigned);
+	H225_RasMessage & BuildReject(unsigned reason);
+
+	void SetupResponseTokens(H225_RasMessage & responsePdu, const endptr & requestingEP);
 
 	typedef Factory<RasMsg, unsigned>::Creator1<GatekeeperMessage *> RasCreator;
 	struct Creator : public RasCreator {
-		Creator() : RasCreator(RasInfo<RAS>::tag) {}
+		Creator() : RasCreator(RasInfo<RAS>::tag) { }
 		virtual RasMsg *operator()(GatekeeperMessage *m) const { return new RasPDU<RAS>(m); }
 	};
 
@@ -174,6 +182,46 @@ protected:
 	RAS & request;
 };
 
+template<class RAS>
+void RasPDU<RAS>::SetupResponseTokens(H225_RasMessage & responsePdu, const endptr & requestingEP)
+{
+	if (m_authenticators == NULL && requestingEP)
+		m_authenticators = requestingEP->GetH235Authenticators();
+
+	if (m_authenticators == NULL) {
+		return;
+	}
+
+	if (GkConfig()->GetBoolean("H235", "UseEndpointIdentifier", true) && requestingEP) {
+		m_authenticators->SetProcedure1RemoteId(requestingEP->GetEndpointIdentifier().GetValue());
+	}
+
+	typedef typename RasInfo<RAS>::ConfirmTag ConfirmTag;
+	typedef typename RasInfo<RAS>::ConfirmType ConfirmType;
+	typedef typename RasInfo<RAS>::RejectTag RejectTag;
+	typedef typename RasInfo<RAS>::RejectType RejectType;
+	if (responsePdu.GetTag() == ConfirmTag()) {
+        ConfirmType & confirm = responsePdu;
+        m_authenticators->PrepareTokens(responsePdu, confirm.m_tokens, confirm.m_cryptoTokens);
+
+        if (confirm.m_tokens.GetSize() > 0)
+            confirm.IncludeOptionalField(ConfirmType::e_tokens);
+        if (confirm.m_cryptoTokens.GetSize() > 0)
+            confirm.IncludeOptionalField(ConfirmType::e_cryptoTokens);
+	} else {
+        if (responsePdu.GetTag() != RejectTag())
+            responsePdu.SetTag(RejectTag());   // create a Ronfirm if not set
+        RejectType & reject = responsePdu;
+        m_authenticators->PrepareTokens(responsePdu, reject.m_tokens, reject.m_cryptoTokens);
+
+        if (reject.m_tokens.GetSize() > 0)
+            reject.IncludeOptionalField(RejectType::e_tokens);
+        if (reject.m_cryptoTokens.GetSize() > 0)
+            reject.IncludeOptionalField(RejectType::e_cryptoTokens);
+	}
+}
+
+
 // abstract factory for listeners
 class GkInterface {
 public:
@@ -222,6 +270,10 @@ protected:
 #ifdef HAS_TLS
 	TLSCallSignalListener *m_tlsCallSignalListener;
 	WORD m_tlsSignalPort;
+#endif
+#ifdef HAS_H46018
+	MultiplexH245Listener *m_multiplexH245Listener;
+	WORD m_h245MultiplexPort;
 #endif
 	RasServer *m_rasSrv;
 
@@ -231,6 +283,9 @@ private:
 	virtual CallSignalListener *CreateCallSignalListener();
 #ifdef HAS_TLS
 	virtual TLSCallSignalListener *CreateTLSCallSignalListener();
+#endif
+#ifdef HAS_H46018
+	virtual MultiplexH245Listener *CreateMultiplexH245Listener();
 #endif
 	virtual StatusListener *CreateStatusListener();
 };
@@ -240,7 +295,7 @@ public:
 	typedef PIPSocket::Address Address;
 
 	RasHandler();
-	virtual ~RasHandler() {}
+	virtual ~RasHandler() { }
 
 	// new virtual function
 
@@ -279,7 +334,7 @@ public:
 	virtual ~RasRequester();
 
 	WORD GetSeqNum() const { return m_seqNum; }
-	bool WaitForResponse(int);
+	bool WaitForResponse(int timeout);
 	RasMsg *GetReply();
 
 	// override from class RasHandler
@@ -288,7 +343,7 @@ public:
 	virtual void Stop();
 
 	// new virtual function
-	virtual bool SendRequest(const Address &, WORD, int = 2);
+	virtual bool SendRequest(const Address &, WORD, int retry = 2);
 	virtual bool OnTimeout();
 
 protected:
@@ -317,7 +372,11 @@ public:
 	typedef typename RasInfo<RAS>::ConfirmTag ConfirmTag;
 	typedef typename RasInfo<RAS>::RejectTag RejectTag;
 	Requester(H225_RasMessage &, const Address &);
-	virtual ~Requester() { this->m_rasSrv->UnregisterHandler(this); } // fix for GCC 3.4.2
+	virtual ~Requester()
+	{
+		typedef typename std::pair< RasServer*, RAS* >::first_type RasServerPtr;
+		static_cast< RasServerPtr >(this->m_rasSrv)->UnregisterHandler(this); // fix for GCC 3.4.2
+	}
 };
 
 template<class RAS>
@@ -328,7 +387,9 @@ Requester<RAS>::Requester(H225_RasMessage & obj_ras, const Address & ip) : RasRe
 	ras.m_requestSeqNum = GetSeqNum();
 	AddFilter(ConfirmTag());
 	AddFilter(RejectTag());
-	this->m_rasSrv->RegisterHandler(this); // fix for GCC 3.4.2
+
+	typedef typename std::pair< RasServer*, RAS* >::first_type RasServerPtr;
+	static_cast< RasServerPtr >(this->m_rasSrv)->RegisterHandler(this); // fix for GCC 3.4.2
 }
 
 #endif // RASPDU_H
