@@ -5,7 +5,7 @@
 // Multi-threaded RAS Server for GNU Gatekeeper
 //
 // Copyright (c) Citron Network Inc. 2001-2003
-// Copyright (c) 2000-2014, Jan Willamowius
+// Copyright (c) 2000-2019, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -15,7 +15,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef RASSRV_H
-#define RASSRV_H "@(#) $Id: RasSrv.h,v 1.87 2014/04/03 16:53:29 willamowius Exp $"
+#define RASSRV_H "@(#) $Id$"
 
 #include <vector>
 #include <list>
@@ -104,10 +104,11 @@ public:
 	H225_TransportAddress GetRasAddress(const Address &) const;
 	H225_TransportAddress GetCallSignalAddress(const Address &) const;
 
-	bool SendRas(const H225_RasMessage &, const Address &, WORD, RasListener * = NULL);
-	bool SendRas(const H225_RasMessage &, const H225_TransportAddress &, RasListener * = NULL);
-	bool SendRas(const H225_RasMessage &, const Address &, WORD, const Address &);
-	bool SendRIP(H225_RequestSeqNum seqNum, unsigned ripDelay, const Address & addr, WORD port);
+	bool SendRas(H225_RasMessage &, const Address &, WORD, RasListener * = NULL, GkH235Authenticators * auth = NULL);
+	bool SendRas(H225_RasMessage &, const H225_TransportAddress &, RasListener * = NULL, GkH235Authenticators * auth = NULL);
+	bool SendRas(H225_RasMessage &, const Address &, WORD, const Address &, GkH235Authenticators * auth);
+    bool SendRas(H225_RasMessage & rasobj, const H225_TransportAddress & dest, const Address & local, GkH235Authenticators * auth);
+	bool SendRIP(H225_RequestSeqNum seqNum, unsigned ripDelay, const Address & addr, WORD port, GkH235Authenticators * auth);
 
 	bool IsRedirected(unsigned = 0) const;
 	bool IsForwardedMessage(const H225_NonStandardParameter *, const Address &) const;
@@ -138,13 +139,13 @@ public:
 		return authList->Validate(ras, reason);
 	}
 
-	bool ValidateAdditivePDU(RasPDU<H225_RegistrationRequest>& ras, RRQAuthData& authData);
+	bool ValidateAdditivePDU(RasPDU<H225_RegistrationRequest>& ras, RRQAuthData & authData);
 
-	bool ValidatePDU(RasPDU<H225_RegistrationRequest> & ras, RRQAuthData& authData)
+	bool ValidatePDU(RasPDU<H225_RegistrationRequest> & ras, RRQAuthData & authData)
 	{
 		return authList->Validate(ras, authData);
 	}
-	
+
 	bool ValidatePDU(RasPDU<H225_AdmissionRequest> & ras, ARQAuthData & authData)
 	{
 		return authList->Validate(ras, authData);
@@ -155,10 +156,15 @@ public:
 		return authList->Validate(setup, authData);
 	}
 
+	bool ValidatePDU(Q931 & msg, Q931AuthData & authData)
+	{
+		return authList->Validate(msg, authData);
+	}
+
 	GkAcctLoggerList * GetAcctList() { return acctList; }
 	GkAuthenticatorList * GetAuthList() { return authList; }
 
-	bool LogAcctEvent(int evt, callptr & call, time_t now = 0);
+	bool LogAcctEvent(int evt, const callptr & call, time_t now = 0);
 
 	bool LogAcctEvent(int evt, const endptr & ep);
 
@@ -194,10 +200,13 @@ public:
 #if HAS_DATABASE
 		Toolkit::Instance()->AlternateGKs().GetAlternateGK(ip.m_address, alternates);
 #endif
+        if (Toolkit::Instance()->IsMaintenanceMode() && alternates.GetSize() == 0) {
+            alternates = Toolkit::Instance()->GetMaintenanceAlternate();
+        }
 		if (alternates.GetSize() > 0) {
 			// add alternates by IP
 			ras.IncludeOptionalField(RAS::e_altGKInfo);
-			ras.m_altGKInfo.m_altGKisPermanent = (redirectGK == e_permanentRedirect);
+			ras.m_altGKInfo.m_altGKisPermanent = true; // not just for this message
 			ras.m_altGKInfo.m_alternateGatekeeper = alternates;
 		} else	if (altGKs.GetSize() > 0) {
 			// use global configuration
@@ -208,15 +217,13 @@ public:
 	}
 
 #ifdef h323v6
-	template<class RAS> bool HasAssignedGK(const PString & alias,const PIPSocket::Address & ip, RAS & ras)
+	template<class RAS> bool HasAssignedGK(const PString & alias, const PIPSocket::Address & ip, RAS & ras)
 	{
         H225_ArrayOf_AlternateGK assignedGK;
 		assignedGK.SetSize(0);
 
 		// Queries a DB for gatekeeper registrations with the first being
 		// the assigned gatekeeper. The alternates are then in preference order
-		// Note: If an assigned gatekeeper is found then the registration is not
-		// handled by this gatekeeper but by the assigned gatekeeper.
 		if (!Toolkit::Instance()->AssignedGKs().GetAssignedGK(alias, ip, assignedGK))
 			return false;
 
@@ -226,12 +233,12 @@ public:
         ras.IncludeOptionalField(RAS::e_assignedGatekeeper);
 		ras.m_assignedGatekeeper = assignedGK[0];
 
-		if (assignedGK.GetSize() > 1) {
-			for (PINDEX i=1; i< assignedGK.GetSize(); i++) {
+		if (assignedGK.GetSize() > 0) {
+			ras.IncludeOptionalField(RAS::e_alternateGatekeeper);
+			for (PINDEX i = 0; i< assignedGK.GetSize(); i++) {
 			   ras.m_alternateGatekeeper.SetSize(i);
 			   ras.m_alternateGatekeeper[i-1] = assignedGK[i];
 			}
-			ras.IncludeOptionalField(RAS::e_alternateGatekeeper);
 		}
 
 		return true;
