@@ -5,7 +5,7 @@
 // Routing Mechanism for GNU Gatekeeper
 //
 // Copyright (c) Citron Network Inc. 2003
-// Copyright (c) 2004-2012, Jan Willamowius
+// Copyright (c) 2004-2021, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -15,7 +15,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef ROUTING_H
-#define ROUTING_H "@(#) $Id: Routing.h,v 1.89 2014/04/23 18:32:33 willamowius Exp $"
+#define ROUTING_H "@(#) $Id$"
 
 #include <map>
 #include <list>
@@ -24,6 +24,10 @@
 #include "RasTbl.h"
 #include "stl_supp.h"
 
+#ifdef HAS_JSON
+#include <nlohmann/json.hpp>
+#endif // HAS_JSON
+
 // forward references to avoid includes
 class H225_AdmissionRequest;
 class H225_LocationRequest;
@@ -74,6 +78,7 @@ public:
 		);
 
 	bool operator< (const Route & rhs) const { return m_priority < rhs.m_priority; }
+	bool operator== (const Route & rhs) const { return (m_destAddr == rhs.m_destAddr) && (m_destEndpoint == rhs.m_destEndpoint) && (m_routeId == rhs.m_routeId); }
 
 	unsigned GetPriority() const { return m_priority; }
 	void SetPriority(unsigned p) { m_priority = p; }
@@ -107,7 +112,7 @@ public:
 		e_fromNeighbor = 8,	// currently not used
 		e_Reject = 16
 	};
-	
+
 	RoutingRequest();
 	RoutingRequest(const std::list<Route> & failedRoutes);
 	virtual ~RoutingRequest();
@@ -126,6 +131,10 @@ public:
 	PString GetSourceIP() const { return m_sourceIP; }
 	void SetCallerID(const PString & id) { m_callerID = id; }
 	PString GetCallerID() const { return m_callerID; }
+	void SetCallerDisplayIE(const PString & display) { m_callerDisplayIE = display; }
+	PString GetCallerDisplayIE() const { return m_callerDisplayIE; }
+	void SetCalledDisplayIE(const PString & display) { m_calledDisplayIE = display; }
+	PString GetCalledDisplayIE() const { return m_calledDisplayIE; }
 	void SetGatewayDestination(const H225_TransportAddress & gw) { m_gwDestination = gw; }
 	bool GetGatewayDestination(H225_TransportAddress & gw ) const;
 
@@ -133,6 +142,10 @@ public:
 	void SetServiceType(const PString & service) { m_serviceType = service; }
 
 	bool SupportLanguages() const;
+	bool HasNewSetupInternalAliases() const { return m_hasNewSetupAliases; }
+	H225_ArrayOf_AliasAddress GetNewSetupInternalAliases() const { return m_newSetupAliases; }
+	void SetNewSetupInternalAliases(H225_ArrayOf_AliasAddress aliases) { m_newSetupAliases = aliases; m_hasNewSetupAliases = true;}
+
 private:
 	RoutingRequest(const RoutingRequest &);
 	RoutingRequest& operator=(const RoutingRequest &);
@@ -144,8 +157,13 @@ private:
 	std::list<Route> m_failedRoutes;
 	PString m_sourceIP;
 	PString m_callerID;
+	PString m_callerDisplayIE;
+	PString m_calledDisplayIE;
 	H225_TransportAddress m_gwDestination;
 	PString m_serviceType;
+	/// new destination alias for this call, to be applied to Setup when it comes in
+	bool m_hasNewSetupAliases;
+	H225_ArrayOf_AliasAddress m_newSetupAliases;
 };
 
 template<class R, class W>
@@ -154,11 +172,11 @@ public:
 	typedef R ReqObj;
 	typedef W Wrapper;
 
-	Request(ReqObj & r, Wrapper *w) : m_request(r), m_wrapper(w), m_clientAuthId(0) {}
+	Request(ReqObj & r, Wrapper *w) : m_request(r), m_wrapper(w), m_clientAuthId(0) { }
 	Request(ReqObj & r, Wrapper *w, const PString & id, PUInt64 authid)
-		: m_request(r), m_wrapper(w), m_callingStationId(id), m_clientAuthId(authid) {}
+		: m_request(r), m_wrapper(w), m_callingStationId(id), m_clientAuthId(authid) { }
 	Request(ReqObj & r, Wrapper *w, const std::list<Route> &failedRoutes)
-		: RoutingRequest(failedRoutes), m_request(r), m_wrapper(w), m_clientAuthId(0) {}
+		: RoutingRequest(failedRoutes), m_request(r), m_wrapper(w), m_clientAuthId(0) { }
 
 	bool Process();
 
@@ -191,7 +209,7 @@ typedef Request<H225_Facility_UUIE, FacilityMsg> FacilityRequest;
 template<class T>
 class PolicyList {
 public:
-	typedef T Base;	// for SimpleCreator template 
+	typedef T Base;	// for SimpleCreator template
 
 	PolicyList() : m_next(NULL) { }
 	virtual ~PolicyList() { delete m_next; }  // delete whole list recursively
@@ -215,7 +233,7 @@ T *PolicyList<T>::Create(const PStringArray & rules)
 	for (int i = rules.GetSize(); --i >= 0; ) {
 		PStringArray id = rules[i].Tokenise("::", false);	// check for <policy>::<ID>
 		if (T * current = Factory<T>::Create(id[0])) {
-			if (id.GetSize() > 1) 
+			if (id.GetSize() > 1)
 				current->SetInstance(id[1]);
 			else
 				current->SetInstance("");
@@ -268,7 +286,7 @@ protected:
 	virtual bool OnRequest(SetupRequest &)	   { return false; }
 	virtual bool OnRequest(FacilityRequest &)  { return false; }
 
-	virtual void LoadConfig(const PString & instance) { }	// should be used to load config, always called after the policy object is created
+	virtual void LoadConfig(const PString & /* instance */) { }	// should be used to load config, always called after the policy object is created
 
 protected:
 	/// human readable name for the policy - it should be set inside constructors
@@ -339,7 +357,7 @@ private:
 };
 
 
-// the classical policy, find the dstination from the RegistrationTable
+// the classical policy, find the destination from the RegistrationTable
 class InternalPolicy : public AliasesPolicy {
 public:
 	InternalPolicy();
@@ -352,9 +370,10 @@ protected:
 	virtual bool FindByAliases(LocationRequest &, H225_ArrayOf_AliasAddress &);
 	virtual bool FindByAliases(SetupRequest &, H225_ArrayOf_AliasAddress &);
 	virtual bool FindByAliases(AdmissionRequest &, H225_ArrayOf_AliasAddress &);
-	
+
 private:
 	bool roundRobin;
+	bool leastUsedRouting;
 };
 
 
@@ -417,7 +436,7 @@ public:
 		int m_minLength;
 		int m_maxLength;
 	};
-	
+
 	NumberAnalysisPolicy();
 
 protected:
@@ -429,7 +448,7 @@ protected:
 private:
 	NumberAnalysisPolicy(const NumberAnalysisPolicy &);
 	NumberAnalysisPolicy& operator=(const NumberAnalysisPolicy &);
-	
+
 private:
 	typedef vector<PrefixEntry> Prefixes;
 
@@ -458,7 +477,7 @@ class DestinationRoutes {
 public:
 	DestinationRoutes();
 	~DestinationRoutes() { }
-	
+
 	bool EndPolicyChain() const { return m_endChain; }
 	bool RejectCall() const { return m_reject; }
 	void SetRejectCall(bool reject) { m_reject = reject; m_endChain = true; }
@@ -482,7 +501,7 @@ protected:
 	PStringList m_language;
 };
 
-// superclass for dynamic policies like sql and lua scripring
+// superclass for dynamic policies like sql and lua scripting
 class DynamicPolicy : public Policy {
 public:
 	DynamicPolicy();
@@ -510,8 +529,8 @@ protected:
 		DestinationRoutes & destination) = 0;
 
 	virtual bool ResolveRoute(
-		RoutingRequest & request, 
-		DestinationRoutes & destination
+		RoutingRequest & /* request */,
+		DestinationRoutes & /* destination */
 		) { return true; }
 
 protected:
@@ -551,6 +570,44 @@ protected:
 	long m_timeout;
 };
 
+// a policy to route calls via an SQL database
+class HttpPolicy : public DynamicPolicy {
+public:
+	HttpPolicy();
+	virtual ~HttpPolicy();
+
+protected:
+	virtual void LoadConfig(const PString & instance);
+
+	virtual void RunPolicy(
+		/*in */
+		const PString & source,
+		const PString & calledAlias,
+		const PString & calledIP,
+		const PString & caller,
+		const PString & callingStationId,
+		const PString & callid,
+		const PString & messageType,
+		const PString & clientauthid,
+		const PString & language,
+		/* out: */
+		DestinationRoutes & destination);
+
+#ifdef HAS_JSON
+		virtual void ParseJSONRoute(const nlohmann::json & jsonRoute, const PString & language, DestinationRoutes & destination);
+#endif // HAS_JSON
+
+protected:
+	PString m_url;
+	PString m_body;
+	PCaselessString m_method;
+	PString m_contentType;
+	PRegularExpression m_resultRegex;
+	PRegularExpression m_deleteRegex;
+	PRegularExpression m_errorRegex;
+	bool m_JSONResponse;
+};
+
 
 // a policy to route all calls to one default endpoint
 class CatchAllPolicy : public Policy {
@@ -559,14 +616,14 @@ public:
 	virtual ~CatchAllPolicy() { }
 
 protected:
-	virtual bool OnRequest(AdmissionRequest & request) { return CatchAllRoute(request); }
-	virtual bool OnRequest(LocationRequest & request) { return CatchAllRoute(request); }
-	virtual bool OnRequest(SetupRequest & request) { return CatchAllRoute(request); }
+	virtual bool OnRequest(AdmissionRequest & request);
+	virtual bool OnRequest(LocationRequest & request);
+	virtual bool OnRequest(SetupRequest & request);
 
-	bool CatchAllRoute(RoutingRequest & request) const;
+	bool CatchAllRoute(RoutingRequest & request, bool & updateAlias) const;
 
 	virtual void LoadConfig(const PString & instance);
-	
+
 	PString m_catchAllAlias;
 	PString m_catchAllIP;
 };
@@ -586,8 +643,8 @@ protected:
 	bool URIServiceRoute(RoutingRequest & request, H225_ArrayOf_AliasAddress * aliases) const;
 
 	virtual void LoadConfig(const PString & instance);
-	
-	map<PString,H225_TransportAddress> m_uriServiceRoute;
+
+	std::map<PString,H225_TransportAddress> m_uriServiceRoute;
 };
 
 
@@ -598,7 +655,7 @@ inline bool Request<R, W>::Process()
 }
 
 /** A class that supports ACD (Automatic Call Distribution). A call
-    made to specified alias(-es) (called virtual queue) is signalled
+    made to specified alias(-es) (called virtual queue) is signaled
 	via the GK status line to an external application (an ACD application)
 	that decides where the call should be routed (e.g. what agent should
 	answe the call). Basically, it rewrites virtual queue alias
@@ -629,7 +686,7 @@ public:
 		True if the external application routed the call (either by specifying
 		an alias or by rejecting the call), false if timed out waiting
 		for the routing decision.
-		If the request was rejected, destinationInfo is set to an epmty array
+		If the request was rejected, destinationInfo is set to an empty array
 		(0 elements).
 	*/
 	bool SendRouteRequest(
@@ -648,19 +705,31 @@ public:
 		PString * bindIP,
 		/// caller ID
 		PString * callerID,
+		/// DisplayIE of caller
+		PString * callerDisplayIE,
+		/// DisplayIE of called party
+		PString * calledDisplayIE,
 		/// should the call be rejected modified by this function on return)
 		bool & reject,
+        /// H.225 ReleaseComplete reason
+        unsigned & rejectReason,
+        /// don't communicate updated route to caller
+        bool & keepRouteInternal,
 		/// an actual virtual queue name (should be present in destinationInfo too)
-		const PString& vqueue,
+		const PString & vqueue,
 		/// a sequence of aliases for the calling endpoint
 		/// (in the "alias:type[=alias:type]..." format)
 		const PString & sourceInfo,
 		/// the callID as string
 		const PString & callID,
 		/// the called IP for unregistered calls
-		const PString & calledip = "unknown",
+		const PString & calledip,
 		/// vendor string of caller
-		const PString & vendorString = "unknown"
+		const PString & vendorString,
+        /// the IP we received this message from
+        const PString & fromIP,
+        /// type of message that caused this RouteRequest
+        const PString & msgType
 		);
 
 	/** Make a routing decision for a pending route request (inserted
@@ -687,7 +756,15 @@ public:
 		// callerID or empty
 		const PString & callerID,
 		/// should this call be rejected
-		bool reject = false
+		bool reject = false,
+        /// don't communicate updated route to caller
+        bool keepRouteInternal = false,
+        /// Display IE or empty
+        const PString & displayIE = PString::Empty(),
+        /// Display IE of called party or empty
+        const PString & calledDisplayIE = PString::Empty(),
+		/// H225_AdmissionRejectReason/H.225 ReleaseComplete reason
+		int reason = -1
 		);
 
 	/** Make a routing decision for a pending route request (inserted
@@ -701,7 +778,7 @@ public:
 		/// will replace the original destination info
 		const PString & agent,
 		/// will replace the original destinationCallSignallAddress
-		const PString & destinationip, 		
+		const PString & destinationip,
 		/// identifier of the endpoint associated with the route request
 		const PString & callingEpId,
 		/// CRV of the call associated with the route request
@@ -713,7 +790,15 @@ public:
 		// callerID or empty
 		const PString & callerID,
 		/// should this call be rejected
-		bool reject = false
+		bool reject = false,
+        /// don't communicate updated route to caller
+        bool keepRouteInternal = false,
+        /// Display IE or empty
+        const PString & displayIE = PString::Empty(),
+        /// Display IE of called party or empty
+        const PString & calledDisplayIE = PString::Empty(),
+		/// H225_AdmissionRejectReason/H.225 ReleaseComplete reason
+		int reason = -1
 		);
 
 	/** Reject a pending route request (inserted by SendRequest).
@@ -727,7 +812,9 @@ public:
 		/// CRV of the call associated with the route request
 		unsigned crv,
 		/// callID of the call associated with the route request
-		const PString & callID
+		const PString & callID,
+		/// H225_AdmissionRejectReason/H.225 ReleaseComplete reason
+		int reason = -1
 		);
 
 	/** @return
@@ -748,12 +835,16 @@ private:
 			H225_ArrayOf_AliasAddress * agent,
 			PString * callsignaladdr,
 			PString * bindIP,
-			PString * callerID
+			PString * callerID,
+			PString * callerDisplayIE,
+			PString * calledDisplayIE
 			)
 			:
 			m_callingEpId((const char*)callingEpId), m_crv(crv), m_callID(callID),
 			m_agent(agent), m_callsignaladdr(callsignaladdr), m_sourceIP(bindIP),
-			m_callerID(callerID), m_reject(false) { }
+			m_callerID(callerID), m_callerDisplayIE(callerDisplayIE), m_calledDisplayIE(calledDisplayIE),
+			m_reject(false), m_rejectReason(-1),
+			m_keepRouteInternal(false) { }
 
 		/// identifier for the endpoint associated with this request
 		PString m_callingEpId;
@@ -770,8 +861,16 @@ private:
 		PString * m_sourceIP;
 		/// callerID or empty
 		PString * m_callerID;
+		/// Display IE of caller or empty
+		PString * m_callerDisplayIE;
+		/// Display IE of called party or empty
+		PString * m_calledDisplayIE;
 		/// should this call be rejected
 		bool m_reject;
+        /// H225_AdmissionRejectReasonH225_AdmissionRejectReasonH.225 ReleaseComplete reason
+        int m_rejectReason;
+        /// don't communicate changed route to caller
+        bool m_keepRouteInternal;
 		/// a synchronization point for signaling that routing decision
 		/// has been made by the external application
 		PSyncPoint m_sync;
@@ -796,6 +895,10 @@ private:
 		PString * bindIP,
 		/// caller ID
 		PString * callerID,
+		/// Display IE of caller
+		PString * callerDisplayIE,
+		/// Display IE of called
+		PString * calledDisplayIE,
 		/// set by the function to true if another route request for the same
 		/// call is pending
 		bool & duplicate
