@@ -2,7 +2,7 @@
 //
 // SoftPBX.cxx
 //
-// Copyright (c) 2000-2012, Jan Willamowius
+// Copyright (c) 2000-2017, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -11,6 +11,7 @@
 //
 //////////////////////////////////////////////////////////////////
 
+#include "config.h"
 #include <ptlib.h>
 #include <h323pdu.h>
 #include "gk_const.h"
@@ -22,6 +23,7 @@
 #include "capctrl.h"
 #include "h323util.h"
 #include "MakeCall.h"
+#include "Neighbor.h"
 
 int SoftPBX::TimeToLive = -1;
 PTime SoftPBX::StartUp;
@@ -164,6 +166,30 @@ void SoftPBX::UnregisterIp(const PString & Ip)
 	GkStatus::Instance()->SignalStatus(msg + "\r\n");
 }
 
+void SoftPBX::UnregisterEndpoint(const PString & idStr)
+{
+	H225_EndpointIdentifier epId;
+	epId = idStr;
+	PTRACE(3, "GK\tSoftPBX: UnregisterEP " << idStr);
+
+	const endptr ep = RegistrationTable::Instance()->FindByEndpointId(epId);
+	if (!ep) {
+		PString msg("Endpoint ID " + idStr + " not found!");
+		PTRACE(1, "GK\tSoftPBX: " + msg);
+		GkStatus::Instance()->SignalStatus(msg + "\r\n");
+		return;
+	}
+	DisconnectEndpoint(ep);
+	ep->Unregister();
+
+	// remove the endpoint (even if we don't get a UCF - the endoint might be dead)
+	RegistrationTable::Instance()->RemoveByEndptr(ep);
+
+	PString msg("Endpoint " + idStr + " unregistered!");
+	PTRACE(2, "GK\tSoftPBX: " + msg);
+	GkStatus::Instance()->SignalStatus(msg + "\r\n");
+}
+
 void SoftPBX::DisconnectAll()
 {
 	CallTable::Instance()->ClearTable();
@@ -289,7 +315,7 @@ void SoftPBX::SendProceeding(const PString & CallId)
 	PTRACE(3, "GK\tSoftPBX: SendProceeding " << CallId);
 
 	H225_CallIdentifier cid = StringToCallId(CallId);
- 
+
 	// CallProceeding will be sent during the routing process
 	// at this time the call won't yet be in the CallTable,
 	// thus we look for it in the PreliminaryCallTable
@@ -305,12 +331,12 @@ void SoftPBX::SendProceeding(const PString & CallId)
 			return;
 		}
 	} else {
-		PString msg("No call to send CallProceeding! " + AsString(cid.m_guid));
+		PString msg("No call to send CallProceeding! " + AsString(cid));
 		PTRACE(1, "GK\tSoftPBX: " + msg);
 		GkStatus::Instance()->SignalStatus(msg + "\r\n");
 		return;
 	}
-	
+
 	Q931 q931;
 	PBYTEArray lBuffer;
 	lForwardedSocket->BuildProceedingPDU(q931, call->GetCallIdentifier(), call->GetCallRef() | 0x8000u);
@@ -405,6 +431,13 @@ void SoftPBX::RerouteCall(const PString & CallId, const PCaselessString & whichL
 {
 	PTRACE(1, "GK\tSoftPBX: RerouteCall " << CallId << " (" << whichLeg << ") -> " << destination);
 
+	if (whichLeg != "CALLER" && whichLeg != "CALLED") {
+        PString msg("SoftPBX: Error: reroute direction must be either CALLER or CALLED");
+        PTRACE(1, "GK\t" + msg);
+        GkStatus::Instance()->SignalStatus(msg + "\r\n");
+        return;
+	}
+
 	CallLeg which = (whichLeg == "called") ? Called : Caller;
 	callptr lCall = CallTable::Instance()->FindCallRec(StringToCallId(CallId));
 
@@ -416,14 +449,14 @@ void SoftPBX::RerouteCall(const PString & CallId, const PCaselessString & whichL
 			lForwardedSocket = lCall->GetCallSignalSocketCalling();
 		}
 		if (lForwardedSocket) {
-			if (!lForwardedSocket->RerouteCall(which, destination, false)) {
+			if (!lForwardedSocket->RerouteCall(which, destination)) {
 				PString msg("SoftPBX: Reroute failed");
 				PTRACE(1, "GK\t" + msg);
 				GkStatus::Instance()->SignalStatus(msg + "\r\n");
 				return;
 			}
 		} else {
-			PString msg("SoftPBX: can't find signalling socket (direct mode ?)");
+			PString msg("SoftPBX: can't find signaling socket (direct mode ?)");
 			PTRACE(1, "GK\t" + msg);
 			GkStatus::Instance()->SignalStatus(msg + "\r\n");
 			return;
@@ -444,10 +477,10 @@ void SoftPBX::RerouteCall(const PString & CallId, const PCaselessString & whichL
 void SoftPBX::TransferCall(const PString & CallId, const PCaselessString & which, const PString & Destination, const PString & method)
 {
 	PTRACE(1, "GK\tSoftPBX: TransferCall " << CallId << " (" << which << ") -> " << Destination << " using " << method);
- 
+
 	callptr lCall = CallTable::Instance()->FindCallRec(StringToCallId(CallId));
 	CallSignalSocket *lForwardedSocket = NULL;
- 
+
 	if (lCall) {
 		if (which == "called") {
 			lForwardedSocket = lCall->GetCallSignalSocketCalled();
@@ -455,7 +488,7 @@ void SoftPBX::TransferCall(const PString & CallId, const PCaselessString & which
 			lForwardedSocket = lCall->GetCallSignalSocketCalling();
 		}
 		if (!lForwardedSocket) {
-			PString msg("SoftPBX: can't find signalling socket (direct mode ?)");
+			PString msg("SoftPBX: can't find signaling socket (direct mode ?)");
 			PTRACE(1, "GK\t" + msg);
 			GkStatus::Instance()->SignalStatus(msg + "\r\n");
 			return;
@@ -466,7 +499,7 @@ void SoftPBX::TransferCall(const PString & CallId, const PCaselessString & which
 		GkStatus::Instance()->SignalStatus(msg + "\r\n");
 		return;
 	}
- 
+
 	Q931 q931;
 	PBYTEArray lBuffer;
 	if (method == "FacilityRouteCallToMC") {
@@ -476,20 +509,20 @@ void SoftPBX::TransferCall(const PString & CallId, const PCaselessString & which
 	}
 	q931.Encode(lBuffer);
 	lForwardedSocket->TransmitData(lBuffer);
- 
+
 	PString msg = PString("SoftPBX: call ") + CallId + " transferred from " + which + " to " + Destination;
 	PTRACE(1, "GK\t" + msg);
 	GkStatus::Instance()->SignalStatus(msg + "\r\n");
 }
 
-void SoftPBX::MakeCall(const PString & SourceAlias, const PString & DestinationAlias)
+void SoftPBX::MakeCall(const PString & SourceAlias, const PString & DestinationAlias, const PString & TransferMethod)
 {
 	PTRACE(3, "GK\tSoftPBX: MakeCall " << SourceAlias << " -> " << DestinationAlias);
 	if (! MakeCallEndPoint::Instance()->IsRegisteredWithGk()) {
 		PThread::Sleep(500);	// give pseudo-endpoint 0.5 sec to register
 	}
 	if (MakeCallEndPoint::Instance()->IsRegisteredWithGk()) {
-		MakeCallEndPoint::Instance()->ThirdPartyMakeCall(SourceAlias, DestinationAlias);
+		MakeCallEndPoint::Instance()->ThirdPartyMakeCall(SourceAlias, DestinationAlias, TransferMethod);
 	} else {
 		PTRACE(1, "GK\tSoftPBX: MakeCall registration of pseudo-endpoint failed");
 		SNMP_TRAP(7, SNMPError, General, "MakeCall endpoint registration failed");
@@ -517,6 +550,50 @@ void SoftPBX::PrintEndpointQoS(USocket *client)
 	RegistrationTable::Instance()->PrintEndpointQoS(client);
 }
 
+void SoftPBX::PrintNeighbors(USocket *client)
+{
+	PTRACE(3, "GK\tSoftPBX: PrintNeighbors");
+    client->TransmitData("Neighbors\r\n");
+	NeighborList::List & neighbors = *RasServer::Instance()->GetNeighbors();
+    for(NeighborList::List::iterator iter = neighbors.begin(); iter != neighbors.end(); ++iter) {
+        Neighbors::Neighbor * n = (*iter);
+        PString msg = "NB|" + n->GetGkId() + "|" + AsString(n->GetIP(), n->GetPort());
+        msg += PString("|") + (n->IsDisabled() ? "disabled" : "enabled") + "|";
+        // IsClient() means we are a client to this neighbor server
+        if (n->IsH46018Client())
+            msg += "H.460.18 server";
+        // IsServer() means we are a server to this neighbor client
+        if (n->IsH46018Server())
+            msg += "H.460.18 client";
+        msg += "\r\n";
+        client->TransmitData(msg);
+    }
+    client->TransmitData(PString("Number of Neighbors: ") + PString(neighbors.size()) + "\r\n;\r\n");
+}
+
+void SoftPBX::PrintCallInfo(USocket *client, const PString & callid)
+{
+	PTRACE(3, "GK\tSoftPBX: PrintCallInfo");
+    client->TransmitData("CallInfo\r\n");
+    CallTable::Instance()->PrintCallInfo(client, callid);
+    client->TransmitData(";\r\n");
+}
+
+void SoftPBX::MaintenanceMode(bool on, const PString & alternate)
+{
+	PTRACE(3, "GK\tSoftPBX: MaintenanceMode " << (on ? "ON" : "OFF") << " " << alternate);
+	Toolkit::Instance()->SetMaintenanceMode(on);
+	if (IsIPAddress(alternate)) {
+        Toolkit::Instance()->SetMaintenanceAlternate(alternate);
+    } else {
+        Toolkit::Instance()->SetMaintenanceAlternate("");
+    }
+    if (on) {
+        RegistrationTable::Instance()->UnregisterAllEndpointsNotInCall();
+    }
+    GkStatus::Instance()->SignalStatus(PString("MaintenanceMode ") + (on ? "ON" : "OFF") + " " + alternate + "\r\n");
+}
+
 PString SoftPBX::Uptime()
 {
 	long total = (PTime() - SoftPBX::StartUp).GetSeconds();
@@ -530,3 +607,8 @@ PString SoftPBX::Uptime()
 			(const char *)SoftPBX::StartUp.AsString(),
 			days, hour, min, sec);
 }
+
+PInt32 SoftPBX::UptimeTicks()
+{
+	return (PTime() - SoftPBX::StartUp).GetMilliSeconds() / 10;
+}
