@@ -2,7 +2,7 @@
 //
 // SoftPBX.h
 //
-// Copyright (c) 2000-2011, Jan Willamowius
+// Copyright (c) 2000-2017, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -12,7 +12,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef SOFTPBX_H
-#define SOFTPBX_H "@(#) $Id: SoftPBX.h,v 1.28 2011/10/18 19:24:51 willamowius Exp $"
+#define SOFTPBX_H "@(#) $Id$"
 
 // nothing to include :)
 
@@ -37,6 +37,7 @@ namespace SoftPBX
 	void UnregisterAllEndpoints();
 	void UnregisterAlias(const PString & Alias);
 	void UnregisterIp(const PString & Ip);
+	void UnregisterEndpoint(const PString & idStr);
 	void DisconnectAll();
 	void DisconnectCall(unsigned CallNumber);
 	void DisconnectCallId(const PString & CallId);
@@ -47,13 +48,17 @@ namespace SoftPBX
 	void SendProceeding(const PString & CallId);
 	void TransferCall(const PString & SourceAlias, const PString & DestinationAlias);
 	void TransferCall(const PString & CallId, const PCaselessString & which, const PString & Destination, const PString & method);
-	void MakeCall(const PString & SourceAlias, const PString & DestinationAlias);
+	void MakeCall(const PString & SourceAlias, const PString & DestinationAlias, const PString & TransferMethod = "");
 	void RerouteCall(const PString & CallId, const PCaselessString & whichLeg, const PString & destination);
 	void PrintPrefixCapacities(USocket *client, const PString & alias);
 	void PrintCapacityControlRules(USocket *client);
 	void PrintEndpointQoS(USocket *client);
-	
+	void PrintNeighbors(USocket *client);
+	void PrintCallInfo(USocket *client, const PString & callid);
+	void MaintenanceMode(bool on, const PString & alternate = "");
+
 	PString Uptime();
+	PInt32 UptimeTicks();
 
 	extern int TimeToLive;
 	extern PTime StartUp;
