@@ -2,7 +2,7 @@
 //
 // Toolkit class for the GnuGk
 //
-// Copyright (c) 2000-2013, Jan Willamowius
+// Copyright (c) 2000-2024, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -12,7 +12,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef TOOLKIT_H
-#define TOOLKIT_H "@(#) $Id: Toolkit.h,v 1.116 2013/10/20 19:43:20 shorne Exp $"
+#define TOOLKIT_H "@(#) $Id$"
 
 #include <vector>
 #include <map>
@@ -32,8 +32,6 @@ extern "C" {
 #include <openssl/err.h>
 #include <openssl/rand.h>
 #include <openssl/ssl.h>
-// on Windows PTLib 2.10.x doesn't set an #pragma include_alias for this file,
-// so the OpenSSL include dir must be added to IDE settings
 #include <openssl/x509v3.h>	// needed for certificate check
 }
 #endif
@@ -48,6 +46,8 @@ template <class> class H225SignalingMsg;
 typedef H225SignalingMsg<H225_Setup_UUIE> SetupMsg;
 struct SetupAuthData;
 
+extern bool g_disableSettingUDPSourceIP;
+
 /// Hold an address of a single host or a whole network
 class NetworkAddress {
 public:
@@ -59,15 +59,18 @@ public:
 	NetworkAddress(const PIPSocket::Address & addr, const PIPSocket::Address & nm);
 	/// Build an address from the string
 	NetworkAddress(
-		const PString &str /// an address in a form A.B.C.D, A.B.C.D/24 or A.B.C.D/255.255.255.0
+		const PString & str /// an address in a form A.B.C.D, A.B.C.D/24 or A.B.C.D/255.255.255.0 or IPv6 format
 		);
 
+    /// @return IP version (4 or 6)
+    unsigned GetVersion() const { return m_address.GetVersion(); }
+
 	/// @return	Length of the network mask (number of significant bits)
 	unsigned GetNetmaskLen() const;
 
 	/** Compare two network addresses and define their relative order.
 	    Ordering is done accordingly to netmask length and then to IP bytes.
-		
+
 	    @return <0 if this address is lesser than #addr#, 0 if both are equal,
 	            >0 if this address is greater than #addr#.
 	*/
@@ -78,21 +81,21 @@ public:
 
 	/// @return	True if this is a wildcard address
 	bool IsAny() const;
-		
+
 	/// @return	True if the given address is equal to this address
-	bool operator==(const PIPSocket::Address &addr) const;
-	bool operator==(const NetworkAddress &addr) const;
+	bool operator==(const PIPSocket::Address & addr) const;
+	bool operator==(const NetworkAddress & addr) const;
 
-	/// @return	True if the given address is contained within this network		
-	bool operator>>(const PIPSocket::Address &addr) const;
-	bool operator>>(const NetworkAddress &addr) const;
-	/// @return	True if the given network contains this network		
-	bool operator<<(const NetworkAddress &addr) const;
+	/// @return	True if the given address is contained within this network
+	bool operator>>(const PIPSocket::Address & addr) const;
+	bool operator>>(const NetworkAddress & addr) const;
+	/// @return	True if the given network contains this network
+	bool operator<<(const NetworkAddress & addr) const;
 
-	bool operator<(const NetworkAddress &addr) const;
-	bool operator<=(const NetworkAddress &addr) const;
-	bool operator>(const NetworkAddress &addr) const;
-	bool operator>=(const NetworkAddress &addr) const;
+	bool operator<(const NetworkAddress & addr) const;
+	bool operator<=(const NetworkAddress & addr) const;
+	bool operator>(const NetworkAddress & addr) const;
+	bool operator>=(const NetworkAddress & addr) const;
 
 	PIPSocket::Address m_address; /// host/network address
 private:
@@ -100,9 +103,9 @@ private:
 };
 
 /// @return	True if the given address equals to this address
-bool operator==(const PIPSocket::Address &addr, const NetworkAddress &net);
+bool operator==(const PIPSocket::Address & addr, const NetworkAddress & net);
 /// @return	True if the given address is contained withing this network
-bool operator<<(const PIPSocket::Address &addr, const NetworkAddress &net);
+bool operator<<(const PIPSocket::Address & addr, const NetworkAddress & net);
 
 #ifdef H323_H350
 class H350_Session;
@@ -114,7 +117,7 @@ class GkTimerManager;
 class CLIRewrite;
 
 #define GW_REWRITE_IN		true
-#define GW_REWRITE_OUT	false
+#define GW_REWRITE_OUT		false
 
 class Toolkit : public Singleton<Toolkit>
 {
@@ -128,7 +131,7 @@ class Toolkit : public Singleton<Toolkit>
 		typedef PIPSocket::InterfaceTable InterfaceTable;
 
 	public:
-		RouteTable() : rtable_begin(NULL), rtable_end(NULL), DynExtIP(false), ExtIP("") { /* initialize later */ }
+		RouteTable() : rtable_begin(NULL), rtable_end(NULL), DynExtIP(false) { /* initialize later */ }
 		virtual ~RouteTable() { ClearTable(); }
 		Address GetLocalAddress(unsigned version) const;
 		Address GetLocalAddress(const Address & dest) const;
@@ -139,6 +142,7 @@ class Toolkit : public Singleton<Toolkit>
 		bool IsEmpty() const { return rtable_begin == NULL; }
 
 		std::vector<NetworkAddress> GetInternalNetworks() const { return m_internalnetworks; }
+		void ClearInternalNetworks();
 		void AddInternalNetwork(const NetworkAddress & network);
 
 	    virtual bool IsMasquerade(PIPSocket::Address &) const { return false; }
@@ -147,7 +151,8 @@ class Toolkit : public Singleton<Toolkit>
 		public:
 			RouteEntry(const PString &);
 			RouteEntry(const PIPSocket::RouteEntry &, const InterfaceTable &);
-			bool Compare(const Address *) const;
+			bool CompareWithoutMask(const Address *) const;
+			bool CompareWithMask(const Address *) const;
 		};
 		virtual bool CreateTable() { return CreateRouteTable(); }
 		bool CreateRouteTable(const PString & extroute = PString::Empty());
@@ -187,7 +192,7 @@ class Toolkit : public Singleton<Toolkit>
 		int SelectRoutingMode(const Address & ip1, const Address & ip2) const;
 		void LoadConfig(PConfig *);
 		int IsInternal(const Address & ip) const;
-	
+
 	private:
 		int ToRoutingMode(const PCaselessString & mode) const;	// returns a CallRec::RoutingMode
 		NetworkAddress FindModeRule(const NetworkAddress & ip) const;
@@ -231,6 +236,7 @@ class Toolkit : public Singleton<Toolkit>
 		PString m_RewriteFastmatch;
 		char m_TrailingChar;
 		PStringArray m_defaultDomain;
+		PString m_externalIP;
 		RewriteData * m_Rewrite;
 	};
 
@@ -286,7 +292,7 @@ class Toolkit : public Singleton<Toolkit>
 	   bool m_sqlactive;
 	   // connection to the SQL database
 	   GkSQLConnection* m_sqlConn;
-	   // parametrized query string for the auth condition string retrieval
+	   // parametrized query string
 	   PString m_query;
 	   // query timeout
 	   long m_timeout;
@@ -308,9 +314,9 @@ class Toolkit : public Singleton<Toolkit>
 	    void LoadConfig(PConfig *);
 		bool QueryAssignedGK(const PString & alias, const PIPSocket::Address & ip, PStringArray & addresses);
 #ifdef H323_H350
-		bool QueryH350Directory(const PString & alias,const PIPSocket::Address & ip, PStringArray & addresses);
+		bool QueryH350Directory(const PString & alias, const PIPSocket::Address & ip, PStringArray & addresses);
 #endif
-        bool GetAssignedGK(const PString & alias,const PIPSocket::Address & ip, H225_ArrayOf_AlternateGK & gklist);
+        bool GetAssignedGK(const PString & alias, const PIPSocket::Address & ip, H225_ArrayOf_AlternateGK & gklist);
 
 	 protected:
 		 std::vector< std::pair<PString, PString> > assignedGKList;
@@ -320,7 +326,7 @@ class Toolkit : public Singleton<Toolkit>
 	   bool m_sqlactive;
 	   // connection to the SQL database
 	   GkSQLConnection* m_sqlConn;
-	   // parametrized query string for the auth condition string retrieval
+	   // parametrized query string
 	   PString m_query;
 	   // query timeout
 	   long m_timeout;
@@ -345,7 +351,7 @@ class Toolkit : public Singleton<Toolkit>
 	   bool m_sqlactive;
 	   // connection to the SQL database
 	   GkSQLConnection* m_sqlConn;
-	   // parametrized query string for the auth condition string retrieval
+	   // parametrized query string
 	   PString m_query;
 	   // query timeout
 	   long m_timeout;
@@ -353,6 +359,27 @@ class Toolkit : public Singleton<Toolkit>
 	AlternateGatekeepers AlternateGKs() const { return m_AlternateGKs; }
 #endif
 
+#if HAS_DATABASE
+	class GnuGkAssignedGatekeepers {
+	 public:
+	    GnuGkAssignedGatekeepers();
+		~GnuGkAssignedGatekeepers();
+
+	    void LoadConfig(PConfig *);
+        bool HasAssignedGk(endptr ep, const PIPSocket::Address & ip);
+
+     private:
+	   bool m_sqlactive;
+	   // connection to the SQL database
+	   GkSQLConnection* m_sqlConn;
+	   // parametrized query string
+	   PString m_query;
+	   // query timeout
+	   long m_timeout;
+	};
+	GnuGkAssignedGatekeepers GnuGkAssignedGKs() const { return m_GnuGkAssignedGKs; }
+#endif
+
 #if HAS_DATABASE
 	class QoSMonitor {
 	  public:
@@ -381,12 +408,20 @@ class Toolkit : public Singleton<Toolkit>
     bool CreateH350Session(H350_Session * session);
 #endif
 
+#ifdef P_SSL
+    void InitOpenSSL();
+#endif
+
 #ifdef HAS_TLS
 	bool IsTLSEnabled() const;
 	SSL_CTX * GetTLSContext();
 	bool MatchHostCert(SSL * ssl, PIPSocket::Address addr);
 #endif
 
+	void CheckLicense();
+	bool IsLicenseValid(PString & message) const;
+	PString GetServerID() const;
+	PString GetLicenseType() const;
 
 #ifdef HAS_LANGUAGE
 	class AssignedLanguage {
@@ -415,6 +450,8 @@ class Toolkit : public Singleton<Toolkit>
 	AssignedLanguage GetAssignedLanguages() const { return m_assignedLanguage; }
 #endif
 
+	bool PASNEqual(PASN_OctetString * str1, PASN_OctetString * str2);
+
 	/// maybe modifies #alias#. returns true if it did
 	bool RewriteE164(H225_AliasAddress & alias);
 	bool RewriteE164(H225_ArrayOf_AliasAddress & aliases);
@@ -455,6 +492,14 @@ class Toolkit : public Singleton<Toolkit>
 	void SetGKHome(const PStringArray &);
 	bool IsGKHome(const PIPSocket::Address & addr) const;
 
+	void SetMaintenanceMode(bool val) { m_maintenanceMode = val; }
+	bool IsMaintenanceMode() const { return m_maintenanceMode; }
+	void SetMaintenanceAlternate(const PString & alternate) { m_maintenanceAlternate = alternate; }
+	H225_ArrayOf_AlternateGK GetMaintenanceAlternate() const;
+
+    PString GetExternalIP() const;
+    PString ReplaceGlobalParams(const PString & str);
+
 	bool RemoveAllH235Tokens() const { return m_alwaysRemoveH235Tokens; }
 	bool RemoveH235TokensFrom(const PIPSocket::Address & addr) const;
 #ifdef HAS_H235_MEDIA
@@ -473,19 +518,16 @@ class Toolkit : public Singleton<Toolkit>
 	bool GetH46023STUN(const PIPSocket::Address & addr,  H323TransportAddress & stun);
 	bool H46023SameNetwork(const PIPSocket::Address & addr1, const PIPSocket::Address & addr2);
 #endif
-#ifdef HAS_H46026
-	bool IsH46026Enabled() const;
-#endif
+	bool IsH46026Enabled() const { return m_H46026Enabled; }
 #ifdef HAS_H460P
 	bool IsH460PEnabled() const;
 	GkPresence & GetPresenceHandler();
 #endif
-	bool IsSNMPEnabled() const;
-	bool IsIPv6Enabled() const;
+	bool IsSNMPEnabled() const { return m_snmpEnabled; }
+	bool IsIPv6Enabled() const { return m_ipv6Enabled; }
 	bool IsPortNotificationActive();
 	void PortNotification(PortType type, PortAction action, const PString & protocol,
-							const PIPSocket::Address & addr, WORD port,
-							const H225_CallIdentifier & callID = 0);
+							const PIPSocket::Address & addr, WORD port, PINDEX callNo = 0);
 
 	// accessors
 	/** Accessor and 'Factory' to the static Toolkit.
@@ -518,7 +560,7 @@ class Toolkit : public Singleton<Toolkit>
 	/** Sets the config that the toolkit uses to a given config.
 	 *  A prior loaded Config is discarded.
 	 */
-	PConfig* SetConfig(const PFilePath &fp, const PString &section);
+	PConfig* SetConfig(const PFilePath & fp, const PString & section);
 
 	/* This method modifies the config from status port
 	 * Warning: don't modify the config via status port and change config file simultaneously,
@@ -542,7 +584,7 @@ class Toolkit : public Singleton<Toolkit>
 	 * @param regexStr the string which is compiled to a regex and executed with #regex.Execute(str, pos)#
 	 * @return TRUE if the regex matched and FALSE if not or any error case.
 	 */
-	static bool MatchRegex(const PString &str, const PString &regexStr);
+	static bool MatchRegex(const PString &str, const PString & regexStr);
 
 	/** returns the #bool# that #str# represents.
 	 * Case insensitive, "t...", "y...", "a...", "1" are #TRUE#, all other values are #FALSE#.
@@ -569,21 +611,24 @@ class Toolkit : public Singleton<Toolkit>
 	enum T35ManufacturerCodes {
 		t35mOpenOrg = 4242,     /// manufacurers code for the "Open Source Organisation"
 		t35mGnuGk = 2,
-		t35mCisco = 18
+		t35mCisco = 18,
+		t35mAvaya = 16726,
+		t35mLucent = 19540
 	};
 	enum T35OpenOrgExtensions {
-		t35eFailoverRAS = 255  /// Defined HERE!
+		t35eFailoverRAS = 255
 	};
 	enum T35GnuGkExtensions {
 		t35eNeighborId = 1,
-		t35eNATTraversal = 2
+		t35eNATTraversal = 2,
+		t35eH245KeepAlive = 3
 	};
-	
+
 	/** If the triple #(country,extension,manufacturer)# represents an
 	 * extension known to the GnuGk this method returns its 'internal extension code'
 	 # #iecXXX' or #iecUnknow# otherwise.
 	 *
-	 * Overwriting methods should use a simlilar scheme and call
+	 * Overwriting methods should use a similar scheme and call
 	 * <code>
 	 * if(inherited::GnuGKExtension(country,extension,menufacturer) == iecUnknown) {
 	 *   ...
@@ -593,11 +638,11 @@ class Toolkit : public Singleton<Toolkit>
 	 * </code>
 	 * This results in 'cascading' calls until a iec!=iecUnkown is returned.
 	 */
-	virtual int GetInternalExtensionCode(const unsigned &country,
-						 const unsigned &extension,
-						 const unsigned &manufacturer) const;
+	virtual int GetInternalExtensionCode(const unsigned & country,
+						 const unsigned & extension,
+						 const unsigned & manufacturer) const;
 
-	int GetInternalExtensionCode(const H225_H221NonStandard& data) const;
+	int GetInternalExtensionCode(const H225_H221NonStandard & data) const;
 
 	/** Generate a call id for accounting purposes, that is unique
 		during subsequent GK start/stop events.
@@ -611,7 +656,7 @@ class Toolkit : public Singleton<Toolkit>
 	    A path to a temp directory.
 	*/
 	PString GetTempDir() const;
-	
+
 	/** @return
 	    A pointer to the GkTimerManager object that allows registration
 	    of time scheduled events.
@@ -622,25 +667,25 @@ class Toolkit : public Singleton<Toolkit>
 	    format string or (if it is an empty string) to the default format string.
 	    The format string conforms to strftime formatting rules or can be a one
 	    of predefined constants: Cisco, ISO8601, RFC822, MySQL.
-		
+
 	    @return	Formatted timestamp string.
 	*/
 	PString AsString(
-		const PTime& tm, /// timestamp to convert into a string
-		const PString& formatStr = PString::Empty() /// format string to use
+		const PTime & tm, /// timestamp to convert into a string
+		const PString & formatStr = PString::Empty() /// format string to use
 		) const;
 
 	/** Read and decrypt a password from the config. As a decryption key
 	    this function is using the given key name padded with bytes of value
 	    specified by the 'KeyFilled' config variable, if it is found
 	    in the given config section, or a global padding byte.
-		
+
 	    @return
 	    A decrypted password or an empty string, if the given key is missing.
-	*/	
+	*/
 	PString ReadPassword(
-		const PString &cfgSection, /// config section to read
-		const PString &cfgKey, /// config key to read an encrypted password from
+		const PString & cfgSection, /// config section to read
+		const PString & cfgKey, /// config key to read an encrypted password from
 		bool forceEncrypted = false /// decrypt even if no KeyFilled is present
 		);
 
@@ -648,12 +693,12 @@ class Toolkit : public Singleton<Toolkit>
 	void RewriteCLI(
 		SetupMsg & msg /// Q.931 Setup message to be rewritten
 		) const;
-		
+
 	/// Outbound rewrite for ANI/CLI
 	void RewriteCLI(
-		SetupMsg &msg, /// Q.931 Setup message to be rewritten
+		SetupMsg & msg, /// Q.931 Setup message to be rewritten
 		SetupAuthData & authData, /// additional data
-		const PIPSocket::Address &destAddr /// callee's IP
+		const PIPSocket::Address & destAddr /// callee's IP
 		) const;
 
 	void RewriteSourceAddress(SetupMsg & msg) const;
@@ -661,29 +706,30 @@ class Toolkit : public Singleton<Toolkit>
 	void SetRerouteCauses(unsigned char *causeMap);
 
 	/** Map H225_ReleaseCompleteReason code to Q.931 cause value.
-	
+
 	@return
 	The corresponding Q.931 cause value or 0, if there is no direct mapping.
-	*/	
+	*/
 	unsigned MapH225ReasonToQ931Cause(int reason);
 
 	void ParseTranslationMap(std::map<unsigned, unsigned> & cause_map, const PString & ini) const;
-	
+
 	/// global translation of cause codes (called from endpoint specific method)
 	unsigned TranslateReceivedCause(unsigned cause) const;
 	unsigned TranslateSentCause(unsigned cause) const;
-	// check if cause contranslation feature is active (global or per endpoint)
+	// check if cause code translation feature is active (global or per endpoint)
 	bool IsCauseCodeTranslationActive() const { return m_causeCodeTranslationActive; }
 	void SetCauseCodeTranslationActive(bool val) { m_causeCodeTranslationActive = val; }
 
-	PStringList GetAuthenticatorList();
+	PStringList GetAuthenticatorList() const;
+	bool IsAuthenticatorEnabled(const PString & algo) const;
 
 protected:
 	void CreateConfig();
 	void ReloadSQLConfig();
 	void LoadCauseMap(PConfig *cfg);
 	void LoadReasonMap(PConfig *cfg);
-	
+
 	PFilePath m_ConfigFilePath;
 	PFilePath m_extConfigFilePath;
 	PString   m_GKName;
@@ -703,6 +749,7 @@ protected:
 	VendorModeTool m_venderMode;
 #if HAS_DATABASE
 	AlternateGatekeepers m_AlternateGKs;
+	GnuGkAssignedGatekeepers m_GnuGkAssignedGKs;
 	QoSMonitor m_qosMonitor;
 #endif
 	RouteTable m_RouteTable;
@@ -719,9 +766,11 @@ protected:
 	PMutex m_acctSessionMutex;
 
 private:
+    bool m_maintenanceMode;
+    PString m_maintenanceAlternate;
 	PFilePath m_tmpconfig;
 	/// global manager for time-based events
-	GkTimerManager* m_timerManager;
+	GkTimerManager * m_timerManager;
 	/// a default timestamp format string
 	PString m_timestampFormatStr;
 	/// a padding byte used during config passwords decryption
@@ -754,6 +803,7 @@ private:
 	std::map<int,std::vector<H323TransportAddress> > m_H46023STUN;
 	PMutex m_stunMutex;
 #endif
+    bool m_H46026Enabled;
 
 #ifdef HAS_H460P
 	GkPresence m_presence;
@@ -765,6 +815,14 @@ private:
 	SSL_CTX * m_sslCtx;
 	PString m_passphrase;
 #endif
+#ifdef P_SSL
+    bool m_OpenSSLInitialized;
+#endif
+	bool m_licenseValid;
+	PString m_licenseError;
+	PString m_licenseType;
+	bool m_snmpEnabled;
+	bool m_ipv6Enabled;
 };
 
 
