@@ -2,7 +2,7 @@
 //
 // gk.h gatekeeper process
 //
-// Copyright (c) 2000-2013, Jan Willamowius
+// Copyright (c) 2000-2023, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -14,7 +14,7 @@
 
 
 #ifndef GK_H
-#define GK_H "@(#) $Id: gk.h,v 1.46 2014/03/20 22:58:49 willamowius Exp $"
+#define GK_H "@(#) $Id$"
 #include "config.h"
 #include "version.h"
 #ifdef COMPILE_AS_SERVICE
@@ -23,7 +23,7 @@
 #include <ptlib/pprocess.h>
 #endif
 
-#ifdef _WIN32
+#if _WIN32 || _WIN64
 #include <mswsock.h>
 #endif
 
@@ -34,16 +34,21 @@
 #endif
 
 extern PSemaphore ShutdownMutex;
+extern bool ShutdownFlag;	// you may only set this flag if you own the ShutdownMutex, once it is set, it can never be cleared!
 extern const char * KnownConfigEntries[][2];
 
-// you must change PTLib 2.10.x configure.ac to set WINVER = 0x0600 to enable
+// you must change PTLib 2.10.x / 2.11.x configure.ac to set WINVER = 0x0600 to enable
 #define WINDOWS_VISTA	0x0600
-#if defined(_WIN32) && (_WIN32_WINNT >= WINDOWS_VISTA)
+#if (defined(_WIN32) || defined(_WIN64)) && (_WIN32_WINNT >= WINDOWS_VISTA)
 extern LPFN_WSASENDMSG g_pfWSASendMsg;
 #endif
 
 class GkTimer;
 
+extern bool IsGatekeeperShutdown();
+extern void ExitGK();
+
+
 #ifdef COMPILE_AS_SERVICE
 class Gatekeeper : public PServiceProcess
 {
@@ -55,7 +60,11 @@ class Gatekeeper : public PProcess
 #endif
  public:
 	Gatekeeper
+#ifdef COMPILE_AS_SERVICE
+		(const char * _manuf = "GnuGk.org",
+#else
 		(const char * _manuf = "GNU",
+#endif
 		 const char * _name = GNUGK_NAME,
 		 WORD _majorVersion = GNUGK_MAJOR_VERSION,
 		 WORD _minorVersion = GNUGK_MINOR_VERSION,
@@ -63,7 +72,7 @@ class Gatekeeper : public PProcess
 		 WORD _buildNumber = GNUGK_BUILD_NUMBER);
 
 	virtual void Main();
-	
+
 #ifdef COMPILE_AS_SERVICE
 	virtual PBoolean OnStart();
 	virtual void OnStop();
@@ -80,12 +89,12 @@ class Gatekeeper : public PProcess
 		Monthly,
 		RotationIntervalMax
 	};
-	
+
 	static bool SetLogFilename(const PString & filename);
 
-		
+
 	static bool RotateLogFile();
-	static bool ReopenLogFile();	
+	static bool ReopenLogFile();
 	static void CloseLogFile();
 
 	static void EnableLogFileRotation(bool enable = true);
@@ -112,9 +121,9 @@ class Gatekeeper : public PProcess
 
 	/** installs the signal handlers; First called init method. */
 	virtual bool InitHandlers(const PArgList & args);
-	
+
 	virtual bool InitConfig(const PArgList & args);
-	
+
 	/** initiates logging and tracing; Called after #InitConfig# */
 	virtual bool InitLogging(const PArgList & args);
 
