@@ -13,7 +13,7 @@
 
 
 #ifndef GK_CONST_H
-#define GK_CONST_H "@(#) $Id: gk_const.h,v 1.35 2014/01/02 23:35:16 willamowius Exp $"
+#define GK_CONST_H "@(#) $Id$"
 
 #include "config.h"
 
@@ -34,6 +34,7 @@
 
 #define GK_DEF_STATUS_PORT			7000
 
+#define GK_DEF_MULTIPLEX_H245_PORT  1722
 #define GK_DEF_MULTIPLEX_RTP_PORT	3000
 #define GK_DEF_MULTIPLEX_RTCP_PORT	3001
 
@@ -41,8 +42,22 @@
 
 #define GK_DEF_BANDWIDTH			1280
 
-extern const char *H225_ProtocolID;
-extern const char *H245_ProtocolID;
+const char * const H225_ProtocolIDv2 = "0.0.8.2250.0.2";
+
+const char * const H225_ProtocolIDv4 = "0.0.8.2250.0.4";
+
+const char * const H225_ProtocolIDv5 = "0.0.8.2250.0.5";
+
+const char * const H225_ProtocolIDv6 = "0.0.8.2250.0.6";
+
+const char * const H245_ProtocolIDv3 = "0.0.8.245.0.3";
+
+const int MAX_H323_VERSION = 7;  // for messages we generate
+const unsigned ProtocolID_BufferSize = 20;
+extern char H225_ProtocolID[];
+extern char H245_ProtocolID[];
+extern const char *H225_Protocol_Version[];
+extern const char *H245_Protocol_Version[];
 
 //+++++++++++++++++++++++++++++++++++++++++++++
 // GnuGk OID
