@@ -5,7 +5,7 @@
  * support for accounting to the gatekeeper.
  *
  * Copyright (c) 2003, Quarcom FHU, Michal Zygmuntowicz
- * Copyright (c) 2005-2013, Jan Willamowius
+ * Copyright (c) 2005-2023, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -14,6 +14,14 @@
  *
  */
 
+// avoid warning in PTLib object.h
+#if (!_WIN32) && (!_WIN64)
+#define GCC_VERSION (__GNUC__ * 10000 + __GNUC_MINOR__ * 100 + __GNUC_PATCHLEVEL__)
+#if (GCC_VERSION >= 40400)
+#pragma GCC diagnostic ignored "-Wstrict-overflow"
+#endif
+#endif
+
 #include <ptlib.h>
 #include <h225.h>
 #include "gk_const.h"
@@ -29,87 +37,84 @@ using std::vector;
 
 /// Name of the config file section for accounting configuration
 namespace {
-const char* GkAcctSectionName = "Gatekeeper::Acct";
+    const char* GkAcctSectionName = "Gatekeeper::Acct";
 }
 
 extern const char* CallTableSection;
 
 
-GkAcctLogger::GkAcctLogger(
-	const char* moduleName,
-	const char* cfgSecName
-	)
-	: NamedObject(moduleName), m_controlFlag(Required), m_defaultStatus(Fail),
+GkAcctLogger::GkAcctLogger(const char* moduleName, const char* cfgSecName)
+  : NamedObject(moduleName), m_controlFlag(Required), m_defaultStatus(Fail),
 	m_enabledEvents(AcctAll), m_supportedEvents(AcctNone), m_config(GkConfig()),
 	m_configSectionName(cfgSecName)
 {
 	if (m_configSectionName.IsEmpty())
 		m_configSectionName = moduleName;
-		
-	const PStringArray control(
-		m_config->GetString(GkAcctSectionName, moduleName, "").Tokenise(";,")
-		);
 
-	if (control.GetSize() < 1)
+	const PStringArray control(m_config->GetString(GkAcctSectionName, moduleName, "").Tokenise(";,"));
+
+	if (control.GetSize() < 1) {
 		PTRACE(1, "GKACCT\tEmpty config entry for module " << moduleName);
-	else if (strcasecmp(moduleName, "default") == 0) {
+	} else if (strcasecmp(moduleName, "default") == 0) {
 		m_controlFlag = Required;
 		m_defaultStatus = Toolkit::AsBool(control[0]) ? Ok : Fail;
 		m_supportedEvents = AcctAll;
-	} else if (control[0] *= "optional")
+	} else if (control[0] *= "optional") {
 		m_controlFlag = Optional;
-	else if (control[0] *= "sufficient")
+	} else if (control[0] *= "sufficient") {
 		m_controlFlag = Sufficient;
-	else if (control[0] *= "alternative")
+	} else if (control[0] *= "alternative") {
 		m_controlFlag = Alternative;
-	
+    }
+
 	if (control.GetSize() > 1)
 		m_enabledEvents = GetEvents(control);
-	
+
 	PTRACE(1, "GKACCT\tCreated module " << moduleName << " with event mask "
-		<< PString(PString::Unsigned, (long)m_enabledEvents, 16)
-		);
+		<< PString(PString::Unsigned, (long)m_enabledEvents, 16));
 }
 
 GkAcctLogger::~GkAcctLogger()
 {
-	PTRACE(1, "GKACCT\tDestroyed module "<<GetName());
+	PTRACE(1, "GKACCT\tDestroyed module " << GetName());
 }
 
-int GkAcctLogger::GetEvents(
-	const PStringArray& tokens
-	) const
+int GkAcctLogger::GetEvents(const PStringArray & tokens) const
 {
 	int mask = 0;
-	
-	for( PINDEX i = 1; i < tokens.GetSize(); i++ ) {
-		const PString& token = tokens[i];
-		if( token *= "start" )
+
+	for (PINDEX i = 1; i < tokens.GetSize(); i++) {
+		const PString & token = tokens[i];
+		if (token *= "start")
 			mask |= AcctStart;
-		else if( token *= "stop" )
+		else if (token *= "stop")
 			mask |= AcctStop;
-		else if( token *= "update" )
+		else if (token *= "update")
 			mask |= AcctUpdate;
-		else if( token *= "alert" )
+		else if (token *= "alert")
 			mask |= AcctAlert;
-		else if( token *= "connect" )
+		else if (token *= "connect")
 			mask |= AcctConnect;
-		else if( token *= "register" )
+		else if (token *= "register")
 			mask |= AcctRegister;
-		else if( token *= "unregister" )
+		else if (token *= "unregister")
 			mask |= AcctUnregister;
-		else if( token *= "on" )
+		else if (token *= "on")
 			mask |= AcctOn;
-		else if( token *= "off" )
+		else if (token *= "off")
 			mask |= AcctOff;
+		else if (token *= "reject")
+			mask |= AcctReject;
+		else if (token *= "mediafail")
+			mask |= AcctMediaFail;
 	}
-	
+
 	return mask;
 }
 
 GkAcctLogger::Status GkAcctLogger::Log(
 	AcctEvent evt, /// accounting event to log
-	const callptr& /*call*/ /// a call associated with the event (if any)
+	const callptr & /*call*/ /// a call associated with the event (if any)
 	)
 {
 	return (evt & m_enabledEvents & m_supportedEvents) ? m_defaultStatus : Next;
@@ -117,7 +122,7 @@ GkAcctLogger::Status GkAcctLogger::Log(
 
 GkAcctLogger::Status GkAcctLogger::Log(
 	AcctEvent evt, /// accounting event to log
-	const endptr& /*ep*/ /// endpoint associated with the event (if any)
+	const endptr & /*ep*/ /// endpoint associated with the event (if any)
 	)
 {
 	return (evt & m_enabledEvents & m_supportedEvents) ? m_defaultStatus : Next;
@@ -125,13 +130,14 @@ GkAcctLogger::Status GkAcctLogger::Log(
 
 void GkAcctLogger::SetupAcctParams(
 	/// CDR parameters (name => value) associations
-	std::map<PString, PString>& params,
+	std::map<PString, PString> & params,
 	/// call (if any) associated with an accounting event being logged
-	const callptr& call,
+	const callptr & call,
 	/// timestamp formatting string
-	const PString& timestampFormat
+	const PString & timestampFormat
 	) const
 {
+    OpalGloballyUniqueID eventID;
 	PIPSocket::Address addr;
 	WORD port = 0;
 	time_t t;
@@ -139,7 +145,17 @@ void GkAcctLogger::SetupAcctParams(
 	Toolkit* const toolkit = Toolkit::Instance();
 	toolkit->GetGKHome(interfaces);
 
+	params["event-uuid"] = eventID.AsString();
+	params["event-time"] = toolkit->AsString(PTime(), timestampFormat);
 	params["g"] = toolkit->GKName();
+	if (interfaces.empty())
+		params["gkip"] = "";
+	else
+		params["gkip"] = interfaces.front().AsString();
+
+	if (!call)
+        return;
+
 	params["n"] = PString(call->GetCallNumber());
 	params["u"] = GetUsername(call);
 	params["d"] = call->GetDuration();
@@ -149,62 +165,76 @@ void GkAcctLogger::SetupAcctParams(
 	params["p"] = call->GetPostDialDelay();
 	params["r"] = call->GetReleaseSource();
 	params["t"] = call->GetTotalCallDuration();
-	if (interfaces.empty())
-		params["gkip"] = "";
-	else
-		params["gkip"] = interfaces.front().AsString();
-	params["CallId"] = ::AsString(call->GetCallIdentifier().m_guid);
+	params["CallId"] = ::AsString(call->GetCallIdentifier());
 	params["ConfId"] = ::AsString(call->GetConferenceIdentifier());
 	params["CallLink"] = call->GetCallLinkage();
-	
+
 	t = call->GetSetupTime();
-	if (t)
-		params["setup-time"] = toolkit->AsString(PTime(t), timestampFormat);
+    params["setup-time"] = t ? toolkit->AsString(PTime(t), timestampFormat) : "";
 	t = call->GetAlertingTime();
-	if (t)
-		params["alerting-time"] = toolkit->AsString(PTime(t), timestampFormat);
+    params["alerting-time"] = t ? toolkit->AsString(PTime(t), timestampFormat) : "";
 	t = call->GetConnectTime();
-	if (t)
-		params["connect-time"] = toolkit->AsString(PTime(t), timestampFormat);
+    params["connect-time"] = t ? toolkit->AsString(PTime(t), timestampFormat) : "";
 	t = call->GetDisconnectTime();
-	if (t)
-		params["disconnect-time"] = toolkit->AsString(PTime(t), timestampFormat);
+    params["disconnect-time"] = t ? toolkit->AsString(PTime(t), timestampFormat) : "";
 	params["ring-time"] = call->GetRingTime();
 
 	if (call->GetSrcSignalAddr(addr, port)) {
 		params["caller-ip"] = addr.AsString();
 		params["caller-port"] = port;
+	} else {
+		params["caller-ip"] = "";
+		params["caller-port"] = 0;
 	}
-	
+
 	params["src-info"] = call->GetSrcInfo();
 	params["Calling-Station-Id"] = GetCallingStationId(call);
-		
+
 	addr = (DWORD)0;
 	port = 0;
-		
+
 	if (call->GetDestSignalAddr(addr, port)) {
 		params["callee-ip"] = addr.AsString();
 		params["callee-port"] = port;
+	} else {
+		params["callee-ip"] = "";
+		params["callee-port"] = 0;
 	}
 
 	params["dest-info"] = call->GetDestInfo();
 	params["Called-Station-Id"] = GetCalledStationId(call);
 	params["Dialed-Number"] = GetDialedNumber(call);
 
+	if (GetConfig()->GetBoolean(CallTableSection, "SetCalledStationIdToDialedIP", false)
+		&& IsIPAddress(params["Dialed-Number"])) {
+		params["Called-Station-Id"] = params["Dialed-Number"];
+	}
+
 	endptr caller;
 	if ((caller = call->GetCallingParty())) {
 		params["caller-epid"] = caller->GetEndpointIdentifier().GetValue();
+	} else {
+		params["caller-epid"] = "";
 	}
 	endptr callee;
 	if ((callee = call->GetCalledParty())) {
 		params["callee-epid"] = callee->GetEndpointIdentifier().GetValue();
+	} else {
+		params["callee-epid"] = "";
 	}
 	params["call-attempts"] = PString(call->GetNoCallAttempts());
 	params["last-cdr"] = call->GetNoRemainingRoutes() > 0 ? "0" : "1";
 
-	if ((call->GetMediaOriginatingIp(addr)))
-		params["media-oip"] = addr.AsString();
-	params["codec"] = call->GetCodec();
+	if ((call->GetCallerAudioIP(addr, port))) {
+		params["caller-media-ip"] = addr.AsString();
+    } else {
+		params["caller-media-ip"] = "";
+    }
+	if ((call->GetCalledAudioIP(addr, port))) {
+		params["callee-media-ip"] = addr.AsString();
+	} else {
+		params["callee-media-ip"] = "";
+	}
 	params["bandwidth"] = call->GetBandwidth();
 	params["client-auth-id"] = call->GetClientAuthId();
 	PString vendor, version;
@@ -212,23 +242,73 @@ void GkAcctLogger::SetupAcctParams(
 	params["caller-vendor"] = vendor + " " + version;
 	call->GetCalledVendor(vendor, version);
 	params["callee-vendor"] = vendor + " " + version;
+	params["sinfo-ip"] = call->GetSInfoIP();
+
+	params["bandwidth-kbps"] = PString(call->GetBandwidth() / 10); // in kbps
+	params["caller-audio-codec"] = call->GetCallerAudioCodec();
+	params["callee-audio-codec"] = call->GetCalledAudioCodec();
+	params["caller-video-codec"] = call->GetCallerVideoCodec();
+	params["callee-video-codec"] = call->GetCalledVideoCodec();
+	params["caller-audio-bitrate"] = PString(call->GetCallerAudioBitrate() / 10); // in kbps
+	params["callee-audio-bitrate"] = PString(call->GetCalledAudioBitrate() / 10); // in kbps
+	params["caller-video-bitrate"] = PString(call->GetCallerVideoBitrate() / 10); // in kbps
+	params["callee-video-bitrate"] = PString(call->GetCalledVideoBitrate() / 10); // in kbps
+
+	PString encryption = "On";
+	if (!params["caller-audio-codec"].IsEmpty() && params["caller-audio-codec"].Find("H.235") == P_MAX_INDEX)
+        encryption = "Off";
+	if (!params["callee-audio-codec"].IsEmpty() && params["callee-audio-codec"].Find("H.235") == P_MAX_INDEX)
+        encryption = "Off";
+	if (!params["caller-video-codec"].IsEmpty() && params["caller-video-codec"].Find("H.235") == P_MAX_INDEX)
+        encryption = "Off";
+	if (!params["callee-video-codec"].IsEmpty() && params["callee-video-codec"].Find("H.235") == P_MAX_INDEX)
+        encryption = "Off";
+	if (params["caller-audio-codec"].IsEmpty() && params["callee-audio-codec"].IsEmpty()
+        && params["caller-video-codec"].IsEmpty() && params["callee-video-codec"].IsEmpty())
+        encryption = "Off";
+	params["encryption"] = encryption;
+
+	params["codec"] = call->GetCallerAudioCodec();  // deprecated
+    params["media-oip"] = addr.AsString(); // deprecated
 }
 
 void GkAcctLogger::SetupAcctEndpointParams(
 	/// parameter (name => value) associations
-	std::map<PString, PString>& params,
+	std::map<PString, PString> & params,
 	/// endpoint associated with an accounting event being logged
-	const endptr& ep
+	const endptr & ep,
+	/// timestamp formatting string
+	const PString & timestampFormat
 	) const
 {
+    OpalGloballyUniqueID eventID;
 	PIPSocket::Address addr;
 	WORD port = 0;
+	vector<PIPSocket::Address> interfaces;
+	Toolkit::Instance()->GetGKHome(interfaces);
+
+	params["event-uuid"] = eventID.AsString();
+	params["event-time"] = Toolkit::Instance()->AsString(PTime(), timestampFormat);
+	params["g"] = Toolkit::GKName();
+	if (interfaces.empty())
+		params["gkip"] = "";
+	else
+		params["gkip"] = interfaces.front().AsString();
+
+	if (!ep)
+        return;
 
 	H225_TransportAddress sigip = ep->GetCallSignalAddress();
 	if (GetIPAndPortFromTransportAddr(sigip, addr, port)) {
 		params["endpoint-ip"] = addr.AsString();
 		params["endpoint-port"] = port;
+	} else {
+		params["endpoint-ip"] = "";
+		params["endpoint-port"] = 0;
 	}
+	PString vendor, version;
+	ep->GetEndpointInfo(vendor, version);
+	params["endpoint-vendor"] = vendor + " " + version;
 
 	PString aliasString = AsString(ep->GetAliases(), false);
 	aliasString.Replace(PString("="), PString(","), true);	// make list comma separated
@@ -236,23 +316,40 @@ void GkAcctLogger::SetupAcctEndpointParams(
 
 	// The username is always the last in the Alias List
 	PStringArray aliasList = aliasString.Tokenise(",");
-    if (aliasList.GetSize() > 0)
+    if (aliasList.GetSize() > 0) {
 	    params["u"] = aliasList[aliasList.GetSize()-1];
+    } else {
+	    params["u"] = "";
+    }
+
+    params["Calling-Station-Id"] = GetBestAliasAddressString(ep->GetAliases(), false,
+                                                             AliasAddressTagMask(H225_AliasAddress::e_dialedDigits) | AliasAddressTagMask(H225_AliasAddress::e_partyNumber));
 
 	params["epid"] = ep->GetEndpointIdentifier().GetValue();
-	params["g"] = Toolkit::GKName();
 }
 
-// avoid warning in PTLib object.h
-#if (!_WIN32) && (GCC_VERSION >= 40400)
-#pragma GCC diagnostic ignored "-Wstrict-overflow"
-#endif
+void GkAcctLogger::SetupAcctParams(
+ 	/// CDR parameters (name => value) associations
+    std::map<PString, PString> & params
+    ) const
+{
+	PIPSocket::Address addr;
+	vector<PIPSocket::Address> interfaces;
+	Toolkit* const toolkit = Toolkit::Instance();
+	toolkit->GetGKHome(interfaces);
+
+	params["g"] = toolkit->GKName();
+	if (interfaces.empty())
+		params["gkip"] = "";
+	else
+		params["gkip"] = interfaces.front().AsString();
+}
 
 PString GkAcctLogger::ReplaceAcctParams(
 	/// parametrized CDR string
-	const PString& cdrStr,
+	const PString & cdrStr,
 	/// parameter values
-	const std::map<PString, PString>& params
+	const std::map<PString, PString> & params
 	) const
 {
 	PString finalCDR((const char*)cdrStr);
@@ -273,19 +370,14 @@ PString GkAcctLogger::ReplaceAcctParams(
 			const PINDEX closingBrace = finalCDR.Find('}', ++pos);
 			if (closingBrace != P_MAX_INDEX) {
 				const PINDEX paramLen = closingBrace - pos;
-				std::map<PString, PString>::const_iterator i = params.find(
-					finalCDR.Mid(pos, paramLen)
-					);
+				std::map<PString, PString>::const_iterator i = params.find(finalCDR.Mid(pos, paramLen));
 				if (i != params.end()) {
 					const PINDEX escapedLen = EscapeAcctParam(i->second).GetLength();
 					finalCDR.Splice(EscapeAcctParam(i->second), pos - 2, paramLen + 3);
 					len = len + escapedLen - paramLen - 3;
 					pos = pos - 2 + escapedLen;
 				} else {
-					// replace out of range parameter with an empty string
-					finalCDR.Delete(pos - 2, paramLen + 3);
-					len -= paramLen + 3;
-					pos -= 2;
+                    // leave unknown placeholders intact so a 2nd stage can replace them
 				}
 			}
 		} else { // simple syntax (%c)
@@ -296,10 +388,7 @@ PString GkAcctLogger::ReplaceAcctParams(
 				len = len + escapedLen - 2;
 				pos = pos - 1 + escapedLen;
 			} else {
-				// replace out of range parameter with an empty string
-				finalCDR.Delete(pos - 1, 2);
-				len -= 2;
-				pos--;
+                // leave unknown placeholders intact so a 2nd stage can replace them
 			}
 		}
 	}
@@ -307,28 +396,28 @@ PString GkAcctLogger::ReplaceAcctParams(
 	return finalCDR;
 }
 
-PString GkAcctLogger::EscapeAcctParam(const PString& param) const
+PString GkAcctLogger::EscapeAcctParam(const PString & param) const
 {
 	return param;	// default implementation: don't escape anything
 }
 
 PString GkAcctLogger::GetUsername(
 	/// call (if any) associated with the RAS message
-	const callptr& call
+	const callptr & call
 	) const
 {
 	if (!call)
 		return PString::Empty();
-		
+
 	const endptr callingEP = call->GetCallingParty();
 	PString username;
-		
+
 	username = GetBestAliasAddressString(call->GetSourceAddress(), true,
 		AliasAddressTagMask(H225_AliasAddress::e_h323_ID),
 		AliasAddressTagMask(H225_AliasAddress::e_email_ID)
 			| AliasAddressTagMask(H225_AliasAddress::e_url_ID)
 		);
-			
+
 	if (callingEP && (username.IsEmpty()
 			|| FindAlias(callingEP->GetAliases(), username) == P_MAX_INDEX))
 		username = GetBestAliasAddressString(callingEP->GetAliases(), false,
@@ -336,7 +425,7 @@ PString GkAcctLogger::GetUsername(
 			AliasAddressTagMask(H225_AliasAddress::e_email_ID)
 				| AliasAddressTagMask(H225_AliasAddress::e_url_ID)
 			);
-		
+
 	if (username.IsEmpty())
 		username = GetBestAliasAddressString(call->GetSourceAddress(), false,
 			AliasAddressTagMask(H225_AliasAddress::e_h323_ID),
@@ -354,13 +443,13 @@ PString GkAcctLogger::GetUsername(
 			&& callingSigAddr.IsValid())
 			username = callingSigAddr.AsString();
 	}
-	
+
 	return username;
 }
 
 PString GkAcctLogger::GetCallingStationId(
 	/// call associated with the accounting event
-	const callptr& call
+	const callptr & call
 	) const
 {
 	if (!call)
@@ -375,7 +464,7 @@ PString GkAcctLogger::GetCallingStationId(
 			AliasAddressTagMask(H225_AliasAddress::e_dialedDigits)
 				| AliasAddressTagMask(H225_AliasAddress::e_partyNumber)
 			);
-				
+
 	if (id.IsEmpty()) {
 		const endptr callingEP = call->GetCallingParty();
 		if (callingEP)
@@ -384,7 +473,7 @@ PString GkAcctLogger::GetCallingStationId(
 					| AliasAddressTagMask(H225_AliasAddress::e_partyNumber)
 				);
 	}
-					
+
 	if (id.IsEmpty()) {
 		PIPSocket::Address callingSigAddr;
 		WORD callingSigPort = 0;
@@ -392,13 +481,13 @@ PString GkAcctLogger::GetCallingStationId(
 			&& callingSigAddr.IsValid())
 			id = ::AsString(callingSigAddr, callingSigPort);
 	}
-	
+
 	return id;
 }
 
 PString GkAcctLogger::GetCalledStationId(
 	/// call associated with the accounting event
-	const callptr& call
+	const callptr & call
 	) const
 {
 	if (!call)
@@ -407,13 +496,13 @@ PString GkAcctLogger::GetCalledStationId(
 	PString id = call->GetCalledStationId();
 	if (!id)
 		return id;
-			
+
 	if (id.IsEmpty())
 		id = GetBestAliasAddressString(call->GetDestinationAddress(), false,
 			AliasAddressTagMask(H225_AliasAddress::e_dialedDigits)
 				| AliasAddressTagMask(H225_AliasAddress::e_partyNumber)
 			);
-		
+
 	if (id.IsEmpty()) {
 		const endptr calledEP = call->GetCalledParty();
 		if (calledEP)
@@ -422,7 +511,7 @@ PString GkAcctLogger::GetCalledStationId(
 					| AliasAddressTagMask(H225_AliasAddress::e_partyNumber)
 				);
 	}
-	
+
 	if (id.IsEmpty()) {
 		PIPSocket::Address calledSigAddr;
 		WORD calledSigPort = 0;
@@ -430,13 +519,13 @@ PString GkAcctLogger::GetCalledStationId(
 		 	&& calledSigAddr.IsValid())
 			id = ::AsString(calledSigAddr, calledSigPort);
 	}
-	
+
 	return id;
 }
 
 PString GkAcctLogger::GetDialedNumber(
 	/// call associated with the accounting event
-	const callptr& call
+	const callptr & call
 	) const
 {
 	if (!call)
@@ -445,13 +534,13 @@ PString GkAcctLogger::GetDialedNumber(
 	PString id = call->GetDialedNumber();
 	if (!id)
 		return id;
-			
+
 	if (id.IsEmpty())
 		id = GetBestAliasAddressString(call->GetDestinationAddress(), false,
 			AliasAddressTagMask(H225_AliasAddress::e_dialedDigits)
 				| AliasAddressTagMask(H225_AliasAddress::e_partyNumber)
 			);
-		
+
 	if (id.IsEmpty()) {
 		const endptr calledEP = call->GetCalledParty();
 		if (calledEP)
@@ -460,7 +549,7 @@ PString GkAcctLogger::GetDialedNumber(
 					| AliasAddressTagMask(H225_AliasAddress::e_partyNumber)
 				);
 	}
-	
+
 	if (id.IsEmpty()) {
 		PIPSocket::Address calledSigAddr;
 		WORD calledSigPort = 0;
@@ -468,7 +557,7 @@ PString GkAcctLogger::GetDialedNumber(
 		 	&& calledSigAddr.IsValid())
 			id = ::AsString(calledSigAddr, calledSigPort);
 	}
-	
+
 	return id;
 }
 
@@ -482,32 +571,21 @@ const char* const FileAcct::m_intervalNames[] =
 	"Hourly", "Daily", "Weekly", "Monthly"
 };
 
-FileAcct::FileAcct(
-	const char* moduleName,
-	const char* cfgSecName
-	)
-	:
-	GkAcctLogger(moduleName, cfgSecName),
+FileAcct::FileAcct(const char* moduleName, const char* cfgSecName)
+  : GkAcctLogger(moduleName, cfgSecName),
 	m_cdrFile(NULL), m_rotateLines(-1), m_rotateSize(-1), m_rotateInterval(-1),
 	m_rotateMinute(-1), m_rotateHour(-1), m_rotateDay(-1),
 	m_rotateTimer(GkTimerManager::INVALID_HANDLE), m_cdrLines(0),
 	m_standardCDRFormat(true)
 {
 	SetSupportedEvents(FileAcctEvents);
-	
+
 	m_cdrString = GetConfig()->GetString(GetConfigSectionName(), "CDRString", "");
-	m_standardCDRFormat = Toolkit::AsBool(
-		GetConfig()->GetString(GetConfigSectionName(), "StandardCDRFormat",
-			m_cdrString.IsEmpty() ? "1" : "0"
-			));
-	m_timestampFormat = GetConfig()->GetString(GetConfigSectionName(),
-		"TimestampFormat", ""
-		);
-		
-	// determine rotation type (by lines, by size, by time)	
-	const PString rotateCondition = GetConfig()->GetString(
-		GetConfigSectionName(), "Rotate", ""
-		).Trim();
+	m_standardCDRFormat = GetConfig()->GetBoolean(GetConfigSectionName(), "StandardCDRFormat", m_cdrString.IsEmpty() ? true : false);
+	m_timestampFormat = GetConfig()->GetString(GetConfigSectionName(), "TimestampFormat", "");
+
+	// determine rotation type (by lines, by size, by time)
+	const PString rotateCondition = GetConfig()->GetString(GetConfigSectionName(), "Rotate", "").Trim();
 	if (!rotateCondition) {
 		const char suffix = rotateCondition[rotateCondition.GetLength()-1];
 		if (rotateCondition[0] == 'L' || rotateCondition[0] == 'l') {
@@ -530,9 +608,7 @@ FileAcct::FileAcct(
 					m_rotateInterval = i;
 
 			if (m_rotateInterval < 0 || m_rotateInterval >= RotationIntervalMax)
-				PTRACE(1, "GKACCT\t" << GetName() << " unsupported rotation "
-					"method: " << rotateCondition << " - rotation disabled"
-					);
+				PTRACE(1, "GKACCT\t" << GetName() << " unsupported rotation method: " << rotateCondition << " - rotation disabled");
 			else {
 				// time based rotation
 				GetRotateInterval(*GetConfig(), GetConfigSectionName());
@@ -543,9 +619,7 @@ FileAcct::FileAcct(
 	m_cdrFilename = GetConfig()->GetString(GetConfigSectionName(), "DetailFile", "");
 	m_cdrFile = OpenCDRFile(m_cdrFilename);
 	if (m_cdrFile && m_cdrFile->IsOpen()) {
-		PTRACE(2, "GKACCT\t" << GetName() << " CDR file: "
-			<< m_cdrFile->GetFilePath()
-			);
+		PTRACE(2, "GKACCT\t" << GetName() << " CDR file: " << m_cdrFile->GetFilePath());
 		// count an initial number of CDR lines
 		if (m_rotateLines > 0) {
 			PString s;
@@ -571,72 +645,50 @@ FileAcct::FileAcct(
 			this, &FileAcct::RotateOnTimer, rotateTime, 60*60
 			);
 		PTRACE(5, "GKACCT\t" << GetName() << " hourly rotation enabled (first "
-			"rotation scheduled at " << rotateTime
-			);
+			"rotation scheduled at " << rotateTime);
 		break;
-		
+
 	case Daily:
 		rotateTime = PTime(0, m_rotateMinute, m_rotateHour, now.GetDay(),
-			now.GetMonth(), now.GetYear(), now.GetTimeZone()
-			);
+			now.GetMonth(), now.GetYear(), now.GetTimeZone());
 		if (rotateTime <= now)
 			rotateTime += PTimeInterval(0, 0, 0, 0, 1); // 1 day
 		m_rotateTimer = Toolkit::Instance()->GetTimerManager()->RegisterTimer(
-			this, &FileAcct::RotateOnTimer, rotateTime, 60*60*24
-			);
-		PTRACE(5, "GKACCT\t" << GetName() << " daily rotation enabled (first "
-			"rotation scheduled at " << rotateTime
-			);
+			this, &FileAcct::RotateOnTimer, rotateTime, 60*60*24);
+		PTRACE(5, "GKACCT\t" << GetName() << " daily rotation enabled (first rotation scheduled at " << rotateTime);
 		break;
-		
+
 	case Weekly:
-		rotateTime = PTime(0, m_rotateMinute, m_rotateHour, now.GetDay(),
-			now.GetMonth(), now.GetYear(), now.GetTimeZone()
-			);
+		rotateTime = PTime(0, m_rotateMinute, m_rotateHour, now.GetDay(), now.GetMonth(), now.GetYear(), now.GetTimeZone());
 		if (rotateTime.GetDayOfWeek() < m_rotateDay)
-			rotateTime += PTimeInterval(0, 0, 0, 0,
-				m_rotateDay - rotateTime.GetDayOfWeek()
-				);
+			rotateTime += PTimeInterval(0, 0, 0, 0, m_rotateDay - rotateTime.GetDayOfWeek());
 		else if (rotateTime.GetDayOfWeek() > m_rotateDay)
-			rotateTime -= PTimeInterval(0, 0, 0, 0,
-				rotateTime.GetDayOfWeek() - m_rotateDay
-				);
+			rotateTime -= PTimeInterval(0, 0, 0, 0, rotateTime.GetDayOfWeek() - m_rotateDay);
 		if (rotateTime <= now)
 			rotateTime += PTimeInterval(0, 0, 0, 0, 7); // 1 week
-		m_rotateTimer = Toolkit::Instance()->GetTimerManager()->RegisterTimer(
-			this, &FileAcct::RotateOnTimer, rotateTime, 60*60*24*7
-			);
-		PTRACE(5, "GKACCT\t" << GetName() << " weekly rotation enabled (first "
-			"rotation scheduled at " << rotateTime
-			);
+		m_rotateTimer = Toolkit::Instance()->GetTimerManager()->RegisterTimer(this, &FileAcct::RotateOnTimer, rotateTime, 60*60*24*7);
+		PTRACE(5, "GKACCT\t" << GetName() << " weekly rotation enabled (first rotation scheduled at " << rotateTime);
 		break;
-		
+
 	case Monthly:
-		rotateTime = PTime(0, m_rotateMinute, m_rotateHour, 1,
-			now.GetMonth(), now.GetYear(), now.GetTimeZone()
-			);
+		rotateTime = PTime(0, m_rotateMinute, m_rotateHour, 1, now.GetMonth(), now.GetYear(), now.GetTimeZone());
 		rotateTime += PTimeInterval(0, 0, 0, 0, m_rotateDay - 1);
-		while (rotateTime.GetMonth() != now.GetMonth())				
+		while (rotateTime.GetMonth() != now.GetMonth())
 			rotateTime -= PTimeInterval(0, 0, 0, 0, 1); // 1 day
 
 		if (rotateTime <= now) {
 			rotateTime = PTime(0, m_rotateMinute, m_rotateHour, 1,
 				now.GetMonth() + (now.GetMonth() == 12 ? -11 : 1),
 				now.GetYear() + (now.GetMonth() == 12 ? 1 : 0),
-				now.GetTimeZone()
-				);
+				now.GetTimeZone());
 			const int month = rotateTime.GetMonth();
 			rotateTime += PTimeInterval(0, 0, 0, 0, m_rotateDay - 1);
-			while (rotateTime.GetMonth() != month)				
+			while (rotateTime.GetMonth() != month)
 				rotateTime -= PTimeInterval(0, 0, 0, 0, 1); // 1 day
 		}
 
-		m_rotateTimer = Toolkit::Instance()->GetTimerManager()->RegisterTimer(
-			this, &FileAcct::RotateOnTimer, rotateTime
-			);
-		PTRACE(5, "GKACCT\t" << GetName() << " monthly rotation enabled (first "
-			"rotation scheduled at " << rotateTime
-			);
+		m_rotateTimer = Toolkit::Instance()->GetTimerManager()->RegisterTimer( this, &FileAcct::RotateOnTimer, rotateTime);
+		PTRACE(5, "GKACCT\t" << GetName() << " monthly rotation enabled (first rotation scheduled at " << rotateTime);
 		break;
 	}
 }
@@ -645,7 +697,7 @@ FileAcct::~FileAcct()
 {
 	if (m_rotateTimer != GkTimerManager::INVALID_HANDLE)
 		Toolkit::Instance()->GetTimerManager()->UnregisterTimer(m_rotateTimer);
-		
+
 	PWaitAndSignal lock(m_cdrFileMutex);
 	if (m_cdrFile) {
 		m_cdrFile->Close();
@@ -653,13 +705,10 @@ FileAcct::~FileAcct()
 	}
 }
 
-void FileAcct::GetRotateInterval(
-	PConfig& cfg,
-	const PString& section
-	)
+void FileAcct::GetRotateInterval(PConfig & cfg, const PString & section)
 {
 	PString s;
-	
+
 	if (m_rotateInterval == Hourly)
 		m_rotateMinute = cfg.GetInteger(section, "RotateTime", 59);
 	else {
@@ -668,17 +717,14 @@ void FileAcct::GetRotateInterval(
 		m_rotateMinute = 0;
 		if (s.Find(':') != P_MAX_INDEX)
 			m_rotateMinute = s.Mid(s.Find(':') + 1).AsInteger();
-			
-		if (m_rotateHour < 0 || m_rotateHour > 23 || m_rotateMinute < 0
-			|| m_rotateMinute > 59) {
-			PTRACE(1, "GKACCT\t" << GetName() << " invalid "
-				"RotateTime specified: " << s
-				);
+
+		if (m_rotateHour < 0 || m_rotateHour > 23 || m_rotateMinute < 0 || m_rotateMinute > 59) {
+			PTRACE(1, "GKACCT\t" << GetName() << " invalid RotateTime specified: " << s);
 			m_rotateMinute = 59;
 			m_rotateHour = 0;
 		}
 	}
-			
+
 	if (m_rotateInterval == Weekly)	{
 		s = cfg.GetString(section, "RotateDay", "Sun");
 		if (strspn(s, "0123456") == (size_t)s.GetLength()) {
@@ -696,52 +742,42 @@ void FileAcct::GetRotateInterval(
 			m_rotateDay = (i != dayNames.end()) ? i->second : -1;
 		}
 		if (m_rotateDay < 0 || m_rotateDay > 6) {
-			PTRACE(1, "GKACCT\t" << GetName() << " invalid "
-				"RotateDay specified: " << s
-				);
+			PTRACE(1, "GKACCT\t" << GetName() << " invalid RotateDay specified: " << s);
 			m_rotateDay = 0;
 		}
 	} else if (m_rotateInterval == Monthly) {
 		m_rotateDay = cfg.GetInteger(section, "RotateDay", 1);
 		if (m_rotateDay < 1 || m_rotateDay > 31) {
-			PTRACE(1, "GKACCT\t" << GetName() << " invalid "
-				"RotateDay specified: " << cfg.GetString(section, "RotateDay", "")
-				);
+			PTRACE(1, "GKACCT\t" << GetName() << " invalid RotateDay specified: " << cfg.GetString(section, "RotateDay", ""));
 			m_rotateDay = 1;
 		}
 	}
 }
 
-GkAcctLogger::Status FileAcct::Log(
-	GkAcctLogger::AcctEvent evt,
-	const callptr& call
-	)
+GkAcctLogger::Status FileAcct::Log(GkAcctLogger::AcctEvent evt, const callptr & call)
 {
 	if ((evt & GetEnabledEvents() & GetSupportedEvents()) == 0)
 		return Next;
-		
+
 	if (!call) {
 		PTRACE(1, "GKACCT\t" << GetName() << " - missing call info for event " << evt);
 		return Fail;
 	}
-	
+
 	PString cdrString;
-	
+
 	if (!GetCDRText(cdrString, evt, call)) {
 		PTRACE(2, "GKACCT\t" << GetName() << " - unable to get CDR text for "
-			"event " << evt << ", call no. " << call->GetCallNumber()
-			);
+			"event " << evt << ", call no. " << call->GetCallNumber());
 		return Fail;
 	}
-	
+
 	PWaitAndSignal lock(m_cdrFileMutex);
-	
+
 	if (m_cdrFile && m_cdrFile->IsOpen()) {
 		if (m_cdrFile->WriteLine(PString(cdrString))) {
 			PTRACE(5, "GKACCT\t" << GetName() << " - CDR string for event "
-				<< evt << ", call no. " << call->GetCallNumber()
-				<< ": " << cdrString
-				);
+				<< evt << ", call no. " << call->GetCallNumber() << ": " << cdrString);
 			m_cdrLines++;
 			if (IsRotationNeeded())
 				Rotate();
@@ -759,24 +795,20 @@ GkAcctLogger::Status FileAcct::Log(
 	return Fail;
 }
 
-bool FileAcct::GetCDRText(
-	PString& cdrString,
-	AcctEvent evt,
-	const callptr& call
-	)
+bool FileAcct::GetCDRText(PString & cdrString, AcctEvent evt, const callptr & call)
 {
 	if ((evt & AcctStop) != AcctStop || !call)
 		return false;
-	
-	if (m_standardCDRFormat)	
+
+	if (m_standardCDRFormat)
 		cdrString = call->GenerateCDR(m_timestampFormat);
 	else {
 		std::map<PString, PString> params;
 
 		SetupAcctParams(params, call, m_timestampFormat);
 		cdrString = ReplaceAcctParams(m_cdrString, params);
-	}	
-	
+	}
+
 	return !cdrString;
 }
 
@@ -789,9 +821,7 @@ bool FileAcct::IsRotationNeeded()
 	return false;
 }
 
-void FileAcct::RotateOnTimer(
-	GkTimer* timer
-	)
+void FileAcct::RotateOnTimer(GkTimer * timer)
 {
 	if (m_rotateInterval == Monthly) {
 		// setup next time for one-shot timer
@@ -800,16 +830,15 @@ void FileAcct::RotateOnTimer(
 			rotateTime.GetHour(), 1,
 			rotateTime.GetMonth() < 12 ? rotateTime.GetMonth() + 1 : 1,
 			rotateTime.GetMonth() < 12 ? rotateTime.GetYear() : rotateTime.GetYear() + 1,
-			rotateTime.GetTimeZone()
-			);
-	
+			rotateTime.GetTimeZone());
+
 		const int month = newRotateTime.GetMonth();
 		newRotateTime += PTimeInterval(0, 0, 0, 0, m_rotateDay - 1);
 		while (newRotateTime.GetMonth() != month)
 			newRotateTime -= PTimeInterval(0, 0, 0, 0, 1);
 		timer->SetExpirationTime(newRotateTime);
 		timer->SetFired(false);
-	}	
+	}
 	PWaitAndSignal lock(m_cdrFileMutex);
 	Rotate();
 }
@@ -822,14 +851,13 @@ void FileAcct::Rotate()
 		delete m_cdrFile;
 		m_cdrFile = NULL;
 	}
-	
+
 	const PFilePath fn = m_cdrFilename;
-	
+
 	if (PFile::Exists(fn)) {
 		if (!PFile::Rename(fn, fn.GetFileName() + PTime().AsString(".yyyyMMdd-hhmmss"))) {
-			PTRACE(1, "GKACCT\t" << GetName() << " rotate failed - could not "
-				"rename the log file");
-			SNMP_TRAP(6, SNMPError, Accounting, GetName() + " failed");
+			PTRACE(1, "GKACCT\t" << GetName() << " rotate failed - could not rename the log file");
+			SNMP_TRAP(6, SNMPError, Accounting, GetName() + " rotate failed");
 		}
 	}
 
@@ -837,18 +865,14 @@ void FileAcct::Rotate()
 	m_cdrLines = 0;
 }
 
-PTextFile* FileAcct::OpenCDRFile(
-	const PFilePath& fn
-	)
+PTextFile* FileAcct::OpenCDRFile(const PFilePath & fn)
 {
-	PTextFile* cdrFile = new PTextFile(fn, PFile::ReadWrite, 
+	PTextFile* cdrFile = new PTextFile(fn, PFile::ReadWrite,
 		PFile::Create | PFile::DenySharedWrite
 		);
 	if (!cdrFile->IsOpen()) {
-		PTRACE(1, "GKACCT\t" << GetName() << " could not open file"
-			" required for plain text accounting \""
-			<< fn << "\" :" << cdrFile->GetErrorText()
-			);
+		PTRACE(1, "GKACCT\t" << GetName() << " could not open file required for plain text accounting \""
+			<< fn << "\" :" << cdrFile->GetErrorText());
 		delete cdrFile;
 		return NULL;
 	}
@@ -859,9 +883,7 @@ PTextFile* FileAcct::OpenCDRFile(
 
 
 GkAcctLoggerList::GkAcctLoggerList()
-	: m_acctUpdateInterval(
-		GkConfig()->GetInteger(CallTableSection, "AcctUpdateInterval", 0)
-		)
+	: m_acctUpdateInterval(GkConfig()->GetInteger(CallTableSection, "AcctUpdateInterval", 0))
 {
 	// should not be less than 10 seconds
 	if (m_acctUpdateInterval)
@@ -876,16 +898,14 @@ GkAcctLoggerList::~GkAcctLoggerList()
 
 void GkAcctLoggerList::OnReload()
 {
-	m_acctUpdateInterval = GkConfig()->GetInteger(CallTableSection, 
-		"AcctUpdateInterval", 0
-		);
+	m_acctUpdateInterval = GkConfig()->GetInteger(CallTableSection, "AcctUpdateInterval", 0);
 	// should not be less than 10 seconds
 	if (m_acctUpdateInterval)
 		m_acctUpdateInterval = PMAX((long)10, m_acctUpdateInterval);
 
 	DeleteObjectsInContainer(m_loggers);
 	m_loggers.clear();
-	
+
 	const PStringArray modules = GkConfig()->GetKeys(GkAcctSectionName);
 	for (PINDEX i = 0; i < modules.GetSize(); i++) {
 		GkAcctLogger* logger = Factory<GkAcctLogger>::Create(modules[i]);
@@ -894,32 +914,32 @@ void GkAcctLoggerList::OnReload()
 	}
 }
 
-bool GkAcctLoggerList::LogAcctEvent( 
+bool GkAcctLoggerList::LogAcctEvent(
 	GkAcctLogger::AcctEvent evt, /// the accounting event to be logged
-	const callptr& call, /// a call associated with the event (if any)
+	const callptr & call, /// a call associated with the event (if any)
 	time_t now /// "now" timestamp for accounting update events
 	)
 {
 	// if this is an accounting update, check the interval
 	if (evt & GkAcctLogger::AcctUpdate) {
-		if ((!call) || m_acctUpdateInterval == 0 
+		if ((!call) || m_acctUpdateInterval == 0
 			|| (now - call->GetLastAcctUpdateTime()) < m_acctUpdateInterval) {
 			return true;
 		} else {
 			call->SetLastAcctUpdateTime(now);
 		}
 	}
-			
+
 	bool finalResult = true;
 	GkAcctLogger::Status status = GkAcctLogger::Ok;
 	std::list<GkAcctLogger*>::const_iterator iter = m_loggers.begin();
-	
+
 	while (iter != m_loggers.end()) {
 		GkAcctLogger* logger = *iter++;
-	
+
 		if ((evt & logger->GetEnabledEvents() & logger->GetSupportedEvents()) == 0)
 			continue;
-		
+
 		status = logger->Log(evt, call);
 		switch (status)
 		{
@@ -932,7 +952,7 @@ bool GkAcctLoggerList::LogAcctEvent(
 				PTrace::End(strm);
 			}
 			break;
-			
+
 		default:
 			if (PTrace::CanTrace(3)) {
 				ostream& strm = PTrace::Begin(3, __FILE__, __LINE__);
@@ -942,15 +962,15 @@ bool GkAcctLoggerList::LogAcctEvent(
 					strm << " for call no. " << call->GetCallNumber();
 				PTrace::End(strm);
 			}
-			// required and sufficient rules always determine 
+			// required and sufficient rules always determine
 			// status of the request
 			if (logger->GetControlFlag() == GkAcctLogger::Required
 				|| logger->GetControlFlag() == GkAcctLogger::Sufficient)
 				finalResult = false;
 		}
-		
+
 		// sufficient and alternative are terminal rules (on log success)
-		if (status == GkAcctLogger::Ok 
+		if (status == GkAcctLogger::Ok
 			&& (logger->GetControlFlag() == GkAcctLogger::Sufficient
 			|| logger->GetControlFlag() == GkAcctLogger::Alternative))
 			break;
@@ -959,11 +979,10 @@ bool GkAcctLoggerList::LogAcctEvent(
 	// a last rule determine status of the the request
 	if (finalResult && status != GkAcctLogger::Ok)
 		finalResult = false;
-		
+
 	if (PTrace::CanTrace(2)) {
-		ostream& strm = PTrace::Begin(2, __FILE__, __LINE__);
-		strm << "GKACCT\t" << (finalResult ? "Successfully logged event " 
-			: "Failed to log event ") << evt;
+		ostream & strm = PTrace::Begin(2, __FILE__, __LINE__);
+		strm << "GKACCT\t" << (finalResult ? "Successfully logged event " : "Failed to log event ") << evt;
 		if (call)
 			strm << " for call no. " << call->GetCallNumber();
 		PTrace::End(strm);
@@ -975,21 +994,21 @@ bool GkAcctLoggerList::LogAcctEvent(
 	return finalResult;
 }
 
-bool GkAcctLoggerList::LogAcctEvent( 
+bool GkAcctLoggerList::LogAcctEvent(
 	GkAcctLogger::AcctEvent evt, /// the accounting event to be logged
-	const endptr& ep /// endpoint associated with the event
+	const endptr & ep /// endpoint associated with the event
 	)
 {
 	bool finalResult = true;
 	GkAcctLogger::Status status = GkAcctLogger::Ok;
 	std::list<GkAcctLogger*>::const_iterator iter = m_loggers.begin();
-	
+
 	while (iter != m_loggers.end()) {
 		GkAcctLogger* logger = *iter++;
-	
+
 		if ((evt & logger->GetEnabledEvents() & logger->GetSupportedEvents()) == 0)
 			continue;
-		
+
 		status = logger->Log(evt, ep);
 		switch (status)
 		{
@@ -1002,7 +1021,7 @@ bool GkAcctLoggerList::LogAcctEvent(
 				PTrace::End(strm);
 			}
 			break;
-			
+
 		default:
 			if (PTrace::CanTrace(3)) {
 				ostream& strm = PTrace::Begin(3, __FILE__, __LINE__);
@@ -1012,15 +1031,15 @@ bool GkAcctLoggerList::LogAcctEvent(
 				PTrace::End(strm);
 				SNMP_TRAP(7, SNMPError, Accounting, logger->GetName() + " failed to log event " + PString(evt));
 			}
-			// required and sufficient rules always determine 
+			// required and sufficient rules always determine
 			// status of the request
 			if (logger->GetControlFlag() == GkAcctLogger::Required
 				|| logger->GetControlFlag() == GkAcctLogger::Sufficient)
 				finalResult = false;
 		}
-		
+
 		// sufficient and alternative are terminal rules (on log success)
-		if (status == GkAcctLogger::Ok 
+		if (status == GkAcctLogger::Ok
 			&& (logger->GetControlFlag() == GkAcctLogger::Sufficient
 			|| logger->GetControlFlag() == GkAcctLogger::Alternative))
 			break;
@@ -1029,11 +1048,10 @@ bool GkAcctLoggerList::LogAcctEvent(
 	// a last rule determine status of the the request
 	if (finalResult && status != GkAcctLogger::Ok)
 		finalResult = false;
-		
+
 	if (PTrace::CanTrace(2)) {
 		ostream& strm = PTrace::Begin(2, __FILE__, __LINE__);
-		strm << "GKACCT\t" << (finalResult ? "Successfully logged event " 
-			: "Failed to log event ") << evt;
+		strm << "GKACCT\t" << (finalResult ? "Successfully logged event " : "Failed to log event ") << evt;
 		if (ep)
 			strm << " for endpoint " << ep->GetEndpointIdentifier().GetValue();
 		PTrace::End(strm);
