@@ -4,7 +4,7 @@
  * Generic interface to access SQL databases
  *
  * Copyright (c) 2004, Michal Zygmuntowicz
- * Copyright (c) 2006-2012, Jan Willamowius
+ * Copyright (c) 2006-2019, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -23,13 +23,15 @@ using std::max;
 using std::min;
 
 namespace {
-const int GKSQL_DEFAULT_MIN_POOL_SIZE = 1;
-const int GKSQL_DEFAULT_MAX_POOL_SIZE = 1;
-const long GKSQL_CLEANUP_TIMEOUT = 5000;
+    const int GKSQL_DEFAULT_CONNECT_TIMEOUT = 10; // in seconds
+    const int GKSQL_DEFAULT_READ_TIMEOUT = 60; // in seconds
+    const long GKSQL_CLEANUP_TIMEOUT = 5000; // in milliseconds !
+    const int GKSQL_DEFAULT_MIN_POOL_SIZE = 1;
+    const int GKSQL_DEFAULT_MAX_POOL_SIZE = 1;
 }
 
 
-GkSQLResult::~GkSQLResult() 
+GkSQLResult::~GkSQLResult()
 {
 }
 
@@ -37,7 +39,9 @@ GkSQLConnection::GkSQLConnection(
 	/// name to use in the log
 	const char * name)
 	: NamedObject(name), m_port(0),
-	m_minPoolSize(GKSQL_DEFAULT_MIN_POOL_SIZE), 
+	m_connectTimeout(GKSQL_DEFAULT_CONNECT_TIMEOUT),
+	m_readTimeout(GKSQL_DEFAULT_READ_TIMEOUT),
+	m_minPoolSize(GKSQL_DEFAULT_MIN_POOL_SIZE),
 	m_maxPoolSize(GKSQL_DEFAULT_MAX_POOL_SIZE),
 	m_destroying(false), m_connected(false)
 {
@@ -55,7 +59,7 @@ bool GkSQLConnection::Initialize(
 	const char * cfgSectionName)
 {
 	PWaitAndSignal lock(m_connectionsMutex);
-	
+
 	if (!(cfg && cfgSectionName)) {
 		PTRACE(1, GetName() << "\tInitialize failed: NULL config or config section not specified!");
 		SNMP_TRAP(4, SNMPError, Database, GetName() + " creation failed");
@@ -67,40 +71,42 @@ bool GkSQLConnection::Initialize(
 	m_database = cfg->GetString(cfgSectionName, "Database", "");
 	m_username = cfg->GetString(cfgSectionName, "Username", "");
 	m_password = Toolkit::Instance()->ReadPassword(cfgSectionName, "Password");
+	m_connectTimeout = cfg->GetInteger(cfgSectionName, "ConnectTimeout", GKSQL_DEFAULT_CONNECT_TIMEOUT);
+	m_readTimeout = cfg->GetInteger(cfgSectionName, "ReadTimeout", GKSQL_DEFAULT_READ_TIMEOUT);
 	m_minPoolSize = cfg->GetInteger(cfgSectionName, "MinPoolSize", GKSQL_DEFAULT_MIN_POOL_SIZE);
 	m_minPoolSize = max(m_minPoolSize, 0);
 	m_maxPoolSize = cfg->GetInteger(cfgSectionName, "MaxPoolSize", m_minPoolSize);
 	if (m_maxPoolSize >= 0)
 		m_maxPoolSize = max(m_minPoolSize, m_maxPoolSize);
-		
+
 	if (m_host.IsEmpty() || m_database.IsEmpty()) {
 		PTRACE(1, GetName() << "\tInitialize failed: database name or host not specified!");
 		SNMP_TRAP(4, SNMPError, Database, GetName() + " creation failed");
 		return false;
 	}
 
-	return Connect();	
+	return Connect();
 }
 
 bool GkSQLConnection::Connect()
 {
 	PWaitAndSignal lock(m_connectionsMutex);
-	
+
 	for (PINDEX i = m_idleConnections.size(); i < m_minPoolSize; ++i) {
 		SQLConnPtr connptr = CreateNewConnection(i);
 		if (connptr != NULL)
 			m_idleConnections.push_back(connptr);
 	}
-	
+
 	if (m_idleConnections.empty() && m_minPoolSize) {
-		PTRACE(1, GetName() << "\tDatabase connection failed: " 
+		PTRACE(1, GetName() << "\tDatabase connection failed: "
 			<< m_username << '@' << m_host << '[' << m_database << ']');
 		SNMP_TRAP(4, SNMPError, Database, GetName() + " connection failed");
 		return false;
 	} else {
-		PTRACE(3, GetName() << "\tDatabase connection pool created: " 
+		PTRACE(3, GetName() << "\tDatabase connection pool created: "
 			<< m_username << '@' << m_host << '[' << m_database << ']');
-		PTRACE(5, GetName() << "\tConnection pool: " 
+		PTRACE(5, GetName() << "\tConnection pool: "
 			<< m_idleConnections.size() << " SQL connections created, "
 			<< (m_minPoolSize - m_idleConnections.size()) << " failed");
 		m_connected = true;
@@ -111,7 +117,7 @@ bool GkSQLConnection::Connect()
 void GkSQLConnection::Disconnect()
 {
 	PWaitAndSignal lock(m_connectionsMutex);
-	
+
 	// disconnect/delete all connections
 	PTRACE(3, GetName() << "\tDisconnecting all SQL connections in pool");
 	for(iterator Iter = m_idleConnections.begin(); Iter != m_idleConnections.end(); ++Iter) {
@@ -124,11 +130,11 @@ void GkSQLConnection::Disconnect()
 GkSQLConnection::~GkSQLConnection()
 {
 	const PTime timeStart;
-	
+
 	m_destroying = true;
-	// wakeup any waiting threads
+	// wake up any waiting threads
 	m_connectionAvailable.Signal();
-	
+
 	// wait for still active connections (should not happen, but...)
 	do {
 		{
@@ -141,15 +147,15 @@ GkSQLConnection::~GkSQLConnection()
 		}
 		PThread::Sleep(250);
 	} while ((PTime()-timeStart).GetMilliSeconds() < GKSQL_CLEANUP_TIMEOUT);
-	
+
 	// close connections from the idle list and leave any on the busy list
 	// busy list should be empty at this moment
 	PWaitAndSignal lock(m_connectionsMutex);
-	
+
 	m_waitingRequests.clear();
 	iterator iter = m_idleConnections.begin();
 	iterator end = m_idleConnections.end();
-	
+
 	while (iter != end) {
 		PTRACE(5, GetName() << "\tDatabase connection (id " << (*iter)->m_id << ") closed");
 		delete *iter++;
@@ -164,7 +170,7 @@ GkSQLConnection::~GkSQLConnection()
 }
 
 bool GkSQLConnection::AcquireSQLConnection(
-	SQLConnPtr& connptr,
+	SQLConnPtr & connptr,
 	long timeout
 	)
 {
@@ -180,7 +186,7 @@ bool GkSQLConnection::AcquireSQLConnection(
 			return false;
 		}
 	}
-	
+
 	const PTime timeStart;
 	connptr = NULL;
 	bool waiting = false;
@@ -250,7 +256,7 @@ void GkSQLConnection::ReleaseSQLConnection(
 			// check if SQLConnPtr* is not NULL and if SQLConnPtr is empty (NULL)
 			if (*iter && *(*iter) == NULL)
 				break;
-			iter++;
+			++iter;
 		}
 		if (iter != end && !m_destroying && !deleteFromPool) {
 			// do not remove itself from the list of busy connections
@@ -275,12 +281,11 @@ void GkSQLConnection::ReleaseSQLConnection(
 
 GkSQLResult* GkSQLConnection::ExecuteQuery(
 	const char* queryStr,
-	const PStringArray* queryParams,
-	long timeout
-	)
+	const PStringArray * queryParams,
+	long timeout)
 {
 	SQLConnPtr connptr;
-	
+
 	if (AcquireSQLConnection(connptr, timeout)) {
 		GkSQLResult* result = NULL;
 		if (queryParams) {
@@ -302,14 +307,14 @@ GkSQLResult* GkSQLConnection::ExecuteQuery(
 
 GkSQLResult* GkSQLConnection::ExecuteQuery(
 	const char* queryStr,
-	const std::map<PString, PString>& queryParams,
+	const std::map<PString, PString> & queryParams,
 	long timeout
 	)
 {
 	SQLConnPtr connptr;
-	
+
 	if (AcquireSQLConnection(connptr, timeout)) {
-		GkSQLResult* result = NULL;
+		GkSQLResult * result = NULL;
 		if (queryParams.empty()) {
 			PTRACE(5, GetName() << "\tExecuting query: " << queryStr);
 			result = ExecuteQuery(connptr, queryStr, timeout);
@@ -333,14 +338,14 @@ PString GkSQLConnection::ReplaceQueryParams(
 	/// parametrized query string
 	const char* queryStr,
 	/// parameter values
-	const PStringArray& queryParams
+	const PStringArray & queryParams
 	)
 {
 	const PINDEX numParams = queryParams.GetSize();
 	PString finalQuery(queryStr);
 	PINDEX queryLen = finalQuery.GetLength();
 	PINDEX pos = 0;
-	char* endChar;
+	char * endChar = NULL;
 
 	while (pos != P_MAX_INDEX && pos < queryLen) {
 		pos = finalQuery.Find('%', pos);
@@ -362,10 +367,8 @@ PString GkSQLConnection::ReplaceQueryParams(
 				queryLen = queryLen + escapedLen - paramLen - 1;
 				pos = pos - 1 + escapedLen;
 			} else if (paramNo && paramLen) {
-				// replace out of range parameter with an empty string
-				finalQuery.Delete(pos - 1, paramLen + 1);
-				queryLen -= paramLen + 1;
-				pos--;
+                // leave unknown placeholders intact so a 2nd stage can replace them
+
 			}
 		} else if (c == '{') { // escaped syntax (%{1})
 			const PINDEX closingBrace = finalQuery.Find('}', ++pos);
@@ -379,10 +382,7 @@ PString GkSQLConnection::ReplaceQueryParams(
 					queryLen = queryLen + escapedLen - paramLen - 3;
 					pos = pos - 2 + escapedLen;
 				} else if (paramNo && paramLen) {
-					// replace out of range parameter with an empty string
-					finalQuery.Delete(pos - 2, paramLen + 3);
-					queryLen -= paramLen + 3;
-					pos -= 2;
+				    // leave unknown placeholders intact so a 2nd stage can replace them
 				}
 			}
 		}
@@ -397,7 +397,7 @@ PString GkSQLConnection::ReplaceQueryParams(
 	/// parametrized query string
 	const char* queryStr,
 	/// parameter values
-	const std::map<PString, PString>& queryParams
+	const std::map<PString, PString> & queryParams
 	)
 {
 	PString finalQuery(queryStr);
@@ -428,10 +428,7 @@ PString GkSQLConnection::ReplaceQueryParams(
 					queryLen = queryLen + escapedLen - paramLen - 3;
 					pos = pos - 2 + escapedLen;
 				} else {
-					// replace out of range parameter with an empty string
-					finalQuery.Delete(pos - 2, paramLen + 3);
-					queryLen -= paramLen + 3;
-					pos -= 2;
+                    // leave unknown placeholders intact so a 2nd stage can replace them
 				}
 			}
 		} else { // simple syntax (%1)
@@ -443,10 +440,7 @@ PString GkSQLConnection::ReplaceQueryParams(
 				queryLen = queryLen + escapedLen - 2;
 				pos = pos - 1 + escapedLen;
 			} else {
-				// replace out of range parameter with an empty string
-				finalQuery.Delete(pos - 1, 2);
-				queryLen -= 2;
-				pos--;
+                // leave unknown placeholders intact so a 2nd stage can replace them
 			}
 		}
 	}
