@@ -4,7 +4,7 @@
  * MySQL driver module for GnuGk
  *
  * Copyright (c) 2004, Michal Zygmuntowicz
- * Copyright (c) 2006-2012, Jan Willamowius
+ * Copyright (c) 2006-2023, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -41,6 +41,9 @@ static const char * (STDCALL *g_mysql_error)(MYSQL *mysql) = NULL;
 static my_ulonglong (STDCALL *g_mysql_affected_rows)(MYSQL *mysql) = NULL;
 
 
+#ifndef my_bool
+#define my_bool int
+#endif
 
 /** Class that encapsulates SQL query result for MySQL backend.
 	It does not provide any multithread safety, so should be accessed
@@ -68,46 +71,46 @@ public:
 		/// MySQL specific error message text
 		const char* errorMsg
 		);
-	
+
 	virtual ~GkMySQLResult();
-	
+
 	/** @return
 	    Backend specific error message, if the query failed.
-	*/	
+	*/
 	virtual PString GetErrorMessage();
-	
+
 	/** @return
 	    Backend specific error code, if the query failed.
-	*/	
+	*/
 	virtual long GetErrorCode();
-	
+
 	/** Fetch a single row from the result set. After each row is fetched,
 	    cursor position is moved to a next row.
-		
+
 	    @return
 	    True if the row has been fetched, false if no more rows are available.
 	*/
 	virtual bool FetchRow(
 		/// array to be filled with string representations of the row fields
-		PStringArray& result
+		PStringArray & result
 		);
 	virtual bool FetchRow(
 		/// array to be filled with string representations of the row fields
-		ResultRow& result
+		ResultRow & result
 		);
 
 private:
 	GkMySQLResult();
-	GkMySQLResult(const GkMySQLResult&);
-	GkMySQLResult& operator=(const GkMySQLResult&);
-	
+	GkMySQLResult(const GkMySQLResult &);
+	GkMySQLResult & operator=(const GkMySQLResult &);
+
 protected:
 	/// query result for SELECT type queries, NULL otherwise
-	MYSQL_RES* m_sqlResult;
+	MYSQL_RES * m_sqlResult;
 	/// the most recent row returned by fetch operation
 	MYSQL_ROW m_sqlRow;
-	/// lenghts (bytes) for each field in m_sqlRow result row
-	unsigned long* m_sqlRowLengths;
+	/// lengths (bytes) for each field in m_sqlRow result row
+	unsigned long * m_sqlRowLengths;
 	/// MySQL specific error code (if the query failed)
 	unsigned int m_errorCode;
 	/// MySQL specific error message text (if the query failed)
@@ -123,7 +126,7 @@ public:
 		/// name to use in the log
 		const char* name = "MySQL"
 		);
-	
+
 	virtual ~GkMySQLConnection();
 
 protected:
@@ -143,32 +146,32 @@ protected:
 
 	private:
 		MySQLConnWrapper();
-		MySQLConnWrapper(const MySQLConnWrapper&);
-		MySQLConnWrapper& operator=(const MySQLConnWrapper&);
+		MySQLConnWrapper(const MySQLConnWrapper &);
+		MySQLConnWrapper & operator=(const MySQLConnWrapper &);
 
 	public:
-		MYSQL* m_conn;
+		MYSQL * m_conn;
 	};
 
 	/** Create a new SQL connection using parameters stored in this object.
 	    When the connection is to be closed, the object is simply deleted
 	    using delete operator.
-	    
+
 	    @return
-	    NULL if database connection could not be established 
+	    NULL if database connection could not be established
 	    or an object of MySQLConnWrapper class.
 	*/
 	virtual SQLConnPtr CreateNewConnection(
 		/// unique identifier for this connection
 		int id
 		);
-	
+
 	/** Execute the query using specified SQL connection.
 
 		@return
 		Query execution result.
 	*/
-	virtual GkSQLResult* ExecuteQuery(
+	virtual GkSQLResult * ExecuteQuery(
 		/// SQL connection to use for query execution
 		SQLConnPtr conn,
 		/// query string
@@ -176,7 +179,7 @@ protected:
 		/// maximum time (ms) for the query execution, -1 means infinite
 		long timeout = -1
 		);
-		
+
 	/** Escape any special characters in the string, so it can be used in a SQL query.
 
 		@return
@@ -190,15 +193,15 @@ protected:
 		);
 
 private:
-	GkMySQLConnection(const GkMySQLConnection&);
-	GkMySQLConnection& operator=(const GkMySQLConnection&);
+	GkMySQLConnection(const GkMySQLConnection &);
+	GkMySQLConnection & operator=(const GkMySQLConnection &);
 };
 
 
 GkMySQLResult::GkMySQLResult(
 	/// SELECT type query result
 	MYSQL_RES* selectResult
-	) 
+	)
 	: GkSQLResult(false), m_sqlResult(selectResult), m_sqlRow(NULL),
 	m_sqlRowLengths(NULL), m_errorCode(0)
 {
@@ -212,19 +215,19 @@ GkMySQLResult::GkMySQLResult(
 GkMySQLResult::GkMySQLResult(
 	/// number of rows affected by the query
 	long numRowsAffected
-	) 
-	: GkSQLResult(false), m_sqlResult(NULL), m_sqlRow(NULL), 
+	)
+	: GkSQLResult(false), m_sqlResult(NULL), m_sqlRow(NULL),
 	m_sqlRowLengths(NULL), m_errorCode(0)
 {
 	m_numRows = numRowsAffected;
 }
-	
+
 GkMySQLResult::GkMySQLResult(
 	/// MySQL specific error code
 	unsigned int errorCode,
 	/// MySQL specific error message text
 	const char* errorMsg
-	) 
+	)
 	: GkSQLResult(true), m_sqlResult(NULL), m_sqlRow(NULL),
 	m_sqlRowLengths(NULL), m_errorCode(errorCode), m_errorMessage(errorMsg)
 {
@@ -240,7 +243,7 @@ PString GkMySQLResult::GetErrorMessage()
 {
 	return m_errorMessage;
 }
-	
+
 long GkMySQLResult::GetErrorCode()
 {
 	return m_errorCode;
@@ -248,12 +251,12 @@ long GkMySQLResult::GetErrorCode()
 
 bool GkMySQLResult::FetchRow(
 	/// array to be filled with string representations of the row fields
-	PStringArray& result
+	PStringArray & result
 	)
 {
 	if (m_sqlResult == NULL || m_numRows <= 0)
 		return false;
-	
+
 	m_sqlRow = (*g_mysql_fetch_row)(m_sqlResult);
 	m_sqlRowLengths = (*g_mysql_fetch_lengths)(m_sqlResult);
 	if (m_sqlRow == NULL || m_sqlRowLengths == NULL) {
@@ -261,23 +264,23 @@ bool GkMySQLResult::FetchRow(
 		m_sqlRowLengths = NULL;
 		return false;
 	}
-		
+
 	result.SetSize(m_numFields);
-	
+
 	for (PINDEX i = 0; i < m_numFields; i++)
 		result[i] = PString(m_sqlRow[i], m_sqlRowLengths[i]);
-	
+
 	return true;
 }
 
 bool GkMySQLResult::FetchRow(
 	/// array to be filled with string representations of the row fields
-	ResultRow& result
+	ResultRow & result
 	)
 {
 	if (m_sqlResult == NULL || m_numRows <= 0)
 		return false;
-	
+
 	m_sqlRow = (g_mysql_fetch_row)(m_sqlResult);
 	m_sqlRowLengths = (*g_mysql_fetch_lengths)(m_sqlResult);
 	MYSQL_FIELD* fields = (*g_mysql_fetch_fields)(m_sqlResult);
@@ -286,14 +289,14 @@ bool GkMySQLResult::FetchRow(
 		m_sqlRowLengths = NULL;
 		return false;
 	}
-		
+
 	result.resize(m_numFields);
-	
+
 	for (PINDEX i = 0; i < m_numFields; i++) {
 		result[i].first = PString(m_sqlRow[i], m_sqlRowLengths[i]);
 		result[i].second = fields[i].name;
 	}
-	
+
 	return true;
 }
 
@@ -304,7 +307,7 @@ GkMySQLConnection::GkMySQLConnection(
 	) : GkSQLConnection(name)
 {
 }
-	
+
 GkMySQLConnection::~GkMySQLConnection()
 {
 }
@@ -321,7 +324,7 @@ GkSQLConnection::SQLConnPtr GkMySQLConnection::CreateNewConnection(
 {
 	if (!g_sharedLibrary.IsLoaded()) {
 		if (m_library.IsEmpty()) {
-#ifdef _WIN32
+#if _WIN32 || _WIN64
 			m_library = "libmysql" + g_sharedLibrary.GetExtension();
 #else
 			m_library = "libmysqlclient" + g_sharedLibrary.GetExtension();
@@ -362,7 +365,8 @@ GkSQLConnection::SQLConnPtr GkMySQLConnection::CreateNewConnection(
 		}
 	}
 
-	const unsigned int CONNECT_TIMEOUT = 10;	// connect timeout in seconds (!)
+	const unsigned int CONNECT_TIMEOUT = m_connectTimeout; // connect timeout in seconds
+	const unsigned int READ_TIMEOUT = m_readTimeout; // read timeout in seconds
 
 	MYSQL* conn = (*g_mysql_init)(NULL);
 	if (conn == NULL) {
@@ -371,6 +375,7 @@ GkSQLConnection::SQLConnPtr GkMySQLConnection::CreateNewConnection(
 		return NULL;
 	}
 	(*g_mysql_options)(conn, MYSQL_OPT_CONNECT_TIMEOUT, (const char*)&CONNECT_TIMEOUT);
+	(*g_mysql_options)(conn, MYSQL_OPT_READ_TIMEOUT, (const char*)&READ_TIMEOUT);
 
 #if (MYSQL_VERSION_ID >= 50013)
 	my_bool reconnect = 1;	// enable auto-reconnect, older versions have it on by default
@@ -382,20 +387,20 @@ GkSQLConnection::SQLConnPtr GkMySQLConnection::CreateNewConnection(
 	(*g_mysql_options)(conn, MYSQL_READ_DEFAULT_GROUP, "gnugk");
 
 	// connect to the MySQL database, try each host on the list in case of failure
-	if ((*g_mysql_real_connect)(conn, m_host, m_username, 
+	if ((*g_mysql_real_connect)(conn, m_host, m_username,
 			m_password.IsEmpty() ? (const char*)NULL : (const char*)m_password,
 			m_database, m_port, NULL, CLIENT_MULTI_STATEMENTS)) {
-		PTRACE(5, GetName() << "\tMySQL connection to " << m_username << '@' << m_host 
+		PTRACE(5, GetName() << "\tMySQL connection to " << m_username << '@' << m_host
 			<< '[' << m_database << "] established successfully");
 		return new MySQLConnWrapper(id, m_host, conn);
 	} else {
-		PTRACE(2, GetName() << "\tMySQL connection to " << m_username << '@' << m_host 
+		PTRACE(2, GetName() << "\tMySQL connection to " << m_username << '@' << m_host
 			<< '[' << m_database << "] failed (mysql_real_connect failed): " << (*g_mysql_error)(conn));
 		SNMP_TRAP(5, SNMPError, Database, GetName() + " connection failed");
 	}
 	return NULL;
 }
-	
+
 GkSQLResult* GkMySQLConnection::ExecuteQuery(
 	/// SQL connection to use for query execution
 	GkSQLConnection::SQLConnPtr conn,
@@ -405,7 +410,7 @@ GkSQLResult* GkMySQLConnection::ExecuteQuery(
 	long /*timeout*/
 	)
 {
-	MYSQL* mysqlconn = ((MySQLConnWrapper*)conn)->m_conn;
+	MYSQL * mysqlconn = ((MySQLConnWrapper*)conn)->m_conn;
 
 	int result = (*g_mysql_real_query)(mysqlconn, queryStr, strlen(queryStr));
 	if (result) {
@@ -414,18 +419,17 @@ GkSQLResult* GkMySQLConnection::ExecuteQuery(
 		return sqlResult;
 	}
 
-	MYSQL_RES* queryResult = (*g_mysql_store_result)(mysqlconn);
+	MYSQL_RES * queryResult = (*g_mysql_store_result)(mysqlconn);
 
 	if (queryResult) {
 		/* loop and discard results after first if any */
-		MYSQL_RES* tmpResult;
 		int tmpstatus;
 		while ( (tmpstatus = (*g_mysql_next_result)(mysqlconn) ) >= 0) {
 			if (tmpstatus > 0) {
 				/* error fetching next result */
 				break;
 			}
-			tmpResult = (*g_mysql_store_result)(mysqlconn);
+			MYSQL_RES * tmpResult = (*g_mysql_store_result)(mysqlconn);
 			(*g_mysql_free_result)(tmpResult);
 		}
 	}
@@ -452,7 +456,7 @@ PString GkMySQLConnection::EscapeString(
 {
 	PString escapedStr;
 	const size_t numChars = str ? strlen(str) : 0;
-	
+
 	if (numChars) {
 		char * buf = (char *)malloc(numChars * 2 + 1);
 		MYSQL* mysqlconn = ((MySQLConnWrapper*)conn)->m_conn;
