@@ -3,7 +3,7 @@
  *
  * native ODBC / unixODBC driver module for GnuGk
  *
- * Copyright (c) 2008-2013, Jan Willamowius
+ * Copyright (c) 2008-2023, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -84,7 +84,7 @@ static SQLRETURN (SQL_API *g_SQLSetStmtAttr)(SQLHSTMT StatementHandle,
 PString GetODBCDiagMsg(SQLRETURN result, SQLSMALLINT handleType, SQLHANDLE handle)
 {
 	if (result == SQL_SUCCESS)
-		return "Sucesss";
+		return "Success";
 	if (result == SQL_INVALID_HANDLE)
 		return "Invalid ODBC handle passed";
 	if (result == SQL_NO_DATA)
@@ -95,7 +95,7 @@ PString GetODBCDiagMsg(SQLRETURN result, SQLSMALLINT handleType, SQLHANDLE handl
 	SQLCHAR *msg = new SQLCHAR[SQL_MAX_MESSAGE_LENGTH];
 	SQLSMALLINT msgSize;
 	SQLRETURN r;
-	
+
 	memset(msg, 0, SQL_MAX_MESSAGE_LENGTH);
 	r = (*g_SQLGetDiagRec)(handleType, handle, 1, reinterpret_cast<SQLCHAR*>(sqlState),
 		&nativeError, msg, SQL_MAX_MESSAGE_LENGTH, &msgSize);
@@ -140,22 +140,22 @@ public:
 		/// error message text
 		const char* errorMsg
 		);
-	
+
 	virtual ~GkODBCResult();
-	
+
 	/** @return
 	    Backend specific error message, if the query failed.
-	*/	
+	*/
 	virtual PString GetErrorMessage();
-	
+
 	/** @return
 	    Backend specific error code, if the query failed.
-	*/	
+	*/
 	virtual long GetErrorCode();
-	
+
 	/** Fetch a single row from the result set. After each row is fetched,
 	    cursor position is moved to a next row.
-		
+
 	    @return
 	    True if the row has been fetched, false if no more rows are available.
 	*/
@@ -167,12 +167,12 @@ public:
 		/// array to be filled with string representations of the row fields
 		ResultRow& result
 		);
-		
+
 private:
 	GkODBCResult();
 	GkODBCResult(const GkODBCResult &);
 	GkODBCResult & operator=(const GkODBCResult &);
-	
+
 protected:
 	/// query result for SELECT type queries
 	std::list<ResultRow*> * m_sqlResult;
@@ -195,7 +195,7 @@ public:
 		/// name to use in the log
 		const char* name = "ODBC"
 		);
-	
+
 	virtual ~GkODBCConnection();
 
 protected:
@@ -225,16 +225,16 @@ protected:
 	/** Create a new SQL connection using parameters stored in this object.
 	    When the connection is to be closed, the object is simply deleted
 	    using delete operator.
-	    
+
 	    @return
-	    NULL if database connection could not be established 
+	    NULL if database connection could not be established
 	    or an object of GkODBCConnWrapper class.
 	*/
 	virtual SQLConnPtr CreateNewConnection(
 		/// unique identifier for this connection
 		int id
 		);
-	
+
 	/** Execute the query using specified SQL connection.
 
 		@return
@@ -248,7 +248,7 @@ protected:
 		/// maximum time (ms) for the query execution, -1 means infinite
 		long timeout = -1
 		);
-		
+
 	/** Escape any special characters in the string, so it can be used in a SQL query.
 
 		@return
@@ -264,7 +264,7 @@ protected:
 private:
 	GkODBCConnection(const GkODBCConnection &);
 	GkODBCConnection & operator=(const GkODBCConnection &);
-	
+
 private:
 	SQLHENV m_env;
 };
@@ -283,10 +283,10 @@ GkODBCResult::GkODBCResult(
 {
 	m_numRows = numRowsAffected;
 	m_numFields = numFields;
-	
+
 	if (m_sqlResult != NULL && !m_sqlResult->empty())
 		m_numRows = m_sqlResult->size();
-	
+
 	m_queryError = false;
 }
 
@@ -295,7 +295,7 @@ GkODBCResult::GkODBCResult(
 	unsigned int errorCode,
 	/// error message text
 	const char* errorMsg
-	) 
+	)
 	: GkSQLResult(true), m_sqlResult(NULL), m_sqlRow(-1),
 	m_errorCode(errorCode), m_errorMessage(errorMsg)
 {
@@ -319,7 +319,7 @@ PString GkODBCResult::GetErrorMessage()
 {
 	return m_errorMessage;
 }
-	
+
 long GkODBCResult::GetErrorCode()
 {
 	return m_errorCode;
@@ -332,22 +332,22 @@ bool GkODBCResult::FetchRow(
 {
 	if (m_sqlResult == NULL || m_numRows <= 0)
 		return false;
-	
+
 	if (m_sqlRow < 0) {
 		m_sqlRow = 0;
 		m_sqlRowIter = m_sqlResult->begin();
 	}
-	
+
 	if (m_sqlRow >= m_numRows)
 		return false;
 
 	result.SetSize(m_numFields);
 	for (int i = 0; i < m_numFields; ++i)
 		result[i] = (**m_sqlRowIter)[i].first;
-	
+
 	++m_sqlRow;
 	++m_sqlRowIter;
-	
+
 	return true;
 }
 
@@ -358,20 +358,20 @@ bool GkODBCResult::FetchRow(
 {
 	if (m_sqlResult == NULL || m_numRows <= 0)
 		return false;
-	
+
 	if (m_sqlRow < 0) {
 		m_sqlRow = 0;
 		m_sqlRowIter = m_sqlResult->begin();
 	}
-	
+
 	if (m_sqlRow >= m_numRows)
 		return false;
 
 	result = *(*m_sqlRowIter);
-	
+
 	++m_sqlRow;
 	++m_sqlRowIter;
-	
+
 	return true;
 }
 
@@ -382,7 +382,7 @@ GkODBCConnection::GkODBCConnection(
 	) : GkSQLConnection(name), m_env(SQL_NULL_HENV)
 {
 }
-	
+
 GkODBCConnection::~GkODBCConnection()
 {
 	if (m_env != SQL_NULL_HENV) {
@@ -409,9 +409,11 @@ GkSQLConnection::SQLConnPtr GkODBCConnection::CreateNewConnection(
 	int id
 	)
 {
+    SQLRETURN r;
+
 	if (!g_sharedLibrary.IsLoaded()) {
 		if (m_library.IsEmpty()) {
-#ifdef _WIN32
+#if _WIN32 || _WIN64
 			m_library = "odbc32" + g_sharedLibrary.GetExtension();
 #else
 			m_library = "libodbc" + g_sharedLibrary.GetExtension();
@@ -447,41 +449,43 @@ GkSQLConnection::SQLConnPtr GkODBCConnection::CreateNewConnection(
 			SNMP_TRAP(5, SNMPError, Database, GetName() + " DLL load error");
 			return NULL;
 		}
-
-		// allocate Environment handle and register version 
-		SQLRETURN r = (*g_SQLAllocHandle)(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &m_env);
-		if (!SQL_SUCCEEDED(r)) {
-			PTRACE(1, GetName() << "\tFailed to allocate an ODBC environment handle: " << GetODBCDiagMsg(r, SQL_HANDLE_ENV, SQL_NULL_HENV));
-			SNMP_TRAP(5, SNMPError, Database, GetName() + " connection failed");
-			return NULL;
-		}
-		r = (*g_SQLSetEnvAttr)(m_env, SQL_ATTR_ODBC_VERSION, reinterpret_cast<SQLPOINTER>(SQL_OV_ODBC3), 0); 
-		if (!SQL_SUCCEEDED(r)) {
-			PTRACE(1, GetName() << "\tFailed to request ODBC interface version 3.0: " << GetODBCDiagMsg(r, SQL_HANDLE_ENV, m_env));
-			(*g_SQLFreeHandle)(SQL_HANDLE_ENV, m_env);
-			m_env = SQL_NULL_HENV;
-			SNMP_TRAP(5, SNMPError, Database, GetName() + " connection failed");
-			return NULL;
-		}
-		PTRACE(5, GetName() << "\tODBC environment created");
 	}
 
+    if (m_env == SQL_NULL_HENV) {
+        // allocate Environment handle and register version
+        r = (*g_SQLAllocHandle)(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &m_env);
+        if (!SQL_SUCCEEDED(r)) {
+            PTRACE(1, GetName() << "\tFailed to allocate an ODBC environment handle: " << GetODBCDiagMsg(r, SQL_HANDLE_ENV, SQL_NULL_HENV));
+            SNMP_TRAP(5, SNMPError, Database, GetName() + " connection failed");
+            return NULL;
+        }
+        r = (*g_SQLSetEnvAttr)(m_env, SQL_ATTR_ODBC_VERSION, reinterpret_cast<SQLPOINTER>(SQL_OV_ODBC3), 0);
+        if (!SQL_SUCCEEDED(r)) {
+            PTRACE(1, GetName() << "\tFailed to request ODBC interface version 3.0: " << GetODBCDiagMsg(r, SQL_HANDLE_ENV, m_env));
+            (*g_SQLFreeHandle)(SQL_HANDLE_ENV, m_env);
+            m_env = SQL_NULL_HENV;
+            SNMP_TRAP(5, SNMPError, Database, GetName() + " connection failed");
+            return NULL;
+        }
+        PTRACE(5, GetName() << "\tODBC environment created");
+    }
+
 	SQLHDBC conn = SQL_NULL_HDBC;
 
 	// allocate connection handle, set timeout
-	SQLRETURN r = (*g_SQLAllocHandle)(SQL_HANDLE_DBC, m_env, &conn); 
+	r = (*g_SQLAllocHandle)(SQL_HANDLE_DBC, m_env, &conn);
 	if (!SQL_SUCCEEDED(r)) {
 		PTRACE(1, GetName() << "\tFailed to allocate an ODBC connection handle: " << GetODBCDiagMsg(r, SQL_HANDLE_ENV, m_env));
 		SNMP_TRAP(5, SNMPError, Database, GetName() + " connection failed");
 		return NULL;
 	}
-	
+
 	r = (g_SQLSetConnectAttr)(conn, SQL_ATTR_LOGIN_TIMEOUT, reinterpret_cast<SQLPOINTER>(10), 0);
 	if (!SQL_SUCCEEDED(r)) {
 		PTRACE(1, GetName() << "\tFailed to set ODBC connection login timeout: " << GetODBCDiagMsg(r, SQL_HANDLE_DBC, conn));
 		SNMP_TRAP(5, SNMPError, Database, GetName() + " connection failed");
 	}
-	
+
 	r = (*g_SQLSetConnectAttr)(conn, SQL_ATTR_CONNECTION_TIMEOUT, reinterpret_cast<SQLPOINTER>(10), 0);
 	if (!SQL_SUCCEEDED(r)) {
 		PTRACE(1, GetName() << "\tFailed to set ODBC connection request timeout: " << GetODBCDiagMsg(r, SQL_HANDLE_DBC, conn));
@@ -518,7 +522,7 @@ GkSQLResult* GkODBCConnection::ExecuteQuery(
 	SQLHDBC conn = ((GkODBCConnWrapper*)con)->m_conn;
 	SQLHSTMT stmt = SQL_NULL_HSTMT;
 
-	SQLRETURN r = (*g_SQLAllocHandle)(SQL_HANDLE_STMT, conn, &stmt); 
+	SQLRETURN r = (*g_SQLAllocHandle)(SQL_HANDLE_STMT, conn, &stmt);
 	if (!SQL_SUCCEEDED(r)) {
 		PString errmsg(GetODBCDiagMsg(r, SQL_HANDLE_DBC, conn));
 		PTRACE(1, GetName() << "\tFailed to allocate an ODBC statement handle: " << errmsg);
@@ -542,7 +546,7 @@ GkSQLResult* GkODBCConnection::ExecuteQuery(
 		Disconnect();
 		return new GkODBCResult(r, errmsg + ", query: " + queryStr);
 	}
-	
+
 	SQLSMALLINT columns = 0;
 	r = (*g_SQLNumResultCols)(stmt, &columns);
 	if (!SQL_SUCCEEDED(r)) {
@@ -553,9 +557,9 @@ GkSQLResult* GkODBCConnection::ExecuteQuery(
 		Disconnect();
 		return new GkODBCResult(r, errmsg + ", query: " + queryStr);
 	}
-	
+
 	SQLLEN rows = 0;
-	
+
 	if (columns == 0) {
 		if (nodata) {
 			(*g_SQLFreeHandle)(SQL_HANDLE_STMT, stmt);
@@ -575,7 +579,7 @@ GkSQLResult* GkODBCConnection::ExecuteQuery(
 		return new GkODBCResult(0, columns, NULL);
 
 	std::vector<PString> fieldNames(columns);
-	
+
 	for (SQLUSMALLINT i = 1; i <= columns; ++i) {
 		SQLCHAR colname[64];
 		r = (*g_SQLDescribeCol)(stmt, i, colname, sizeof(colname), NULL, NULL, NULL, NULL, NULL);
@@ -591,16 +595,16 @@ GkSQLResult* GkODBCConnection::ExecuteQuery(
 		r = (*g_SQLFetch)(stmt);
 		if (r == SQL_NO_DATA)
 			break;
-			
+
 		if (!SQL_SUCCEEDED(r)) {
 			PTRACE(1, GetName() << "\tFailed to fetch an ODBC result row: " << GetODBCDiagMsg(r, SQL_HANDLE_STMT, stmt));
 			SNMP_TRAP(5, SNMPError, Database, GetName() + " query failed")
 			// we should return an error instead
 			break;
 		}
-		
+
 		GkSQLResult::ResultRow * row = new GkSQLResult::ResultRow(columns);
-		
+
 		for (SQLUSMALLINT i = 1; i <= columns; ++i) {
 			SQLLEN indicator;
 			char data[512];
@@ -625,7 +629,7 @@ GkSQLResult* GkODBCConnection::ExecuteQuery(
 	} while (SQL_SUCCEEDED(r));
 
 	(*g_SQLFreeHandle)(SQL_HANDLE_STMT, stmt);
-	
+
 	return new GkODBCResult(rows, columns, resultRows);
 }
 
