@@ -2,7 +2,7 @@
 //
 // H.323 utility functions
 //
-// Copyright (c) 2000-2013, Jan Willamowius
+// Copyright (c) 2000-2024, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -11,18 +11,22 @@
 //
 //////////////////////////////////////////////////////////////////
 
+#include "config.h"
 #include <ptlib.h>
 #include <h323pdu.h>
 #include "gk_const.h"
-#include "config.h"
 #include "h323util.h"
+#include "Toolkit.h"
 
 #ifdef HAS_H460
 #include <h460/h4601.h>
 #endif
 
+#if (!_WIN32) && (GCC_VERSION >= 40600)
+#pragma GCC diagnostic ignored "-Wstrict-overflow"
+#endif
 
-#if defined(_WIN32) && defined(hasIPV6)
+#if (defined(_WIN32) || defined(_WIN64)) && defined(hasIPV6)
 // Windows doesn't have inet_ntop, so we fake it
 const char * inet_ntop(int family, const void *src, char *dst, socklen_t cnt)
 {
@@ -72,6 +76,14 @@ PString AsString(const PIPSocket::Address & ip, WORD pt)
 	return ip.AsString() + ":" + PString(pt);
 }
 
+PString AsString(const IPAndPortAddress & addr) {
+    if (addr.IsSet()) {
+        return AsString(addr.GetIP(), addr.GetPort());
+    } else {
+        return "0.0.0.0:0";
+    }
+}
+
 PString AsString(const H245_UnicastAddress & ip)
 {
 	if (ip.GetTag() == H245_UnicastAddress::e_iPAddress) {
@@ -125,9 +137,10 @@ void SetH245Port(H245_UnicastAddress & addr, WORD port)
 
 PString AsString(const H225_TransportAddress & ta)
 {
-	PStringStream stream;
-	stream << ta;
-	return stream;
+//	PStringStream stream;
+//	stream << ta;
+//	return stream;
+    return AsDotString(ta);
 }
 
 PString AsDotString(const H225_TransportAddress & addr, bool showPort)
@@ -169,27 +182,29 @@ WORD GetH225Port(const H225_TransportAddress & addr)
 
 PString AsString(const H323TransportAddress & ta)
 {
-	if (!IsSet(ta))
-		return "";
-
 	PIPSocket::Address ip;
 	WORD port = 0;
-	if (ta.GetIpAndPort(ip, port))
+	if (IsSet(ta) && ta.GetIpAndPort(ip, port))
 		return AsString(ip, port);
 	else
 		return "";
 }
 
+PString AsString(const NetworkAddress & na)
+{
+    return na.AsString();
+}
+
 PString AsString(const H225_EndpointType & terminalType)
 {
 	PString terminalTypeString;
-			
+
 	if (terminalType.HasOptionalField(H225_EndpointType::e_terminal))
 		terminalTypeString = ",terminal";
 
 	if (terminalType.HasOptionalField(H225_EndpointType::e_gateway))
 		terminalTypeString += ",gateway";
-	
+
 	if (terminalType.HasOptionalField(H225_EndpointType::e_mcu))
 		terminalTypeString += ",mcu";
 
@@ -203,7 +218,7 @@ PString AsString(const H225_EndpointType & terminalType)
 
 	if (terminalTypeString.IsEmpty())
 		terminalTypeString = ",unknown";
-	
+
 	return terminalTypeString.Mid(1);
 }
 
@@ -257,7 +272,7 @@ PString AsString(const H225_ArrayOf_AliasAddress & terminalAlias, bool includeAl
 	return (aliasListString);
 }
 
-PString AsString(const PASN_OctetString & Octets)
+PString AsString(const PASN_OctetString & Octets, bool noSpaces)
 {
 	PString result;
 	if (Octets.GetDataLength() > 0) {
@@ -265,9 +280,16 @@ PString AsString(const PASN_OctetString & Octets)
 		for (PINDEX i = 1; i < Octets.GetDataLength(); ++i)
 			result += PString(PString::Printf, " %02x", Octets[i]);
 	}
+	if (noSpaces)
+        result.Replace(" ", "-", true);
 	return result;
 }
 
+PString AsString(const H225_CallIdentifier & callid, bool noSpaces)
+{
+    return AsString(callid.m_guid, noSpaces);
+}
+
 PString AsString(const PBYTEArray & array)
 {
 	PString result;
@@ -338,7 +360,7 @@ H245_TransportAddress IPToH245TransportAddr(const PIPSocket::Address & ip, WORD
 	return Result;
 }
 
-// convert a string (dot notation without port) into an H245 transport address
+// convert a string (dot notation without port) into an H.245 transport address
 //H245_TransportAddress StringToH245TransportAddr(const PString & Addr, WORD Port)
 //{
 //	H245_TransportAddress Result;
@@ -348,26 +370,26 @@ H245_TransportAddress IPToH245TransportAddr(const PIPSocket::Address & ip, WORD
 //	uniaddr.SetTag(H245_UnicastAddress::e_iPAddress);
 //	H245_UnicastAddress_iPAddress & ipaddr = uniaddr;
 //	PIPSocket::Address ip(Addr);
-//	for (int i = 0; i < 4; ++i)
+//	for (int i = 0; i < 4; ++i) // TODO: add IPv6
 //		ipaddr.m_network[i] = ip[i];
 //	ipaddr.m_tsapIdentifier = Port;
 //
 //	return Result;
 //}
 
-// convert a socket IP address into an H225 transport address
+// convert a socket IP address into an H.225 transport address
 H225_TransportAddress SocketToH225TransportAddr(const PIPSocket::Address & Addr, WORD Port)
 {
 	H225_TransportAddress Result;
 
 	if (Addr.GetVersion() == 6) {
-		Result.SetTag( H225_TransportAddress::e_ip6Address );
+		Result.SetTag(H225_TransportAddress::e_ip6Address);
 		H225_TransportAddress_ip6Address & ResultIP = Result;
 		for (int i = 0; i < 16; ++i)
 			ResultIP.m_ip[i] = Addr[i];
 		ResultIP.m_port = Port;
 	} else {
-		Result.SetTag( H225_TransportAddress::e_ipAddress );
+		Result.SetTag(H225_TransportAddress::e_ipAddress);
 		H225_TransportAddress_ipAddress & ResultIP = Result;
 		for (int i = 0; i < 4; ++i)
 			ResultIP.m_ip[i] = Addr[i];
@@ -377,15 +399,61 @@ H225_TransportAddress SocketToH225TransportAddr(const PIPSocket::Address & Addr,
 	return Result;
 }
 
-// convert a H.323 transport address into an H225 transport address
+// convert a socket address into a H.245 unicast address
+H245_UnicastAddress SocketToH245UnicastAddr(const PIPSocket::Address & Addr, WORD Port)
+{
+    H245_UnicastAddress h245unicast;
+
+	if (Addr.GetVersion() == 6) {
+		h245unicast.SetTag(H245_UnicastAddress::e_iP6Address);
+		H245_UnicastAddress_iP6Address & ipv6 = h245unicast;
+		for (int i = 0; i < 16; ++i)
+			ipv6.m_network[i] = Addr[i];
+		ipv6.m_tsapIdentifier = Port;
+	} else {
+		h245unicast.SetTag(H245_UnicastAddress::e_iPAddress);
+        H245_UnicastAddress_iPAddress & ipv4 = h245unicast;
+		for (int i = 0; i < 4; ++i)
+			ipv4.m_network[i] = Addr[i];
+		ipv4.m_tsapIdentifier = Port;
+	}
+
+	return h245unicast;
+}
+
+// convert a H.245 transport address into an H.225 transport address
+H225_TransportAddress H245ToH225TransportAddress(const H245_TransportAddress & h245addr)
+{
+	PIPSocket::Address ip;
+	WORD port = 0;
+    (void)GetIPAndPortFromTransportAddr(h245addr, ip, port);
+    return SocketToH225TransportAddr(ip, port);
+}
+
+// convert a H.323 transport address into an H.225 transport address
 H225_TransportAddress H323ToH225TransportAddress(const H323TransportAddress & h323addr)
 {
 	PIPSocket::Address ip;
 	WORD port = 0;
-	h323addr.GetIpAndPort(ip, port);
+	if (IsSet(h323addr))
+        h323addr.GetIpAndPort(ip, port);
 	return SocketToH225TransportAddr(ip, port);
 }
 
+// convert a H.245 unicast address into a socket address
+PIPSocket::Address H245UnicastToSocketAddr(const H245_UnicastAddress & h245unicast)
+{
+	if (h245unicast.GetTag() == H245_UnicastAddress::e_iPAddress) {
+		const H245_UnicastAddress_iPAddress & ipv4 = h245unicast;
+		return PIPSocket::Address(ipv4.m_network.GetSize(), ipv4.m_network.GetValue());
+	} else if (h245unicast.GetTag() == H245_UnicastAddress::e_iP6Address) {
+		const H245_UnicastAddress_iP6Address & ipv6 = h245unicast;
+		return PIPSocket::Address(ipv6.m_network.GetSize(), ipv6.m_network.GetValue());
+	}
+	PTRACE(1, "Unsupported H245_UnicastAddress: " << h245unicast.GetTagName());
+	return PIPSocket::Address();
+}
+
 // convert a H.245 unicast address into an H.323 transport address
 H323TransportAddress H245UnicastToH323TransportAddr(const H245_UnicastAddress & h245unicast)
 {
@@ -442,6 +510,34 @@ bool GetIPAndPortFromTransportAddr(const H225_TransportAddress & addr, PIPSocket
 	return false;
 }
 
+bool GetIPFromTransportAddr(const H245_TransportAddress & addr, PIPSocket::Address & ip)
+{
+    WORD notused;
+    return GetIPAndPortFromTransportAddr(addr, ip, notused);
+}
+
+bool GetIPAndPortFromTransportAddr(const H245_TransportAddress & h245addr, PIPSocket::Address & ip, WORD & port)
+{
+	if (h245addr.GetTag() == H245_TransportAddress::e_unicastAddress) {
+		const H245_UnicastAddress & h245unicast = h245addr;
+        if (h245unicast.GetTag() == H245_UnicastAddress::e_iPAddress) {
+            const H245_UnicastAddress_iPAddress & ipv4 = h245unicast;
+            ip = PIPSocket::Address(ipv4.m_network.GetSize(), ipv4.m_network.GetValue());
+            port = ipv4.m_tsapIdentifier;
+            return true;
+        } else if (h245unicast.GetTag() == H245_UnicastAddress::e_iP6Address) {
+            const H245_UnicastAddress_iP6Address & ipv6 = h245unicast;
+            ip = PIPSocket::Address(ipv6.m_network.GetSize(), ipv6.m_network.GetValue());
+            port = ipv6.m_tsapIdentifier;
+            return true;
+        }
+        PTRACE(1, "Unsupported H245_UnicastAddress: " << h245unicast.GetTagName());
+        return false;
+	}
+    PTRACE(1, "Unsupported H245_TransportAddress: " << h245addr.GetTagName());
+    return false;
+}
+
 PStringArray SplitIPAndPort(const PString & str, WORD default_port)
 {
 	PStringArray result;
@@ -471,6 +567,14 @@ PStringArray SplitIPAndPort(const PString & str, WORD default_port)
 	}
 }
 
+PIPSocket::Address GetIP(const PString & str)
+{
+    PIPSocket::Address ip((DWORD)0);
+    H323TransportAddress h323ta = H323TransportAddress(str);
+    h323ta.GetIpAddress(ip);
+    return ip;
+}
+
 void SetSockaddr(sockaddr_in & sin, const PIPSocket::Address & ip, WORD port)
 {
 	memset(&sin, 0, sizeof(sin));
@@ -483,11 +587,12 @@ void SetSockaddr(sockaddr_in & sin, const H323TransportAddress & addr)
 {
 	PIPSocket::Address ip;
 	WORD port = 0;
-	addr.GetIpAndPort(ip, port);
 	memset(&sin, 0, sizeof(sin));
-	sin.sin_family = AF_INET;
-	sin.sin_addr = ip;
-	sin.sin_port = htons(port);
+	if (IsSet(addr) && addr.GetIpAndPort(ip, port)) {
+        sin.sin_family = AF_INET;
+        sin.sin_addr = ip;
+        sin.sin_port = htons(port);
+	}
 }
 
 void SetSockaddr(sockaddr_in & sin, const H245_UnicastAddress & addr)
@@ -520,16 +625,17 @@ void SetSockaddr(sockaddr_in6 & sin6, const H323TransportAddress & addr)
 {
 	PIPSocket::Address ip;
 	WORD port = 0;
-	addr.GetIpAndPort(ip, port);
 	memset(&sin6, 0, sizeof(sin6));
-	if (ip.GetVersion() == 6) {
-		sin6.sin6_family = AF_INET6;
-		sin6.sin6_addr = ip;
-		sin6.sin6_port = htons(port);
-	} else {
-		((struct sockaddr_in*)&sin6)->sin_family = AF_INET;
-		((struct sockaddr_in*)&sin6)->sin_addr = ip;
-		((struct sockaddr_in*)&sin6)->sin_port = htons(port);
+	if (IsSet(addr) && addr.GetIpAndPort(ip, port)) {
+        if (ip.GetVersion() == 6) {
+            sin6.sin6_family = AF_INET6;
+            sin6.sin6_addr = ip;
+            sin6.sin6_port = htons(port);
+        } else {
+            ((struct sockaddr_in*)&sin6)->sin_family = AF_INET;
+            ((struct sockaddr_in*)&sin6)->sin_addr = ip;
+            ((struct sockaddr_in*)&sin6)->sin_port = htons(port);
+        }
 	}
 }
 
@@ -635,12 +741,71 @@ bool IsLoopback(const PIPSocket::Address & addr)
 	return addr.IsLoopback() != 0;
 }
 
+bool IsPrivate(const PIPSocket::Address & ip)
+{
+    bool result = false;
+    if (ip.IsRFC1918())
+        result = true;
+    NetworkAddress sharedNetworkSpace("100.64.0.0/10"); // RFC 6598 shared network space, carrier grade NAT
+    if (IsInNetwork(ip, sharedNetworkSpace))
+        result = true;
+    NetworkAddress zeroconf("169.254.0.0/16"); // RFC 3927 Zeroconf
+    if (IsInNetwork(ip, zeroconf))
+        result = true;
+#ifdef hasIPV6
+    if (ip.GetVersion() == 6) {
+        if (ip.IsLinkLocal())
+            result = true;
+        // check for 'site local' (fec0::/10, outdated); IsSiteLocal() is not available in PTLib 2.10.x
+        NetworkAddress siteLocal("fec0::/10");
+        if (IsInNetwork(ip, siteLocal))
+            result = true;
+        // check for 'unique local' (RFC 4193 - fc00::/7)
+        NetworkAddress uniqueLocal("fc00::/7");
+        if (IsInNetwork(ip, uniqueLocal))
+            result = true;
+    }
+#endif
+    return result;
+}
+
+void SetInvalid(PIPSocket::Address & ip)
+{
+    ip = INADDR_ANY; // detected as !IsValid()
+}
+
+// is this IP part of this network
+bool IsInNetwork(const PIPSocket::Address & ip, const NetworkAddress & net)
+{
+    return (ip << net);
+}
+
+// is this IP part of this network list
+bool IsInNetworks(const PIPSocket::Address & ip, const list<NetworkAddress> & nets)
+{
+    for (list<NetworkAddress>::const_iterator i = nets.begin(); i != nets.end(); ++i) {
+        if (IsInNetwork(ip, *i))
+            return true;
+    }
+    return false;
+}
+
+bool IsSet(const H225_TransportAddress & addr)
+{
+	return (addr != H225_TransportAddress());   // TODO: use IsValid() ?
+}
+
 bool IsSet(const H323TransportAddress & addr)
 {
 	return (addr != H323TransportAddress());
 }
 
-bool IsValidE164(const PString & s) 
+bool IsSet(const IPAndPortAddress & addr)
+{
+	return addr.IsSet();
+}
+
+bool IsValidE164(const PString & s)
 {
 	return (!s.IsEmpty() && strspn(s, "1234567890*#+,") == strlen(s));
 }
@@ -653,11 +818,11 @@ PString GetGUIDString(
 {
 	if (id.GetSize() < 16)
 		return "Invalid";
-		
+
 	PString idstr;
-					
+
 	for (int j = 0, i = 0; j < 4; j++) {
-		const unsigned hex = ((unsigned)(id[i])<<24) | ((unsigned)(id[i+1])<<16) 
+		const unsigned hex = ((unsigned)(id[i])<<24) | ((unsigned)(id[i+1])<<16)
 			| ((unsigned)(id[i+2])<<8) | ((unsigned)(id[i+3]));
 		i += 4;
 
@@ -682,6 +847,21 @@ H225_CallIdentifier StringToCallId(PString CallId)
 	return result;
 }
 
+/** compare two OIDs in string format numerically
+ */
+int OIDCmp(const PString & oid1, const PString & oid2)
+{
+	if (oid1 == oid2)
+		return 0;
+
+	PASN_ObjectId o1;
+	o1.SetValue(oid1);
+	PASN_ObjectId o2;
+	o2.SetValue(oid2);
+
+	return (o1 < o2 ? -1 : +1);
+}
+
 PINDEX GetBestAliasAddressIndex(
 	const H225_ArrayOf_AliasAddress & aliases, /// aliases to be searched
 	bool exactMatch, /// search only specified tags or find any alias
@@ -728,7 +908,7 @@ PINDEX FindAlias(
 	for (PINDEX i = 0; i < sz; i++)
 		if (alias == AsString(aliases[i], FALSE))
 			return i;
-			
+
 	return P_MAX_INDEX;
 }
 
@@ -741,10 +921,10 @@ int MatchPrefix(
 		return 0;
 
 	const bool negative = (prefix[0] == '!');
-	
+
 	int i = 0;
 	int j = (negative ? 1 : 0);
-	
+
 	while (prefix[j] != 0) {
 		const char c = prefix[j];
 		if (alias[i] == 0 || (c != '.' && c != '%' && c != alias[i]))
@@ -752,7 +932,7 @@ int MatchPrefix(
 		i++;
 		j++;
 	}
-	
+
 	return negative ? -j + 1 : j;
 }
 
@@ -766,7 +946,7 @@ PString RewriteString(
 {
 	if (prefix == NULL || value == NULL)
 		return s;
-	
+
 	PString result = value + s.Mid(strlen(prefix));
 
 	if (postdialmatch)
@@ -780,8 +960,8 @@ PString RewriteString(
 			PTRACE(1, "GK\tInvalid rewrite rule (dots do not match) - " << prefix << " = " << value);
 			break;
 		}
-		int dotDstOffset = (long)lastDstDot - (long)value;
-		int dotSrcOffset = (long)lastSrcDot - (long)prefix;
+		int dotDstOffset = long(lastDstDot - value);
+		int dotSrcOffset = long(lastSrcDot - prefix);
 		while (*lastDstDot++ == '.' && *lastSrcDot++ == '.') {
 			if (postdialmatch) {
 				postdialdigits += s[dotSrcOffset++];
@@ -789,7 +969,7 @@ PString RewriteString(
 				result[dotDstOffset++] = s[dotSrcOffset++];
 			}
 		}
-				
+
 		lastDstDot = strchr(lastDstDot, '.');
 	}
 
@@ -805,7 +985,7 @@ PString RewriteWildcard(const PString & s, const PString & expression)
 
 	if (o.IsEmpty())
 		return s;
-	
+
 	int b1 = o.Find('{');
 	if (b1 == P_MAX_INDEX) return o; /// No rewrite
 
@@ -822,7 +1002,7 @@ PString RewriteWildcard(const PString & s, const PString & expression)
 	   int start = -1;
        if (exp.Find('^') != P_MAX_INDEX)  start = 1;
 	   else if (exp.Find('$') != P_MAX_INDEX) start = 0;
-	  
+
 	   if (start < 0) return s;   /// Logic error ignore rewrite
 
 	   int c1 = exp.Find('(');
@@ -845,7 +1025,6 @@ PString ReplaceParameters(const PString & queryStr, const std::map<PString, PStr
 	PString finalQuery = queryStr;
 	PINDEX pos = 0;
 	std::map<PString, PString>::const_iterator it;
-	PString var;
 
 	while (pos != P_MAX_INDEX) {
 		pos = finalQuery.Find('%', pos);
@@ -894,7 +1073,7 @@ void RemoveH460Descriptor(unsigned feat, H225_ArrayOf_FeatureDescriptor & featur
                     features[j-1] = features[j];
                 features.SetSize(features.GetSize() - 1);
                 return;
-            }   
+            }
         }
     }
 }
@@ -908,3 +1087,63 @@ void AddH460Feature(H225_ArrayOf_FeatureDescriptor & desc, const H460_Feature &
 }
 #endif
 
+unsigned ProtocolVersion(const char * protocolIdentifier)
+{
+    PString protocolId(protocolIdentifier);
+    PINDEX lastDot = protocolId.FindLast('.');
+    if (lastDot == P_MAX_INDEX)
+        return 0;
+    return protocolId.Mid(lastDot + 1).AsUnsigned();
+}
+
+// replace a regular expression in a string
+void ReplaceRegEx(PString & str, const PRegularExpression & re, const PString & subs, PBoolean all, PINDEX offset)
+{
+    PINDEX pos, len;
+    while (str.FindRegEx(re, pos, len, offset)) {
+        str.Delete(pos, len);
+        str.Splice(subs, pos);
+        if (!all)
+            break;
+        offset = pos + 1;
+    }
+}
+
+bool GetUUIE(const Q931 & q931, H225_H323_UserInformation & uuie)
+{
+	if (q931.HasIE(Q931::UserUserIE)) {
+		PPER_Stream strm(q931.GetIE(Q931::UserUserIE));
+		if (uuie.Decode(strm))
+			return true;
+	}
+	return false;
+}
+
+void SetUUIE(Q931 & q931, const H225_H323_UserInformation & uuie)
+{
+	PPER_Stream strm;
+	uuie.Encode(strm);
+	strm.CompleteEncoding();
+	q931.SetIE(Q931::UserUserIE, strm);
+}
+
+// get the name for Q.931 message types
+PString Q931MessageName(unsigned messageType)
+{
+    switch (messageType) {
+        case Q931::AlertingMsg: return "Alerting";
+        case Q931::CallProceedingMsg: return "CallProceeding";
+        case Q931::ConnectMsg: return "Connect";
+        case Q931::ProgressMsg: return "Progress";
+        case Q931::SetupMsg: return "Setup";
+        case Q931::SetupAckMsg: return "SetupAck";
+        case Q931::ReleaseCompleteMsg: return "ReleaseComplete";
+        case Q931::InformationMsg: return "Information";
+        case Q931::NotifyMsg: return "Notify";
+        case Q931::StatusMsg: return "Status";
+        case Q931::StatusEnquiryMsg: return "StatusEnquiry";
+        case Q931::FacilityMsg: return "Facility";
+        default: return "unknown";
+    }
+}
+
