@@ -3,7 +3,7 @@
  *
  * unit tests for h323util.cxx
  *
- * Copyright (c) 2011-2013, Jan Willamowius
+ * Copyright (c) 2011-2024, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -14,6 +14,8 @@
 
 #include "config.h"
 #include "h323util.h"
+#include "gk_const.h"
+#include "Toolkit.h"
 #include <h323pdu.h>
 #include "gtest/gtest.h"
 
@@ -82,7 +84,8 @@ TEST_F(H323UtilTest, H245Port) {
 }
 
 TEST_F(H323UtilTest, H225TransportAddressAsString) {
-	EXPECT_TRUE(AsString(h225transport_withipv4).Find("03 04 05 06") != P_MAX_INDEX);
+	EXPECT_STREQ("3.4.5.6:999", AsString(h225transport_withipv4));
+	//EXPECT_TRUE(AsString(h225transport_withipv4).Find("03 04 05 06") != P_MAX_INDEX); // behaviour changed!
 	EXPECT_STREQ("3.4.5.6:999", AsDotString(h225transport_withipv4));
 	EXPECT_STREQ("3.4.5.6:999", AsDotString(h225transport_withipv4));
 	EXPECT_STREQ("3.4.5.6:999", AsDotString(h225transport_withipv4, true));
@@ -283,6 +286,78 @@ TEST_F(H323UtilTest, IsLoopback) {
 	EXPECT_TRUE(IsLoopback(ip));
 }
 
+TEST_F(H323UtilTest, IsLocal) {
+	PIPSocket::Address ip4_1("192.168.1.12");// private
+	PIPSocket::Address ip4_2("193.0.0.1");   // public
+	PIPSocket::Address ip4_3("100.64.0.1");  // Shared Network Address
+	PIPSocket::Address ip4_4("169.254.0.1"); // Zeroconf
+
+	PIPSocket::Address ip6_1("fe80::1");    // link local
+	PIPSocket::Address ip6_2("fec0::1");    // site local
+	PIPSocket::Address ip6_2_n("ff00::1");  // NOT site local
+	PIPSocket::Address ip6_3("fc00::1");    // unique local
+	PIPSocket::Address ip6_3_n("ff00::1");  // NOT unique local
+	PIPSocket::Address ip6_4("fd9e:21a7:a92c:2323::1");    // unique local
+	PIPSocket::Address ip6_5("fb9e:21a7:a92c:2323::1");    // public
+
+	EXPECT_TRUE(IsPrivate(ip4_1));
+	EXPECT_FALSE(IsPrivate(ip4_2));
+	EXPECT_TRUE(IsPrivate(ip4_3));
+	EXPECT_TRUE(IsPrivate(ip4_4));
+
+	EXPECT_TRUE(IsPrivate(ip6_1));
+	EXPECT_TRUE(IsPrivate(ip6_2));
+	EXPECT_FALSE(IsPrivate(ip6_2_n));
+	EXPECT_TRUE(IsPrivate(ip6_3));
+	EXPECT_FALSE(IsPrivate(ip6_3_n));
+	EXPECT_TRUE(IsPrivate(ip6_4));
+	EXPECT_FALSE(IsPrivate(ip6_5));
+}
+
+TEST_F(H323UtilTest, IsInNetwork) {
+	PIPSocket::Address ip1("4.5.6.7");
+	PIPSocket::Address ip2("4.5.7.1");
+	PIPSocket::Address ip3("5.6.7.1");
+	PIPSocket::Address ip4("192.168.86.48");
+	PIPSocket::Address ip5("172.21.221.144");
+    NetworkAddress net("4.5.6.0/24");
+    NetworkAddress net2("5.6.7.0/24");
+    NetworkAddress net3("172.21.0.0/16");
+    list<NetworkAddress> net_list;
+    net_list.push_back(net);
+    net_list.push_back(net2);
+    list<NetworkAddress> net_list2;
+    net_list2.push_back(net3);
+	EXPECT_TRUE(IsInNetwork(ip1, net));
+	EXPECT_FALSE(IsInNetwork(ip2, net));
+	EXPECT_TRUE(IsInNetwork(ip5, net3));
+	EXPECT_TRUE(IsInNetworks(ip1, net_list));
+	EXPECT_FALSE(IsInNetworks(ip2, net_list));
+	EXPECT_TRUE(IsInNetworks(ip3, net_list));
+	EXPECT_FALSE(IsInNetworks(ip4, net_list2));
+	EXPECT_TRUE(IsInNetworks(ip5, net_list2));
+}
+
+TEST_F(H323UtilTest, IsSetH225) {
+	PIPSocket::Address ip("4.5.6.7");
+	WORD port = 123;
+    H323TransportAddress h225addr;
+	EXPECT_FALSE(IsSet(h225addr));
+	h225addr = SocketToH225TransportAddr(ip, port);
+	EXPECT_TRUE(IsSet(h225addr));
+	h225addr = (DWORD)0;
+	EXPECT_FALSE(IsSet(h225addr));
+}
+
+TEST_F(H323UtilTest, IsSetH323) {
+	PIPSocket::Address ip("4.5.6.7");
+	WORD port = 123;
+    H323TransportAddress h323addr;
+	EXPECT_FALSE(IsSet(h323addr));
+	h323addr = H323TransportAddress(ip, port);
+	EXPECT_TRUE(IsSet(h323addr));
+}
+
 TEST_F(H323UtilTest, SplitIPAndPort) {
 	PStringArray parts;
 	parts = SplitIPAndPort("1.2.3.4", 1234);
@@ -299,6 +374,15 @@ TEST_F(H323UtilTest, SplitIPAndPort) {
 	EXPECT_STREQ(parts[1], "1234");
 }
 
+TEST_F(H323UtilTest, GetIP) {
+    PString str1 = "1.2.3.4";
+    PString str2 = "4.5.6.7";
+	PIPSocket::Address ip1("1.2.3.4");
+	EXPECT_TRUE(GetIP(str1) == ip1);
+	EXPECT_TRUE(AsString(GetIP(str2)) == "4.5.6.7");
+	EXPECT_FALSE(AsString(GetIP(str1)) == "4.5.6.7");
+}
+
 TEST_F(H323UtilTest, GetGUIDString) {
 	H225_GloballyUniqueID id;
 	EXPECT_STREQ("00000000 00000000 00000000 00000000", GetGUIDString(id, true));
@@ -353,5 +437,42 @@ TEST_F(H323UtilTest, RewriteWildcard) {
 	EXPECT_STREQ("5678@mydomain.com", RewriteWildcard("12345678", "{\\d(4)$}@mydomain.com"));
 }
 
+TEST_F(H323UtilTest, ProtocolVersion) {
+	EXPECT_EQ(0u, ProtocolVersion("invalid"));
+	EXPECT_EQ(2u, ProtocolVersion(H225_ProtocolIDv2));
+	EXPECT_EQ(4u, ProtocolVersion(H225_ProtocolIDv4));
+	EXPECT_EQ(6u, ProtocolVersion(H225_ProtocolIDv6));
+	EXPECT_EQ(7u, ProtocolVersion("0.0.8.2250.0.7"));
+	EXPECT_EQ(3u, ProtocolVersion(H245_ProtocolIDv3));
+}
+
+TEST_F(H323UtilTest, IPAndPortAddress) {
+	PIPSocket::Address ip1("1.2.3.4");
+	PIPSocket::Address ip2("4.5.6.7");
+	WORD port = 123;
+    IPAndPortAddress addr1;
+    IPAndPortAddress addr2(ip2, port);
+
+    EXPECT_FALSE(IsSet(addr1));
+    EXPECT_FALSE(addr1.IsSet());
+    EXPECT_TRUE(IsSet(addr2));
+    EXPECT_TRUE(addr2.IsSet());
+
+    EXPECT_FALSE(addr1 == addr2);
+    addr1.Set(ip1, port);
+    EXPECT_FALSE(addr1 == addr2);
+    EXPECT_TRUE(addr1 != addr2);
+    addr1.Set(ip2, port);
+    EXPECT_TRUE(addr1 == addr2);
+    EXPECT_FALSE(addr1 != addr2);
+}
+
+TEST_F(H323UtilTest, OIDCompare) {
+	PString oid1 = "1.2.9.4";
+	PString oid2 = "1.2.10.4";
+    EXPECT_TRUE(OIDCmp(oid1, oid1) == 0);
+    EXPECT_TRUE(OIDCmp(oid1, oid2) < 0);
+    EXPECT_TRUE(OIDCmp(oid2, oid1) > 0);
+}
 
 }  // namespace
