@@ -3,7 +3,7 @@
 // Presence in H.323 gatekeeper
 //
 // Copyright (c) 2009-2012, Simon Horne
-// Copyright (c) 2009-2012, Jan Willamowius
+// Copyright (c) 2009-2020, Jan Willamowius
 //
 // This work is published under the GNU Public License (GPL)
 // see file COPYING for details.
@@ -65,7 +65,7 @@ private:
 };
 
 PresWorker::PresWorker(GkPresence * _handler , int _waitTime)
-: PThread(5000, AutoDeleteThread), handler(_handler), waitTime(_waitTime), processNow(false), shutDown(false)
+: PThread(5000, AutoDeleteThread, NormalPriority, "PresenceWorker"), handler(_handler), waitTime(_waitTime), processNow(false), shutDown(false)
 {
 	PTRACE(4, "PRES\tPresence Thread instance fire every " << waitTime << " sec");
 	Resume();
@@ -76,7 +76,7 @@ PresWorker::~PresWorker()
 	if (!shutDown)
 		Stop();
 }
-	
+
 void PresWorker::Main()
 {
 	int wint=100;
@@ -156,7 +156,7 @@ void PresWorker::ProcessMessages()
 			endptr ep = RegistrationTable::Instance()->FindByEndpointId(*i);
 			if (ep) {
 #ifdef HAS_H460P_VER_3
-				bool dataToSend=false;
+				bool dataToSend = false;
 				list<PASN_OctetString> element;
 				if (ep->HasPresenceData())
 					dataToSend = handler->BuildPresenceElement(H225_RasMessage::e_registrationRequest, *i, element);
@@ -166,12 +166,12 @@ void PresWorker::ProcessMessages()
 				if (dataToSend) {
 					PASN_OctetString data;
 					H225_RasMessage sci_ras;
-					while (element.size() > 0) {
+					while (!element.empty()) {
 						data = element.front();
 						element.pop_front();
 						BuildSCI(sci_ras,data);
 						RasServer::Instance()->SendRas(sci_ras, ep->GetRasAddress());
-						if (element.size() > 0)
+						if (!element.empty())
 							PThread::Sleep(10);
 					}
 				}
@@ -179,7 +179,7 @@ void PresWorker::ProcessMessages()
 				PASN_OctetString element;
 				if (handler->BuildPresenceElement(H225_RasMessage::e_serviceControlIndication, *i, element)) {
 					H225_RasMessage sci_ras;
-					BuildSCI(sci_ras,element);
+					BuildSCI(sci_ras, element);
 					RasServer::Instance()->SendRas(sci_ras, ep->GetRasAddress());
 				}
 #endif
@@ -196,10 +196,10 @@ void PresWorker::ProcessMessages()
 		while (i != gkip.end()) {
 #ifdef HAS_H460P_VER_3
 			list<PASN_OctetString> element;
-			if (handler->BuildPresenceElement(H225_RasMessage::e_locationRequest,*i, element)) {
+			if (handler->BuildPresenceElement(H225_RasMessage::e_locationRequest, *i, element)) {
 				PASN_OctetString data;
 				H225_RasMessage lrq_ras;
-				while (element.size() > 0) {
+				while (!element.empty()) {
 					data = element.front();
 					element.pop_front();
 					BuildLRQ(lrq_ras,data);
@@ -209,9 +209,9 @@ void PresWorker::ProcessMessages()
 			}
 #else
 			PASN_OctetString element;
-			if (handler->BuildPresenceElement(H225_RasMessage::e_locationRequest,*i, element)) {
+			if (handler->BuildPresenceElement(H225_RasMessage::e_locationRequest, *i, element)) {
 				H225_RasMessage lrq_ras;
-				BuildLRQ(lrq_ras,element);
+				BuildLRQ(lrq_ras, element);
 				RasServer::Instance()->SendRas(lrq_ras, *i);
 			}
 #endif
@@ -234,11 +234,11 @@ H460P_PresenceInstruction & BuildInstructionMsg(const H460P_PresenceInstruction
 	return msg;
 }
 
-H460P_PresenceInstruction & BuildInstructionMsg(unsigned type, const H225_AliasAddress & addr, H460P_PresencePDU & msg, 
+H460P_PresenceInstruction & BuildInstructionMsg(unsigned type, const H225_AliasAddress & addr, H460P_PresencePDU & msg,
 												const PString & display = PString(), const PString & avatar = PString())
 {
-	H323PresenceInstruction instruct((H323PresenceInstruction::Instruction)type, AsString(addr,0), display, avatar);
-	return BuildInstructionMsg(instruct,msg);
+	H323PresenceInstruction instruct((H323PresenceInstruction::Instruction)type, AsString(addr, 0), display, avatar);
+	return BuildInstructionMsg(instruct, msg);
 }
 
 H460P_PresenceSubscription & BuildSubscriptionMsg(const H460P_PresenceSubscription & subscription, H460P_PresencePDU & msg)
@@ -265,8 +265,8 @@ H460P_PresenceSubscription & BuildSubscriptionMsg(const OpalGloballyUniqueID & i
 {
 	H323PresenceSubscription sub;
 	sub.SetSubscription(id);
-	sub.SetSubscriptionDetails(pid.m_subscriber,pid.m_Alias,pid.m_Display,pid.m_Avatar,pid.m_Category);
-	return BuildSubscriptionMsg(sub,msg);
+	sub.SetSubscriptionDetails(pid.m_subscriber, pid.m_Alias, pid.m_Display, pid.m_Avatar, pid.m_Category);
+	return BuildSubscriptionMsg(sub, msg);
 }
 #endif
 
@@ -294,7 +294,7 @@ bool RemoveInstruction(const H225_AliasAddress & addr, H323PresenceInstructions
 {
 	bool found = false;
 	int sz = list.GetSize();
-	for (PINDEX i=0; i<sz;  ++i) {
+	for (PINDEX i = 0; i < sz;  ++i) {
 		if (addr == list[i]) {
 			found = true;
 			continue;
@@ -312,14 +312,14 @@ bool RemoveInstruction(const H225_AliasAddress & addr, H323PresenceInstructions
 bool UpdateInstruction(const H460P_PresenceInstruction & addr, H323PresenceInstructions & inst)
 {
 	bool found = false;
-	for (PINDEX i=0; i< inst.GetSize(); ++i) {
+	for (PINDEX i = 0; i < inst.GetSize(); ++i) {
 #ifdef HAS_H460P_VER_1
 		const H225_AliasAddress & alias = addr;
 #else
 		const H460P_PresenceAlias & palias = addr;
 		const H225_AliasAddress & alias = palias.m_alias;
 #endif
-		if (AsString(alias,0) == inst[i].GetAlias()) {
+		if (AsString(alias, false) == inst[i].GetAlias()) {
 			found = true;
 			inst[i] = (const H323PresenceInstruction &)addr;
 		}
@@ -424,7 +424,6 @@ GkPresence::GkPresence()
 
 GkPresence::~GkPresence()
 {
-
 }
 
 void GkPresence::Stop()
@@ -457,7 +456,7 @@ bool GkPresence::IsEnabled() const
 
 void GkPresence::LoadConfig(PConfig * cfg)
 {
-	m_enabled = cfg->GetBoolean("RoutedMode", "EnableH460P", 0);
+	m_enabled = cfg->GetBoolean("RoutedMode", "EnableH460P", false);
 	if (!m_enabled)
 		return;
 
@@ -470,61 +469,52 @@ void GkPresence::LoadConfig(PConfig * cfg)
 
 	const PString driverName = cfg->GetString(authName, "Driver", "");
 	if (driverName.IsEmpty()) {
-		PTRACE(0, "H460PSQL\tModule creation failed: "
-			"no SQL driver selected"
-			);
-		PTRACE(0, "H460PSQL\tFATAL: Shutting down");
+		PTRACE(1, "H460PSQL\tModule creation failed: no SQL driver selected");
+		PTRACE(1, "H460PSQL\tFATAL: Shutting down");
 		return;
 	}
-	
+
 	m_sqlConn = GkSQLConnection::Create(driverName, "GkPresence");
 	if (m_sqlConn == NULL) {
-		PTRACE(0, "H460PSQL\tModule creation failed: "
-			"Could not find " << driverName << " database driver"
-			);
-		PTRACE(0, "H460PSQL\tFATAL: Shutting down");
+		PTRACE(1, "H460PSQL\tModule creation failed: Could not find " << driverName << " database driver");
+		PTRACE(1, "H460PSQL\tFATAL: Shutting down");
 		return;
 	}
-		
+
 	m_queryList = cfg->GetString(authName, "QueryList", "");
 	if (m_queryList.IsEmpty()) {
-		PTRACE(0, "H460PSQL\tModule creation failed: No QueryList configured"
-			);
-		PTRACE(0, "H460PSQL\tFATAL: Shutting down");
+		PTRACE(1, "H460PSQL\tModule creation failed: No QueryList configured");
+		PTRACE(1, "H460PSQL\tFATAL: Shutting down");
 		return;
 	} else
 		PTRACE(4, "H460PSQL\tQuerylist: " << m_queryList);
 
 	m_queryAdd = cfg->GetString(authName, "QueryAdd", "");
 	if (m_queryAdd.IsEmpty()) {
-		PTRACE(0, "H460PSQL\tModule creation failed: No QueryAdd configured"
-			);
-		PTRACE(0, "H460PSQL\tFATAL: Shutting down");
+		PTRACE(1, "H460PSQL\tModule creation failed: No QueryAdd configured");
+		PTRACE(1, "H460PSQL\tFATAL: Shutting down");
 		return;
 	} else
 		PTRACE(4, "H460PSQL\tQueryAdd: " << m_queryAdd);
 
 	m_queryDelete = cfg->GetString(authName, "QueryDelete", "");
 	if (m_queryDelete.IsEmpty()) {
-		PTRACE(0, "H460PSQL\tModule creation failed: No QueryDelete configured"
-			);
-		PTRACE(0, "H460PSQL\tFATAL: Shutting down");
+		PTRACE(1, "H460PSQL\tModule creation failed: No QueryDelete configured");
+		PTRACE(1, "H460PSQL\tFATAL: Shutting down");
 		return;
 	} else
 		PTRACE(4, "H460PSQL\tQueryDelete: " << m_queryDelete);
 
 	m_queryUpdate = cfg->GetString(authName, "QueryUpdate", "");
 	if (m_queryUpdate.IsEmpty()) {
-		PTRACE(0, "H460PSQL\tModule creation failed: No QueryUpdate configured"
-			);
-		PTRACE(0, "H460PSQL\tFATAL: Shutting down");
+		PTRACE(1, "H460PSQL\tModule creation failed: No QueryUpdate configured");
+		PTRACE(1, "H460PSQL\tFATAL: Shutting down");
 		return;
 	} else
 		PTRACE(4, "H460PSQL\tQueryUpdate: " << m_queryUpdate);
 
 	if (!m_sqlConn->Initialize(cfg, authName)) {
-		PTRACE(0, "H460PSQL\tModule creation failed: Could not connect to the database"
-			);
+		PTRACE(1, "H460PSQL\tModule creation failed: Could not connect to the database");
 		return;
 	}
 
@@ -586,9 +576,9 @@ void UpdateLocalPresence(H460P_PresenceNotification & local, const H460P_Presenc
 	if (r.HasOptionalField(H460P_Presentity::e_genericData)) {
 		l.IncludeOptionalField(H460P_Presentity::e_genericData);
 		bool found;
-		for (PINDEX i=0; i < r.m_genericData.GetSize(); ++i) {
+		for (PINDEX i = 0; i < r.m_genericData.GetSize(); ++i) {
 		   found = false;
-			for (PINDEX j=0; i < l.m_genericData.GetSize(); ++j) {
+			for (PINDEX j = 0; i < l.m_genericData.GetSize(); ++j) {
 				if (r.m_genericData[i].m_id == l.m_genericData[j].m_id) {
 					l.m_genericData[j] = r.m_genericData[i];
 					found = true;
@@ -597,7 +587,7 @@ void UpdateLocalPresence(H460P_PresenceNotification & local, const H460P_Presenc
 			}
 			if (!found) {
 				int sz = l.m_genericData.GetSize();
-				l.m_genericData.SetSize(sz+1);
+				l.m_genericData.SetSize(sz + 1);
 				l.m_genericData[sz] = r.m_genericData[i];
 			}
 		}
@@ -613,15 +603,14 @@ bool GkPresence::DatabaseLoad(PBoolean incremental)
 	GkSQLResult* result = m_sqlConn->ExecuteQuery(m_queryList,params, m_timeout);
 	if (result == NULL) {
 		PTRACE(2, "H460PSQL\tSubList failed - timeout or fatal error");
-		SNMP_TRAP(5, SNMPError, Database, "Presense SQL query failed");
+		SNMP_TRAP(5, SNMPError, Database, "Presence SQL query failed");
 		return false;
 	}
 
 	int fieldCount = 7;
 	if (result->GetNumRows() > 0 && result->GetNumFields() < fieldCount) {
-		PTRACE(2, "H460PSQL\tBad-formed query - "
-			"insufficient columns found in the result set expect " << fieldCount);
-		SNMP_TRAP(5, SNMPError, Database, "Presense SQL query failed");
+		PTRACE(2, "H460PSQL\tBad-formed query - insufficient columns found in the result set expect " << fieldCount);
+		SNMP_TRAP(5, SNMPError, Database, "Presence SQL query failed");
 		delete result;
 		return false;
 	}
@@ -644,20 +633,20 @@ bool GkPresence::DatabaseLoad(PBoolean incremental)
 	}
 
 #if PTRACING
-	PINDEX j=0;
+	PINDEX j = 0;
 #endif
 	PStringArray retval;
 	while (result->FetchRow(retval)) {
 		if (retval[0].IsEmpty()) {
 			PTRACE(1, "H460PSQL\tQuery Invalid value found.");
-			SNMP_TRAP(5, SNMPError, Database, "Presense SQL query failed");
+			SNMP_TRAP(5, SNMPError, Database, "Presence SQL query failed");
 			continue;
 		}
 
 #if PTRACING
 		PStringStream results;
-		for (PINDEX i=0; i < retval.GetSize(); ++i) {
-			results << " " << presenceFieldName(i) << " "; 
+		for (PINDEX i = 0; i < retval.GetSize(); ++i) {
+			results << " " << presenceFieldName(i) << " ";
 			if (i == 4) results << H323PresenceInstruction::GetInstructionString(retval[i].AsInteger());
 #ifndef HAS_H460P_VER_1
 			else if (i == 9) results << H323PresenceInstruction::GetCategoryString(retval[i].AsInteger());
@@ -685,7 +674,7 @@ bool GkPresence::DatabaseLoad(PBoolean incremental)
 			if (retval.GetSize() >= 10) id.m_Category = (H323PresenceInstruction::Category)retval[9].AsInteger();
 								else id.m_Category = H323PresenceInstruction::e_UnknownCategory;
 #endif
-		
+
 #ifdef HAS_H460P_VER_1
 			H323PresenceInstruction instruct(id.m_Status, retval[2]);
 #else
@@ -695,7 +684,7 @@ bool GkPresence::DatabaseLoad(PBoolean incremental)
 		if (id.m_Active || (id.m_isSubscriber && (id.m_Status == H323PresenceInstruction::e_pending))) {
 			H323PresenceStore::iterator itx = localStore.find(id.m_subscriber);
 			if (itx != localStore.end()) {
-				if (id.m_subscriber != id.m_Alias) 
+				if (id.m_subscriber != id.m_Alias)
 					itx->second.m_Instruction.Add(instruct);
 			} else {
 				H323PresenceEndpoint store;
@@ -704,10 +693,10 @@ bool GkPresence::DatabaseLoad(PBoolean incremental)
 						H460P_PresenceNotification n;
 						n.m_presentity.m_state = H460P_PresenceState::e_offline;
 						store.m_Notify.SetSize(1);
-						UpdateLocalPresence(store.m_Notify[0],n);
+						UpdateLocalPresence(store.m_Notify[0], n);
 						// Load Instructions
 						store.m_Instruction.Add(instruct);
-					localStore.insert(pair<H225_AliasAddress,H323PresenceEndpoint>(id.m_subscriber,store));
+					localStore.insert(pair<H225_AliasAddress, H323PresenceEndpoint>(id.m_subscriber, store));
 				}
 			}
 		}
@@ -715,15 +704,15 @@ bool GkPresence::DatabaseLoad(PBoolean incremental)
 		if (id.m_subscriber == id.m_Alias)
 			continue;
 
-		if (incremental && IsLocalAvailable(id.m_subscriber,aliasList)) {
+		if (incremental && IsLocalAvailable(id.m_subscriber, aliasList)) {
 			// Queue up any new messages
 			H460P_PresencePDU msg;
-			BuildInstructionMsg(instruct,msg);
-			EnQueuePresence(id.m_subscriber,msg);
+			BuildInstructionMsg(instruct, msg);
+			EnQueuePresence(id.m_subscriber, msg);
 		}
 
 		// Load the identifier store
-		AddPresenceMap(pid,id,remoteIds,remoteIdmap);
+		AddPresenceMap(pid, id, remoteIds, remoteIdmap);
 	}
 	delete result;
 
@@ -750,8 +739,8 @@ bool GkPresence::DatabaseAdd(const PString & identifier, const H323PresenceID &
 	GkSQLResult::ResultRow resultRow;
 	std::map<PString, PString> params;
 	params["i"] = identifier;
-	params["u"] = AsString(id.m_subscriber,0);
-	params["a"] = AsString(id.m_Alias,0);
+	params["u"] = AsString(id.m_subscriber, 0);
+	params["a"] = AsString(id.m_Alias, 0);
 	params["s"] = (id.m_isSubscriber ? "1" : "0");
 #ifndef HAS_H460P_VER_1
 	params["d"] = id.m_Display;
@@ -764,7 +753,7 @@ bool GkPresence::DatabaseAdd(const PString & identifier, const H323PresenceID &
 	return false;
 #endif
 }
-	
+
 bool GkPresence::DatabaseDelete(const PString & identifier)
 {
 #ifdef HAS_DATABASE
@@ -791,7 +780,7 @@ bool GkPresence::DatabaseUpdate(unsigned tag, const PString & identifier)
 	std::map<PString, PString> params;
 	params["i"] = identifier;
 	params["b"] = tag;
-	
+
 	return (m_sqlConn->ExecuteQuery(m_queryUpdate, params, m_timeout) != NULL);
 #else
 	return false;
@@ -807,13 +796,14 @@ bool GkPresence::HandleNewAlias(const H225_AliasAddress & addr)
 		id.m_Alias = addr;
 		id.m_Active = true;
 		id.m_isSubscriber = true;
+		id.m_Category = H323PresenceInstruction::e_UnknownCategory;
 
-	DatabaseAdd(uid.AsString().ToUpper(),id);
+	DatabaseAdd(uid.AsString().ToUpper(), id);
     DatabaseUpdate(H323PresenceInstruction::e_subscribe,uid.AsString());
 
 	// load the cache with empty store
 	H323PresenceEndpoint store;
-	localStore.insert(pair<H225_AliasAddress,H323PresenceEndpoint>(addr,store));
+	localStore.insert(pair<H225_AliasAddress,H323PresenceEndpoint>(addr, store));
 	return true;
 }
 
@@ -824,9 +814,9 @@ H460P_PresenceSubscription & GkPresence::HandleSubscription(bool isNew, const H4
 
 	H460P_PresencePDU msg;
 #ifdef HAS_H460P_VER_1
-	H460P_PresenceSubscription & sub = BuildSubscriptionMsg(pid.m_guid,id.m_Alias,id.m_subscriber,msg);
+	H460P_PresenceSubscription & sub = BuildSubscriptionMsg(pid.m_guid, id.m_Alias, id.m_subscriber, msg);
 #else
-	H460P_PresenceSubscription & sub = BuildSubscriptionMsg(pid.m_guid,id,msg);
+	H460P_PresenceSubscription & sub = BuildSubscriptionMsg(pid.m_guid, id, msg);
 #endif
 	EnQueuePresence(id.m_subscriber,msg);
 	return *(H460P_PresenceSubscription *)sub.Clone();
@@ -840,8 +830,8 @@ bool GkPresence::RegisterEndpoint(const H225_EndpointIdentifier & ep, const H225
 		// register the endpoint as being at this endpoint identifier
 		H323PresenceAlias::iterator it = aliasList.find(addr[j]);
 		if (it == aliasList.end())
-			aliasList.insert(pair<H225_AliasAddress,H225_EndpointIdentifier>(addr[j],ep));
-			
+			aliasList.insert(pair<H225_AliasAddress,H225_EndpointIdentifier>(addr[j], ep));
+
 		// handle subscriptions/notifications from local store
 		H323PresenceStore::iterator itx = localStore.find(addr[j]);
 		if (itx == localStore.end()) {
@@ -857,11 +847,11 @@ bool GkPresence::RegisterEndpoint(const H225_EndpointIdentifier & ep, const H225
 		}
 
 		// Queue up any notifications of existing registrations
-		for (PINDEX i=0; i< itx->second.m_Instruction.GetSize(); i++) {
+		for (PINDEX i = 0; i < itx->second.m_Instruction.GetSize(); i++) {
 		   // Queue up info from the datastore to supply to the endpoint
 			H460P_PresencePDU msg;
-			BuildInstructionMsg(itx->second.m_Instruction[i],msg);
-			EnQueuePresence(addr[j],msg);
+			BuildInstructionMsg(itx->second.m_Instruction[i], msg);
+			EnQueuePresence(addr[j], msg);
 
 			// look in the local store to see if we can send presence info to people logged in.
 #ifdef HAS_H460P_VER_1
@@ -881,14 +871,14 @@ bool GkPresence::RegisterEndpoint(const H225_EndpointIdentifier & ep, const H225
 			while (i != plist.end()) {
 				H323PresenceID local;
 				if (GetSubscription(*i,local) && (local.m_subscriber == addr[j]) && !local.m_Active) {
-					H460P_PresenceSubscription & sub = HandleSubscription(false,*i,local);
+					H460P_PresenceSubscription & sub = HandleSubscription(false, *i, local);
 					int sz = itx->second.m_Authorize.GetSize();
-					itx->second.m_Authorize.SetSize(sz+1);
+					itx->second.m_Authorize.SetSize(sz + 1);
 					itx->second.m_Authorize[sz] = (H323PresenceSubscription &)sub;
 				}
 				i++;
 			}
-		}		
+		}
 	}
 	m_worker->ProcessNow();
 	return true;
@@ -903,14 +893,14 @@ void GkPresence::UnRegisterEndpoint(const H225_ArrayOf_AliasAddress & addr)
 	H460P_PresenceNotification & n = msg;
 	n.m_presentity.m_state = H460P_PresenceState::e_offline;
 
-	for (PINDEX i=0; i < addr.GetSize(); i++) {
+	for (PINDEX i = 0; i < addr.GetSize(); i++) {
 		// Send Notification
 		OnNotification(H323PresenceHandler::e_Status, n, addr[i]);
 
 		H323PresenceAlias::iterator it = aliasList.find(addr[i]);
 		if (it != aliasList.end())
 			aliasList.erase(it);
-	}	
+	}
 }
 
 bool GkPresence::GetPendingIdentifiers(list<H225_EndpointIdentifier> & epid)
@@ -980,7 +970,7 @@ void GkPresence::ProcessPresenceElement(const PASN_OctetString & pdu)
 
 	if (!ReceivedPDU(pdu)) {
 		PTRACE(4, "H460P\tError processing PDU");
-		SNMP_TRAP(9, SNMPError, Network, "Error decoding Presense PDU");
+		SNMP_TRAP(9, SNMPError, Network, "Error decoding Presence PDU");
 	}
 }
 
@@ -1022,13 +1012,13 @@ bool GkPresence::EnQueueFullNotification(const H225_AliasAddress & local, const
         return false;
 
 	H460P_PresencePDU msg;
-    if (itm->second.m_Notify.GetSize() == 0) 
+    if (itm->second.m_Notify.GetSize() == 0)
 		return false;
 
-	H460P_PresenceNotification & notification = BuildNotificationMsg(itm->second.m_Notify[0],msg);
+	H460P_PresenceNotification & notification = BuildNotificationMsg(itm->second.m_Notify[0], msg);
 	notification.IncludeOptionalField(H460P_PresenceNotification::e_aliasAddress);
 	notification.m_aliasAddress = local;
-	EnQueuePresence(remote,msg);
+	EnQueuePresence(remote, msg);
 
 	// Send remote status to local
 	H323PresenceStore::iterator itx = localStore.find(remote);
@@ -1042,7 +1032,7 @@ bool GkPresence::EnQueueFullNotification(const H225_AliasAddress & local, const
 	H460P_PresenceNotification & notification2 = BuildNotificationMsg(itx->second.m_Notify[0],msg2);
 	notification2.IncludeOptionalField(H460P_PresenceNotification::e_aliasAddress);
 	notification2.m_aliasAddress = remote;
-	EnQueuePresence(local,msg2);
+	EnQueuePresence(local, msg2);
 	return true;
 }
 
@@ -1060,18 +1050,18 @@ bool GkPresence::EnQueuePresence(const H225_AliasAddress & addr, const H460P_Pre
 			else {
 				list<H460P_PresencePDU> m_Indication;
 				m_Indication.push_back(msg);
-				xlist.insert(pair<H225_AliasAddress,list<H460P_PresencePDU> >(addr,m_Indication));
+				xlist.insert(pair<H225_AliasAddress,list<H460P_PresencePDU> >(addr, m_Indication));
 			}
 		} else {
 			list<H460P_PresencePDU> m_Indication;
 			m_Indication.push_back(msg);
 			H323PresenceInd xlist;
-			xlist.insert(pair<H225_AliasAddress,list<H460P_PresencePDU> >(addr,m_Indication));	
-			pendingStore.insert(pair<H225_EndpointIdentifier,H323PresenceInd>(it->second,xlist));
+			xlist.insert(pair<H225_AliasAddress,list<H460P_PresencePDU> >(addr, m_Indication));
+			pendingStore.insert(pair<H225_EndpointIdentifier,H323PresenceInd>(it->second, xlist));
 		}
 		return true;
 	}
-	
+
 	// check if the alias is registered remotely
 	H323PresenceExternal::iterator rt = remoteList.find(addr);
 	if (rt != remoteList.end()) {
@@ -1084,14 +1074,14 @@ bool GkPresence::EnQueuePresence(const H225_AliasAddress & addr, const H460P_Pre
 			else {
 				list<H460P_PresencePDU> m_Indication;
 				m_Indication.push_back(msg);
-				xlist.insert(pair<H225_AliasAddress,list<H460P_PresencePDU> >(addr,m_Indication));
+				xlist.insert(pair<H225_AliasAddress,list<H460P_PresencePDU> >(addr, m_Indication));
 			}
 		} else {
 			list<H460P_PresencePDU> m_Indication;
 			m_Indication.push_back(msg);
 			H323PresenceInd xlist;
-			xlist.insert(pair<H225_AliasAddress,list<H460P_PresencePDU> >(addr,m_Indication));	
-			remoteStore.insert(pair<H225_TransportAddress,H323PresenceInd>(rt->second,xlist));
+			xlist.insert(pair<H225_AliasAddress,list<H460P_PresencePDU> >(addr, m_Indication));
+			remoteStore.insert(pair<H225_TransportAddress,H323PresenceInd>(rt->second, xlist));
 		}
 		return true;
 	}
@@ -1121,7 +1111,7 @@ PBoolean GkPresence::BuildSubscription(const H225_EndpointIdentifier & ep, H323P
 					if (k == subscription.end())  {
 						H323PresenceEndpoint pe;
 						pe.m_Authorize.Add((const H323PresenceSubscription &)n);
-						subscription.insert(pair<H225_AliasAddress,H323PresenceEndpoint>(i->first,pe));	
+						subscription.insert(pair<H225_AliasAddress,H323PresenceEndpoint>(i->first,pe));
 					} else
 						k->second.m_Authorize.Add((const H323PresenceSubscription &)n);
 
@@ -1169,7 +1159,7 @@ PBoolean GkPresence::BuildNotification(const H225_EndpointIdentifier & ep, H323P
 					if (k == notify.end())  {
 						H323PresenceEndpoint pe;
 						pe.m_Notify.Add((const H323PresenceNotification &)n);
-						notify.insert(pair<H225_AliasAddress,H323PresenceEndpoint>(i->first,pe));	
+						notify.insert(pair<H225_AliasAddress,H323PresenceEndpoint>(i->first, pe));
 					} else
 						k->second.m_Notify.Add((const H323PresenceNotification &)n);
 
@@ -1217,7 +1207,7 @@ PBoolean GkPresence::BuildInstructions(const H225_EndpointIdentifier & ep, H323P
 					if (k == instruction.end())  {
 						H323PresenceEndpoint pe;
 						pe.m_Instruction.Add((const H323PresenceInstruction &)n);
-						instruction.insert(pair<H225_AliasAddress,H323PresenceEndpoint>(i->first,pe));	
+						instruction.insert(pair<H225_AliasAddress,H323PresenceEndpoint>(i->first, pe));
 					} else
 						k->second.m_Instruction.Add((const H323PresenceInstruction &)n);
 
@@ -1265,7 +1255,7 @@ PBoolean GkPresence::BuildSubscription(bool request, const H225_TransportAddress
 						if (k == subscription.end())  {
 							H323PresenceEndpoint pe;
 							pe.m_Authorize.Add((const H323PresenceSubscription &)n);
-							subscription.insert(pair<H225_TransportAddress,H323PresenceEndpoint>(ip,pe));	
+							subscription.insert(pair<H225_TransportAddress,H323PresenceEndpoint>(ip, pe));
 						} else
 							k->second.m_Authorize.Add((const H323PresenceSubscription &)n);
 
@@ -1313,7 +1303,7 @@ PBoolean GkPresence::BuildNotification(const H225_TransportAddress & ip, H323Pre
 					if (k == notify.end())  {
 						H323PresenceEndpoint pe;
 						pe.m_Notify.Add((const H323PresenceNotification &)n);
-						notify.insert(pair<H225_TransportAddress,H323PresenceEndpoint>(ip,pe));	
+						notify.insert(pair<H225_TransportAddress,H323PresenceEndpoint>(ip, pe));
 					} else
 						k->second.m_Notify.Add((const H323PresenceNotification &)n);
 
@@ -1361,7 +1351,7 @@ PBoolean GkPresence::BuildIdentifiers(bool alive, const H225_TransportAddress &
 						if (k == identifiers.end())  {
 							H323PresenceEndpoint pe;
 							pe.m_Identifiers.Add(n.m_guid);
-							identifiers.insert(pair<H225_TransportAddress,H323PresenceEndpoint>(ip,pe));	
+							identifiers.insert(pair<H225_TransportAddress,H323PresenceEndpoint>(ip, pe));
 						} else
 							k->second.m_Identifiers.Add(n.m_guid);
 
@@ -1392,8 +1382,7 @@ PBoolean GkPresence::BuildIdentifiers(bool alive, const H225_TransportAddress &
 
 
 bool GkPresence::HandleStatusUpdates(const H460P_PresenceIdentifier & pid, const H225_AliasAddress & local,
-									 unsigned type, const H225_AliasAddress & remote,
-									 const H323PresenceID * id)
+									 unsigned type, const H225_AliasAddress & remote, const H323PresenceID * id)
 {
 		H460P_PresencePDU msg;
 		H460P_PresenceInstruction & inst = BuildInstructionMsg(type,remote,msg);
@@ -1404,11 +1393,11 @@ bool GkPresence::HandleStatusUpdates(const H460P_PresenceIdentifier & pid, const
 		if (it != localStore.end()) {
 			switch (type) {
 			  case H460P_PresenceInstruction::e_subscribe:
-				  DatabaseUpdate(type,spid);
+				  DatabaseUpdate(type, spid);
 				  break;
 
 			  case H460P_PresenceInstruction::e_unsubscribe:
-				  DatabaseUpdate(type,spid);
+				  DatabaseUpdate(type, spid);
 				  break;
 
 			  case H460P_PresenceInstruction::e_block:
@@ -1416,19 +1405,19 @@ bool GkPresence::HandleStatusUpdates(const H460P_PresenceIdentifier & pid, const
 					DatabaseAdd(spid,*id);
 					DatabaseUpdate(type,spid);
 					AddPresenceMap(pid, *id, remoteIds, remoteIdmap);
-					UpdateInstruction(inst,it->second.m_Instruction);
+					UpdateInstruction(inst, it->second.m_Instruction);
 				  }
 				  break;
 
 			  case H460P_PresenceInstruction::e_unblock:
-				  DatabaseUpdate(type,spid);
+				  DatabaseUpdate(type, spid);
 				  break;
 
 			  case H460P_PresenceInstruction::e_pending:
 				  if (id) {
 					  DatabaseAdd(spid,*id);
 					  AddPresenceMap(pid, *id, remoteIds, remoteIdmap);
-					  UpdateInstruction(inst,it->second.m_Instruction);
+					  UpdateInstruction(inst, it->second.m_Instruction);
 				  }
 				  break;
 			}
@@ -1436,10 +1425,10 @@ bool GkPresence::HandleStatusUpdates(const H460P_PresenceIdentifier & pid, const
 
 		PTRACE(4, "PRES\tChanged Subscription for " << local << " : " << PresMsgType(type) << " " << remote);
 
-		if (!IsLocalAvailable(local,aliasList))
+		if (!IsLocalAvailable(local, aliasList))
 			return true;
 		else
-			return EnQueuePresence(local,msg);
+			return EnQueuePresence(local, msg);
 }
 
 bool GkPresence::HandleForwardPresence(const H460P_PresenceIdentifier & identifier, const H460P_PresencePDU & msg)
@@ -1452,7 +1441,7 @@ bool GkPresence::HandleForwardPresence(const H460P_PresenceIdentifier & identifi
 		H225_AliasAddress a;
 		H323SetAliasAddress(PString("Relay"), a);
 		PTRACE(5, "PRES\tRelaying Identifier " << identifier << " to " << itx->second);
-		xlist.insert(pair<H225_AliasAddress,list<H460P_PresencePDU> >(a, m_Indication));	
+		xlist.insert(pair<H225_AliasAddress,list<H460P_PresencePDU> >(a, m_Indication));
 		remoteStore.insert(pair<H225_TransportAddress,H323PresenceInd>(itx->second, xlist));
 		return true;
 	}
@@ -1473,8 +1462,8 @@ bool GkPresence::HandleSubscriptionLocal(const H460P_PresenceSubscription & subs
 		approved = subscription.m_approved;
 		if (approved) {
 			HandleStatusUpdates(subscription.m_identifier, it->second.m_subscriber, e_subscribe, it->second.m_Alias);
-			if (IsLocalAvailable(it->second.m_subscriber,aliasList))
-				EnQueueFullNotification(it->second.m_subscriber,it->second.m_Alias);
+			if (IsLocalAvailable(it->second.m_subscriber, aliasList))
+				EnQueueFullNotification(it->second.m_subscriber, it->second.m_Alias);
 		} else
 			RemoveSubscription(e_unsubscribe,subscription.m_identifier);
 
@@ -1498,7 +1487,7 @@ bool GkPresence::HandleSubscriptionLocal(const H460P_PresenceSubscription & subs
 #endif
 			id.m_Active = false;
 			id.m_isSubscriber = false;
-		HandleSubscription(true,subscription.m_identifier,id);
+		HandleSubscription(true, subscription.m_identifier, id);
 		return true;
 	}
 	return false;
@@ -1537,7 +1526,7 @@ bool GkPresence::HandleNewInstruction(unsigned tag, const H225_AliasAddress & ad
 		return true;
 
 	// Check to see if we already have this instruction
-	for (PINDEX i=0; i< instructions.GetSize(); i++) {
+	for (PINDEX i = 0; i < instructions.GetSize(); i++) {
 #ifdef HAS_H460P_VER_1
 		const H225_AliasAddress & b = instruction;
 #else
@@ -1585,7 +1574,7 @@ bool GkPresence::HandleNewInstruction(unsigned tag, const H225_AliasAddress & ad
 #ifndef HAS_H460P_VER_1
 					idx.m_Display = displayName;
 #endif
-				HandleSubscription(true,pid,idx);
+				HandleSubscription(true, pid, idx);
 
 				//Build local side
 				idx.m_isSubscriber = true;
@@ -1622,7 +1611,7 @@ void GkPresence::OnNotification(MsgType tag, const H460P_PresenceNotification &
 				UpdateLocalPresence(ep.m_Notify[0], notify);
 			}
 
-			for (PINDEX i=0; i < ep.m_Instruction.GetSize(); i++) {
+			for (PINDEX i = 0; i < ep.m_Instruction.GetSize(); i++) {
 				if (ep.m_Instruction[i].GetTag() == H460P_PresenceInstruction::e_subscribe) {
 #ifdef HAS_H460P_VER_1
 					const H225_AliasAddress & a = ep.m_Instruction[i];
@@ -1631,10 +1620,10 @@ void GkPresence::OnNotification(MsgType tag, const H460P_PresenceNotification &
 					const H225_AliasAddress & a = palias.m_alias;
 #endif
 					H460P_PresencePDU msg;
-					H460P_PresenceNotification & notification = BuildNotificationMsg(notify,msg);
+					H460P_PresenceNotification & notification = BuildNotificationMsg(notify, msg);
 					notification.IncludeOptionalField(H460P_PresenceNotification::e_aliasAddress);
 					notification.m_aliasAddress = addr;
-					EnQueuePresence(a,msg);
+					EnQueuePresence(a, msg);
 				}
 			}
 		}
@@ -1647,7 +1636,7 @@ void GkPresence::OnInstructions(MsgType tag, const H460P_ArrayOf_PresenceInstruc
 	H323PresenceStore::iterator itx = localStore.find(addr);
 	if (itx != localStore.end()) {
 		H323PresenceEndpoint & ep = itx->second;
-		for (PINDEX i=0; i<instruction.GetSize(); i++) {
+		for (PINDEX i = 0; i<instruction.GetSize(); i++) {
 			HandleNewInstruction(instruction[i].GetTag(), addr, instruction[i], ep.m_Instruction);
 		}
 	}
@@ -1656,17 +1645,17 @@ void GkPresence::OnInstructions(MsgType tag, const H460P_ArrayOf_PresenceInstruc
 void GkPresence::OnSubscription(MsgType tag, const H460P_PresenceSubscription & subscription, const H225_AliasAddress & addr)
 {
 	bool approved = false;
-	if (!HandleSubscriptionLocal(subscription,approved)) {
+	if (!HandleSubscriptionLocal(subscription, approved)) {
 		H460P_PresencePDU msg;
-		BuildSubscriptionMsg(subscription,msg);
+		BuildSubscriptionMsg(subscription, msg);
 		if (!HandleForwardPresence(subscription.m_identifier, msg)) {
-			PTRACE(2, "PRES\tSubscription received " << AsString(subscription.m_identifier.m_guid) << " from " << AsString(addr,0) << " not handled");
+			PTRACE(2, "PRES\tSubscription received " << AsString(subscription.m_identifier.m_guid) << " from " << AsString(addr, 0) << " not handled");
 			return;
 		}
 	}
 
 	// Handle the senders side
-	for (PINDEX i=0; i<subscription.m_aliases.GetSize(); i++) {
+	for (PINDEX i = 0; i<subscription.m_aliases.GetSize(); i++) {
 #ifdef HAS_H460P_VER_1
 		const H225_AliasAddress & sub = subscription.m_aliases[i];
 #else
@@ -1690,16 +1679,16 @@ void GkPresence::OnNotification(MsgType tag, const H460P_PresenceNotification &
 	if (!notify.HasOptionalField(H460P_PresenceNotification::e_subscribers))
 			return;
 
-	for (PINDEX i=0; i<notify.m_subscribers.GetSize(); ++i) {
+	for (PINDEX i = 0; i<notify.m_subscribers.GetSize(); ++i) {
 		// need to find the local alias
 		H323PresenceIds::const_iterator it = remoteIds.find(notify.m_subscribers[i]);
 		if (it != remoteIds.end()) {
 			H225_AliasAddress addr = it->second.m_subscriber;
 			PTRACE(5, "PRES\tReceived Notification " << notify.m_subscribers[i] << " for " << addr << " from " << ip);
 			H460P_PresencePDU msg;
-			H460P_PresenceNotification & notification = BuildNotificationMsg(notify,msg);
+			H460P_PresenceNotification & notification = BuildNotificationMsg(notify, msg);
 			notification.RemoveOptionalField(H460P_PresenceNotification::e_subscribers);
-			EnQueuePresence(addr,msg);
+			EnQueuePresence(addr, msg);
 		} else {
 			// not one of ours see if we are passing on.
 			H460P_PresencePDU msg;
@@ -1707,7 +1696,7 @@ void GkPresence::OnNotification(MsgType tag, const H460P_PresenceNotification &
 			notification.m_subscribers.SetSize(1);
 			notification.m_subscribers[0] = notify.m_subscribers[i];
 			if (!HandleForwardPresence(notification.m_subscribers[0], msg)) {
-				PTRACE(4, "PRES\tUnknown Notification received " << notification.m_subscribers[0] << " from " << ip << " disgarding.");
+				PTRACE(4, "PRES\tUnknown Notification received " << notification.m_subscribers[0] << " from " << ip << " disregarding.");
 			}
 		}
 	}
@@ -1715,12 +1704,12 @@ void GkPresence::OnNotification(MsgType tag, const H460P_PresenceNotification &
 
 void GkPresence::OnSubscription(MsgType tag,const H460P_PresenceSubscription & subscription,const H225_TransportAddress & ip)
 {
-	bool approved;
-	if (!HandleSubscriptionLocal(subscription,approved)) {
+	bool approved = false;
+	if (!HandleSubscriptionLocal(subscription, approved)) {
 		H460P_PresencePDU msg;
-		BuildSubscriptionMsg(subscription,msg);
+		BuildSubscriptionMsg(subscription, msg);
 		if (!HandleForwardPresence(subscription.m_identifier, msg)) {
-			PTRACE(4, "PRES\tUnknown Subscription received " << subscription.m_identifier << " from " << ip << " disgarding.");
+			PTRACE(4, "PRES\tUnknown Subscription received " << subscription.m_identifier << " from " << ip << " disregarding.");
 			return;
 		}
 	}
@@ -1748,10 +1737,8 @@ void GkPresence::OnIdentifiers(MsgType tag, const H460P_PresenceIdentifier & ide
 		H460P_PresencePDU msg;
 		BuildIdentifierMsg(identifier,msg);
 		if (!HandleForwardPresence(identifier, msg)) {
-			PTRACE(4, "PRES\tUnknown Identifier received " << identifier << " from " << ip << " disgarding.");
+			PTRACE(4, "PRES\tUnknown Identifier received " << identifier << " from " << ip << " disregarding.");
 		}
 	}
 }
 #endif
-
-
