@@ -4,7 +4,7 @@
  * IP based authentication modules
  *
  * Copyright (c) 2005, Michal Zygmuntowicz
- * Copyright (c) 2006-2013, Jan Willamowius
+ * Copyright (c) 2006-2015, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -14,61 +14,78 @@
  */
 
 #ifndef IPAUTH_H
-#define IPAUTH_H "@(#) $Id: ipauth.h,v 1.6 2013/08/03 12:26:19 willamowius Exp $"
+#define IPAUTH_H "@(#) $Id$"
 
 #include "gkauth.h"
 
 /// Generic IP based authentication
 class IPAuthBase : public GkAuthenticator {
 public:
-	enum SupportedChecks {
-		IPAuthRasChecks = RasInfo<H225_GatekeeperRequest>::flag
-			| RasInfo<H225_RegistrationRequest>::flag
-			| RasInfo<H225_LocationRequest>::flag,
-		IPAuthMiscChecks = e_Setup | e_SetupUnreg
+	enum SupportedRasChecks {
+		/// bitmask of RAS checks implemented by this module
+		IPAuthRasChecks = RasInfo<H225_RegistrationRequest>::flag
+			| RasInfo<H225_AdmissionRequest>::flag
+			| RasInfo<H225_GatekeeperRequest>::flag
+			| RasInfo<H225_UnregistrationRequest>::flag
+			| RasInfo<H225_BandwidthRequest>::flag
+			| RasInfo<H225_DisengageRequest>::flag
+			| RasInfo<H225_LocationRequest>::flag
+			| RasInfo<H225_InfoRequest>::flag
+			| RasInfo<H225_ResourcesAvailableIndicate>::flag
+	};
+	enum SupportedMiscChecks {
+		/// bitmask of Misc checks implemented by this module
+        IPAuthMiscChecks = e_Setup
+            | e_SetupUnreg
+            | e_Connect
+            | e_CallProceeding
+            | e_Alerting
+            | e_Information
+            | e_ReleaseComplete
+            | e_Facility
+            | e_Progress
+            | e_Empty
+            | e_Status
+            | e_StatusEnquiry
+            | e_SetupAck
+            | e_Notify
 	};
 
 	/// Destroy the authenticator
 	virtual ~IPAuthBase();
 
-	/** Authenticate using data from GRQ RAS message.
-	
-		@return:
-		#GkAuthenticator::Status enum# with the result of authentication.
-	*/
-	virtual int Check(
-		/// GRQ RAS message to be authenticated
-		RasPDU<H225_GatekeeperRequest> & grqPdu, 
-		/// gatekeeper request reject reason
-		unsigned & rejectReason
-		);
-
 	/** Authenticate using data from RRQ RAS message.
-	
 		@return:
 		#GkAuthenticator::Status enum# with the result of authentication.
 	*/
 	virtual int Check(
 		/// RRQ RAS message to be authenticated
-		RasPDU<H225_RegistrationRequest> & rrqPdu, 
+		RasPDU<H225_RegistrationRequest> & rrqPdu,
 		/// authorization data (reject reason, ...)
 		RRQAuthData & authData
 		);
-		
-	/** Authenticate using data from LRQ RAS message.
-	
+
+	/** Authenticate using data from ARQ RAS message.
 		@return:
 		#GkAuthenticator::Status enum# with the result of authentication.
 	*/
 	virtual int Check(
-		/// LRQ nessage to be authenticated
-		RasPDU<H225_LocationRequest> & lrqPdu, 
-		/// location request reject reason
-		unsigned & rejectReason
+		/// ARQ to be authenticated/authorized
+		RasPDU<H225_AdmissionRequest> & request,
+		/// authorization data (call duration limit, reject reason, ...)
+		ARQAuthData & authData
 		);
 
+    // authenticate other RAS messages
+	virtual int Check(RasPDU<H225_GatekeeperRequest> & grqPdu, unsigned & rejectReason);
+	virtual int Check(RasPDU<H225_UnregistrationRequest> & urqPdu, unsigned & rejectReason);
+	virtual int Check(RasPDU<H225_BandwidthRequest> & brqPdu, unsigned & rejectReason);
+	virtual int Check(RasPDU<H225_DisengageRequest> & drqPdu, unsigned & rejectReason);
+	virtual int Check(RasPDU<H225_LocationRequest> & lrqPdu, unsigned & rejectReason);
+	virtual int Check(RasPDU<H225_InfoRequest> & req, unsigned & rejectReason);
+	virtual int Check(RasPDU<H225_ResourcesAvailableIndicate> & req, unsigned & rejectReason);
+
 	/** Authenticate using data from Q.931/H.225.0 Setup message.
-	
 		@return:
 		#GkAuthenticator::Status enum# with the result of authentication.
 	*/
@@ -78,10 +95,17 @@ public:
 		/// authorization data (call duration limit, reject reason, ...)
 		SetupAuthData & authData
 		);
-	
-protected:		
+
+    virtual int Check(
+		/// Q931 message to be authenticated/authorized
+		Q931 & msg,
+		/// authorization data
+		Q931AuthData & authData
+		);
+
+protected:
 	/// Create IP based authenticator
-	IPAuthBase( 
+	IPAuthBase(
 		/// authenticator name from Gatekeeper::Auth section
 		const char * authName,
 		/// bitmask with supported RAS checks
@@ -107,7 +131,7 @@ private:
 	/* No copy constructor allowed */
 	IPAuthBase(const IPAuthBase &);
 	/* No operator= allowed */
-	IPAuthBase& operator=(const IPAuthBase&);
+	IPAuthBase& operator=(const IPAuthBase &);
 };
 
 #endif /* #ifndef IPAUTH_H */
