@@ -3,7 +3,7 @@
  *
  * LDAP authentication/authorization modules for GNU Gatekeeper
  *
- * Copyright (c) 2013, Jan Willamowius
+ * Copyright (c) 2013-2020, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -59,14 +59,14 @@ class LDAPPasswordAuth : public LDAPBase, public SimplePasswordAuth
 public:
 	LDAPPasswordAuth(const char* authName);
 	virtual ~LDAPPasswordAuth() { }
-	
+
 protected:
 	/** Override from SimplePasswordAuth.
-	
+
 	    @return
 	    True if the password has been found for the given alias.
 	*/
-	virtual bool GetPassword(const PString & alias, PString & password);
+	virtual bool GetPassword(const PString & alias, PString & password, std::map<PString, PString> & params);
 
 private:
 	LDAPPasswordAuth();
@@ -82,10 +82,10 @@ public:
 	virtual ~LDAPAliasAuth() { }
 
 protected:
-	/** Get auth condition string for the given alias. 
+	/** Get auth condition string for the given alias.
 	    This implementation searches the LDAP database for the string.
 	    Override from AliasAuth.
-		
+
 	    @return
 	    The AliasAuth condition string for the given alias.
 	*/
@@ -132,7 +132,7 @@ PLDAPSession * LDAPBase::CreateConnection()
 	if (m_startTLS) {
 #ifdef hasLDAPStartTLS
 		if (!session->StartTLS()) {
-			PTRACE(1,"LDAP\tStartTLS failed");
+			PTRACE(1, "LDAP\tStartTLS failed");
 			return NULL;
 		}
 #else
@@ -163,7 +163,7 @@ LDAPPasswordAuth::LDAPPasswordAuth(const char* authName)
 	m_attribute = GetConfig()->GetString(LDAPAttributeSect, "H235PassWord", "plaintextPassword");
 }
 
-bool LDAPPasswordAuth::GetPassword(const PString & alias, PString & password)
+bool LDAPPasswordAuth::GetPassword(const PString & alias, PString & password, std::map<PString, PString> & params)
 {
 	PLDAPSession * ldapClient = CreateConnection();
 	if (!ldapClient)
@@ -230,8 +230,6 @@ public:
 protected:
 	virtual bool FindByAliases(Routing::RoutingRequest &, H225_ArrayOf_AliasAddress &);
 	virtual bool FindByAliases(Routing::LocationRequest &, H225_ArrayOf_AliasAddress &);
-
-	PString m_attribute;
 };
 
 LDAPPolicy::LDAPPolicy()
