@@ -2,7 +2,7 @@
 //
 // snmp.cxx for GNU Gatekeeper
 //
-// Copyright (c) 2012-2014, Jan Willamowius
+// Copyright (c) 2012-2019, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -20,8 +20,18 @@
 #include "job.h"
 #include "SoftPBX.h"
 
+#ifdef HAS_PTLIBSNMP
+#include "RasSrv.h"
+#endif
+
 void ReloadHandler();
 
+// NOTE: all OIDs must also be in m_nextList for PTLib-SNMP !
+const char * const sysDescrOIDStr        = "1.3.6.1.2.1.1.1";
+const char * const sysObjectIDOIDStr     = "1.3.6.1.2.1.1.2";
+const char * const sysUpTimeOIDStr       = "1.3.6.1.2.1.1.3";
+const char * const sysNameOIDStr         = "1.3.6.1.2.1.1.5";
+
 const char * const GnuGkMIBStr           = "1.3.6.1.4.1.27938.11";
 const char * const ShortVersionOIDStr    = "1.3.6.1.4.1.27938.11.1.1";
 const char * const LongVersionOIDStr     = "1.3.6.1.4.1.27938.11.1.2";
@@ -31,10 +41,14 @@ const char * const TraceLevelOIDStr      = "1.3.6.1.4.1.27938.11.1.5";
 const char * const CatchAllOIDStr        = "1.3.6.1.4.1.27938.11.1.6";
 const char * const TotalCallsOIDStr      = "1.3.6.1.4.1.27938.11.1.7";
 const char * const SuccessfulCallsOIDStr = "1.3.6.1.4.1.27938.11.1.8";
+const char * const TotalBandwidthOIDStr  = "1.3.6.1.4.1.27938.11.1.9";
+
 const char * const severityOIDStr        = "1.3.6.1.4.1.27938.11.2.1";
 const char * const groupOIDStr           = "1.3.6.1.4.1.27938.11.2.2";
 const char * const displayMsgOIDStr      = "1.3.6.1.4.1.27938.11.2.3";
 
+const char * const NoNextOIDStr          = "99.99";
+
 
 #ifdef HAS_NETSNMP
 
@@ -44,6 +58,7 @@ const char * const displayMsgOIDStr      = "1.3.6.1.4.1.27938.11.2.3";
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 
 const char * agent_name = "gnugk-agent";
+PMutex g_NetSNMPMutex;
 
 static oid snmptrap_oid[]       = { 1, 3, 6, 1, 6, 3, 1, 1, 4, 1, 0 };
 static oid ShortVersionOID[]    = { 1, 3, 6, 1, 4, 1, 27938, 11, 1, 1 };
@@ -54,6 +69,8 @@ static oid TraceLevelOID[]      = { 1, 3, 6, 1, 4, 1, 27938, 11, 1, 5 };
 static oid CatchAllOID[]        = { 1, 3, 6, 1, 4, 1, 27938, 11, 1, 6 };
 static oid TotalCallsOID[]      = { 1, 3, 6, 1, 4, 1, 27938, 11, 1, 7 };
 static oid SuccessfulCallsOID[] = { 1, 3, 6, 1, 4, 1, 27938, 11, 1, 8 };
+static oid TotalBandwidthOID[]  = { 1, 3, 6, 1, 4, 1, 27938, 11, 1, 9 };
+
 static oid severityOID[]        = { 1, 3, 6, 1, 4, 1, 27938, 11, 2, 1 };
 static oid groupOID[]           = { 1, 3, 6, 1, 4, 1, 27938, 11, 2, 2 };
 static oid displayMsgOID[]      = { 1, 3, 6, 1, 4, 1, 27938, 11, 2, 3 };
@@ -61,13 +78,15 @@ static oid displayMsgOID[]      = { 1, 3, 6, 1, 4, 1, 27938, 11, 2, 3 };
 
 void SendNetSNMPTrap(unsigned trapNumber, SNMPLevel severity, SNMPGroup group, const PString & msg)
 {
+    PWaitAndSignal lock(g_NetSNMPMutex);
+
 	PTRACE(5, "SNMP\tSendSNMPTrap " << trapNumber << ", " << severity << ", " << group << ", " << msg);
-	oid trapOID[]              = { 1, 3, 6, 1, 4, 1, 27938, 11, 0, 99999 };
+	oid trapOID[] = { 1, 3, 6, 1, 4, 1, 27938, 11, 0, 99999 };
 
 	// insert trapNumber as last digit
 	trapOID[ OID_LENGTH(trapOID) - 1 ] = trapNumber;
 
-	netsnmp_variable_list *var_list = NULL;
+	netsnmp_variable_list * var_list = NULL;
 	// set snmpTrapOid.0 value
     snmp_varlist_add_variable(&var_list,
                               snmptrap_oid, OID_LENGTH(snmptrap_oid),
@@ -98,6 +117,8 @@ int short_version_handler(netsnmp_mib_handler * /* handler */,
 							netsnmp_agent_request_info * reqinfo,
 							netsnmp_request_info * requests)
 {
+    PWaitAndSignal lock(g_NetSNMPMutex);
+
     if (reqinfo->mode != MODE_GET)
 		return SNMPERR_SUCCESS;
     for (netsnmp_request_info *request = requests; request; request = request->next) {
@@ -112,6 +133,8 @@ int long_version_handler(netsnmp_mib_handler * /* handler */,
 							netsnmp_agent_request_info * reqinfo,
 							netsnmp_request_info * requests)
 {
+    PWaitAndSignal lock(g_NetSNMPMutex);
+
     if (reqinfo->mode != MODE_GET)
 		return SNMPERR_SUCCESS;
     for (netsnmp_request_info *request = requests; request; request = request->next) {
@@ -125,6 +148,8 @@ int registrations_handler(netsnmp_mib_handler * /* handler */,
 							netsnmp_agent_request_info * reqinfo,
 							netsnmp_request_info * requests)
 {
+    PWaitAndSignal lock(g_NetSNMPMutex);
+
     if (reqinfo->mode != MODE_GET)
 		return SNMPERR_SUCCESS;
     for (netsnmp_request_info *request = requests; request; request = request->next) {
@@ -139,6 +164,8 @@ int calls_handler(netsnmp_mib_handler * /* handler */,
 							netsnmp_agent_request_info * reqinfo,
 							netsnmp_request_info * requests)
 {
+    PWaitAndSignal lock(g_NetSNMPMutex);
+
     if (reqinfo->mode != MODE_GET)
 		return SNMPERR_SUCCESS;
     for (netsnmp_request_info *request = requests; request; request = request->next) {
@@ -148,11 +175,29 @@ int calls_handler(netsnmp_mib_handler * /* handler */,
 	return SNMPERR_SUCCESS;
 }
 
+int bandwidth_handler(netsnmp_mib_handler * /* handler */,
+							netsnmp_handler_registration * /* reg */,
+							netsnmp_agent_request_info * reqinfo,
+							netsnmp_request_info * requests)
+{
+    PWaitAndSignal lock(g_NetSNMPMutex);
+
+    if (reqinfo->mode != MODE_GET)
+		return SNMPERR_SUCCESS;
+    for (netsnmp_request_info *request = requests; request; request = request->next) {
+		unsigned total_bandwidth = CallTable::Instance()->GetTotalAllocatedBandwidth();
+		snmp_set_var_typed_integer(request->requestvb, ASN_UNSIGNED, total_bandwidth);
+	}
+	return SNMPERR_SUCCESS;
+}
+
 int totalcalls_handler(netsnmp_mib_handler * /* handler */,
 							netsnmp_handler_registration * /* reg */,
 							netsnmp_agent_request_info * reqinfo,
 							netsnmp_request_info * requests)
 {
+    PWaitAndSignal lock(g_NetSNMPMutex);
+
     if (reqinfo->mode != MODE_GET)
 		return SNMPERR_SUCCESS;
     for (netsnmp_request_info *request = requests; request; request = request->next) {
@@ -167,6 +212,8 @@ int successfulcalls_handler(netsnmp_mib_handler * /* handler */,
 							netsnmp_agent_request_info * reqinfo,
 							netsnmp_request_info * requests)
 {
+    PWaitAndSignal lock(g_NetSNMPMutex);
+
     if (reqinfo->mode != MODE_GET)
 		return SNMPERR_SUCCESS;
     for (netsnmp_request_info *request = requests; request; request = request->next) {
@@ -181,6 +228,8 @@ int tracelevel_handler(netsnmp_mib_handler * /* handler */,
 							netsnmp_agent_request_info * reqinfo,
 							netsnmp_request_info * requests)
 {
+    PWaitAndSignal lock(g_NetSNMPMutex);
+
     for (netsnmp_request_info *request = requests; request; request = request->next) {
 		if (reqinfo->mode == MODE_GET) {
 			snmp_set_var_typed_integer(request->requestvb, ASN_UNSIGNED, PTrace::GetLevel());
@@ -200,6 +249,8 @@ int catchall_handler(netsnmp_mib_handler * /* handler */,
 							netsnmp_agent_request_info * reqinfo,
 							netsnmp_request_info * requests)
 {
+    PWaitAndSignal lock(g_NetSNMPMutex);
+
     for (netsnmp_request_info *request = requests; request; request = request->next) {
 		if (reqinfo->mode == MODE_GET) {
 			PString catchAllDest = GkConfig()->GetString("Routing::CatchAll", "CatchAllIP", "");
@@ -285,7 +336,7 @@ void NetSNMPAgent::Run()
 	} else {
 		// run as AgentX sub-agent
 		netsnmp_ds_set_boolean(NETSNMP_DS_APPLICATION_ID, NETSNMP_DS_AGENT_ROLE, 1);
-		// use 127.0.0.1:705 by default, can be overriden in $HOME/.snmp/agentx.conf
+		// use 127.0.0.1:705 by default, can be overridden in $HOME/.snmp/agentx.conf
 		netsnmp_ds_set_string(NETSNMP_DS_APPLICATION_ID, NETSNMP_DS_AGENT_X_SOCKET, "tcp:127.0.0.1:705");
 		netsnmp_enable_subagent();
 	}
@@ -308,6 +359,8 @@ void NetSNMPAgent::Run()
 		netsnmp_create_handler_registration("catchall", tracelevel_handler, TraceLevelOID, OID_LENGTH(TraceLevelOID), HANDLER_CAN_RWRITE));
 	netsnmp_register_scalar(
 		netsnmp_create_handler_registration("catchall", catchall_handler, CatchAllOID, OID_LENGTH(CatchAllOID), HANDLER_CAN_RWRITE));
+	netsnmp_register_scalar(
+		netsnmp_create_handler_registration("bandwidth", bandwidth_handler, TotalBandwidthOID, OID_LENGTH(TotalBandwidthOID), HANDLER_CAN_RONLY));
 
 	init_snmp(agent_name);   // reads $HOME/.snmp/gnugk-agent.conf + $HOME/.snmp/agentx.conf
 
@@ -333,13 +386,11 @@ void NetSNMPAgent::Run()
 
 class PTLibSNMPAgent;
 
-static PTLibSNMPAgent * g_ptlibAgentPtr = NULL;
-
 void SendPTLibSNMPTrap(unsigned trapNumber, SNMPLevel severity, SNMPGroup group, const PString & msg)
 {
-	PTRACE(5, "SNMP\tSendSNMPTrap " << trapNumber << ", " << severity << ", " << group << ", " << msg);
 	PString trapHost = GkConfig()->GetString(SNMPSection, "TrapHost", "");
 	if (!trapHost.IsEmpty()) {
+    	PTRACE(5, "SNMP\tSendSNMPTrap " << trapNumber << ", " << severity << ", " << group << ", " << msg);
 		// DNS resolve TrapHost
 		PIPSocket::Address trapHostIP;
 		H323TransportAddress addr = H323TransportAddress(trapHost);
@@ -374,7 +425,7 @@ public:
 	virtual PBoolean MIB_LocalMatch(PSNMP_PDU & answerPDU);
 
   protected:
-    PRFC1155_SimpleSyntax sys_description;
+    list<PString> m_nextList;
 };
 
 
@@ -383,6 +434,21 @@ PTLibSNMPAgent::PTLibSNMPAgent()
 		(WORD)GkConfig()->GetInteger(SNMPSection, "AgentListenPort", 161))
 {
 	PTRACE(1, "SNMP\tStarting SNMP agent (PTLib)");
+	// NOTE: the list must have _all_ OIDs we support in _numerical_order_
+	m_nextList.push_back(sysDescrOIDStr);
+	m_nextList.push_back(sysObjectIDOIDStr);
+	m_nextList.push_back(sysUpTimeOIDStr);
+	m_nextList.push_back(sysNameOIDStr);
+
+	m_nextList.push_back(ShortVersionOIDStr);
+	m_nextList.push_back(LongVersionOIDStr);
+	m_nextList.push_back(RegistrationsOIDStr);
+	m_nextList.push_back(CallsOIDStr);
+	m_nextList.push_back(TraceLevelOIDStr);
+	m_nextList.push_back(CatchAllOIDStr);
+	m_nextList.push_back(TotalCallsOIDStr);
+	m_nextList.push_back(SuccessfulCallsOIDStr);
+	m_nextList.push_back(TotalBandwidthOIDStr);
 }
 
 PTLibSNMPAgent::~PTLibSNMPAgent()
@@ -393,14 +459,22 @@ PTLibSNMPAgent::~PTLibSNMPAgent()
 PBoolean PTLibSNMPAgent::Authorise(const PIPSocket::Address & ip)
 {
 	PStringArray networks = GkConfig()->GetString(SNMPSection, "AllowRequestsFrom", "").Tokenise(",", FALSE);
-	for (PINDEX n=0; n < networks.GetSize(); ++n) {
-		if (networks[n].Find('/') == P_MAX_INDEX)
-			networks[n] += "/32";	// add netmask to pure IPs
+	for (PINDEX n = 0; n < networks.GetSize(); ++n) {
+		if (networks[n].Find('/') == P_MAX_INDEX) {
+            if (IsIPv4Address(networks[n])) {
+                networks[n] += "/32";	// add netmask to pure IPs
+            } else {
+                networks[n] += "/128";	// add netmask to pure IPs
+            }
+        }
+
 		NetworkAddress net = NetworkAddress(networks[n]);
 		if (ip << net) {
+            PTRACE(4, "SNMP\tAccepting SNMP request from " << ip);
 			return PTrue;
 		}
 	}
+    PTRACE(2, "SNMP\tWarning: Rejecting SNMP request from " << ip);
 	return PFalse;
 }
 
@@ -412,18 +486,68 @@ PBoolean PTLibSNMPAgent::ConfirmCommunity(PASN_OctetString & community)
 
 PBoolean PTLibSNMPAgent::OnGetRequest(PINDEX reqID, PSNMP::BindingList & vars, PSNMP::ErrorType & errCode)
 {
-	return PTrue; // allow GET requests
+	return PTrue; // respond to GET request
 }
 
 PBoolean PTLibSNMPAgent::OnGetNextRequest(PINDEX reqID, PSNMP::BindingList & vars, PSNMP::ErrorType & errCode)
 {
-	return PTrue; // allow GETNEXT requests
+    PString oid = vars.front().first;
+    if (oid.Right(2) == ".0")
+        oid = oid.Left(oid.GetLength() - 2);
+    list<PString>::const_iterator it = m_nextList.begin();
+    while(it != m_nextList.end() && (OIDCmp(*it, oid) <= 0)) {
+        ++it;
+    }
+    if (it != m_nextList.end() /* TODO: && is in same space? */) {
+        vars.front().first = *it + PString(".0");
+	    return PTrue; // respond to GET-NEXT request
+    } else {
+        vars.front().first = NoNextOIDStr + PString(".0");
+        return PTrue; // no further OID found
+    }
 }
 
 PBoolean PTLibSNMPAgent::OnSetRequest(PINDEX reqID, PSNMP::BindingList & vars, PSNMP::ErrorType & errCode)
 {
-	// TODO: SET operation is broken in PTLib PSNMPServer
-	return PFalse;	// doesn't work
+	// SET operation is broken in PTLib PSNMPServer, you need a patched 2.10.9 version later than 2019-05-26
+    list< pair<PString, PRFC1155_ObjectSyntax> >::const_iterator iter = vars.begin();
+    while (iter != vars.end()) {
+        const PString & oid = iter->first;
+        const PRFC1155_ObjectSyntax & value = iter->second;
+
+		if (oid == TraceLevelOIDStr + PString(".0")) {
+            if (value.GetTag() == PRFC1155_SimpleSyntax::e_number) {
+                const PRFC1155_SimpleSyntax & simple = value;
+                const PASN_Integer & num = simple;
+                PTRACE(4, "SNMP\tSetting trace level to " << num.GetValue());
+                PTrace::SetLevel(num.GetValue());
+            } else {
+                errCode = BadValue;
+            }
+		} else if (oid == CatchAllOIDStr + PString(".0")) {
+            if (value.GetTag() == PRFC1155_SimpleSyntax::e_string) {
+                const PRFC1155_SimpleSyntax & simple = value;
+                const PASN_OctetString & str = simple;
+                PString dest = AsString(str.GetValue());
+                PTRACE(4, "SNMP\tSetting CatchAll to " << dest);
+                if (IsIPAddress(dest)) {
+                    Toolkit::Instance()->SetConfig(1, "Routing::CatchAll", "CatchAllIP", dest);
+                    Toolkit::Instance()->SetConfig(1, "Routing::CatchAll", "CatchAllAlias", "");
+                } else {
+                    Toolkit::Instance()->SetConfig(1, "Routing::CatchAll", "CatchAllIP", "");
+                    Toolkit::Instance()->SetConfig(1, "Routing::CatchAll", "CatchAllAlias", dest);
+                }
+                ReloadHandler();
+            } else {
+                errCode = BadValue;
+            }
+		} else {
+		    PTRACE(2, "SNMP\tWarning: SNMP SET for unsupported OID " << oid);
+		    errCode = NoSuchName;
+		}
+        ++iter;
+    }
+	return PTrue; // send response
 }
 
 // 0=SNMPv1, 1=SNMPv2
@@ -434,7 +558,7 @@ PBoolean PTLibSNMPAgent::ConfirmVersion(PASN_Integer vers)
 
 void SetRFC1155Object(PRFC1155_ObjectSyntax & obj, unsigned i)
 {
-	// according to RFC 1902 Gauge32 is identical ro Unsigned32; PTLib only has Gauge
+	// according to RFC 1902 Gauge32 is identical to Unsigned32; PTLib only has Gauge
 	PRFC1155_ApplicationSyntax appl(PRFC1155_ApplicationSyntax::e_gauge);
 	PRFC1155_ObjectSyntax * newObj = (PRFC1155_ObjectSyntax*)&appl;
 	PASN_Integer * num = (PASN_Integer *)&appl.GetObject();
@@ -451,6 +575,15 @@ void SetRFC1155CounterObject(PRFC1155_ObjectSyntax & obj, unsigned i)
 	obj = *newObj;
 }
 
+void SetRFC1155TicksObject(PRFC1155_ObjectSyntax & obj, unsigned i)
+{
+	PRFC1155_ApplicationSyntax appl(PRFC1155_ApplicationSyntax::e_ticks);
+	PRFC1155_ObjectSyntax * newObj = (PRFC1155_ObjectSyntax*)&appl;
+	PASN_Integer * num = (PASN_Integer *)&appl.GetObject();
+	num->SetValue(i);
+	obj = *newObj;
+}
+
 void SetRFC1155Object(PRFC1155_ObjectSyntax & obj, const PString & str)
 {
 	PRFC1155_SimpleSyntax simple(PRFC1155_SimpleSyntax::e_string);
@@ -460,191 +593,73 @@ void SetRFC1155Object(PRFC1155_ObjectSyntax & obj, const PString & str)
 	obj = *newObj;
 }
 
+void SetRFC1155OIDObject(PRFC1155_ObjectSyntax & obj, const PString & str)
+{
+	PRFC1155_SimpleSyntax simple(PRFC1155_SimpleSyntax::e_object);
+	PRFC1155_ObjectSyntax * newObj = (PRFC1155_ObjectSyntax*)&simple;
+	PASN_ObjectId * idObj = (PASN_ObjectId *)&simple.GetObject();
+	idObj->SetValue(str);
+	obj = *newObj;
+}
+
 PBoolean PTLibSNMPAgent::MIB_LocalMatch(PSNMP_PDU & answerPDU)
 {
-	PSNMP_VarBindList & vars = answerPDU.m_variable_bindings;
-	bool found = false;
+	const PSNMP_VarBindList & vars = answerPDU.m_variable_bindings;
 
-	for (PINDEX i = 0; i < vars.GetSize(); i++){
-		if (vars[i].m_name == ShortVersionOIDStr + PString(".0")) {
+	for (PINDEX i = 0; i < vars.GetSize(); i++) {
+        const PRFC1155_ObjectName & oid = vars[i].m_name;
+        // GnuGk's OIDs
+		if (oid == ShortVersionOIDStr + PString(".0")) {
 			SetRFC1155Object(vars[i].m_value, PProcess::Current().GetVersion(true));
-			found = true;
-		} else if (vars[i].m_name == LongVersionOIDStr + PString(".0")) {
-			SetRFC1155Object(vars[i].m_value, Toolkit::GKVersion());
-			found = true;
-		} else if (vars[i].m_name == RegistrationsOIDStr + PString(".0")) {
+		} else if (oid == LongVersionOIDStr + PString(".0")) {
+			SetRFC1155Object(vars[i].m_value, Toolkit::GKVersion().Trim());
+		} else if (oid == RegistrationsOIDStr + PString(".0")) {
 			SetRFC1155Object(vars[i].m_value, RegistrationTable::Instance()->Size());
-			found = true;
-		} else if (vars[i].m_name == CallsOIDStr + PString(".0")) {
+		} else if (oid == CallsOIDStr + PString(".0")) {
 			SetRFC1155Object(vars[i].m_value, CallTable::Instance()->Size());
-			found = true;
-		} else if (vars[i].m_name == TotalCallsOIDStr + PString(".0")) {
+		} else if (oid == TotalBandwidthOIDStr + PString(".0")) {
+			SetRFC1155Object(vars[i].m_value, CallTable::Instance()->GetTotalAllocatedBandwidth());
+		} else if (oid == TotalCallsOIDStr + PString(".0")) {
 			SetRFC1155CounterObject(vars[i].m_value, CallTable::Instance()->TotalCallCount());
-			found = true;
-		} else if (vars[i].m_name == SuccessfulCallsOIDStr + PString(".0")) {
+		} else if (oid == SuccessfulCallsOIDStr + PString(".0")) {
 			SetRFC1155CounterObject(vars[i].m_value, CallTable::Instance()->SuccessfulCallCount());
-			found = true;
-		} else if (vars[i].m_name == TraceLevelOIDStr + PString(".0")) {
+		} else if (oid == TraceLevelOIDStr + PString(".0")) {
 			SetRFC1155Object(vars[i].m_value, PTrace::GetLevel());
-			found = true;
-		} else if (vars[i].m_name == CatchAllOIDStr + PString(".0")) {
+		} else if (oid == CatchAllOIDStr + PString(".0")) {
 			PString catchAllDest = GkConfig()->GetString("Routing::CatchAll", "CatchAllIP", "");
 			if (catchAllDest.IsEmpty())
 				catchAllDest = GkConfig()->GetString("Routing::CatchAll", "CatchAllAlias", "catchall");
 			SetRFC1155Object(vars[i].m_value, catchAllDest);
-			found = true;
+
+        // generic OIDs
+		} else if (oid == sysDescrOIDStr + PString(".0")) {
+			SetRFC1155Object(vars[i].m_value, "GNU Gatekeeper " + PProcess::Current().GetVersion(true));
+		} else if (oid == sysObjectIDOIDStr + PString(".0")) {
+			SetRFC1155OIDObject(vars[i].m_value, LongVersionOIDStr);
+		} else if (oid == sysUpTimeOIDStr + PString(".0")) {
+			SetRFC1155TicksObject(vars[i].m_value, SoftPBX::UptimeTicks());
+		} else if (oid == sysNameOIDStr + PString(".0")) {
+			SetRFC1155Object(vars[i].m_value, Toolkit::Instance()->GKName());
+
+        // dummy OID to signal end in Get-Next
+		} else if (oid == NoNextOIDStr + PString(".0")) {
+			SetRFC1155Object(vars[i].m_value, 0);
+
+		} else {
+		    PTRACE(2, "SNMP\tWarning: SNMP GET for unsupported OID " << oid);
+		    answerPDU.m_error_status = PSNMP::NoSuchName;
 		}
 	}
 
-	return found;
+	return true;
 }
 
 #endif	// HAS_PTLIBSNMP
 
 
-#ifdef HAS_WINSNMP
-
-#define BUFSIZE 512
-const char * const getPipename = "\\\\.\\pipe\\GnuGkGetSNMP";
-const char * const trapPipename = "\\\\.\\pipe\\GnuGkTrapSNMP";
-
-class WindowsSNMPAgent : public Singleton<WindowsSNMPAgent>
+PCaselessString SelectSNMPImplementation()
 {
-public:
-	WindowsSNMPAgent();
-	virtual ~WindowsSNMPAgent();
-
-	virtual void Run();
-	virtual void Stop();
-	virtual PString HandleRequest(const PString & request);
-
-	virtual void SendWindowsSNMPTrap(unsigned trapNumber, SNMPLevel severity, SNMPGroup group, const PString & msg);
-
-protected:
-	HANDLE m_getPipe;
-	bool m_shutdown;
-};
-
-WindowsSNMPAgent::WindowsSNMPAgent() : Singleton<WindowsSNMPAgent>("WindowsSNMPAgent")
-{
-	PTRACE(1, "SNMP\tStarting SNMP agent (Windows)");
-	m_shutdown = false;
-	m_getPipe = CreateNamedPipe(getPipename,
-		PIPE_ACCESS_DUPLEX, PIPE_TYPE_MESSAGE | PIPE_READMODE_MESSAGE | PIPE_WAIT,
-		1, BUFSIZE, BUFSIZE, 0, NULL);
-}
-
-WindowsSNMPAgent::~WindowsSNMPAgent()
-{
-	PTRACE(1, "SNMP\tDeleting SNMP agent (Windows)");
-	// CloseHandle(m_getPipe);	// don't close get pipe, will hang in blocking ConnectNamedPipe()
-}
-
-void WindowsSNMPAgent::Stop()
-{
-	PTRACE(1, "SNMP\tStopping SNMP agent (Windows)");
-	m_shutdown = true;
-}
-
-void WindowsSNMPAgent::Run()
-{
-	SNMP_TRAP(1, SNMPInfo, General, "GnuGk started");
-	while (!m_shutdown) {	// main loop: wait for connection from extension DLL until shutdown
-		if (ConnectNamedPipe(m_getPipe, NULL)) {	// wait for DLL to connect
-			// the pipe is connected; change to message-read mode.
-			DWORD dwMode = PIPE_READMODE_MESSAGE;
-			if (!SetNamedPipeHandleState(m_getPipe, &dwMode, NULL, NULL)) {
-				PTRACE(1, "SNMP\tSetNamedPipeHandleState failed. GLE=" << GetLastError());
-			}
-
-			// read server request and respond, then disconnect
-			char buffer[BUFSIZE];
-			DWORD bytesRead;
-			if (ReadFile(m_getPipe, buffer, sizeof(buffer), &bytesRead, NULL)) {
-				PString request((const char *)&buffer, bytesRead);
-				PString response = HandleRequest(request);
-				// Send response to the pipe server
-				DWORD bytesWritten;
-				WriteFile(m_getPipe, response.GetPointer(), response.GetLength(), &bytesWritten, NULL);
-			}
-			DisconnectNamedPipe(m_getPipe);
-		}
-	} // while running
-}
-
-PString WindowsSNMPAgent::HandleRequest(const PString & request)
-{
-	PStringArray token = request.Tokenise(" ", FALSE);
-	if ((token.GetSize() == 2) && (token[0] == "GET")) {
-		if (token[1] == ShortVersionOIDStr + PString(".0")) {
-			return "GET_RESPONSE s " + PProcess::Current().GetVersion(true);
-		}
-		if (token[1] == LongVersionOIDStr + PString(".0")) {
-			return "GET_RESPONSE s " + Toolkit::GKVersion();
-		}
-		if (token[1] == RegistrationsOIDStr + PString(".0")) {
-			return "GET_RESPONSE u " + PString(PString::Unsigned, RegistrationTable::Instance()->Size());
-		}
-		if (token[1] == CallsOIDStr + PString(".0")) {
-			return "GET_RESPONSE u " + PString(PString::Unsigned, CallTable::Instance()->Size());
-		}
-		if (token[1] == TotalCallsOIDStr + PString(".0")) {
-			return "GET_RESPONSE c " + PString(PString::Unsigned, CallTable::Instance()->TotalCallCount());
-		}
-		if (token[1] == SuccessfulCallsOIDStr + PString(".0")) {
-			return "GET_RESPONSE c " + PString(PString::Unsigned, CallTable::Instance()->SuccessfulCallCount());
-		}
-		if (token[1] == TraceLevelOIDStr + PString(".0")) {
-			return "GET_RESPONSE u " + PString(PString::Unsigned, PTrace::GetLevel());
-		}
-		if (token[1] == CatchAllOIDStr + PString(".0")) {
-			PString catchAllDest = GkConfig()->GetString("Routing::CatchAll", "CatchAllIP", "");
-			if (catchAllDest.IsEmpty())
-				catchAllDest = GkConfig()->GetString("Routing::CatchAll", "CatchAllAlias", "catchall");
-			return "GET_RESPONSE s " + catchAllDest;
-		}
-	} else if ((token.GetSize() == 3) && (token[0] == "SET")) {
-		if (token[1] == TraceLevelOIDStr + PString(".0")) {
-			PTrace::SetLevel(token[2].AsUnsigned());
-			return "SET_RESPONSE u " + PString(PString::Unsigned, PTrace::GetLevel());
-		}
-		if (token[1] == CatchAllOIDStr + PString(".0")) {
-			if (IsIPAddress(token[2])) {
-				Toolkit::Instance()->SetConfig(1, "Routing::CatchAll", "CatchAllIP", token[2]);
-				Toolkit::Instance()->SetConfig(1, "Routing::CatchAll", "CatchAllAlias", "");
-			} else {
-				Toolkit::Instance()->SetConfig(1, "Routing::CatchAll", "CatchAllIP", "");
-				Toolkit::Instance()->SetConfig(1, "Routing::CatchAll", "CatchAllAlias", token[2]);
-			}
-			ReloadHandler();
-			return "SET_RESPONSE s " + token[2];
-		}
-	}
-	return "ERROR";
-}
-
-void WindowsSNMPAgent::SendWindowsSNMPTrap(unsigned trapNumber, SNMPLevel severity, SNMPGroup group, const PString & msg)
-{
-	PTRACE(5, "SNMP\tSendSNMPTrap " << trapNumber << ", " << severity << ", " << group << ", " << msg);
-	if (!m_shutdown) {
-		PString trap = "TRAP " + PString(PString::Unsigned, trapNumber)
-						+ " " + PString(PString::Unsigned, severity)
-						+ " " + PString(PString::Unsigned, group)
-						+ " " + msg;
-		CallNamedPipe(trapPipename, (void *)trap.GetPointer(), trap.GetLength(), NULL, 0, NULL, 1000);
-	}
-}
-
-#endif // HAS_WINSNMP
-
-
-PCaselessString SelectSNMPImplementation() 
-{
-#ifdef HAS_NETSNMP
-	PCaselessString implementation = GkConfig()->GetString(SNMPSection, "Implementation", "Net-SNMP");
-#else
 	PCaselessString implementation = GkConfig()->GetString(SNMPSection, "Implementation", "PTLib");
-#endif
 
 	// switch to other implementation if only one is available
 #ifndef HAS_NETSNMP
@@ -658,21 +673,15 @@ PCaselessString SelectSNMPImplementation()
 		PTRACE(1, "SNMP\tPTLib implementation not available, using Net-SNMP implementation");
 		implementation = "Net-SNMP";
 	}
-#endif
-#ifndef HAS_WINSNMP
-	if (implementation == "Windows") {
-		PTRACE(1, "SNMP\tWindows implementation not available, using PTLib implementation");
-		implementation = "PTLib";
-	}
 #endif
 	return implementation;
 }
 
 void SendSNMPTrap(unsigned trapNumber, SNMPLevel severity, SNMPGroup group, const PString & msg)
 {
-	if ((severity == SNMPWarning)
-		&& !GkConfig()->GetBoolean(SNMPSection, "EnableWarningTraps", false)) {
-		// don't throw warning trap
+	if (!GkConfig()->GetBoolean(SNMPSection, "EnableTraps", true)
+        || ( (severity == SNMPWarning) && !GkConfig()->GetBoolean(SNMPSection, "EnableWarningTraps", false)) ) {
+		// don't throw trap
 		return;
 	}
 
@@ -687,14 +696,9 @@ void SendSNMPTrap(unsigned trapNumber, SNMPLevel severity, SNMPGroup group, cons
 		SendPTLibSNMPTrap(trapNumber, severity, group, msg);
 	}
 #endif
-#ifdef HAS_WINSNMP
-	if (implementation == "Windows") {
-		WindowsSNMPAgent::Instance()->SendWindowsSNMPTrap(trapNumber, severity, group, msg);
-	}
-#endif
 }
 
-void StartSNMPAgent() 
+void StartSNMPAgent()
 {
 	PCaselessString implementation = SelectSNMPImplementation();
 #ifdef HAS_NETSNMP
@@ -705,19 +709,19 @@ void StartSNMPAgent()
 #endif
 #ifdef HAS_PTLIBSNMP
 	if (implementation == "PTLib") {
-		new PTLibSNMPAgent();
-		return;
-	}
-#endif
-#ifdef HAS_WINSNMP
-	if (implementation == "Windows") {
-		CreateJob(WindowsSNMPAgent::Instance(), &WindowsSNMPAgent::Run, "SNMPAgent");
+		PTLibSNMPAgent * agent = new PTLibSNMPAgent();   // will be deleted when GnuGk shuts down
+		if (!agent->IsOpen()) {
+		    PTRACE(1, "SNMP\tFATAL: Error starting PTLib SNMP agent - shutting down");
+		    cout <<  "Error starting PTLib SNMP agent - shutting down" << endl;
+            RasServer::Instance()->Stop();
+            _exit(1);
+		}
 		return;
 	}
 #endif
 }
 
-void StopSNMPAgent() 
+void StopSNMPAgent()
 {
 #ifdef HAS_NETSNMP
 	if (NetSNMPAgent::InstanceExists()) {
@@ -727,14 +731,9 @@ void StopSNMPAgent()
 #ifdef HAS_PTLIBSNMP
 	// nothing to do
 #endif
-#ifdef HAS_WINSNMP
-	if (WindowsSNMPAgent::InstanceExists()) {
-		WindowsSNMPAgent::Instance()->Stop();
-	}
-#endif
 }
 
-void DeleteSNMPAgent() 
+void DeleteSNMPAgent()
 {
 #ifdef HAS_NETSNMP
 	if (NetSNMPAgent::InstanceExists()) {
@@ -742,15 +741,7 @@ void DeleteSNMPAgent()
 	}
 #endif
 #ifdef HAS_PTLIBSNMP
-	if (g_ptlibAgentPtr) {
-		delete g_ptlibAgentPtr;
-		g_ptlibAgentPtr = NULL;
-	}
-#endif
-#ifdef HAS_WINSNMP
-	if (WindowsSNMPAgent::InstanceExists()) {
-		delete WindowsSNMPAgent::Instance();
-	}
+    // nothing to delete
 #endif
 }
 
