@@ -4,7 +4,7 @@
  * SQL authentication/authorization modules for GNU Gatekeeper
  *
  * Copyright (c) 2004, Michal Zygmuntowicz
- * Copyright (c) 2006-2012, Jan Willamowius
+ * Copyright (c) 2006-2020, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -13,6 +13,7 @@
  *
  */
 
+#include "config.h"
 #include <ptlib.h>
 #include <h235.h>
 #include <h323pdu.h>
@@ -42,34 +43,36 @@ public:
 	/// build authenticator reading settings from the config
 	SQLPasswordAuth(
 		/// name for this authenticator and for the config section to read settings from
-		const char* authName
+		const char * authName
 		);
-	
+
 	virtual ~SQLPasswordAuth();
 
 	virtual PString GetInfo();
-	
+
 protected:
 	/** Override from SimplePasswordAuth.
-	
+
 	    @return
 	    True if the password has been found for the given alias.
 	*/
 	virtual bool GetPassword(
 		/// alias to check the password for
-		const PString& alias,
+		const PString & alias,
 		/// password string, if the match is found
-		PString& password
+		PString & password,
+		/// map of authentication parameters
+		std::map<PString, PString> & params
 		);
 
 private:
 	SQLPasswordAuth();
-	SQLPasswordAuth(const SQLPasswordAuth&);
-	SQLPasswordAuth& operator=(const SQLPasswordAuth&);
-	
+	SQLPasswordAuth(const SQLPasswordAuth &);
+	SQLPasswordAuth & operator=(const SQLPasswordAuth &);
+
 protected:
 	/// connection to the SQL database
-	GkSQLConnection* m_sqlConn;
+	GkSQLConnection * m_sqlConn;
 	/// parametrized query string for password retrieval
 	PString m_query;
 };
@@ -83,31 +86,31 @@ public:
 		/// name for this authenticator and for the config section to read settings from
 		const char* authName
 		);
-	
+
 	virtual ~SQLAliasAuth();
 
 	virtual PString GetInfo();
 
 protected:
-	/** Get auth condition string for the given alias. 
+	/** Get auth condition string for the given alias.
 	    This implementation searches the SQL database for the string.
 	    Override from AliasAuth.
-		
+
 	    @return
 	    The AliasAuth condition string for the given alias.
 	*/
 	virtual bool GetAuthConditionString(
 		/// an alias the condition string is to be retrieved for
-		const PString& alias,
+		const PString & alias,
 		/// filled with auth condition string that has been found
-		PString& authCond
+		PString & authCond
 		);
 
 private:
 	SQLAliasAuth();
-	SQLAliasAuth(const SQLAliasAuth&);
-	SQLAliasAuth& operator=(const SQLAliasAuth&);
-	
+	SQLAliasAuth(const SQLAliasAuth &);
+	SQLAliasAuth & operator=(const SQLAliasAuth &);
+
 protected:
 	/// connection to the SQL database
 	GkSQLConnection* m_sqlConn;
@@ -125,7 +128,7 @@ public:
 			| RasInfo<H225_LocationRequest>::flag,
 		SQLAuthMiscChecks = e_Setup | e_SetupUnreg
 	};
-	
+
 	/// build authenticator reading settings from the config
 	SQLAuth(
 		/// name for this authenticator and for the config section to read settings from
@@ -135,61 +138,58 @@ public:
 		/// Misc check events supported by this module
 		unsigned supportedMiscChecks = SQLAuthMiscChecks
 		);
-	
+
 	virtual ~SQLAuth();
 
 	/** Authenticate using data from RRQ RAS message.
-	
+
 		@return:
 		#GkAuthenticator::Status enum# with the result of authentication.
 	*/
 	virtual int Check(
 		/// RRQ RAS message to be authenticated
-		RasPDU<H225_RegistrationRequest>& rrqPdu, 
+		RasPDU<H225_RegistrationRequest> & rrqPdu,
 		/// authorization data (reject reason, ...)
-		RRQAuthData& authData
+		RRQAuthData & authData
 		);
-		
+
 	/** Authenticate using data from ARQ RAS message.
-	
+
 		@return:
 		#GkAuthenticator::Status enum# with the result of authentication.
 	*/
 	virtual int Check(
-		/// ARQ nessage to be authenticated
-		RasPDU<H225_AdmissionRequest> & arqPdu, 
+		/// ARQ message to be authenticated
+		RasPDU<H225_AdmissionRequest> & arqPdu,
 		/// authorization data (call duration limit, reject reason, ...)
-		ARQAuthData& authData
+		ARQAuthData & authData
 		);
 
 	/** Authenticate using data from LRQ RAS message.
-	
+
 		@return:
 		#GkAuthenticator::Status enum# with the result of authentication.
 	*/
-	virtual int Check(
-		RasPDU<H225_LocationRequest>& req,
-		unsigned& rejectReason
-		);
-		
+	virtual int Check(RasPDU<H225_LocationRequest> & req, unsigned & rejectReason);
+
 	/** Authenticate using data from Q.931 Setup message.
-	
+
 		@return:
 		#GkAuthenticator::Status enum# with the result of authentication.
 	*/
 	virtual int Check(
 		/// Q.931/H.225 Setup message to be authenticated
-		SetupMsg &setup,
+		SetupMsg & setup,
 		/// authorization data (call duration limit, reject reason, ...)
-		SetupAuthData& authData
+		SetupAuthData & authData
 		);
 
 	virtual PString GetInfo();
 
 private:
 	SQLAuth();
-	SQLAuth(const SQLAuth&);
-	SQLAuth& operator=(const SQLAuth&);
+	SQLAuth(const SQLAuth &);
+	SQLAuth & operator=(const SQLAuth &);
 
 protected:
 	/// connection to the SQL database
@@ -216,19 +216,19 @@ bool RunQuery(
 	)
 {
 	resultRow.clear();
-	
+
 	if (conn == NULL) {
 		PTRACE(2, traceStr << ": query failed - SQL connection not active");
 		SNMP_TRAP(5, SNMPError, Authentication, "SQL connection failed");
 		return false;
 	}
-	
+
 	if (query.IsEmpty()) {
 		PTRACE(2, traceStr << ": query failed - query string not configured");
 		SNMP_TRAP(5, SNMPError, Authentication, "SQL connection failed");
 		return false;
 	}
-	
+
 	GkSQLResult* result = conn->ExecuteQuery(query, params, timeout);
 	if (result == NULL) {
 		PTRACE(2, traceStr << ": query failed - timeout or fatal error");
@@ -243,7 +243,7 @@ bool RunQuery(
 		delete result;
 		return false;
 	}
-	
+
 	if (result->GetNumRows() < 1)
 		PTRACE(3, traceStr << ": query returned no rows");
 	else if (result->GetNumFields() < 1) {
@@ -262,37 +262,31 @@ bool RunQuery(
 	return false;
 }
 
-inline GkSQLResult::ResultRow::iterator FindField(
-	GkSQLResult::ResultRow& result,
-	const PString& fieldName
-	)
+inline GkSQLResult::ResultRow::iterator FindField( GkSQLResult::ResultRow & result, const PString & fieldName)
 {
 	GkSQLResult::ResultRow::iterator i = result.begin();
 	while (i != result.end() && i->second != fieldName)
-		i++;
+		++i;
 	return i;
 }
 
 } /* namespace */
 
 
-SQLPasswordAuth::SQLPasswordAuth(
-	const char* authName
-	)
+SQLPasswordAuth::SQLPasswordAuth(const char * authName)
 	: SimplePasswordAuth(authName), m_sqlConn(NULL)
 {
 	PConfig* cfg = GetConfig();
 
 	const PString driverName = cfg->GetString(authName, "Driver", "");
 	if (driverName.IsEmpty()) {
-		PTRACE(0, "SQLAUTH\t" << GetName() << " module creation failed: "
-			"no SQL driver selected");
+		PTRACE(0, "SQLAUTH\t" << GetName() << " module creation failed: no SQL driver selected");
 		SNMP_TRAP(4, SNMPError, Authentication, GetName() + " creation failed");
 		PTRACE(0, "SQLAUTH\tFATAL: Shutting down");
 		RasServer::Instance()->Stop();
 		return;
 	}
-	
+
 	m_sqlConn = GkSQLConnection::Create(driverName, authName);
 	if (m_sqlConn == NULL) {
 		PTRACE(0, "SQLAUTH\t" << GetName() << " module creation failed: "
@@ -304,21 +298,19 @@ SQLPasswordAuth::SQLPasswordAuth(
 	}
 
 	SetCacheTimeout(cfg->GetInteger(authName, "CacheTimeout", 0));
-		
+
 	m_query = cfg->GetString(authName, "Query", "");
 	if (m_query.IsEmpty()) {
-		PTRACE(0, "SQLAUTH\t" << GetName() << " module creation failed: "
-			"no query configured");
+		PTRACE(0, "SQLAUTH\t" << GetName() << " module creation failed: no query configured");
 		SNMP_TRAP(4, SNMPError, Authentication, GetName() + " creation failed");
 		PTRACE(0, "SQLAUTH\tFATAL: Shutting down");
 		RasServer::Instance()->Stop();
 		return;
 	} else
 		PTRACE(4, "SQLAUTH\t" << GetName() << " query: " << m_query);
-		
+
 	if (!m_sqlConn->Initialize(cfg, authName)) {
-		PTRACE(0, "SQLAUTH\t" << GetName() << " module creation failed: "
-			"could not connect to the database");
+		PTRACE(0, "SQLAUTH\t" << GetName() << " module creation failed: could not connect to the database");
 		SNMP_TRAP(5, SNMPError, Authentication, GetName() + " creation failed");
 		return;
 	}
@@ -329,21 +321,15 @@ SQLPasswordAuth::~SQLPasswordAuth()
 	delete m_sqlConn;
 }
 
-bool SQLPasswordAuth::GetPassword(
-	const PString& alias,
-	PString& password
-	)
+bool SQLPasswordAuth::GetPassword(const PString & alias, PString & password, std::map<PString, PString> & params)
 {
 	GkSQLResult::ResultRow result;
-	std::map<PString, PString> params;
 	params["1"] = alias;
-	params["u"] = alias;
 	params["2"] = Toolkit::GKName();
-	params["g"] = Toolkit::GKName();
 
 	if (!RunQuery("SQLAUTH\t" + GetName() + "('" + alias + "')", m_sqlConn, m_query, params, result, -1))
 		return false;
-		
+
 	password = result[0].first;
 	return true;
 }
@@ -351,7 +337,7 @@ bool SQLPasswordAuth::GetPassword(
 PString SQLPasswordAuth::GetInfo()
 {
 	PString result;
-	
+
 	if (m_sqlConn == NULL)
 		result += "  No SQL connection available\r\n";
 	else {
@@ -364,29 +350,26 @@ PString SQLPasswordAuth::GetInfo()
 		result += "  Busy Connections::           " + PString(info.m_busyConnections) + "\r\n";
 		result += "  Waiting Requests:            " + PString(info.m_waitingRequests) + "\r\n";
 	}
-	
+
 	result += ";\r\n";
-	
+
 	return result;
 }
 
-SQLAliasAuth::SQLAliasAuth(
-	const char* authName
-	)
+SQLAliasAuth::SQLAliasAuth(const char * authName)
 	: AliasAuth(authName), m_sqlConn(NULL)
 {
 	PConfig* cfg = GetConfig();
 
 	const PString driverName = cfg->GetString(authName, "Driver", "");
 	if (driverName.IsEmpty()) {
-		PTRACE(0, "SQLAUTH\t" << GetName() << " module creation failed: "
-			"no SQL driver selected");
+		PTRACE(0, "SQLAUTH\t" << GetName() << " module creation failed: no SQL driver selected");
 		SNMP_TRAP(5, SNMPError, Authentication, GetName() + " creation failed");
 		PTRACE(0, "SQLAUTH\tFATAL: Shutting down");
 		RasServer::Instance()->Stop();
 		return;
 	}
-	
+
 	m_sqlConn = GkSQLConnection::Create(driverName, authName);
 	if (m_sqlConn == NULL) {
 		PTRACE(0, "SQLAUTH\t" << GetName() << " module creation failed: "
@@ -398,18 +381,17 @@ SQLAliasAuth::SQLAliasAuth(
 	}
 
 	SetCacheTimeout(cfg->GetInteger(authName, "CacheTimeout", 0));
-	
+
 	m_query = cfg->GetString(authName, "Query", "");
 	if (m_query.IsEmpty()) {
-		PTRACE(1, "SQLAUTH\t" << GetName() << " module creation failed: "
-			"no query configured");
+		PTRACE(1, "SQLAUTH\t" << GetName() << " module creation failed: no query configured");
 		SNMP_TRAP(5, SNMPError, Authentication, GetName() + " creation failed");
 		PTRACE(0, "SQLAUTH\tFATAL: Shutting down");
 		RasServer::Instance()->Stop();
 		return;
 	} else
 		PTRACE(4, "SQLAUTH\t" << GetName() << " query: " << m_query);
-		
+
 	if (!m_sqlConn->Initialize(cfg, authName)) {
 		PTRACE(0, "SQLAUTH\t" << GetName() << " module creation failed: "
 			"could not connect to the database");
@@ -423,10 +405,7 @@ SQLAliasAuth::~SQLAliasAuth()
 	delete m_sqlConn;
 }
 
-bool SQLAliasAuth::GetAuthConditionString(
-	const PString& alias,
-	PString& authCond
-	)
+bool SQLAliasAuth::GetAuthConditionString(const PString & alias, PString & authCond)
 {
 	GkSQLResult::ResultRow result;
 	std::map<PString, PString> params;
@@ -437,7 +416,7 @@ bool SQLAliasAuth::GetAuthConditionString(
 
 	if (!RunQuery("SQLAUTH\t" + GetName() + "('" + alias + "')", m_sqlConn, m_query, params, result, -1))
 		return false;
-		
+
 	authCond = result[0].first;
 	return true;
 }
@@ -445,7 +424,7 @@ bool SQLAliasAuth::GetAuthConditionString(
 PString SQLAliasAuth::GetInfo()
 {
 	PString result;
-	
+
 	if (m_sqlConn == NULL)
 		result += "  No SQL connection available\r\n";
 	else {
@@ -458,33 +437,26 @@ PString SQLAliasAuth::GetInfo()
 		result += "  Busy Connections::           " + PString(info.m_busyConnections) + "\r\n";
 		result += "  Waiting Requests:            " + PString(info.m_waitingRequests) + "\r\n";
 	}
-	
+
 	result += ";\r\n";
-	
+
 	return result;
 }
 
-SQLAuth::SQLAuth(
-	const char* authName,
-	unsigned supportedRasChecks,
-	unsigned supportedMiscChecks
-	) 
-	: 
-	GkAuthenticator(authName, supportedRasChecks, supportedMiscChecks), 
-	m_sqlConn(NULL)
+SQLAuth::SQLAuth(const char* authName, unsigned supportedRasChecks, unsigned supportedMiscChecks)
+    : GkAuthenticator(authName, supportedRasChecks, supportedMiscChecks), m_sqlConn(NULL)
 {
 	PConfig* cfg = GetConfig();
 
 	const PString driverName = cfg->GetString(authName, "Driver", "");
 	if (driverName.IsEmpty()) {
-		PTRACE(0, "SQLAUTH\t" << GetName() << " module creation failed: "
-			"no SQL driver selected");
+		PTRACE(0, "SQLAUTH\t" << GetName() << " module creation failed: no SQL driver selected");
 		SNMP_TRAP(5, SNMPError, Authentication, GetName() + " creation failed");
 		PTRACE(0, "SQLAUTH\tFATAL: Shutting down");
 		RasServer::Instance()->Stop();
 		return;
 	}
-	
+
 	m_sqlConn = GkSQLConnection::Create(driverName, authName);
 	if (m_sqlConn == NULL) {
 		PTRACE(0, "SQLAUTH\t" << GetName() << " module creation failed: "
@@ -497,8 +469,7 @@ SQLAuth::SQLAuth(
 
 	m_regQuery = cfg->GetString(authName, "RegQuery", "");
 	if (m_regQuery.IsEmpty() && IsRasCheckEnabled(RasInfo<H225_RegistrationRequest>::flag)) {
-		PTRACE(1, "SQLAUTH\t" << GetName() << " module creation failed: "
-			"no RRQ query configured");
+		PTRACE(1, "SQLAUTH\t" << GetName() << " module creation failed: no RRQ query configured");
 		SNMP_TRAP(5, SNMPError, Authentication, GetName() + " creation failed");
 		PTRACE(0, "SQLAUTH\tFATAL: Shutting down");
 		RasServer::Instance()->Stop();
@@ -509,8 +480,7 @@ SQLAuth::SQLAuth(
 
 	m_nbQuery = cfg->GetString(authName, "NbQuery", "");
 	if (m_nbQuery.IsEmpty() && IsRasCheckEnabled(RasInfo<H225_LocationRequest>::flag)) {
-		PTRACE(1, "SQLAUTH\t" << GetName() << " module creation failed: "
-			"no LRQ query configured");
+		PTRACE(1, "SQLAUTH\t" << GetName() << " module creation failed: no LRQ query configured");
 		SNMP_TRAP(5, SNMPError, Authentication, GetName() + " creation failed");
 		PTRACE(0, "SQLAUTH\tFATAL: Shutting down");
 		RasServer::Instance()->Stop();
@@ -522,8 +492,7 @@ SQLAuth::SQLAuth(
 	m_callQuery = cfg->GetString(authName, "CallQuery", "");
 	if (m_callQuery.IsEmpty() && (IsRasCheckEnabled(RasInfo<H225_AdmissionRequest>::flag)
 			|| IsMiscCheckEnabled(e_Setup) || IsMiscCheckEnabled(e_SetupUnreg))) {
-		PTRACE(1, "SQLAUTH\t" << GetName() << " module creation failed: "
-			"no ARQ/Setup query configured");
+		PTRACE(1, "SQLAUTH\t" << GetName() << " module creation failed: no ARQ/Setup query configured");
 		SNMP_TRAP(5, SNMPError, Authentication, GetName() + " creation failed");
 		PTRACE(0, "SQLAUTH\tFATAL: Shutting down");
 		RasServer::Instance()->Stop();
@@ -533,8 +502,7 @@ SQLAuth::SQLAuth(
 	}
 
 	if (!m_sqlConn->Initialize(cfg, authName)) {
-		PTRACE(0, "SQLAUTH\t" << GetName() << " module creation failed: "
-			"could not connect to the database");
+		PTRACE(0, "SQLAUTH\t" << GetName() << " module creation failed: could not connect to the database");
 		SNMP_TRAP(5, SNMPError, Authentication, GetName() + " creation failed");
 		return;
 	}
@@ -547,28 +515,29 @@ SQLAuth::~SQLAuth()
 
 int SQLAuth::Check(
 	/// RRQ RAS message to be authenticated
-	RasPDU<H225_RegistrationRequest>& rrqPdu, 
+	RasPDU<H225_RegistrationRequest> & rrqPdu,
 	/// authorization data (reject reason, ...)
-	RRQAuthData& authData
+	RRQAuthData & authData
 	)
 {
-	H225_RegistrationRequest &rrq = rrqPdu;
+	H225_RegistrationRequest & rrq = rrqPdu;
 	std::map<PString, PString> params;
-	
-	// get the username for User-Name attribute		
-	params["u"] = GetUsername(rrqPdu);
-	params["g"] = Toolkit::GKName();
 
-	// Whether a full or additive registration
-	params["additive-rrq"] = rrq.HasOptionalField(H225_RegistrationRequest::e_additiveRegistration) ? 1 : 0;
-	
 	PIPSocket::Address addr = (rrqPdu.operator->())->m_peerAddr;
 
 	const PString traceStr = "SQLAUTH\t" + GetName() + "(RRQ from "
-		+ addr.AsString() + " Username=" + params["u"]
-		+ ")";
+		+ addr.AsString() + " Username=" + params["u"] + ")";
+
+	// get the username for User-Name attribute
+	params["u"] = GetUsername(rrqPdu);
+	params["g"] = Toolkit::GKName();
+	params["Calling-Station-Id"] = GetCallingStationId(rrqPdu, authData);
+
+	// Whether a full or additive registration
+	params["additive-rrq"] = rrq.HasOptionalField(H225_RegistrationRequest::e_additiveRegistration) ? 1 : 0;
+
 	params["callerip"] = addr.AsString();
-	
+
 	addr = (rrqPdu.operator->())->m_localAddr;
 	params["gkip"] = addr.AsString();
 
@@ -581,8 +550,17 @@ int SQLAuth::Check(
 		}
 		params["aliases"] = aliasList;
 	}
+    params["Vendor"] = "";
+    if (rrq.m_endpointVendor.HasOptionalField(H225_EndpointType::e_vendor)) {
+        if (rrq.m_endpointVendor.HasOptionalField(H225_VendorIdentifier::e_productId)) {
+            params["Vendor"] += rrq.m_endpointVendor.m_productId.AsString();
+        }
+        if (rrq.m_endpointVendor.HasOptionalField(H225_VendorIdentifier::e_versionId)) {
+            params["Vendor"] += rrq.m_endpointVendor.m_versionId.AsString();
+        }
+    }
 
-	GkSQLResult::ResultRow result;	
+	GkSQLResult::ResultRow result;
 	if (!RunQuery(traceStr, m_sqlConn, m_regQuery, params, result, -1)) {
 		authData.m_rejectReason = H225_RegistrationRejectReason::e_resourceUnavailable;
 		return GetDefaultStatus();
@@ -599,16 +577,14 @@ int SQLAuth::Check(
 	if (iter != result.end()) {
 		const PString &s = iter->first;
 		if (!s) {
-			if (strspn((const char*)s,"0123456789.") == (size_t)s.GetLength()) {
+			if (strspn((const char*)s, "0123456789.") == (size_t)s.GetLength()) {
 				const int intVal = s.AsInteger();
 				if (intVal == 0)
 					authData.m_billingMode = H225_CallCreditServiceControl_billingMode::e_credit;
 				else if (intVal == 1 || intVal == 2)
 					authData.m_billingMode = H225_CallCreditServiceControl_billingMode::e_debit;
 			} else {
-				PTRACE(3, traceStr << " - invalid billingmode attribute '"
-					<< s << '\''
-					);
+				PTRACE(3, traceStr << " - invalid billingmode attribute '" << s << '\'');
 			}
 		}
 	}
@@ -622,7 +598,7 @@ int SQLAuth::Check(
 	iter = FindField(result, "aliases");
 	if (iter != result.end()) {
 		PStringArray aliases = iter->first.Tokenise(",");
-		if (aliases.GetSize() > 0 
+		if (aliases.GetSize() > 0
 				&& rrq.HasOptionalField(H225_RegistrationRequest::e_terminalAlias)) {
 			PINDEX i = 0;
 			while (i < rrq.m_terminalAlias.GetSize()) {
@@ -645,31 +621,29 @@ int SQLAuth::Check(
 			H323SetAliasAddress(aliases[i], rrq.m_terminalAlias[rrq.m_terminalAlias.GetSize()-1]);
 		}
 	}
-			
+
 	return e_ok;
 }
-		
+
 int SQLAuth::Check(
-	/// ARQ nessage to be authenticated
-	RasPDU<H225_AdmissionRequest> & arqPdu, 
+	/// ARQ message to be authenticated
+	RasPDU<H225_AdmissionRequest> & arqPdu,
 	/// authorization data (call duration limit, reject reason, ...)
-	ARQAuthData& authData
-	)
+	ARQAuthData & authData)
 {
-	const H225_AdmissionRequest &arq = arqPdu;
+	const H225_AdmissionRequest & arq = arqPdu;
 	std::map<PString, PString> params;
-	
+
 	PIPSocket::Address addr = (arqPdu.operator->())->m_peerAddr;
 
 	const PString traceStr = "SQLAUTH\t" + GetName() + "(ARQ from "
-		+ addr.AsString() + " CRV=" + PString(arq.m_callReferenceValue.GetValue() & 0x7fff)
-		+ ")";
+		+ addr.AsString() + " CRV=" + PString(arq.m_callReferenceValue.GetValue() & 0x7fff) + ")";
 	params["callerip"] = addr.AsString();
-		
-	// get the username for User-Name attribute		
+
+	// get the username for User-Name attribute
 	params["u"] = GetUsername(arqPdu, authData);
 	params["g"] = Toolkit::GKName();
-	
+
 	addr = (arqPdu.operator->())->m_localAddr;
 	params["gkip"] = addr.AsString();
 
@@ -679,9 +653,24 @@ int SQLAuth::Check(
 	params["bandwidth"] = PString(arq.m_bandWidth.GetValue());
 	params["answer"] = arq.m_answerCall ? "1" : "0";
 	params["arq"] = "1";
-	params["CallId"] = AsString(arq.m_callIdentifier.m_guid);
-	
-	GkSQLResult::ResultRow result;	
+	params["from-registered"] = "1";
+	params["from-neighbor"] = "0";
+	params["CallId"] = AsString(arq.m_callIdentifier);
+	params["SrcInfo"] = AsString(arq.m_srcInfo, false);
+    params["Vendor"] = "";
+    endptr ep = RegistrationTable::Instance()->FindByEndpointId(arq.m_endpointIdentifier);
+    if (ep) {
+		if (ep->GetEndpointType().HasOptionalField(H225_EndpointType::e_vendor)) {
+			if (ep->GetEndpointType().m_vendor.HasOptionalField(H225_VendorIdentifier::e_productId)) {
+				params["Vendor"] += ep->GetEndpointType().m_vendor.m_productId.AsString();
+			}
+			if (ep->GetEndpointType().m_vendor.HasOptionalField(H225_VendorIdentifier::e_versionId)) {
+				params["Vendor"] += ep->GetEndpointType().m_vendor.m_versionId.AsString();
+			}
+        }
+    }
+
+	GkSQLResult::ResultRow result;
 	if (!RunQuery(traceStr, m_sqlConn, m_callQuery, params, result, -1)) {
 		authData.m_rejectReason = H225_AdmissionRejectReason::e_resourceUnavailable;
 		return GetDefaultStatus();
@@ -696,16 +685,14 @@ int SQLAuth::Check(
 	if (iter != result.end()) {
 		const PString &s = iter->first;
 		if (!s) {
-			if (strspn((const char*)s,"0123456789.") == (size_t)s.GetLength()) {
+			if (strspn((const char*)s, "0123456789.") == (size_t)s.GetLength()) {
 				const int intVal = s.AsInteger();
 				if (intVal == 0)
 					authData.m_billingMode = H225_CallCreditServiceControl_billingMode::e_credit;
 				else if (intVal == 1 || intVal == 2)
 					authData.m_billingMode = H225_CallCreditServiceControl_billingMode::e_debit;
 			} else {
-				PTRACE(3, traceStr << " - invalid billingmode attribute '"
-					<< s << '\''
-					);
+				PTRACE(3, traceStr << " - invalid billingmode attribute '" << s << '\'');
 			}
 		}
 	}
@@ -753,16 +740,16 @@ int SQLAuth::Check(
 			PTRACE(5, traceStr << " - call redirected to the number " << s);
 		}
 	}
-			
+
 	iter = FindField(result, "redirectip");
 	if (iter != result.end()) {
-		const PString &s = iter->first;
+		const PString & s = iter->first;
 		if (!s) {
 			PStringArray tokens(s.Tokenise("; \t", FALSE));
 			for (PINDEX i = 0; i < tokens.GetSize(); ++i) {
 				PIPSocket::Address raddr;
 				WORD port = 0;
-					
+
 				if (GetTransportAddress(tokens[i], GK_DEF_ENDPOINT_SIGNAL_PORT, raddr, port)
 						&& raddr.IsValid() && port != 0) {
 					Route route("SQLAuth", raddr, port);
@@ -778,9 +765,7 @@ int SQLAuth::Check(
 						}
 					}
 					authData.m_destinationRoutes.push_back(route);
-					PTRACE(5, traceStr << " - call redirected to the address " <<
-						route.AsString()
-						);
+					PTRACE(5, traceStr << " - call redirected to the address " << route.AsString());
 				}
 			}
 		}
@@ -788,13 +773,12 @@ int SQLAuth::Check(
 
 	iter = FindField(result, "proxy");
 	if (iter != result.end()) {
-		const PString &s = iter->first;
+		const PString & s = iter->first;
 		if (!s) {
 			authData.m_proxyMode = Toolkit::AsBool(s)
 				? CallRec::ProxyEnabled : CallRec::ProxyDisabled;
 			PTRACE(5, traceStr << " - proxy mode "
-				<< (authData.m_proxyMode == CallRec::ProxyEnabled ? "enabled" : "disabled")
-				);
+				<< (authData.m_proxyMode == CallRec::ProxyEnabled ? "enabled" : "disabled"));
 		}
 	}
 
@@ -807,26 +791,22 @@ int SQLAuth::Check(
 			PTRACE(5, traceStr << " - clientAuthId = " << authData.m_clientAuthId);
 		}
 	}
-	
+
 	return e_ok;
 }
 
-int SQLAuth::Check(
-	RasPDU<H225_LocationRequest>& lrqPdu,
-	unsigned& rejectReason
-	)
+int SQLAuth::Check(RasPDU<H225_LocationRequest> & lrqPdu, unsigned & rejectReason)
 {
-	H225_LocationRequest &lrq = lrqPdu;
+	H225_LocationRequest & lrq = lrqPdu;
 	std::map<PString, PString> params;
-	
+
 	PIPSocket::Address addr = (lrqPdu.operator->())->m_peerAddr;
 
-	const PString traceStr = "SQLAUTH\t" + GetName() + "(LRQ from "
-		+ addr.AsString() + ")";
+	const PString traceStr = "SQLAUTH\t" + GetName() + "(LRQ from " + addr.AsString() + ")";
 	params["nbip"] = addr.AsString();
 	params["nbid"] = RasServer::Instance()->GetNeighbors()->GetNeighborIdBySigAdr(addr);
 	params["g"] = Toolkit::GKName();
-	
+
 	addr = (lrqPdu.operator->())->m_localAddr;
 	params["gkip"] = addr.AsString();
 
@@ -834,24 +814,21 @@ int SQLAuth::Check(
 		params["u"] = GetBestAliasAddressString(lrq.m_sourceInfo, false,
 			AliasAddressTagMask(H225_AliasAddress::e_h323_ID),
 			AliasAddressTagMask(H225_AliasAddress::e_email_ID)
-				| AliasAddressTagMask(H225_AliasAddress::e_url_ID)
-			);
+				| AliasAddressTagMask(H225_AliasAddress::e_url_ID));
 		params["Calling-Station-Id"] = GetBestAliasAddressString(lrq.m_sourceInfo,
 			false, AliasAddressTagMask(H225_AliasAddress::e_dialedDigits)
-				| AliasAddressTagMask(H225_AliasAddress::e_partyNumber)
-			);
+				| AliasAddressTagMask(H225_AliasAddress::e_partyNumber));
 		params["src-info"] = AsString(lrq.m_sourceInfo);
 	}
 	params["Called-Station-Id"] = GetBestAliasAddressString(lrq.m_destinationInfo,
 		false, AliasAddressTagMask(H225_AliasAddress::e_dialedDigits)
-			| AliasAddressTagMask(H225_AliasAddress::e_partyNumber)
-		);
+			| AliasAddressTagMask(H225_AliasAddress::e_partyNumber));
 	params["dest-info"] = AsString(lrq.m_destinationInfo);
 
 	if (lrq.HasOptionalField(H225_LocationRequest::e_bandWidth))
 		params["bandwidth"] = PString(lrq.m_bandWidth.GetValue());
 
-	GkSQLResult::ResultRow result;	
+	GkSQLResult::ResultRow result;
 	if (!RunQuery(traceStr, m_sqlConn, m_nbQuery, params, result, -1)) {
 		rejectReason = H225_LocationRejectReason::e_resourceUnavailable;
 		return GetDefaultStatus();
@@ -883,31 +860,30 @@ int SQLAuth::Check(
 			H323SetAliasAddress(aliases[i], lrq.m_destinationInfo[sz]);
 		}
 	}
-			
+
 	return e_ok;
 }
-	
+
 int SQLAuth::Check(
 	/// Q.931/H.225 Setup message to be authenticated
-	SetupMsg &setup,
+	SetupMsg & setup,
 	/// authorization data (call duration limit, reject reason, ...)
-	SetupAuthData& authData
+	SetupAuthData & authData
 	)
 {
 	std::map<PString, PString> params;
-	
+
 	PIPSocket::Address addr;
 	setup.GetPeerAddr(addr);
 
 	const PString traceStr = "SQLAUTH\t" + GetName() + "(Setup from "
-		+ addr.AsString() + " CRV=" + PString(setup.GetQ931().GetCallReference())
-		+ ")";
+		+ addr.AsString() + " CRV=" + PString(setup.GetQ931().GetCallReference()) + ")";
 	params["callerip"] = addr.AsString();
-		
-	// get the username for User-Name attribute		
+
+	// get the username for User-Name attribute
 	params["u"] = GetUsername(setup, authData);
 	params["g"] = Toolkit::GKName();
-	
+
 	setup.GetLocalAddr(addr);
 	params["gkip"] = addr.AsString();
 
@@ -916,12 +892,31 @@ int SQLAuth::Check(
 	params["Dialed-Number"] = GetDialedNumber(setup, authData);
 	params["answer"] = "0";
 	params["arq"] = "0";
-	params["CallId"] = AsString(setup.GetUUIEBody().m_callIdentifier.m_guid);
+	params["from-registered"] = (authData.m_call && authData.m_call->GetCallingParty()) ? "1" : "0";
+	params["from-neighbor"] = authData.m_fromNeighbor ? "1" : "0";
+	params["CallId"] = AsString(setup.GetUUIEBody().m_callIdentifier);
+    params["SrcInfo"] = "";
+	if (setup.GetUUIEBody().HasOptionalField(H225_Setup_UUIE::e_sourceAddress)
+        && setup.GetUUIEBody().m_sourceAddress.GetSize() > 0) {
+        params["SrcInfo"] = AsString(setup.GetUUIEBody().m_sourceAddress, false);
+	}
+    params["Vendor"] = "";
+    if (setup.GetUUIEBody().m_sourceInfo.HasOptionalField(H225_EndpointType::e_vendor)) {
+        if (setup.GetUUIEBody().m_sourceInfo.m_vendor.HasOptionalField(H225_VendorIdentifier::e_productId)) {
+            params["Vendor"] += setup.GetUUIEBody().m_sourceInfo.m_vendor.m_productId.AsString();
+        }
+        if (setup.GetUUIEBody().m_sourceInfo.m_vendor.HasOptionalField(H225_VendorIdentifier::e_versionId)) {
+            params["Vendor"] += setup.GetUUIEBody().m_sourceInfo.m_vendor.m_versionId.AsString();
+        }
+    }
 
-	if (authData.m_call)
+	if (authData.m_call) {
 		params["bandwidth"] = PString(authData.m_call->GetBandwidth());
+	} else {
+		params["bandwidth"] = "";
+	}
 
-	GkSQLResult::ResultRow result;	
+	GkSQLResult::ResultRow result;
 	if (!RunQuery(traceStr, m_sqlConn, m_callQuery, params, result, -1)) {
 		authData.m_rejectCause = Q931::TemporaryFailure;
 		return GetDefaultStatus();
@@ -957,7 +952,7 @@ int SQLAuth::Check(
 	// check for extra fields for accepted calls
 	GkSQLResult::ResultRow::const_iterator iter = FindField(result, "credittime");
 	if (iter != result.end()) {
-		const PString &s = iter->first;
+		const PString & s = iter->first;
 		if (s.GetLength() > 0
 			&& strspn((const char*)s, "0123456789") == (size_t)s.GetLength()) {
 			PUInt64 limit = s.AsUnsigned64();
@@ -978,7 +973,7 @@ int SQLAuth::Check(
 	PStringArray numbersToDial;
 	iter = FindField(result, "redirectnumber");
 	if (iter != result.end()) {
-		const PString &s = iter->first;
+		const PString & s = iter->first;
 		if (!s) {
 			numbersToDial = s.Tokenise("; \t", FALSE);
 			if (numbersToDial.GetSize() > 0) {
@@ -991,16 +986,16 @@ int SQLAuth::Check(
 			}
 		}
 	}
-			
+
 	iter = FindField(result, "redirectip");
 	if (iter != result.end()) {
-		const PString &s = iter->first;
+		const PString & s = iter->first;
 		if (!s) {
 			PStringArray tokens(s.Tokenise("; \t", FALSE));
 			for (PINDEX i = 0; i < tokens.GetSize(); ++i) {
 				PIPSocket::Address raddr;
 				WORD port = 0;
-					
+
 				if (GetTransportAddress(tokens[i], GK_DEF_ENDPOINT_SIGNAL_PORT, raddr, port)
 						&& raddr.IsValid() && port != 0) {
 					Route route("SQLAuth", raddr, port);
@@ -1016,9 +1011,7 @@ int SQLAuth::Check(
 						}
 					}
 					authData.m_destinationRoutes.push_back(route);
-					PTRACE(5, traceStr << " - call redirected to the address " <<
-						route.AsString()
-						);
+					PTRACE(5, traceStr << " - call redirected to the address " << route.AsString());
 				}
 			}
 		}
@@ -1026,13 +1019,12 @@ int SQLAuth::Check(
 
 	iter = FindField(result, "proxy");
 	if (iter != result.end()) {
-		const PString &s = iter->first;
+		const PString & s = iter->first;
 		if (!s) {
 			authData.m_proxyMode = Toolkit::AsBool(s)
 				? CallRec::ProxyEnabled : CallRec::ProxyDisabled;
 			PTRACE(5, traceStr << " - proxy mode "
-				<< (authData.m_proxyMode == CallRec::ProxyEnabled ? "enabled" : "disabled")
-				);
+				<< (authData.m_proxyMode == CallRec::ProxyEnabled ? "enabled" : "disabled"));
 		}
 	}
 
@@ -1052,7 +1044,7 @@ int SQLAuth::Check(
 PString SQLAuth::GetInfo()
 {
 	PString result;
-	
+
 	if (m_sqlConn == NULL)
 		result += "  No SQL connection available\r\n";
 	else {
@@ -1065,9 +1057,9 @@ PString SQLAuth::GetInfo()
 		result += "  Busy Connections::           " + PString(info.m_busyConnections) + "\r\n";
 		result += "  Waiting Requests:            " + PString(info.m_waitingRequests) + "\r\n";
 	}
-	
+
 	result += ";\r\n";
-	
+
 	return result;
 }
 
