@@ -1,9 +1,9 @@
 /*
  * statusacct.h
  *
- * accounting module for GNU Gatekeeper that send it's output to the status port. 
+ * accounting module for GNU Gatekeeper that sends it's output to the status port.
  *
- * Copyright (c) 2005-2010, Jan Willamowius
+ * Copyright (c) 2005-2018, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -13,7 +13,7 @@
  */
 
 #ifndef __STATUSACCT_H
-#define __STATUSACCT_H "@(#) $Id: statusacct.h,v 1.6 2010/10/25 15:01:51 willamowius Exp $"
+#define __STATUSACCT_H "@(#) $Id$"
 
 #include "gkacct.h"
 
@@ -28,49 +28,40 @@ public:
 	enum Constants
 	{
 		/// events recognized by this module
-		StatusAcctEvents = AcctStart | AcctStop | AcctUpdate | AcctConnect | AcctAlert | AcctRegister | AcctUnregister
+		StatusAcctEvents = AcctStart | AcctStop | AcctUpdate | AcctConnect | AcctAlert | AcctRegister | AcctUnregister | AcctReject | AcctMediaFail
 	};
 
-	StatusAcct( 
+	StatusAcct(
 		/// name from Gatekeeper::Acct section
 		const char* moduleName,
 		/// config section name to be used with an instance of this module,
 		/// pass NULL to use a default section (named "moduleName")
 		const char* cfgSecName = NULL
 		);
-		
+
 	/// Destroy the accounting logger
 	virtual ~StatusAcct();
 
-	/// overriden from GkAcctLogger
-	virtual Status Log(
-		AcctEvent evt,
-		const callptr& call
-		);
+	/// overridden from GkAcctLogger
+	virtual Status Log(AcctEvent evt, const callptr & call);
 
-	/// overriden from GkAcctLogger
-	virtual Status Log(
-		AcctEvent evt,
-		const endptr& ep
-		);
+	/// overridden from GkAcctLogger
+	virtual Status Log(AcctEvent evt, const endptr & ep);
 
-	/// overriden from GkAcctLogger
-	virtual PString EscapeAcctParam(const PString& param) const;
-
-	/// overriden from GkAcctLogger
+	/// overridden from GkAcctLogger
 	PString ReplaceAcctParams(
 		/// parametrized accounting string
-		const PString& cdrStr,
+		const PString & cdrStr,
 		/// parameter values
-		const std::map<PString, PString>& params
+		const std::map<PString, PString> & params
 	) const;
-		
+
 private:
 	StatusAcct();
 	/* No copy constructor allowed */
-	StatusAcct(const StatusAcct&);
+	StatusAcct(const StatusAcct &);
 	/* No operator= allowed */
-	StatusAcct& operator=(const StatusAcct&);
+	StatusAcct & operator=(const StatusAcct &);
 
 private:
 	/// parametrized string for the call start event
@@ -87,6 +78,10 @@ private:
 	PString m_registerEvent;
 	/// parametrized string for the endpoint un-register event
 	PString m_unregisterEvent;
+	/// parametrized string for the call reject event
+	PString m_rejectEvent;
+	/// parametrized string for the call mediafail event
+	PString m_mediaFailEvent;
 	/// timestamp formatting string
 	PString m_timestampFormat;
 };
