@@ -3,7 +3,7 @@
  *
  * accounting module for GNU Gatekeeper for the syslog.
  *
- * Copyright (c) 2006-2010, Jan Willamowius
+ * Copyright (c) 2006-2023, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -11,13 +11,11 @@
  * with the OpenH323/H323Plus and OpenSSL library.
  *
  */
- 
-#ifndef _WIN32
 
+#if !defined(_WIN32) && !defined(_WIN64)
+
+#include "config.h"
 #include <syslog.h>
-#include <ptlib.h>
-#include <h323pdu.h>
-#include "GkStatus.h"
 #include "syslogacct.h"
 #include "Toolkit.h"
 
@@ -25,20 +23,16 @@ const char* const SyslogSec = "SyslogAcct";
 static int syslog_level = LOG_INFO;
 static int syslog_facility = LOG_USER;
 
-SyslogAcct::SyslogAcct( 
-	const char* moduleName,
-	const char* cfgSecName
-	)
-	:
-	GkAcctLogger(moduleName, cfgSecName)
+SyslogAcct::SyslogAcct(const char* moduleName, const char* cfgSecName)
+	: GkAcctLogger(moduleName, cfgSecName)
 {
 	// it is very important to set what type of accounting events
 	// are supported for each accounting module, otherwise the Log method
 	// will no get called
 	SetSupportedEvents(SyslogAcctEvents);
 
-	PConfig* cfg = GetConfig();
-	const PString& cfgSec = GetConfigSectionName();
+	PConfig * cfg = GetConfig();
+	const PString & cfgSec = GetConfigSectionName();
 	m_timestampFormat = cfg->GetString(cfgSec, "TimestampFormat", "");
 	m_startEvent = cfg->GetString(cfgSec, "StartEvent", "CALL|Start|%{caller-ip}:%{caller-port}|%{callee-ip}:%{callee-port}|%{CallId}");
 	m_stopEvent = cfg->GetString(cfgSec, "StopEvent", "CALL|Stop|%{caller-ip}:%{caller-port}|%{callee-ip}:%{callee-port}|%{CallId}");
@@ -50,16 +44,13 @@ SyslogAcct::~SyslogAcct()
 {
 }
 
-GkAcctLogger::Status SyslogAcct::Log(
-	GkAcctLogger::AcctEvent evt, 
-	const callptr& call
-	)
+GkAcctLogger::Status SyslogAcct::Log(GkAcctLogger::AcctEvent evt, const callptr & call)
 {
 	// a workaround to prevent processing end on "sufficient" module
 	// if it is not interested in this event type
 	if ((evt & GetEnabledEvents() & GetSupportedEvents()) == 0)
 		return Next;
-		
+
 	if (!call) {
 		PTRACE(1, "SYSLOGACCT\t" << GetName() << " - missing call info for event " << evt);
 		return Fail;
@@ -138,7 +129,7 @@ GkAcctLogger::Status SyslogAcct::Log(
 	return Ok;
 }
 
-PString SyslogAcct::EscapeAcctParam(const PString& param) const
+PString SyslogAcct::EscapeAcctParam(const PString & param) const
 {
 	return "\"" + param + "\"";	// test: quote
 }
