@@ -3,7 +3,7 @@
 // PURPOSE OF THIS FILE: Give version info
 //
 // Copyright (C) 2003 Nils.Bokermann@mediaways.net
-// Copyright (c) 2006-2012, Jan Willamowius
+// Copyright (c) 2006-2023, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -14,7 +14,6 @@
 
 #include <ptlib.h>
 #include <ptlib/sockets.h>
-#include "versionts.h"
 #include "version.h"
 #include "Toolkit.h"
 #include "config.h"
@@ -22,26 +21,26 @@
 const PString Toolkit::GKVersion()
 {
 	return PString(PString::Printf,
-		       "Gatekeeper(%s) Version(%s) Ext(pthreads=%d,radius=%d,mysql=%d,pgsql=%d,firebird=%d,odbc=%d,sqlite=%d,large_fdset=%d,crypto/ssl=%d,h46018=%d,h46023=%d,ldap=%d,ssh=%d,ipv6=%d,h235media=%d,lua=%d,h46017=%d,snmp=%d,h46026=%d)"
+		       "Gatekeeper(%s) Version(%s) Ext(amqp=%d,avaya=%d,curl=%d,crypto/ssl=%d,firebird=%d,geoip=%d,h235media=%d,h46017=%d,h46018=%d,h46023=%d,h46026=%d,ipv6=%d,json=%d,large_fdset=%d,ldap=%d,lua=%d,mqtt=%d,mysql=%d,odbc=%d,olm=%d,oracle=%d,pgsql=%d,pthreads=%d,radius=%d,redis=%d,snmp=%d,ssh=%d,sqlite=%d)"
 		       " H323Plus(%d.%d.%d) PTLib(%d.%d.%d) Build(%s, %s) Sys(%s %s %s)\r\n",
 		       (const unsigned char*)(PProcess::Current().GetManufacturer()),
 		       (const unsigned char*)(PProcess::Current().GetVersion(true)),
-#ifdef P_PTHREADS
+#if HAS_LIBRABBITMQ
 				(int)1,
 #else
 				(int)0,
 #endif
-#if HAS_RADIUS
+#if HAS_AVAYA_SUPPORT
 				(int)1,
 #else
 				(int)0,
 #endif
-#if HAS_MYSQL
+#if HAS_LIBCURL
 				(int)1,
 #else
 				(int)0,
 #endif
-#if HAS_PGSQL
+#if P_SSL
 				(int)1,
 #else
 				(int)0,
@@ -51,22 +50,17 @@ const PString Toolkit::GKVersion()
 #else
 				(int)0,
 #endif
-#if HAS_ODBC
+#if HAS_GEOIP
 				(int)1,
 #else
 				(int)0,
 #endif
-#if HAS_SQLITE
+#ifdef HAS_H235_MEDIA
 				(int)1,
 #else
 				(int)0,
 #endif
-#ifdef LARGE_FDSET
-				(int)LARGE_FDSET,
-#else
-				(int)0,
-#endif
-#if P_SSL
+#if HAS_H46017
 				(int)1,
 #else
 				(int)0,
@@ -81,12 +75,7 @@ const PString Toolkit::GKVersion()
 #else
 				(int)0,
 #endif
-#if P_LDAP
-				(int)1,
-#else
-				(int)0,
-#endif
-#if HAS_LIBSSH
+#if HAS_H46026
 				(int)1,
 #else
 				(int)0,
@@ -96,7 +85,17 @@ const PString Toolkit::GKVersion()
 #else
 				(int)0,
 #endif
-#ifdef HAS_H235_MEDIA
+#ifdef HAS_JSON
+				(int)1,
+#else
+				(int)0,
+#endif
+#ifdef LARGE_FDSET
+				(int)1,
+#else
+				(int)0,
+#endif
+#if P_LDAP
 				(int)1,
 #else
 				(int)0,
@@ -106,7 +105,47 @@ const PString Toolkit::GKVersion()
 #else
 				(int)0,
 #endif
-#if HAS_H46017
+#if HAS_LIBMOSQUITTO
+				(int)1,
+#else
+				(int)0,
+#endif
+#if HAS_MYSQL
+				(int)1,
+#else
+				(int)0,
+#endif
+#if HAS_ODBC
+				(int)1,
+#else
+				(int)0,
+#endif
+#if HAS_OLM
+				(int)1,
+#else
+				(int)0,
+#endif
+#if HAS_ORACLE
+				(int)1,
+#else
+				(int)0,
+#endif
+#if HAS_PGSQL
+				(int)1,
+#else
+				(int)0,
+#endif
+#ifdef P_PTHREADS
+				(int)1,
+#else
+				(int)0,
+#endif
+#if HAS_RADIUS
+				(int)1,
+#else
+				(int)0,
+#endif
+#if HAS_REDIS
 				(int)1,
 #else
 				(int)0,
@@ -116,7 +155,12 @@ const PString Toolkit::GKVersion()
 #else
 				(int)0,
 #endif
-#if HAS_H46026
+#if HAS_LIBSSH
+				(int)1,
+#else
+				(int)0,
+#endif
+#if HAS_SQLITE
 				(int)1,
 #else
 				(int)0,
