@@ -3,7 +3,7 @@
 // yasocket.cxx
 //
 // Copyright (c) Citron Network Inc. 2002-2003
-// Copyright (c) 2004-2013, Jan Willamowius
+// Copyright (c) 2004-2023, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -12,6 +12,7 @@
 //
 //////////////////////////////////////////////////////////////////
 
+#include "config.h"
 #include <ptlib.h>
 #include "h323util.h"
 #include "stl_supp.h"
@@ -19,16 +20,36 @@
 #include "yasocket.h"
 #include "Toolkit.h"
 #include "snmp.h"
+#include "factory.h"
 #include "gk.h"
+#include "gnugkbuildopts.h"
+#include "ptbuildopts.h"	// sets WINVER needed for WSAPoll
 
-#ifdef _WIN32
+#if _WIN32 || _WIN64
 #	ifndef SHUT_RDWR
 #   	define SHUT_RDWR SD_BOTH
 #	endif
-#endif
+#	ifndef SHUT_WR
+#   	define SHUT_WR SD_SEND
+#	endif
+#else
+#include <unistd.h>
+#endif // _WIN32
 
+#ifdef LARGE_FDSET
+#if _WIN32 || _WIN64
+#	include <winsock2.h>
+#	define poll WSAPoll
+#else
+#	include <poll.h>
+#endif // _WIN32
+#endif // LARGE_FDSET
+
+
+#if (__cplusplus < 201703L) // before C++17
 using std::mem_fun;
 using std::bind1st;
+#endif
 using std::partition;
 using std::distance;
 using std::copy;
@@ -38,7 +59,7 @@ using std::find;
 namespace {
 /// time to recheck state of closed sockets owned by a proxy handler
 const long SOCKETSREADER_IDLE_TIMEOUT = 1000;
-const long SOCKET_CHUNK_PAUSE = 250;	// send in 10K chunks
+const long SOCKET_CHUNK_PAUSE = 250;	// pause 250ms between chunks
 const int MAX_SOCKET_CHUNK = 10240;	// send in 10K chunks
 }
 
@@ -48,83 +69,32 @@ int g_maxSocketQueue = 100;	// set with [Gatekeeper::Main] MaxSocketQueue=
 
 bool YaSelectList::Select(SelectType t, const PTimeInterval & timeout)
 {
-	large_fd_set fdset;
-	// add handles to the fdset
-	const_iterator i = fds.begin();
-	const_iterator endIter = fds.end();
-	while (i != endIter)
-		fdset.add((*i++)->GetHandle());
+	struct pollfd * pfds = new pollfd[GetSize()]; // dynamic alloc for VS2008
+    memset(pfds, 0 , sizeof(*pfds));
+    // add handles to pollfd
+    for (int i = 0; i < GetSize(); ++i) {
+        pfds[i].fd = fds[i]->GetHandle();
+        pfds[i].events = (t == Read) ? POLLIN : POLLOUT;
+    }
 
-	fd_set *readfds, *writefds;
-	if (t == Read)
-		readfds = fdset, writefds = NULL;
-	else
-		writefds = fdset, readfds = NULL;
-
-	const unsigned long msec = timeout.GetInterval();
-	struct timeval tval;
-	tval.tv_sec = msec / 1000;
-	tval.tv_usec = (msec - tval.tv_sec * 1000) * 1000;
-	int r = ::select(maxfd + 1, readfds, writefds, 0, &tval);
+	const int msec = timeout.GetInterval();
+	int r = ::poll(pfds, GetSize(), msec);
 	if (r > 0) {
-#if 1
-		std::vector<YaSocket*>::iterator last = remove_if(
-			fds.begin(), fds.end(),
-			not1(compose1(
-				bind1st(mem_fun(&large_fd_set::has), &fdset),
-				mem_fun(&YaSocket::GetHandle)
-				)));
-		fds.erase(last, fds.end());
-#else
-		/* This unrolled implementation of the above code may give
-		   another 10-15% of performance gain. As it is not much under normal
-		   conditions, I leave it for thouse who want to squeeze a few more
-		   calls from their proxies;-)
-
-		   I did some performance tests (Duron 1.1GHz) with simulation of
-		   various fds selected sockets coverage (10%, 33%, 50%, 75%, 90%):
-
-		   LARGE_FDSET=1024  - 12% performance gain
-		   LARGE_FDSET=4096  - 15% performance gain
-		   LARGE_FDSET=16384 - 13% performance gain
-
-		   For LARGE_FDSET=4096 it took less than 1ms to manipulate the fdset
-		   and this grows in a linear fashion (LARGE_FDSET=16384 takes a few
-		   milliseconds to perform the same task).
-		*/
-		std::vector<YaSocket*>::reverse_iterator j = fds.rbegin();
-		std::vector<YaSocket*>::iterator k = fds.end();
-		const std::vector<YaSocket*>::reverse_iterator rendIter = fds.rend();
-		bool hasfd = false;
-		
-		// start from the end of the list, skip consecutive sockets
-		// that were not selected (find the first one selected)
-		while (j != rendIter) {
-			k--;
-			if (fdset.has((*j)->GetHandle())) {
-				hasfd = true;
-				break;
-			} else
-				++j;
-		}
-		// reorder remaining sockets, so non-selected sockets
-		// are moved to the end of the vector
-		if (hasfd) {
-			while (++j != rendIter) {
-				if (!fdset.has((*j)->GetHandle())) {
-					*j = *k--;
-				}
-			}
-			// at this point the vector [begin(),k] should contain
-			// all selected sockets, so erase the remaining vector elements
-			fds.erase(++k, fds.end());
-		} else {
-			fds.clear();
-		}
-#endif
+    	iterator i = fds.begin();
+    	int j = 0;
+	    while (i != fds.end()) {
+            if (pfds[j].revents & (t == Read ? POLLIN : POLLOUT)) {
+                // keep and move to next element
+	            ++i;
+            } else {
+                i = fds.erase(i);	// first erase then move to next valid element
+            }
+            ++j; // always to to next element in pfds array
+        }
 	} else if (r < 0) {
-		PTRACE(3, GetName() << "\tSelect " << (t == Read ? "read" : "write") << " error - errno: " << errno);
+		PTRACE(3, GetName() << "\tSelect (poll) " << (t == Read ? "read" : "write") << " error - errno: " << errno);
 	}
+	delete [] pfds;
 	return r > 0;
 }
 
@@ -157,16 +127,24 @@ bool YaSocket::Close()
 	struct linger so_linger;
 	so_linger.l_onoff = 0;
 	so_linger.l_linger = 0;
-	::setsockopt(handle, SOL_SOCKET, SO_LINGER, &so_linger, sizeof(so_linger));
+	(void)::setsockopt(handle, SOL_SOCKET, SO_LINGER, (const char *)&so_linger, sizeof(so_linger));
+#if _WIN32 || _WIN64
+	::closesocket(handle);
+#else
 	::close(handle);
+#endif
 	return true;
 }
 
-bool YaSocket::Read(void * buf, int sz)
+bool YaSocket::Read(void * buf, int sz, bool wantZeroReads)
 {
 	int r = os_recv(buf, sz);
-	lastReadCount = ConvertOSError(r, PSocket::LastReadError) ? r : 0;
-	return lastReadCount > 0;
+    lastReadCount = ConvertOSError(r, PSocket::LastReadError) ? r : 0;
+	if (wantZeroReads) {
+        return lastReadCount >= 0;
+	} else {
+        return lastReadCount > 0;
+	}
 }
 
 bool YaSocket::CanRead(long timeout) const
@@ -175,13 +153,11 @@ bool YaSocket::CanRead(long timeout) const
 	if (h < 0)
 		return false;
 
-	YaSelectList::large_fd_set fdset;
-	fdset.add(h);
-
-	struct timeval tval;
-	tval.tv_sec = timeout / 1000;
-	tval.tv_usec = (timeout - tval.tv_sec * 1000) * 1000;
-	return ::select(h + 1, (fd_set*)fdset, NULL, NULL, &tval) > 0;
+    struct pollfd fds[1];
+    memset(fds, 0 , sizeof(fds));
+    fds[0].fd = h;
+    fds[0].events = POLLIN;
+    return ::poll(fds, 1, timeout) > 0;
 }
 
 bool YaSocket::CanWrite(long timeout) const
@@ -189,14 +165,12 @@ bool YaSocket::CanWrite(long timeout) const
 	const int h = os_handle;
 	if (h < 0)
 		return false;
-		
-	YaSelectList::large_fd_set fdset;
-	fdset.add(h);
 
-	struct timeval tval;
-	tval.tv_sec = timeout / 1000;
-	tval.tv_usec = (timeout - tval.tv_sec * 1000) * 1000;
-	return ::select(h + 1, NULL, (fd_set*)fdset, NULL, &tval) > 0;
+    struct pollfd fds[1];
+    memset(fds, 0 , sizeof(fds));
+    fds[0].fd = h;
+    fds[0].events = POLLOUT;
+    return ::poll(fds, 1, timeout) > 0;
 }
 
 bool YaSocket::Write(const void * buf, int sz)
@@ -218,26 +192,27 @@ PBoolean YaSocket::GetLocalAddress(Address & addr) const
 	return GetLocalAddress(addr, pt);
 }
 
+// now returns the IP used by the client for sockets bound to INADDR_ANY, unlike PTLib
 PBoolean YaSocket::GetLocalAddress(Address & addr, WORD & pt) const
 {
 #ifdef hasIPV6
-	sockaddr_in6 inaddr;
+   	sockaddr_in6 inaddr;
 #else
-	sockaddr_in inaddr;
+    sockaddr_in inaddr;
 #endif
-	socklen_t insize = sizeof(inaddr);
-	if (::getsockname(os_handle, (struct sockaddr*)&inaddr, &insize) == 0) {
+    socklen_t insize = sizeof(inaddr);
+    if (::getsockname(os_handle, (struct sockaddr*)&inaddr, &insize) == 0) {
 #ifdef hasIPV6
-		if (((struct sockaddr*)&inaddr)->sa_family == AF_INET6) {
-			addr = ((struct sockaddr_in6*)&inaddr)->sin6_addr;
-			pt = ntohs(((struct sockaddr_in6*)&inaddr)->sin6_port);
-		} else
+	    if (((struct sockaddr*)&inaddr)->sa_family == AF_INET6) {
+		    addr = ((struct sockaddr_in6*)&inaddr)->sin6_addr;
+		    pt = ntohs(((struct sockaddr_in6*)&inaddr)->sin6_port);
+	    } else
 #endif
-		{
-			addr = ((struct sockaddr_in*)&inaddr)->sin_addr;
-			pt = ntohs(((struct sockaddr_in*)&inaddr)->sin_port);
-		}
-	}
+	    {
+		    addr = ((struct sockaddr_in*)&inaddr)->sin_addr;
+		    pt = ntohs(((struct sockaddr_in*)&inaddr)->sin_port);
+	    }
+    }
 	return true;
 }
 
@@ -282,11 +257,17 @@ bool YaSocket::SetNonBlockingMode()
 {
 	if (!IsOpen())
 		return false;
-	// is call to F_SETFD with F_CLOEXEC really neccessary?
+#if _WIN32 || _WIN64
+	u_long cmd = 1;
+	if (ConvertOSError(::ioctlsocket(os_handle, FIONBIO, &cmd)))
+		return true;
+#else
+	// is call to F_SETFD with F_CLOEXEC really necessary ?
 	int cmd = 1;
 	if (ConvertOSError(::ioctl(os_handle, FIONBIO, &cmd))
 			&& ConvertOSError(::fcntl(os_handle, F_SETFD, 1)))
 		return true;
+#endif
 	Close();
 	return false;
 
@@ -327,6 +308,7 @@ bool YaSocket::Bind(const Address & addr, WORD pt)
 // class YaTCPSocket
 YaTCPSocket::YaTCPSocket(WORD pt)
 {
+    memset(&peeraddr, 0, sizeof(peeraddr));
 	((struct sockaddr*)&peeraddr)->sa_family = AF_INET;		// overwritten in Connect()
 	SetPort(pt);
 }
@@ -387,7 +369,7 @@ bool YaTCPSocket::Listen(const Address & addr, unsigned qs, WORD pt, PSocket::Re
 
 	if (!SetOption(SO_REUSEADDR, reuse == PSocket::CanReuseAddress ? 1 : 0))
 		return false;
-	
+
 //	SetNonBlockingMode();
 	if (Bind(addr, pt) && ConvertOSError(::listen(os_handle, qs)))
 		return true;
@@ -412,10 +394,12 @@ bool YaTCPSocket::Accept(YaTCPSocket & socket)
 			break;
 		}
 
-		YaSelectList::large_fd_set fdset;
-		fdset.add(fd);
-		struct timeval tval = { 1, 0 };
-		int r = ::select(fd + 1, fdset, 0, 0, &tval);
+        struct pollfd fds[1];
+        memset(fds, 0 , sizeof(fds));
+        fds[0].fd = fd;
+        fds[0].events = POLLIN;
+        int timeout = 1000; // 1 sec
+        int r = ::poll(fds, 1, timeout);
 		if (r < 0)
 			break;
 		else if (r == 0)
@@ -472,33 +456,34 @@ bool YaTCPSocket::Connect(const Address & iface, WORD localPort, const Address &
 	SetSockaddr(peeraddr, addr, peerPort);
 	SetName(AsString(addr, port));
 
-#ifdef P_FREEBSD
-	// FreeBSD is extra picky about the size argument, not sure if thats the correct size for Linux
-	int r = ::connect(os_handle, (struct sockaddr*)&peeraddr, sizeof(struct sockaddr));
-#else
-	int r = ::connect(os_handle, (struct sockaddr*)&peeraddr, sizeof(peeraddr));
-#endif
+    size_t addr_len = sizeof(sockaddr_in);
+#ifdef hasIPV6
+    if (((struct sockaddr*)&peeraddr)->sa_family == AF_INET6)
+        addr_len = sizeof(sockaddr_in6);
+#endif  // hasIPV6
+	int r = ::connect(os_handle, (struct sockaddr*)&peeraddr, addr_len);
 
-#ifdef _WIN32
+#if _WIN32 || _WIN64
 	if ((r != 0) && (WSAGetLastError() != WSAEWOULDBLOCK))
 #else
 	if (r == 0 || errno != EINPROGRESS)
 #endif
 		return ConvertOSError(r);
 
-	YaSelectList::large_fd_set fdset;
-	fdset.add(os_handle);
-	YaSelectList::large_fd_set exset = fdset;
-	struct timeval tval = { 6, 0 };
-	if ((r = ::select(os_handle + 1, 0, fdset, exset, &tval)) > 0) {
+    struct pollfd fds[1];
+    memset(fds, 0 , sizeof(fds));
+    fds[0].fd = os_handle;
+    fds[0].events = POLLOUT;
+    int timeout = 6000; // 6 sec
+    if ((r = ::poll(fds, 1, timeout)) > 0) {
 		optval = -1;
-		::getsockopt(os_handle, SOL_SOCKET, SO_ERROR, &optval, &optlen);
+		(void)::getsockopt(os_handle, SOL_SOCKET, SO_ERROR, (char *)&optval, &optlen);
 		if (optval == 0) // connected
 			return SetLinger();
 		errno = optval;
 	} else {
 		if (r == 0) {
-#ifdef _WIN32
+#if _WIN32 || _WIN64
 			errno = WSAETIMEDOUT;
 #else
 			errno = ETIMEDOUT;
@@ -518,7 +503,7 @@ int YaTCPSocket::os_recv(void * buf, int sz)
 #if HAS_MSG_NOSIGNAL
 	return ::recv(os_handle, buf, sz, MSG_NOSIGNAL);
 #else
-	return ::recv(os_handle, buf, sz, 0);
+	return ::recv(os_handle, (char *)buf, sz, 0);
 #endif
 }
 
@@ -527,7 +512,7 @@ int YaTCPSocket::os_send(const void * buf, int sz)
 #if HAS_MSG_NOSIGNAL
 	return ::send(os_handle, buf, sz, MSG_NOSIGNAL);
 #else
-	return ::send(os_handle, buf, sz, 0);
+	return ::send(os_handle, (const char *)buf, sz, 0);
 #endif
 }
 
@@ -538,6 +523,7 @@ YaUDPSocket::YaUDPSocket(WORD port, int iAddressFamily)
 	memset(&recvaddr, 0, sizeof(recvaddr));
 	((struct sockaddr*)&sendaddr)->sa_family = iAddressFamily;
 	((struct sockaddr_in*)&sendaddr)->sin_port = htons(port);
+	SetInvalid(lastDestAddress);
 }
 
 bool YaUDPSocket::Listen(unsigned, WORD pt, PSocket::Reusability reuse)
@@ -550,7 +536,7 @@ bool YaUDPSocket::Listen(const Address & addr, unsigned, WORD pt, PSocket::Reusa
 #ifdef hasIPV6
 	if (addr.GetVersion() == 6) {
 		os_handle = ::socket(PF_INET6, SOCK_DGRAM, 0);
-		if (addr.IsAny() && !SetOption(IPV6_V6ONLY, 0, IPPROTO_IPV6)) {
+		if (os_handle >= 0 && addr.IsAny() && !SetOption(IPV6_V6ONLY, 0, IPPROTO_IPV6)) {
 			PTRACE(1, "Removing of IPV6_V6ONLY failed");
 			SNMP_TRAP(10, SNMPWarning, Network, "IPv6 error");
 		}
@@ -627,12 +613,103 @@ bool YaUDPSocket::WriteTo(const void * buf, PINDEX len, const Address & addr, WO
 int YaUDPSocket::os_recv(void * buf, int sz)
 {
 	socklen_t addrlen = sizeof(recvaddr);
-	return ::recvfrom(os_handle, buf, sz, 0, (struct sockaddr *)&recvaddr, &addrlen);
+#if !defined(IP_PKTINFO) && !defined(IP_RECVDSTADDR)
+	return ::recvfrom(os_handle, (char *)buf, sz, 0, (struct sockaddr *)&recvaddr, &addrlen);
+#endif
+
+#if defined(IP_PKTINFO) || defined(IP_RECVDSTADDR)
+    // TODO: move setsockopts right after socket creation and do it only once ?
+    int yes = 1;
+    int e = 0;
+#ifdef IP_PKTINFO
+    // Linux
+    e = setsockopt(os_handle, IPPROTO_IP, IP_PKTINFO, &yes, sizeof(yes));
+#else
+#ifdef IP_RECVDSTADDR
+    // FreeBSD (also *BSD, MacOS X ?)
+    e = setsockopt(os_handle, IPPROTO_IP, IP_RECVDSTADDR, &yes, sizeof(yes));
+#endif
+#endif
+    if (e != 0) {
+        PTRACE(1, "Error: setsockopt IP_PKTINFO=" << errno);
+    }
+#ifdef hasIPV6
+	if (Toolkit::Instance()->IsIPv6Enabled()) {
+#ifdef IPV6_RECVPKTINFO
+        // Linux
+        e = setsockopt(os_handle, IPPROTO_IPV6, IPV6_RECVPKTINFO, &yes, sizeof(yes));
+#else
+#ifdef IPV6_PKTINFO
+        // Solaris (also BSD, Windows ?)
+        e = setsockopt(os_handle, IPPROTO_IPV6, IPV6_PKTINFO, &yes, sizeof(yes));
+#endif
+#endif
+        if (e != 0) {
+            PTRACE(1, "Error: setsockopt IPV6_PKTINFO=" << errno);
+        }
+	}
+#endif // hasIPV6
+    struct iovec vec;
+    const size_t CONTROL_DATA_SIZE = 1024;
+    char cmsg[CONTROL_DATA_SIZE];
+    struct msghdr hdr = {};
+    memset(cmsg, 0, CONTROL_DATA_SIZE);
+
+    vec.iov_base = buf;
+    vec.iov_len = sz;
+
+    hdr.msg_name = &recvaddr;
+    hdr.msg_namelen = addrlen;
+    hdr.msg_iov = &vec;
+    hdr.msg_iovlen = 1;
+    hdr.msg_control = cmsg;
+    hdr.msg_controllen = sizeof(cmsg);
+
+    int result = ::recvmsg(os_handle, &hdr, 0);
+    PIPSocket::Address raddr;
+    WORD rpt;
+    GetLastReceiveAddress(raddr, rpt);
+
+    for ( // iterate through all control headers
+        struct cmsghdr *cmsg = CMSG_FIRSTHDR(&hdr);
+        cmsg != NULL;
+        cmsg = CMSG_NXTHDR(&hdr, cmsg))
+    {
+#ifdef IP_PKTINFO
+        if (cmsg->cmsg_level == IPPROTO_IP && cmsg->cmsg_type == IP_PKTINFO) {
+            struct in_pktinfo * pi = (struct in_pktinfo *)CMSG_DATA(cmsg);
+            // pi->ipi_addr is our IP that the endpoint sent to (in_addr)
+            lastDestAddress = pi->ipi_addr;
+        }
+#endif
+#ifdef IP_RECVDSTADDR
+        if (cmsg->cmsg_level == IPPROTO_IP && cmsg->cmsg_type == IP_RECVDSTADDR) {
+            struct in_addr * i = (struct in_addr *)CMSG_DATA(cmsg);
+            lastDestAddress = *i;
+        }
+#endif
+#if defined(hasIPV6) && defined (IPV6_PKTINFO)
+        if (cmsg->cmsg_level == IPPROTO_IPV6 && cmsg->cmsg_type == IPV6_PKTINFO) {
+            struct in6_pktinfo * pi = (struct in6_pktinfo *)CMSG_DATA(cmsg);
+            // pi->ipi_addr is our IP that the endpoint sent to (in_addr)
+            lastDestAddress = pi->ipi6_addr;
+        }
+#endif
+    }
+	return result;
+#endif
 }
 
 int YaUDPSocket::os_send(const void * buf, int sz)
 {
-	return ::sendto(os_handle, buf, sz, 0, (struct sockaddr *)&sendaddr, sizeof(sendaddr));
+	// must pass short len when sending to IPv4 address on Solaris 11, OpenBSD and NetBSD
+	// sizeof(sockaddr) would be OK on Linux and FreeBSD
+	size_t addr_len = sizeof(sockaddr_in);
+#ifdef hasIPV6
+	if (((struct sockaddr*)&sendaddr)->sa_family == AF_INET6)
+		addr_len = sizeof(sockaddr_in6);
+#endif  // hasIPV6
+	return ::sendto(os_handle, (const char *)buf, sz, 0, (struct sockaddr *)&sendaddr, addr_len);
 }
 
 #else // LARGE_FDSET
@@ -850,8 +927,9 @@ bool USocket::Flush()
 
 bool USocket::WriteData(const BYTE * buf, int len)
 {
-	if (!IsSocketOpen())
+	if (!IsSocketOpen()) {
 		return false;
+	}
 
 	int remaining = len;
 	if (qsize == 0 && writeMutex.Wait(0)) {
@@ -891,7 +969,7 @@ bool USocket::ErrorHandler(PSocket::ErrorGroup group)
 	switch (e)
 	{
 		case PSocket::Timeout:
-			PTRACE(4, msg << " Error(" << group << "): Timeout");
+			PTRACE(4, msg << " Error(" << group << "): Timeout: " + Name());
 			SNMP_TRAP(10, SNMPError, Network, "Socket timeout: " + Name());
 			break;
 		case PSocket::NoError:
@@ -904,7 +982,6 @@ bool USocket::ErrorHandler(PSocket::ErrorGroup group)
 			PTRACE(3, msg << " Error(" << group << "): "
 				<< PSocket::GetErrorText(e) << " (" << e << ':'
 				<< self->GetErrorNumber(group) << ')');
-			SNMP_TRAP(10, SNMPError, Network, "Socket error " + PSocket::GetErrorText(e));
 			CloseSocket();
 			break;
 	}
@@ -927,7 +1004,7 @@ bool USocket::InternalWriteData(const BYTE * buf, int len)
 		PTRACE(4, type << '\t' << Name() << " blocked, " << wcount << " bytes written, " << len << " bytes queued");
 		// InternalWriteData can be called
 		// either when flushing the queue (so any remaining unflushed data
-		// should be put back at the queue front) or when the queue is epmty
+		// should be put back at the queue front) or when the queue is empty
 		PWaitAndSignal lock(queueMutex);
 		std::list<PBYTEArray*>::iterator i = queue.begin();
 		while (len > 0) {
@@ -967,11 +1044,15 @@ SocketsReader::~SocketsReader()
 void SocketsReader::Stop()
 {
 	PWaitAndSignal lock(m_deletionPreventer);
-	
+
 	m_listmutex.StartWrite();
+#if (__cplusplus >= 201703L) // C++17
+	ForEachInContainer(m_sockets, mem_fn(&IPSocket::Close));
+#else
 	ForEachInContainer(m_sockets, mem_fun(&IPSocket::Close));
+#endif
 	m_listmutex.EndWrite();
-	
+
 	RegularJob::Stop();
 }
 
@@ -994,7 +1075,11 @@ void SocketsReader::AddSocket(IPSocket * socket)
 bool SocketsReader::BuildSelectList(SocketSelectList & slist)
 {
 	ReadLock lock(m_listmutex);
+#if (__cplusplus >= 201703L) // C++17
+	ForEachInContainer(m_sockets, bind(&SocketSelectList::Append, &slist, std::placeholders::_1));
+#else
 	ForEachInContainer(m_sockets, bind1st(mem_fun(&SocketSelectList::Append), &slist));
+#endif
 	return !slist.IsEmpty();
 }
 
@@ -1025,7 +1110,11 @@ bool SocketsReader::SelectSockets(SocketSelectList & slist)
 void SocketsReader::RemoveClosed(bool bDeleteImmediately)
 {
 	WriteLock listlock(m_listmutex);
+#if (__cplusplus >= 201703L) // C++17
+	iterator iter = partition(m_sockets.begin(), m_sockets.end(), mem_fn(&IPSocket::IsOpen));
+#else
 	iterator iter = partition(m_sockets.begin(), m_sockets.end(), mem_fun(&IPSocket::IsOpen));
+#endif
 	if (ptrdiff_t rmsize = distance(iter, m_sockets.end())) {
 		if (bDeleteImmediately)
 			DeleteObjects(iter, m_sockets.end());
@@ -1121,25 +1210,12 @@ bool TCPServer::CloseListener(TCPListenSocket * socket)
 void TCPServer::ReadSocket(IPSocket * socket)
 {
 	// don't accept new calls when shutdown is already in progress
-	bool shutdown = false;
-#ifdef hasNoMutexWillBlock
-	// TODO: this might be a performance bottleneck on high load
-	if (!ShutdownMutex.Wait(0)) {
-		shutdown = true;
-	} else {
-		ShutdownMutex.Signal();	// release mutex, we were just checking for shutdown
-	}
-#else
-	if (ShutdownMutex.WillBlock())
-		shutdown = true;
-#endif
-
-	if (shutdown) {
+	if (IsGatekeeperShutdown()) {
 		PTRACE(4, GetName() << "\tShutdown: Rejecting call on " << socket->GetName());
 		int rej = ::accept(socket->GetHandle(), NULL, NULL);
 		if (rej >= 0) {
 			::shutdown(rej, SHUT_RDWR);
-#if defined(_WIN32)
+#if _WIN32 || _WIN64
 			::closesocket(rej);
 #else
 			::close(rej);
@@ -1153,8 +1229,13 @@ void TCPServer::ReadSocket(IPSocket * socket)
 	if (cps_limit > 0) {
 		time_t now = time(NULL);
 		// clear old values
+#if (__cplusplus >= 201703L) // C++17
+		one_sec.remove_if(bind(std::not_equal_to<time_t>(), std::placeholders::_1, now));
+		many_sec.remove_if(bind(std::less<time_t>(), std::placeholders::_1, now - check_interval));
+#else
 		one_sec.remove_if(bind2nd(not_equal_to<time_t>(), now));
 		many_sec.remove_if(bind2nd(less<time_t>(), now - check_interval));
+#endif
 		PTRACE(4, GetName() << "\tcurrent cps=" << one_sec.size() << " calls in interval=" << many_sec.size());
 		if ((many_sec.size() > (cps_limit * check_interval)) && (one_sec.size() > cps_limit)) {
 			// reject call
@@ -1162,7 +1243,7 @@ void TCPServer::ReadSocket(IPSocket * socket)
 			int rej = ::accept(socket->GetHandle(), NULL, NULL);
 			if (rej >= 0) {
 				::shutdown(rej, SHUT_RDWR);
-#if defined(_WIN32)
+#if _WIN32 || _WIN64
 				::closesocket(rej);
 #else
 				::close(rej);
@@ -1180,7 +1261,7 @@ void TCPServer::ReadSocket(IPSocket * socket)
 		return;
 
 	ServerSocket *acceptor = listener->CreateAcceptor();
-	if (acceptor->Accept(*listener)) {
+	if (acceptor && acceptor->Accept(*listener)) {
 		PTRACE(6, GetName() << "\tAccepted new connection on " << socket->GetName() << " from " << acceptor->GetName());
 		CreateJob(acceptor, &ServerSocket::Dispatch, "Acceptor");
 	} else {
