@@ -3,7 +3,7 @@
 // yasocket.h
 //
 // Copyright (c) Citron Network Inc. 2002-2003
-// Copyright (c) 2004-2013, Jan Willamowius
+// Copyright (c) 2004-2021, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -13,17 +13,17 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef YASOCKET_H
-#define YASOCKET_H "@(#) $Id: yasocket.h,v 1.54 2013/11/14 10:44:21 willamowius Exp $"
+#define YASOCKET_H "@(#) $Id$"
 
+#include "config.h"
 #include <list>
 #include <vector>
 #include <ptlib/sockets.h>
 #include "job.h"
-#include "config.h"
 
 #ifdef LARGE_FDSET
 
-// yet another socket class to replace PSocket (Unix only)
+// yet another socket class to replace PSocket (Unix and Windows >= Vista)
 
 class YaSocket : public NamedObject {
 public:
@@ -37,11 +37,11 @@ public:
 	bool Close();
 
 	void SetReadTimeout(const PTimeInterval & time) { readTimeout = time; }
-	virtual bool Read(void *, int);
+	virtual bool Read(void * buf, int sz, bool wantZeroReads = false);
 	virtual int GetLastReadCount() const { return lastReadCount; }
 
 	void SetWriteTimeout(const PTimeInterval & time) { writeTimeout = time; }
-	virtual bool Write(const void *, int);
+	virtual bool Write(const void * buf, int sz);
 	virtual int GetLastWriteCount() const { return lastWriteCount; }
 
 	void SetPort(WORD pt) { port = pt; }
@@ -60,7 +60,7 @@ public:
 
 	virtual bool CanRead(long timeout) const;
 	virtual bool CanWrite(long timeout) const;
-		
+
 protected:
 	virtual int os_recv(void *, int) = 0;
 	virtual int os_send(const void *, int) = 0;
@@ -140,6 +140,8 @@ public:
 	virtual bool ReadFrom(void *, PINDEX, Address &, WORD);
 	virtual bool WriteTo(const void *, PINDEX, const Address &, WORD);
 
+	virtual PBoolean GetLastDestAddress(Address & addr) const { addr = lastDestAddress; return true; }
+
 protected:
 	// override from class YaSocket
 	virtual int os_recv(void *, int);
@@ -155,6 +157,7 @@ private:
 #else
 	sockaddr_in recvaddr, sendaddr;
 #endif
+	Address lastDestAddress;
 };
 
 class YaSelectList {
@@ -166,7 +169,7 @@ public:
 	YaSelectList(
 		/// estimated number of sockets to be put in this select list
 		size_t reserve = 512
-	) : maxfd(0) { fds.reserve(reserve); }
+	) { fds.reserve(reserve); }
 
 	/// build a select list for more than one socket
 	YaSelectList(
@@ -174,19 +177,19 @@ public:
 		const PString & name,
 		/// estimated number of sockets to be put in this select list
 		size_t reserve = 512
-	) : maxfd(0), m_name(name) { fds.reserve(reserve); }
+	) : m_name(name) { fds.reserve(reserve); }
 
-	/// build a select list for signle socket only
+	/// build a select list for single socket only
 	YaSelectList(
 		YaSocket * singleSocket /// socket to be put on the list
-		) : fds(1, singleSocket), maxfd(singleSocket->GetHandle()) { }
+		) : fds(1, singleSocket) { }
 
-	/// build a select list for signle socket only
+	/// build a select list for single socket only
 	YaSelectList(
 		/// name for this select list
 		const PString & name,
 		YaSocket* singleSocket /// socket to be put on the list
-		) : fds(1, singleSocket), maxfd(singleSocket->GetHandle()), m_name(name) { }
+		) : fds(1, singleSocket), m_name(name) { }
 
 	void Append(
 		YaSocket * s /// the socket to be appended
@@ -194,8 +197,6 @@ public:
 	{
 		if (s && s->IsOpen()) {
 			fds.push_back(s);
-			if (s->GetHandle() > maxfd)
-				maxfd = s->GetHandle();
 		}
 	}
 
@@ -210,23 +211,10 @@ public:
 
 	bool Select(SelectType, const PTimeInterval &);
 
-	struct large_fd_set {
-		large_fd_set() { memset(this, 0, sizeof(large_fd_set)); }
-		void add(int fd) { if (fd >= 0 && fd < LARGE_FDSET) FD_SET(fd, &__fdset__); }
-		bool has(int fd) { return (fd >= 0 && fd < LARGE_FDSET) ? FD_ISSET(fd, &__fdset__) : false; }
-		operator fd_set *() { return &__fdset__; }
-
-		union {
-			fd_set __fdset__;
-			char __mem__[LARGE_FDSET / 8];
-		};
-	};
-
 	PString GetName() const { return m_name; }
 
 private:
 	std::vector<YaSocket *> fds;
-	int maxfd;
 	PString m_name;
 };
 
@@ -245,12 +233,12 @@ public:
 	};
 	SocketSelectList(size_t) { }
 	SocketSelectList(PIPSocket * s = NULL) { if (s && s->IsOpen()) Append(s); }
-	SocketSelectList(const PString &name, size_t) : m_name(name) { }
-	SocketSelectList(const PString &name, PIPSocket * s = NULL) : m_name(name) { if (s && s->IsOpen()) Append(s); }
+	SocketSelectList(const PString & name, size_t) : m_name(name) { }
+	SocketSelectList(const PString & name, PIPSocket * s = NULL) : m_name(name) { if (s && s->IsOpen()) Append(s); }
 	bool Select(SelectType, const PTimeInterval &);
 	PSocket *operator[](int i) const;
 
-	PString GetName() const { return m_name; }	
+	PString GetName() const { return m_name; }
 
 private:
 	PString m_name;
@@ -354,7 +342,7 @@ public:
 protected:
 	virtual bool WriteData(const BYTE *, int);
 	bool InternalWriteData(const BYTE *, int);
-	
+
 	int GetQueueSize() const { return qsize; }
 	void QueuePacket(const BYTE * buf, int len)
 	{
@@ -376,7 +364,7 @@ protected:
 		return packet;
 	}
 	void ClearQueue();
-		
+
 	virtual bool ErrorHandler(PSocket::ErrorGroup);
 
 	IPSocket * self;
@@ -457,7 +445,7 @@ public:
 
 	// dispatch this socket to an appropriate handler
 	virtual void Dispatch() = 0;
-	
+
 private:
 	ServerSocket(const ServerSocket &);
 	ServerSocket & operator=(const ServerSocket &);
@@ -508,7 +496,7 @@ private:
 	virtual void ReadSocket(IPSocket *);
 	virtual void CleanUp();
 
-private:	
+private:
 	// rate limiting
 	unsigned int cps_limit;	// max cps per one sec
 	unsigned int check_interval; // number of sec. to check if cps_limit applies
